{% extends "core/base.html" %}
{% load i18n %}


{% block content %}
  <div class="container">
    <div class="card red">
      <div class="card-content white-text">
        <i class="material-icons iconify small left" data-icon="mdi:alert-octagon-outline"></i>
        <span class="card-title">{% trans "Error" %} (500): {% blocktrans %}An unexpected error has
          occurred.{% endblocktrans %}</span>
        <p>
          {% blocktrans %}
            Your site administrators will automatically be notified about this
            error. You can also contact them directly:
          {% endblocktrans %}
        </p>
        {% include "core/partials/admins_list.html" %}
        <a onClick="window.location.reload();" class="btn secondary waves-effect waves-light">
          <i class="material-icons left">refresh</i>
          {% trans "Retry" %}
        </a>
      </div>
    </div>
  </div>
{% endblock %}
