# Generated by Django 3.2.11 on 2022-01-08 10:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0031_oauthapplication_icon'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='person',
            name='is_active',
        ),
    ]
