# Generated by Django 4.1.7 on 2023-03-23 18:30

from django.db import migrations, models
import django.db.models.deletion
import aleksis.core.managers


class Migration(migrations.Migration):
    dependencies = [
        ("core", "0053_freebusy"),
    ]

    operations = [
        migrations.CreateModel(
            name="Organisation",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("extended_data", models.JSONField(default=dict, editable=False)),
                ("name", models.CharField(max_length=255, verbose_name="Name")),
                ("email", models.CharField(max_length=255, verbose_name="Email")),
                (
                    "related_group",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="core.group",
                        verbose_name="Related group",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            managers=[
            ],
        ),
        migrations.AlterModelOptions(
            name="organisation",
            options={"verbose_name": "Organisation", "verbose_name_plural": "Organisations"},
        ),
        migrations.AlterModelManagers(
            name="organisation",
            managers=[
                ("objects", aleksis.core.managers.AlekSISBaseManager()),
            ],
        ),
        migrations.AddField(
            model_name="organisation",
            name="managed_by_app_label",
            field=models.CharField(
                blank=True,
                editable=False,
                max_length=255,
                verbose_name="App label of app responsible for managing this instance",
            ),
        ),
    ]
