{% extends "core/base.html" %}

{% load i18n material_form %}

{% block browser_title %}{% trans "Login cancelled" %}{% endblock %}
{% block page_title %}{% trans "Login cancelled" %}{% endblock %}

{% block content %}
  <div class="container">
    <div class="card red">
      <div class="card-content white-text">
        <div class="material-icons iconify small left" data-icon="mdi:alert-octagon-outline"></div>
        <span class="card-title">{% blocktrans %}Login cancelled{% endblocktrans %}</span>
        <p>
          {% blocktrans %}
            You decided to cancel logging in to our site using one of your existing accounts. If this was a mistake, please proceed to <a href="{{login_url}}">sign in</a>.
          {% endblocktrans %}
        </p>
      </div>
    </div>
  </div>
{% endblock %}
