# Generated by Django 4.1.9 on 2023-07-06 21:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0049_oauthapplication_post_logout_redirect_uris"),
    ]

    operations = [
        migrations.AddField(
            model_name="activity",
            name="managed_by_app_label",
            field=models.CharField(
                blank=True,
                editable=False,
                max_length=255,
                verbose_name="App label of app responsible for managing this instance",
            ),
        ),
        migrations.AddField(
            model_name="additionalfield",
            name="managed_by_app_label",
            field=models.CharField(
                blank=True,
                editable=False,
                max_length=255,
                verbose_name="App label of app responsible for managing this instance",
            ),
        ),
        migrations.AddField(
            model_name="announcement",
            name="managed_by_app_label",
            field=models.CharField(
                blank=True,
                editable=False,
                max_length=255,
                verbose_name="App label of app responsible for managing this instance",
            ),
        ),
        migrations.AddField(
            model_name="announcementrecipient",
            name="managed_by_app_label",
            field=models.CharField(
                blank=True,
                editable=False,
                max_length=255,
                verbose_name="App label of app responsible for managing this instance",
            ),
        ),
        migrations.AddField(
            model_name="custommenu",
            name="managed_by_app_label",
            field=models.CharField(
                blank=True,
                editable=False,
                max_length=255,
                verbose_name="App label of app responsible for managing this instance",
            ),
        ),
        migrations.AddField(
            model_name="custommenuitem",
            name="managed_by_app_label",
            field=models.CharField(
                blank=True,
                editable=False,
                max_length=255,
                verbose_name="App label of app responsible for managing this instance",
            ),
        ),
        migrations.AddField(
            model_name="dashboardwidgetorder",
            name="managed_by_app_label",
            field=models.CharField(
                blank=True,
                editable=False,
                max_length=255,
                verbose_name="App label of app responsible for managing this instance",
            ),
        ),
        migrations.AddField(
            model_name="datacheckresult",
            name="managed_by_app_label",
            field=models.CharField(
                blank=True,
                editable=False,
                max_length=255,
                verbose_name="App label of app responsible for managing this instance",
            ),
        ),
        migrations.AddField(
            model_name="group",
            name="managed_by_app_label",
            field=models.CharField(
                blank=True,
                editable=False,
                max_length=255,
                verbose_name="App label of app responsible for managing this instance",
            ),
        ),
        migrations.AddField(
            model_name="grouptype",
            name="managed_by_app_label",
            field=models.CharField(
                blank=True,
                editable=False,
                max_length=255,
                verbose_name="App label of app responsible for managing this instance",
            ),
        ),
        migrations.AddField(
            model_name="notification",
            name="managed_by_app_label",
            field=models.CharField(
                blank=True,
                editable=False,
                max_length=255,
                verbose_name="App label of app responsible for managing this instance",
            ),
        ),
        migrations.AddField(
            model_name="pdffile",
            name="managed_by_app_label",
            field=models.CharField(
                blank=True,
                editable=False,
                max_length=255,
                verbose_name="App label of app responsible for managing this instance",
            ),
        ),
        migrations.AddField(
            model_name="person",
            name="managed_by_app_label",
            field=models.CharField(
                blank=True,
                editable=False,
                max_length=255,
                verbose_name="App label of app responsible for managing this instance",
            ),
        ),
        migrations.AddField(
            model_name="persongroupthrough",
            name="managed_by_app_label",
            field=models.CharField(
                blank=True,
                editable=False,
                max_length=255,
                verbose_name="App label of app responsible for managing this instance",
            ),
        ),
        migrations.AddField(
            model_name="room",
            name="managed_by_app_label",
            field=models.CharField(
                blank=True,
                editable=False,
                max_length=255,
                verbose_name="App label of app responsible for managing this instance",
            ),
        ),
        migrations.AddField(
            model_name="schoolterm",
            name="managed_by_app_label",
            field=models.CharField(
                blank=True,
                editable=False,
                max_length=255,
                verbose_name="App label of app responsible for managing this instance",
            ),
        ),
        migrations.AddField(
            model_name="taskuserassignment",
            name="managed_by_app_label",
            field=models.CharField(
                blank=True,
                editable=False,
                max_length=255,
                verbose_name="App label of app responsible for managing this instance",
            ),
        ),
    ]
