{% extends "core/base.html" %}

{% load i18n material_form %}

{% block browser_title %}{% trans "Set password" %}{% endblock %}
{% block page_title %}{% trans "Set password" %}{% endblock %}

{% block content %}
  <form method="post" action="{% url 'account_set_password' %}">
    {% csrf_token %}
    {% form form=form %}{% endform %}
    {% trans "Set password" as caption %}
    {% include "core/partials/save_button.html" with caption=caption icon="mdi:exclamation" %}
  </form>

{% endblock %}
