{% extends "core/base.html" %}

{% load i18n %}

{% block browser_title %}{% trans "Change password" %}{% endblock %}
{% block page_title %}{% trans "Change password" %}{% endblock %}

{% block content %}
  <div class="container">
    <div class="card green">
      <div class="card-content white-text">
        <i class="material-icons iconify small left" data-icon="mdi:check"></i>
        <span class="card-title">{% blocktrans %}Password changed!{% endblocktrans %}</span>
        <p>
          {% blocktrans %}
            Your password is now changed!
          {% endblocktrans %}
        </p>
      </div>
    </div>
  </div>
{% endblock %}
