{
  myNotifications: whoAmI {
    id
    person {
      notifications {
        id
        title
        description
        link
        icon
        created
        sender
        read
      }
    }
  }
}
