# Generated by Django 4.2.7 on 2023-12-09 11:29

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('core', '0059_drop_site'),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name='person',
            name='unique_short_name',
        ),
        migrations.AlterField(
            model_name='person',
            name='short_name',
            field=models.CharField(blank=True, default='', max_length=255, verbose_name='Short name'),
            preserve_default=False,
        ),
        migrations.AddConstraint(
            model_name='person',
            constraint=models.UniqueConstraint(condition=models.Q(('short_name', ''), _negated=True), fields=('short_name',), name='unique_short_name'),
        ),
        migrations.AddConstraint(
            model_name='person',
            constraint=models.UniqueConstraint(condition=models.Q(('email', ''), _negated=True), fields=('email',), name='unique_email'),
        ),
    ]
