# Generated by Django 4.1.9 on 2023-06-17 10:59

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("core", "0048_delete_personalicalurl"),
    ]

    operations = [
        migrations.AddField(
            model_name="oauthapplication",
            name="post_logout_redirect_uris",
            field=models.TextField(
                blank=True, help_text="Allowed Post Logout URIs list, space separated"
            ),
        ),
    ]
