# Generated by Django 3.2.16 on 2022-12-16 11:50

from django.db import migrations, models
import django.utils.timezone
import oauth2_provider.generators
import oauth2_provider.models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0044_task_assignment_result_fetched'),
    ]

    operations = [
        migrations.RenameField(
            model_name='datacheckresult',
            old_name='check',
            new_name='data_check',
        ),
    ]
