{% load l10n material_form material_form_internal %}
{% part bound_field.field %}<{{ field.widget.component|default:'dmc-select' }}>
  <label>{{ bound_field.label }}</label>
  <div class="row">
    <div{% attrs bound_field 'group' %}
        id="id_{{ bound_field.html_name }}_container"
        class="input-field col s12{% if field.required %} required{% endif %}{% if bound_field.errors %} has-error{% endif %}"
        style="margin-top: 0"
    {% endattrs %}>
        {% part field prefix %}{% endpart %}
        {% part field control %}
        {{ bound_field }}
        {% endpart %}
        {% part field help_text %}{% if field.help_text %}
            <div class="help-block">{{ bound_field.help_text|safe }}</div>
        {% endif %}{% endpart %}{% part field errors %}
        {% if bound_field.errors %}
            {% include  'material/field_errors.html' %}
        {% endif %}{% endpart %}{{ hidden_initial }}
    </div>
</div></{{ field.widget.component|default:'dmc-select' }}>{% endpart %}
