{% if msg %}
  <div class="alert {{ status }}">
    <div>
      {% if icon != "" %}
        <i class="material-icons iconify left" data-icon="{{ icon }}"></i>
      {% endif %}
      {{ msg }}
    </div>
  </div>
{% endif %}
