{
  twoFactor {
    activated
    backupTokensCount
    defaultDevice {
      persistentId
      name
      confirmed
      action
      verboseAction
      verboseName
      methodCode
      methodVerboseName
    }
    otherDevices {
      persistentId
      name
      confirmed
      action
      verboseAction
      verboseName
      methodCode
      methodVerboseName
    }
  }
}
