{% extends "core/base.html" %}

{% load i18n material_form %}

{% block browser_title %}{% trans "Change password" %}{% endblock %}
{% block no_page_title %}{% endblock %}

{% block content %}
  <div class="row">
    <div class="col m1 l2 xl3"></div>
    <div class="col s12 m10 l8 xl6">
      <div class="card">
        <form method="post" action="{% url 'account_change_password' %}">
          <div class="card-content">
            <div class="card-title">{% trans "Change password" %}</div>
            <div class="alert warning margin-bottom">
              <p>
                <i class="material-icons iconify left" data-icon="mdi:alert-outline"></i>
                {% trans "Forgot your current password?" %}
                <a href="{% url 'account_reset_password' %}">{% trans "Reset password" %}</a>
              </p>
            </div>

            {% csrf_token %}
            {% form form=form %}{% endform %}
          </div>
          <div class="card-action-light">
            {% include "core/partials/save_button.html" with caption=_("Change password") icon="mdi:key-outline" %}
          </div>
        </form>
      </div>
    </div>
  </div>
{% endblock %}
