//////////////
// HEADINGS //
//////////////

p,
h1,
h2,
h3,
h4,
h5,
h6,
.card-title {
  overflow-wrap: break-word;
  hyphens: auto;
}

/////////////
// HELPERS //
/////////////

[v-cloak] {
  display: none;
}

.v-card--reveal {
  bottom: 0;
  opacity: 1 !important;
  position: absolute;
  width: 100%;
  height: 100%;
  overflow-y: scroll;
}

.fullsize {
  width: 100%;
  height: 100%;
}

.full-width {
  width: 100%;
}
