<div class="col draggable s{{ widget.size_s }} m{{ widget.size_m }} l{{ widget.size_l }} xl{{ widget.size_xl }}"
     data-pk="{{ widget.pk }}">
  <div class="card placeholder">
    <div class="card-content">
      <i class="material-icons iconify left small" data-icon="mdi:drag-horizontal-variant"></i>
      <span class="card-title">{{ widget.title }}</span>
    </div>
  </div>
</div>
