# Generated by Django 4.2.7 on 2023-11-08 13:00

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0055_customevent"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="CustomEvent",
            new_name="PersonalEvent",
        ),
    ]
