{# -*- engine:django -*- #}

<div class="row">
  <a href="{{ url_prev }}" class="btn-flat left">
    <i class="material-icons iconify small"data-icon="mdi:chevron-left"></i>
  </a>
  <a href="{{ url_next }}" class="btn-flat right">
    <i class="material-icons iconify small" data-icon="mdi:chevron-right"></i>
  </a>
</div>
