# Generated by Django 4.2.8 on 2023-12-10 12:44

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0060_person_unique_short_name_email'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='group',
            name='additional_fields',
        ),
        migrations.DeleteModel(
            name='AdditionalField',
        ),
    ]
