# Generated by Django 4.2.6 on 2023-10-19 23:10

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0054_create_organisation_model"),
    ]

    operations = [
        migrations.CreateModel(
            name="CustomEvent",
            fields=[
                (
                    "calendarevent_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="core.calendarevent",
                    ),
                ),
                ("title", models.CharField(max_length=255, verbose_name="Title")),
                ("description", models.TextField(blank=True, verbose_name="Description")),
                ("location", models.CharField(blank=True, max_length=255, verbose_name="Location")),
                ("groups", models.ManyToManyField(blank=True, related_name="+", to="core.group")),
                (
                    "owner",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="owned_events",
                        to="core.person",
                        verbose_name="Owner",
                    ),
                ),
                ("persons", models.ManyToManyField(blank=True, related_name="+", to="core.person")),
            ],
            options={
                "abstract": False,
            },
            bases=("core.calendarevent",),
        ),
    ]
