{% extends "core/base.html" %}

{% load i18n %}

{% block browser_title %}
  {% trans "Enable Two-Factor Authentication" %}
{% endblock %}

{% block content_wrapper %}
  {% block content %}{% endblock %}
{% endblock %}
