{% extends "core/base.html" %}

{% load i18n material_form %}

{% block browser_title %}{% trans "Reset password" %}{% endblock %}
{% block no_page_title %}{% endblock %}

{% block content %}
  <div class="row">
    <div class="col m1 l2 xl3"></div>
    <div class="col s12 m10 l8 xl6">
      <div class="card">
        <form method="post" action="{% url 'account_reset_password' %}" class="password_reset">
          <div class="card-content">
            <div class="card-title">{% trans "Reset password" %}</div>
              <p class="margin-bottom">
                {% blocktrans %}Forgotten your password? Enter your e-mail address below, and we'll send you an e-mail allowing you to reset it.{% endblocktrans %}
              </p>
              {% csrf_token %}
              {% form form=form %}{% endform %}
            </div>
          <div class="card-action-light">
            {% trans "Reset password" as caption %}
            {% include "core/partials/save_button.html" with caption=caption icon="mdi:send-outline" %}
          </div>
        </form>
      </div>

      <div class="grey-text text-darken-2">
        {% blocktrans %}Please contact one of your site administrators, if you
        have any trouble resetting your password:{% endblocktrans %}
        {% include "core/partials/admins_list.html" %}
      </div>
    </div>
  </div>
{% endblock %}
