{% load html_helpers %}

<div class="card" style="padding: 2em">
	<div class="card-title">
		{{ title }}
	</div>
        <div class="card-text">
		{{ content|add_class_to_el:"ul, browser-default"|safe }}
        </div>
</div>
