{% load i18n %}
<ul class="tabs">
  <li class="tab ">
    {% for section in registry.section_objects.values %}
      <li class="tab">
        <a class="{% if active_section == section.name %}active{% endif %}"
           href="{% url registry_url section.name %}"
           target="_self">
          {{ section.verbose_name }}
        </a>
      </li>
    {% endfor %}
</ul>
