# Generated by Django 3.2.11 on 2022-01-08 13:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0030_user_attributes'),
    ]

    operations = [
        migrations.AddField(
            model_name='oauthapplication',
            name='icon',
            field=models.ImageField(blank=True, help_text='This image will be shown as icon in the authorization flow. It should be squared.', null=True, upload_to='', verbose_name='Icon'),
        ),
    ]
