# Generated by Django 3.0.7 on 2020-06-28 11:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0002_school_term'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='person',
            name='photo_cropping',
        ),
        migrations.AlterField(
            model_name='person',
            name='photo',
            field=models.ImageField(blank=True, null=True, upload_to='', verbose_name='Photo'),
        ),
    ]
