{
  dynamicRoutes {
    parentRouteName

    routePath
    routeName

    displayAccountMenu
    displaySidenavMenu
    menuNewTab

    menuTitle
    menuIcon

    menuPermission
    routePermission
  }
}
