# Generated by Django 4.2.4 on 2023-08-16 20:44

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0052_site_related_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="FreeBusy",
            fields=[
                (
                    "calendarevent_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="core.calendarevent",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("core.calendarevent",),
        ),
    ]
