# Generated by Django 4.1.10 on 2023-07-20 20:35

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0051_calendarevent_and_holiday"),
    ]

    operations = [
        migrations.AlterField(
            model_name="custommenuitem",
            name="icon",
            field=models.CharField(
                blank=True,
                choices=[
                    ("ab-testing", "ab-testing"),
                    ("abacus", "abacus"),
                    ("abjad-arabic", "abjad-arabic"),
                    ("abjad-hebrew", "abjad-hebrew"),
                    ("abugida-devanagari", "abugida-devanagari"),
                    ("abugida-thai", "abugida-thai"),
                    ("access-point", "access-point"),
                    ("access-point-check", "access-point-check"),
                    ("access-point-minus", "access-point-minus"),
                    ("access-point-network", "access-point-network"),
                    ("access-point-network-off", "access-point-network-off"),
                    ("access-point-off", "access-point-off"),
                    ("access-point-plus", "access-point-plus"),
                    ("access-point-remove", "access-point-remove"),
                    ("account", "account"),
                    ("account-alert", "account-alert"),
                    ("account-alert-outline", "account-alert-outline"),
                    ("account-arrow-down", "account-arrow-down"),
                    ("account-arrow-down-outline", "account-arrow-down-outline"),
                    ("account-arrow-left", "account-arrow-left"),
                    ("account-arrow-left-outline", "account-arrow-left-outline"),
                    ("account-arrow-right", "account-arrow-right"),
                    ("account-arrow-right-outline", "account-arrow-right-outline"),
                    ("account-arrow-up", "account-arrow-up"),
                    ("account-arrow-up-outline", "account-arrow-up-outline"),
                    ("account-badge", "account-badge"),
                    ("account-badge-outline", "account-badge-outline"),
                    ("account-box", "account-box"),
                    ("account-box-multiple", "account-box-multiple"),
                    ("account-box-multiple-outline", "account-box-multiple-outline"),
                    ("account-box-outline", "account-box-outline"),
                    ("account-cancel", "account-cancel"),
                    ("account-cancel-outline", "account-cancel-outline"),
                    ("account-card", "account-card"),
                    ("account-card-outline", "account-card-outline"),
                    ("account-cash", "account-cash"),
                    ("account-cash-outline", "account-cash-outline"),
                    ("account-check", "account-check"),
                    ("account-check-outline", "account-check-outline"),
                    ("account-child", "account-child"),
                    ("account-child-circle", "account-child-circle"),
                    ("account-child-outline", "account-child-outline"),
                    ("account-circle", "account-circle"),
                    ("account-circle-outline", "account-circle-outline"),
                    ("account-clock", "account-clock"),
                    ("account-clock-outline", "account-clock-outline"),
                    ("account-cog", "account-cog"),
                    ("account-cog-outline", "account-cog-outline"),
                    ("account-convert", "account-convert"),
                    ("account-convert-outline", "account-convert-outline"),
                    ("account-cowboy-hat", "account-cowboy-hat"),
                    ("account-cowboy-hat-outline", "account-cowboy-hat-outline"),
                    ("account-credit-card", "account-credit-card"),
                    ("account-credit-card-outline", "account-credit-card-outline"),
                    ("account-details", "account-details"),
                    ("account-details-outline", "account-details-outline"),
                    ("account-edit", "account-edit"),
                    ("account-edit-outline", "account-edit-outline"),
                    ("account-eye", "account-eye"),
                    ("account-eye-outline", "account-eye-outline"),
                    ("account-filter", "account-filter"),
                    ("account-filter-outline", "account-filter-outline"),
                    ("account-group", "account-group"),
                    ("account-group-outline", "account-group-outline"),
                    ("account-hard-hat", "account-hard-hat"),
                    ("account-hard-hat-outline", "account-hard-hat-outline"),
                    ("account-heart", "account-heart"),
                    ("account-heart-outline", "account-heart-outline"),
                    ("account-injury", "account-injury"),
                    ("account-injury-outline", "account-injury-outline"),
                    ("account-key", "account-key"),
                    ("account-key-outline", "account-key-outline"),
                    ("account-lock", "account-lock"),
                    ("account-lock-open", "account-lock-open"),
                    ("account-lock-open-outline", "account-lock-open-outline"),
                    ("account-lock-outline", "account-lock-outline"),
                    ("account-minus", "account-minus"),
                    ("account-minus-outline", "account-minus-outline"),
                    ("account-multiple", "account-multiple"),
                    ("account-multiple-check", "account-multiple-check"),
                    ("account-multiple-check-outline", "account-multiple-check-outline"),
                    ("account-multiple-minus", "account-multiple-minus"),
                    ("account-multiple-minus-outline", "account-multiple-minus-outline"),
                    ("account-multiple-outline", "account-multiple-outline"),
                    ("account-multiple-plus", "account-multiple-plus"),
                    ("account-multiple-plus-outline", "account-multiple-plus-outline"),
                    ("account-multiple-remove", "account-multiple-remove"),
                    ("account-multiple-remove-outline", "account-multiple-remove-outline"),
                    ("account-music", "account-music"),
                    ("account-music-outline", "account-music-outline"),
                    ("account-network", "account-network"),
                    ("account-network-off", "account-network-off"),
                    ("account-network-off-outline", "account-network-off-outline"),
                    ("account-network-outline", "account-network-outline"),
                    ("account-off", "account-off"),
                    ("account-off-outline", "account-off-outline"),
                    ("account-outline", "account-outline"),
                    ("account-plus", "account-plus"),
                    ("account-plus-outline", "account-plus-outline"),
                    ("account-question", "account-question"),
                    ("account-question-outline", "account-question-outline"),
                    ("account-reactivate", "account-reactivate"),
                    ("account-reactivate-outline", "account-reactivate-outline"),
                    ("account-remove", "account-remove"),
                    ("account-remove-outline", "account-remove-outline"),
                    ("account-school", "account-school"),
                    ("account-school-outline", "account-school-outline"),
                    ("account-search", "account-search"),
                    ("account-search-outline", "account-search-outline"),
                    ("account-settings", "account-settings"),
                    ("account-settings-outline", "account-settings-outline"),
                    ("account-settings-variant", "account-settings-variant"),
                    ("account-star", "account-star"),
                    ("account-star-outline", "account-star-outline"),
                    ("account-supervisor", "account-supervisor"),
                    ("account-supervisor-circle", "account-supervisor-circle"),
                    ("account-supervisor-circle-outline", "account-supervisor-circle-outline"),
                    ("account-supervisor-outline", "account-supervisor-outline"),
                    ("account-switch", "account-switch"),
                    ("account-switch-outline", "account-switch-outline"),
                    ("account-sync", "account-sync"),
                    ("account-sync-outline", "account-sync-outline"),
                    ("account-tag", "account-tag"),
                    ("account-tag-outline", "account-tag-outline"),
                    ("account-tie", "account-tie"),
                    ("account-tie-hat", "account-tie-hat"),
                    ("account-tie-hat-outline", "account-tie-hat-outline"),
                    ("account-tie-outline", "account-tie-outline"),
                    ("account-tie-voice", "account-tie-voice"),
                    ("account-tie-voice-off", "account-tie-voice-off"),
                    ("account-tie-voice-off-outline", "account-tie-voice-off-outline"),
                    ("account-tie-voice-outline", "account-tie-voice-outline"),
                    ("account-tie-woman", "account-tie-woman"),
                    ("account-voice", "account-voice"),
                    ("account-voice-off", "account-voice-off"),
                    ("account-wrench", "account-wrench"),
                    ("account-wrench-outline", "account-wrench-outline"),
                    ("accusoft", "accusoft"),
                    ("ad-choices", "ad-choices"),
                    ("adchoices", "adchoices"),
                    ("adjust", "adjust"),
                    ("adobe", "adobe"),
                    ("advertisements", "advertisements"),
                    ("advertisements-off", "advertisements-off"),
                    ("air-conditioner", "air-conditioner"),
                    ("air-filter", "air-filter"),
                    ("air-horn", "air-horn"),
                    ("air-humidifier", "air-humidifier"),
                    ("air-humidifier-off", "air-humidifier-off"),
                    ("air-purifier", "air-purifier"),
                    ("air-purifier-off", "air-purifier-off"),
                    ("airbag", "airbag"),
                    ("airballoon", "airballoon"),
                    ("airballoon-outline", "airballoon-outline"),
                    ("airplane", "airplane"),
                    ("airplane-alert", "airplane-alert"),
                    ("airplane-check", "airplane-check"),
                    ("airplane-clock", "airplane-clock"),
                    ("airplane-cog", "airplane-cog"),
                    ("airplane-edit", "airplane-edit"),
                    ("airplane-landing", "airplane-landing"),
                    ("airplane-marker", "airplane-marker"),
                    ("airplane-minus", "airplane-minus"),
                    ("airplane-off", "airplane-off"),
                    ("airplane-plus", "airplane-plus"),
                    ("airplane-remove", "airplane-remove"),
                    ("airplane-search", "airplane-search"),
                    ("airplane-settings", "airplane-settings"),
                    ("airplane-takeoff", "airplane-takeoff"),
                    ("airport", "airport"),
                    ("alarm", "alarm"),
                    ("alarm-bell", "alarm-bell"),
                    ("alarm-check", "alarm-check"),
                    ("alarm-light", "alarm-light"),
                    ("alarm-light-off", "alarm-light-off"),
                    ("alarm-light-off-outline", "alarm-light-off-outline"),
                    ("alarm-light-outline", "alarm-light-outline"),
                    ("alarm-multiple", "alarm-multiple"),
                    ("alarm-note", "alarm-note"),
                    ("alarm-note-off", "alarm-note-off"),
                    ("alarm-off", "alarm-off"),
                    ("alarm-panel", "alarm-panel"),
                    ("alarm-panel-outline", "alarm-panel-outline"),
                    ("alarm-plus", "alarm-plus"),
                    ("alarm-snooze", "alarm-snooze"),
                    ("album", "album"),
                    ("alert", "alert"),
                    ("alert-box", "alert-box"),
                    ("alert-box-outline", "alert-box-outline"),
                    ("alert-circle", "alert-circle"),
                    ("alert-circle-check", "alert-circle-check"),
                    ("alert-circle-check-outline", "alert-circle-check-outline"),
                    ("alert-circle-outline", "alert-circle-outline"),
                    ("alert-decagram", "alert-decagram"),
                    ("alert-decagram-outline", "alert-decagram-outline"),
                    ("alert-minus", "alert-minus"),
                    ("alert-minus-outline", "alert-minus-outline"),
                    ("alert-octagon", "alert-octagon"),
                    ("alert-octagon-outline", "alert-octagon-outline"),
                    ("alert-octagram", "alert-octagram"),
                    ("alert-octagram-outline", "alert-octagram-outline"),
                    ("alert-outline", "alert-outline"),
                    ("alert-plus", "alert-plus"),
                    ("alert-plus-outline", "alert-plus-outline"),
                    ("alert-remove", "alert-remove"),
                    ("alert-remove-outline", "alert-remove-outline"),
                    ("alert-rhombus", "alert-rhombus"),
                    ("alert-rhombus-outline", "alert-rhombus-outline"),
                    ("alien", "alien"),
                    ("alien-outline", "alien-outline"),
                    ("align-horizontal-center", "align-horizontal-center"),
                    ("align-horizontal-distribute", "align-horizontal-distribute"),
                    ("align-horizontal-left", "align-horizontal-left"),
                    ("align-horizontal-right", "align-horizontal-right"),
                    ("align-vertical-bottom", "align-vertical-bottom"),
                    ("align-vertical-center", "align-vertical-center"),
                    ("align-vertical-distribute", "align-vertical-distribute"),
                    ("align-vertical-top", "align-vertical-top"),
                    ("all-inclusive", "all-inclusive"),
                    ("all-inclusive-box", "all-inclusive-box"),
                    ("all-inclusive-box-outline", "all-inclusive-box-outline"),
                    ("allergy", "allergy"),
                    ("allo", "allo"),
                    ("alpha", "alpha"),
                    ("alpha-a", "alpha-a"),
                    ("alpha-a-box", "alpha-a-box"),
                    ("alpha-a-box-outline", "alpha-a-box-outline"),
                    ("alpha-a-circle", "alpha-a-circle"),
                    ("alpha-a-circle-outline", "alpha-a-circle-outline"),
                    ("alpha-b", "alpha-b"),
                    ("alpha-b-box", "alpha-b-box"),
                    ("alpha-b-box-outline", "alpha-b-box-outline"),
                    ("alpha-b-circle", "alpha-b-circle"),
                    ("alpha-b-circle-outline", "alpha-b-circle-outline"),
                    ("alpha-c", "alpha-c"),
                    ("alpha-c-box", "alpha-c-box"),
                    ("alpha-c-box-outline", "alpha-c-box-outline"),
                    ("alpha-c-circle", "alpha-c-circle"),
                    ("alpha-c-circle-outline", "alpha-c-circle-outline"),
                    ("alpha-d", "alpha-d"),
                    ("alpha-d-box", "alpha-d-box"),
                    ("alpha-d-box-outline", "alpha-d-box-outline"),
                    ("alpha-d-circle", "alpha-d-circle"),
                    ("alpha-d-circle-outline", "alpha-d-circle-outline"),
                    ("alpha-e", "alpha-e"),
                    ("alpha-e-box", "alpha-e-box"),
                    ("alpha-e-box-outline", "alpha-e-box-outline"),
                    ("alpha-e-circle", "alpha-e-circle"),
                    ("alpha-e-circle-outline", "alpha-e-circle-outline"),
                    ("alpha-f", "alpha-f"),
                    ("alpha-f-box", "alpha-f-box"),
                    ("alpha-f-box-outline", "alpha-f-box-outline"),
                    ("alpha-f-circle", "alpha-f-circle"),
                    ("alpha-f-circle-outline", "alpha-f-circle-outline"),
                    ("alpha-g", "alpha-g"),
                    ("alpha-g-box", "alpha-g-box"),
                    ("alpha-g-box-outline", "alpha-g-box-outline"),
                    ("alpha-g-circle", "alpha-g-circle"),
                    ("alpha-g-circle-outline", "alpha-g-circle-outline"),
                    ("alpha-h", "alpha-h"),
                    ("alpha-h-box", "alpha-h-box"),
                    ("alpha-h-box-outline", "alpha-h-box-outline"),
                    ("alpha-h-circle", "alpha-h-circle"),
                    ("alpha-h-circle-outline", "alpha-h-circle-outline"),
                    ("alpha-i", "alpha-i"),
                    ("alpha-i-box", "alpha-i-box"),
                    ("alpha-i-box-outline", "alpha-i-box-outline"),
                    ("alpha-i-circle", "alpha-i-circle"),
                    ("alpha-i-circle-outline", "alpha-i-circle-outline"),
                    ("alpha-j", "alpha-j"),
                    ("alpha-j-box", "alpha-j-box"),
                    ("alpha-j-box-outline", "alpha-j-box-outline"),
                    ("alpha-j-circle", "alpha-j-circle"),
                    ("alpha-j-circle-outline", "alpha-j-circle-outline"),
                    ("alpha-k", "alpha-k"),
                    ("alpha-k-box", "alpha-k-box"),
                    ("alpha-k-box-outline", "alpha-k-box-outline"),
                    ("alpha-k-circle", "alpha-k-circle"),
                    ("alpha-k-circle-outline", "alpha-k-circle-outline"),
                    ("alpha-l", "alpha-l"),
                    ("alpha-l-box", "alpha-l-box"),
                    ("alpha-l-box-outline", "alpha-l-box-outline"),
                    ("alpha-l-circle", "alpha-l-circle"),
                    ("alpha-l-circle-outline", "alpha-l-circle-outline"),
                    ("alpha-m", "alpha-m"),
                    ("alpha-m-box", "alpha-m-box"),
                    ("alpha-m-box-outline", "alpha-m-box-outline"),
                    ("alpha-m-circle", "alpha-m-circle"),
                    ("alpha-m-circle-outline", "alpha-m-circle-outline"),
                    ("alpha-n", "alpha-n"),
                    ("alpha-n-box", "alpha-n-box"),
                    ("alpha-n-box-outline", "alpha-n-box-outline"),
                    ("alpha-n-circle", "alpha-n-circle"),
                    ("alpha-n-circle-outline", "alpha-n-circle-outline"),
                    ("alpha-o", "alpha-o"),
                    ("alpha-o-box", "alpha-o-box"),
                    ("alpha-o-box-outline", "alpha-o-box-outline"),
                    ("alpha-o-circle", "alpha-o-circle"),
                    ("alpha-o-circle-outline", "alpha-o-circle-outline"),
                    ("alpha-p", "alpha-p"),
                    ("alpha-p-box", "alpha-p-box"),
                    ("alpha-p-box-outline", "alpha-p-box-outline"),
                    ("alpha-p-circle", "alpha-p-circle"),
                    ("alpha-p-circle-outline", "alpha-p-circle-outline"),
                    ("alpha-q", "alpha-q"),
                    ("alpha-q-box", "alpha-q-box"),
                    ("alpha-q-box-outline", "alpha-q-box-outline"),
                    ("alpha-q-circle", "alpha-q-circle"),
                    ("alpha-q-circle-outline", "alpha-q-circle-outline"),
                    ("alpha-r", "alpha-r"),
                    ("alpha-r-box", "alpha-r-box"),
                    ("alpha-r-box-outline", "alpha-r-box-outline"),
                    ("alpha-r-circle", "alpha-r-circle"),
                    ("alpha-r-circle-outline", "alpha-r-circle-outline"),
                    ("alpha-s", "alpha-s"),
                    ("alpha-s-box", "alpha-s-box"),
                    ("alpha-s-box-outline", "alpha-s-box-outline"),
                    ("alpha-s-circle", "alpha-s-circle"),
                    ("alpha-s-circle-outline", "alpha-s-circle-outline"),
                    ("alpha-t", "alpha-t"),
                    ("alpha-t-box", "alpha-t-box"),
                    ("alpha-t-box-outline", "alpha-t-box-outline"),
                    ("alpha-t-circle", "alpha-t-circle"),
                    ("alpha-t-circle-outline", "alpha-t-circle-outline"),
                    ("alpha-u", "alpha-u"),
                    ("alpha-u-box", "alpha-u-box"),
                    ("alpha-u-box-outline", "alpha-u-box-outline"),
                    ("alpha-u-circle", "alpha-u-circle"),
                    ("alpha-u-circle-outline", "alpha-u-circle-outline"),
                    ("alpha-v", "alpha-v"),
                    ("alpha-v-box", "alpha-v-box"),
                    ("alpha-v-box-outline", "alpha-v-box-outline"),
                    ("alpha-v-circle", "alpha-v-circle"),
                    ("alpha-v-circle-outline", "alpha-v-circle-outline"),
                    ("alpha-w", "alpha-w"),
                    ("alpha-w-box", "alpha-w-box"),
                    ("alpha-w-box-outline", "alpha-w-box-outline"),
                    ("alpha-w-circle", "alpha-w-circle"),
                    ("alpha-w-circle-outline", "alpha-w-circle-outline"),
                    ("alpha-x", "alpha-x"),
                    ("alpha-x-box", "alpha-x-box"),
                    ("alpha-x-box-outline", "alpha-x-box-outline"),
                    ("alpha-x-circle", "alpha-x-circle"),
                    ("alpha-x-circle-outline", "alpha-x-circle-outline"),
                    ("alpha-y", "alpha-y"),
                    ("alpha-y-box", "alpha-y-box"),
                    ("alpha-y-box-outline", "alpha-y-box-outline"),
                    ("alpha-y-circle", "alpha-y-circle"),
                    ("alpha-y-circle-outline", "alpha-y-circle-outline"),
                    ("alpha-z", "alpha-z"),
                    ("alpha-z-box", "alpha-z-box"),
                    ("alpha-z-box-outline", "alpha-z-box-outline"),
                    ("alpha-z-circle", "alpha-z-circle"),
                    ("alpha-z-circle-outline", "alpha-z-circle-outline"),
                    ("alphabet-aurebesh", "alphabet-aurebesh"),
                    ("alphabet-cyrillic", "alphabet-cyrillic"),
                    ("alphabet-greek", "alphabet-greek"),
                    ("alphabet-latin", "alphabet-latin"),
                    ("alphabet-piqad", "alphabet-piqad"),
                    ("alphabet-tengwar", "alphabet-tengwar"),
                    ("alphabetical", "alphabetical"),
                    ("alphabetical-off", "alphabetical-off"),
                    ("alphabetical-variant", "alphabetical-variant"),
                    ("alphabetical-variant-off", "alphabetical-variant-off"),
                    ("altimeter", "altimeter"),
                    ("amazon", "amazon"),
                    ("amazon-alexa", "amazon-alexa"),
                    ("amazon-drive", "amazon-drive"),
                    ("ambulance", "ambulance"),
                    ("ammunition", "ammunition"),
                    ("ampersand", "ampersand"),
                    ("amplifier", "amplifier"),
                    ("amplifier-off", "amplifier-off"),
                    ("anchor", "anchor"),
                    ("android", "android"),
                    ("android-auto", "android-auto"),
                    ("android-debug-bridge", "android-debug-bridge"),
                    ("android-head", "android-head"),
                    ("android-messages", "android-messages"),
                    ("android-studio", "android-studio"),
                    ("angle-acute", "angle-acute"),
                    ("angle-obtuse", "angle-obtuse"),
                    ("angle-right", "angle-right"),
                    ("angular", "angular"),
                    ("angularjs", "angularjs"),
                    ("animation", "animation"),
                    ("animation-outline", "animation-outline"),
                    ("animation-play", "animation-play"),
                    ("animation-play-outline", "animation-play-outline"),
                    ("ansible", "ansible"),
                    ("antenna", "antenna"),
                    ("anvil", "anvil"),
                    ("apache-kafka", "apache-kafka"),
                    ("api", "api"),
                    ("api-off", "api-off"),
                    ("apple", "apple"),
                    ("apple-finder", "apple-finder"),
                    ("apple-icloud", "apple-icloud"),
                    ("apple-ios", "apple-ios"),
                    ("apple-keyboard-caps", "apple-keyboard-caps"),
                    ("apple-keyboard-command", "apple-keyboard-command"),
                    ("apple-keyboard-control", "apple-keyboard-control"),
                    ("apple-keyboard-option", "apple-keyboard-option"),
                    ("apple-keyboard-shift", "apple-keyboard-shift"),
                    ("apple-safari", "apple-safari"),
                    ("application", "application"),
                    ("application-array", "application-array"),
                    ("application-array-outline", "application-array-outline"),
                    ("application-braces", "application-braces"),
                    ("application-braces-outline", "application-braces-outline"),
                    ("application-brackets", "application-brackets"),
                    ("application-brackets-outline", "application-brackets-outline"),
                    ("application-cog", "application-cog"),
                    ("application-cog-outline", "application-cog-outline"),
                    ("application-edit", "application-edit"),
                    ("application-edit-outline", "application-edit-outline"),
                    ("application-export", "application-export"),
                    ("application-import", "application-import"),
                    ("application-outline", "application-outline"),
                    ("application-parentheses", "application-parentheses"),
                    ("application-parentheses-outline", "application-parentheses-outline"),
                    ("application-settings", "application-settings"),
                    ("application-settings-outline", "application-settings-outline"),
                    ("application-variable", "application-variable"),
                    ("application-variable-outline", "application-variable-outline"),
                    ("appnet", "appnet"),
                    ("approximately-equal", "approximately-equal"),
                    ("approximately-equal-box", "approximately-equal-box"),
                    ("apps", "apps"),
                    ("apps-box", "apps-box"),
                    ("arch", "arch"),
                    ("archive", "archive"),
                    ("archive-alert", "archive-alert"),
                    ("archive-alert-outline", "archive-alert-outline"),
                    ("archive-arrow-down", "archive-arrow-down"),
                    ("archive-arrow-down-outline", "archive-arrow-down-outline"),
                    ("archive-arrow-up", "archive-arrow-up"),
                    ("archive-arrow-up-outline", "archive-arrow-up-outline"),
                    ("archive-cancel", "archive-cancel"),
                    ("archive-cancel-outline", "archive-cancel-outline"),
                    ("archive-check", "archive-check"),
                    ("archive-check-outline", "archive-check-outline"),
                    ("archive-clock", "archive-clock"),
                    ("archive-clock-outline", "archive-clock-outline"),
                    ("archive-cog", "archive-cog"),
                    ("archive-cog-outline", "archive-cog-outline"),
                    ("archive-edit", "archive-edit"),
                    ("archive-edit-outline", "archive-edit-outline"),
                    ("archive-eye", "archive-eye"),
                    ("archive-eye-outline", "archive-eye-outline"),
                    ("archive-lock", "archive-lock"),
                    ("archive-lock-open", "archive-lock-open"),
                    ("archive-lock-open-outline", "archive-lock-open-outline"),
                    ("archive-lock-outline", "archive-lock-outline"),
                    ("archive-marker", "archive-marker"),
                    ("archive-marker-outline", "archive-marker-outline"),
                    ("archive-minus", "archive-minus"),
                    ("archive-minus-outline", "archive-minus-outline"),
                    ("archive-music", "archive-music"),
                    ("archive-music-outline", "archive-music-outline"),
                    ("archive-off", "archive-off"),
                    ("archive-off-outline", "archive-off-outline"),
                    ("archive-outline", "archive-outline"),
                    ("archive-plus", "archive-plus"),
                    ("archive-plus-outline", "archive-plus-outline"),
                    ("archive-refresh", "archive-refresh"),
                    ("archive-refresh-outline", "archive-refresh-outline"),
                    ("archive-remove", "archive-remove"),
                    ("archive-remove-outline", "archive-remove-outline"),
                    ("archive-search", "archive-search"),
                    ("archive-search-outline", "archive-search-outline"),
                    ("archive-settings", "archive-settings"),
                    ("archive-settings-outline", "archive-settings-outline"),
                    ("archive-star", "archive-star"),
                    ("archive-star-outline", "archive-star-outline"),
                    ("archive-sync", "archive-sync"),
                    ("archive-sync-outline", "archive-sync-outline"),
                    ("arm-flex", "arm-flex"),
                    ("arm-flex-outline", "arm-flex-outline"),
                    ("arrange-bring-forward", "arrange-bring-forward"),
                    ("arrange-bring-to-front", "arrange-bring-to-front"),
                    ("arrange-send-backward", "arrange-send-backward"),
                    ("arrange-send-to-back", "arrange-send-to-back"),
                    ("arrow-all", "arrow-all"),
                    ("arrow-bottom-left", "arrow-bottom-left"),
                    ("arrow-bottom-left-bold-box", "arrow-bottom-left-bold-box"),
                    ("arrow-bottom-left-bold-box-outline", "arrow-bottom-left-bold-box-outline"),
                    ("arrow-bottom-left-bold-outline", "arrow-bottom-left-bold-outline"),
                    ("arrow-bottom-left-thick", "arrow-bottom-left-thick"),
                    ("arrow-bottom-left-thin", "arrow-bottom-left-thin"),
                    (
                        "arrow-bottom-left-thin-circle-outline",
                        "arrow-bottom-left-thin-circle-outline",
                    ),
                    ("arrow-bottom-right", "arrow-bottom-right"),
                    ("arrow-bottom-right-bold-box", "arrow-bottom-right-bold-box"),
                    ("arrow-bottom-right-bold-box-outline", "arrow-bottom-right-bold-box-outline"),
                    ("arrow-bottom-right-bold-outline", "arrow-bottom-right-bold-outline"),
                    ("arrow-bottom-right-thick", "arrow-bottom-right-thick"),
                    ("arrow-bottom-right-thin", "arrow-bottom-right-thin"),
                    (
                        "arrow-bottom-right-thin-circle-outline",
                        "arrow-bottom-right-thin-circle-outline",
                    ),
                    ("arrow-collapse", "arrow-collapse"),
                    ("arrow-collapse-all", "arrow-collapse-all"),
                    ("arrow-collapse-down", "arrow-collapse-down"),
                    ("arrow-collapse-horizontal", "arrow-collapse-horizontal"),
                    ("arrow-collapse-left", "arrow-collapse-left"),
                    ("arrow-collapse-right", "arrow-collapse-right"),
                    ("arrow-collapse-up", "arrow-collapse-up"),
                    ("arrow-collapse-vertical", "arrow-collapse-vertical"),
                    ("arrow-decision", "arrow-decision"),
                    ("arrow-decision-auto", "arrow-decision-auto"),
                    ("arrow-decision-auto-outline", "arrow-decision-auto-outline"),
                    ("arrow-decision-outline", "arrow-decision-outline"),
                    ("arrow-down", "arrow-down"),
                    ("arrow-down-bold", "arrow-down-bold"),
                    ("arrow-down-bold-box", "arrow-down-bold-box"),
                    ("arrow-down-bold-box-outline", "arrow-down-bold-box-outline"),
                    ("arrow-down-bold-circle", "arrow-down-bold-circle"),
                    ("arrow-down-bold-circle-outline", "arrow-down-bold-circle-outline"),
                    ("arrow-down-bold-hexagon-outline", "arrow-down-bold-hexagon-outline"),
                    ("arrow-down-bold-outline", "arrow-down-bold-outline"),
                    ("arrow-down-box", "arrow-down-box"),
                    ("arrow-down-circle", "arrow-down-circle"),
                    ("arrow-down-circle-outline", "arrow-down-circle-outline"),
                    ("arrow-down-drop-circle", "arrow-down-drop-circle"),
                    ("arrow-down-drop-circle-outline", "arrow-down-drop-circle-outline"),
                    ("arrow-down-left", "arrow-down-left"),
                    ("arrow-down-left-bold", "arrow-down-left-bold"),
                    ("arrow-down-right", "arrow-down-right"),
                    ("arrow-down-right-bold", "arrow-down-right-bold"),
                    ("arrow-down-thick", "arrow-down-thick"),
                    ("arrow-down-thin", "arrow-down-thin"),
                    ("arrow-down-thin-circle-outline", "arrow-down-thin-circle-outline"),
                    ("arrow-expand", "arrow-expand"),
                    ("arrow-expand-all", "arrow-expand-all"),
                    ("arrow-expand-down", "arrow-expand-down"),
                    ("arrow-expand-horizontal", "arrow-expand-horizontal"),
                    ("arrow-expand-left", "arrow-expand-left"),
                    ("arrow-expand-right", "arrow-expand-right"),
                    ("arrow-expand-up", "arrow-expand-up"),
                    ("arrow-expand-vertical", "arrow-expand-vertical"),
                    ("arrow-horizontal-lock", "arrow-horizontal-lock"),
                    ("arrow-left", "arrow-left"),
                    ("arrow-left-bold", "arrow-left-bold"),
                    ("arrow-left-bold-box", "arrow-left-bold-box"),
                    ("arrow-left-bold-box-outline", "arrow-left-bold-box-outline"),
                    ("arrow-left-bold-circle", "arrow-left-bold-circle"),
                    ("arrow-left-bold-circle-outline", "arrow-left-bold-circle-outline"),
                    ("arrow-left-bold-hexagon-outline", "arrow-left-bold-hexagon-outline"),
                    ("arrow-left-bold-outline", "arrow-left-bold-outline"),
                    ("arrow-left-bottom", "arrow-left-bottom"),
                    ("arrow-left-bottom-bold", "arrow-left-bottom-bold"),
                    ("arrow-left-box", "arrow-left-box"),
                    ("arrow-left-circle", "arrow-left-circle"),
                    ("arrow-left-circle-outline", "arrow-left-circle-outline"),
                    ("arrow-left-drop-circle", "arrow-left-drop-circle"),
                    ("arrow-left-drop-circle-outline", "arrow-left-drop-circle-outline"),
                    ("arrow-left-right", "arrow-left-right"),
                    ("arrow-left-right-bold", "arrow-left-right-bold"),
                    ("arrow-left-right-bold-outline", "arrow-left-right-bold-outline"),
                    ("arrow-left-thick", "arrow-left-thick"),
                    ("arrow-left-thin", "arrow-left-thin"),
                    ("arrow-left-thin-circle-outline", "arrow-left-thin-circle-outline"),
                    ("arrow-left-top", "arrow-left-top"),
                    ("arrow-left-top-bold", "arrow-left-top-bold"),
                    ("arrow-oscillating", "arrow-oscillating"),
                    ("arrow-oscillating-off", "arrow-oscillating-off"),
                    ("arrow-projectile", "arrow-projectile"),
                    ("arrow-projectile-multiple", "arrow-projectile-multiple"),
                    ("arrow-right", "arrow-right"),
                    ("arrow-right-bold", "arrow-right-bold"),
                    ("arrow-right-bold-box", "arrow-right-bold-box"),
                    ("arrow-right-bold-box-outline", "arrow-right-bold-box-outline"),
                    ("arrow-right-bold-circle", "arrow-right-bold-circle"),
                    ("arrow-right-bold-circle-outline", "arrow-right-bold-circle-outline"),
                    ("arrow-right-bold-hexagon-outline", "arrow-right-bold-hexagon-outline"),
                    ("arrow-right-bold-outline", "arrow-right-bold-outline"),
                    ("arrow-right-bottom", "arrow-right-bottom"),
                    ("arrow-right-bottom-bold", "arrow-right-bottom-bold"),
                    ("arrow-right-box", "arrow-right-box"),
                    ("arrow-right-circle", "arrow-right-circle"),
                    ("arrow-right-circle-outline", "arrow-right-circle-outline"),
                    ("arrow-right-drop-circle", "arrow-right-drop-circle"),
                    ("arrow-right-drop-circle-outline", "arrow-right-drop-circle-outline"),
                    ("arrow-right-thick", "arrow-right-thick"),
                    ("arrow-right-thin", "arrow-right-thin"),
                    ("arrow-right-thin-circle-outline", "arrow-right-thin-circle-outline"),
                    ("arrow-right-top", "arrow-right-top"),
                    ("arrow-right-top-bold", "arrow-right-top-bold"),
                    ("arrow-split-horizontal", "arrow-split-horizontal"),
                    ("arrow-split-vertical", "arrow-split-vertical"),
                    ("arrow-top-left", "arrow-top-left"),
                    ("arrow-top-left-bold-box", "arrow-top-left-bold-box"),
                    ("arrow-top-left-bold-box-outline", "arrow-top-left-bold-box-outline"),
                    ("arrow-top-left-bold-outline", "arrow-top-left-bold-outline"),
                    ("arrow-top-left-bottom-right", "arrow-top-left-bottom-right"),
                    ("arrow-top-left-bottom-right-bold", "arrow-top-left-bottom-right-bold"),
                    ("arrow-top-left-thick", "arrow-top-left-thick"),
                    ("arrow-top-left-thin", "arrow-top-left-thin"),
                    ("arrow-top-left-thin-circle-outline", "arrow-top-left-thin-circle-outline"),
                    ("arrow-top-right", "arrow-top-right"),
                    ("arrow-top-right-bold-box", "arrow-top-right-bold-box"),
                    ("arrow-top-right-bold-box-outline", "arrow-top-right-bold-box-outline"),
                    ("arrow-top-right-bold-outline", "arrow-top-right-bold-outline"),
                    ("arrow-top-right-bottom-left", "arrow-top-right-bottom-left"),
                    ("arrow-top-right-bottom-left-bold", "arrow-top-right-bottom-left-bold"),
                    ("arrow-top-right-thick", "arrow-top-right-thick"),
                    ("arrow-top-right-thin", "arrow-top-right-thin"),
                    ("arrow-top-right-thin-circle-outline", "arrow-top-right-thin-circle-outline"),
                    ("arrow-u-down-left", "arrow-u-down-left"),
                    ("arrow-u-down-left-bold", "arrow-u-down-left-bold"),
                    ("arrow-u-down-right", "arrow-u-down-right"),
                    ("arrow-u-down-right-bold", "arrow-u-down-right-bold"),
                    ("arrow-u-left-bottom", "arrow-u-left-bottom"),
                    ("arrow-u-left-bottom-bold", "arrow-u-left-bottom-bold"),
                    ("arrow-u-left-top", "arrow-u-left-top"),
                    ("arrow-u-left-top-bold", "arrow-u-left-top-bold"),
                    ("arrow-u-right-bottom", "arrow-u-right-bottom"),
                    ("arrow-u-right-bottom-bold", "arrow-u-right-bottom-bold"),
                    ("arrow-u-right-top", "arrow-u-right-top"),
                    ("arrow-u-right-top-bold", "arrow-u-right-top-bold"),
                    ("arrow-u-up-left", "arrow-u-up-left"),
                    ("arrow-u-up-left-bold", "arrow-u-up-left-bold"),
                    ("arrow-u-up-right", "arrow-u-up-right"),
                    ("arrow-u-up-right-bold", "arrow-u-up-right-bold"),
                    ("arrow-up", "arrow-up"),
                    ("arrow-up-bold", "arrow-up-bold"),
                    ("arrow-up-bold-box", "arrow-up-bold-box"),
                    ("arrow-up-bold-box-outline", "arrow-up-bold-box-outline"),
                    ("arrow-up-bold-circle", "arrow-up-bold-circle"),
                    ("arrow-up-bold-circle-outline", "arrow-up-bold-circle-outline"),
                    ("arrow-up-bold-hexagon-outline", "arrow-up-bold-hexagon-outline"),
                    ("arrow-up-bold-outline", "arrow-up-bold-outline"),
                    ("arrow-up-box", "arrow-up-box"),
                    ("arrow-up-circle", "arrow-up-circle"),
                    ("arrow-up-circle-outline", "arrow-up-circle-outline"),
                    ("arrow-up-down", "arrow-up-down"),
                    ("arrow-up-down-bold", "arrow-up-down-bold"),
                    ("arrow-up-down-bold-outline", "arrow-up-down-bold-outline"),
                    ("arrow-up-drop-circle", "arrow-up-drop-circle"),
                    ("arrow-up-drop-circle-outline", "arrow-up-drop-circle-outline"),
                    ("arrow-up-left", "arrow-up-left"),
                    ("arrow-up-left-bold", "arrow-up-left-bold"),
                    ("arrow-up-right", "arrow-up-right"),
                    ("arrow-up-right-bold", "arrow-up-right-bold"),
                    ("arrow-up-thick", "arrow-up-thick"),
                    ("arrow-up-thin", "arrow-up-thin"),
                    ("arrow-up-thin-circle-outline", "arrow-up-thin-circle-outline"),
                    ("arrow-vertical-lock", "arrow-vertical-lock"),
                    ("artboard", "artboard"),
                    ("artstation", "artstation"),
                    ("aspect-ratio", "aspect-ratio"),
                    ("assistant", "assistant"),
                    ("asterisk", "asterisk"),
                    ("asterisk-circle-outline", "asterisk-circle-outline"),
                    ("at", "at"),
                    ("atlassian", "atlassian"),
                    ("atm", "atm"),
                    ("atom", "atom"),
                    ("atom-variant", "atom-variant"),
                    ("attachment", "attachment"),
                    ("attachment-check", "attachment-check"),
                    ("attachment-lock", "attachment-lock"),
                    ("attachment-minus", "attachment-minus"),
                    ("attachment-off", "attachment-off"),
                    ("attachment-plus", "attachment-plus"),
                    ("attachment-remove", "attachment-remove"),
                    ("atv", "atv"),
                    ("audio-input-rca", "audio-input-rca"),
                    ("audio-input-stereo-minijack", "audio-input-stereo-minijack"),
                    ("audio-input-xlr", "audio-input-xlr"),
                    ("audio-video", "audio-video"),
                    ("audio-video-off", "audio-video-off"),
                    ("augmented-reality", "augmented-reality"),
                    ("aurora", "aurora"),
                    ("auto-download", "auto-download"),
                    ("auto-fix", "auto-fix"),
                    ("auto-mode", "auto-mode"),
                    ("auto-upload", "auto-upload"),
                    ("autorenew", "autorenew"),
                    ("autorenew-off", "autorenew-off"),
                    ("av-timer", "av-timer"),
                    ("awning", "awning"),
                    ("awning-outline", "awning-outline"),
                    ("aws", "aws"),
                    ("axe", "axe"),
                    ("axe-battle", "axe-battle"),
                    ("axis", "axis"),
                    ("axis-arrow", "axis-arrow"),
                    ("axis-arrow-info", "axis-arrow-info"),
                    ("axis-arrow-lock", "axis-arrow-lock"),
                    ("axis-lock", "axis-lock"),
                    ("axis-x-arrow", "axis-x-arrow"),
                    ("axis-x-arrow-lock", "axis-x-arrow-lock"),
                    ("axis-x-rotate-clockwise", "axis-x-rotate-clockwise"),
                    ("axis-x-rotate-counterclockwise", "axis-x-rotate-counterclockwise"),
                    ("axis-x-y-arrow-lock", "axis-x-y-arrow-lock"),
                    ("axis-y-arrow", "axis-y-arrow"),
                    ("axis-y-arrow-lock", "axis-y-arrow-lock"),
                    ("axis-y-rotate-clockwise", "axis-y-rotate-clockwise"),
                    ("axis-y-rotate-counterclockwise", "axis-y-rotate-counterclockwise"),
                    ("axis-z-arrow", "axis-z-arrow"),
                    ("axis-z-arrow-lock", "axis-z-arrow-lock"),
                    ("axis-z-rotate-clockwise", "axis-z-rotate-clockwise"),
                    ("axis-z-rotate-counterclockwise", "axis-z-rotate-counterclockwise"),
                    ("babel", "babel"),
                    ("baby", "baby"),
                    ("baby-bottle", "baby-bottle"),
                    ("baby-bottle-outline", "baby-bottle-outline"),
                    ("baby-buggy", "baby-buggy"),
                    ("baby-buggy-off", "baby-buggy-off"),
                    ("baby-carriage", "baby-carriage"),
                    ("baby-carriage-off", "baby-carriage-off"),
                    ("baby-face", "baby-face"),
                    ("baby-face-outline", "baby-face-outline"),
                    ("backburger", "backburger"),
                    ("backspace", "backspace"),
                    ("backspace-outline", "backspace-outline"),
                    ("backspace-reverse", "backspace-reverse"),
                    ("backspace-reverse-outline", "backspace-reverse-outline"),
                    ("backup-restore", "backup-restore"),
                    ("bacteria", "bacteria"),
                    ("bacteria-outline", "bacteria-outline"),
                    ("badge-account", "badge-account"),
                    ("badge-account-alert", "badge-account-alert"),
                    ("badge-account-alert-outline", "badge-account-alert-outline"),
                    ("badge-account-horizontal", "badge-account-horizontal"),
                    ("badge-account-horizontal-outline", "badge-account-horizontal-outline"),
                    ("badge-account-outline", "badge-account-outline"),
                    ("badminton", "badminton"),
                    ("bag-carry-on", "bag-carry-on"),
                    ("bag-carry-on-check", "bag-carry-on-check"),
                    ("bag-carry-on-off", "bag-carry-on-off"),
                    ("bag-checked", "bag-checked"),
                    ("bag-personal", "bag-personal"),
                    ("bag-personal-off", "bag-personal-off"),
                    ("bag-personal-off-outline", "bag-personal-off-outline"),
                    ("bag-personal-outline", "bag-personal-outline"),
                    ("bag-personal-plus", "bag-personal-plus"),
                    ("bag-personal-plus-outline", "bag-personal-plus-outline"),
                    ("bag-personal-tag", "bag-personal-tag"),
                    ("bag-personal-tag-outline", "bag-personal-tag-outline"),
                    ("bag-suitcase", "bag-suitcase"),
                    ("bag-suitcase-off", "bag-suitcase-off"),
                    ("bag-suitcase-off-outline", "bag-suitcase-off-outline"),
                    ("bag-suitcase-outline", "bag-suitcase-outline"),
                    ("baguette", "baguette"),
                    ("balcony", "balcony"),
                    ("balloon", "balloon"),
                    ("ballot", "ballot"),
                    ("ballot-outline", "ballot-outline"),
                    ("ballot-recount", "ballot-recount"),
                    ("ballot-recount-outline", "ballot-recount-outline"),
                    ("bandage", "bandage"),
                    ("bandcamp", "bandcamp"),
                    ("bank", "bank"),
                    ("bank-check", "bank-check"),
                    ("bank-circle", "bank-circle"),
                    ("bank-circle-outline", "bank-circle-outline"),
                    ("bank-minus", "bank-minus"),
                    ("bank-off", "bank-off"),
                    ("bank-off-outline", "bank-off-outline"),
                    ("bank-outline", "bank-outline"),
                    ("bank-plus", "bank-plus"),
                    ("bank-remove", "bank-remove"),
                    ("bank-transfer", "bank-transfer"),
                    ("bank-transfer-in", "bank-transfer-in"),
                    ("bank-transfer-out", "bank-transfer-out"),
                    ("barcode", "barcode"),
                    ("barcode-off", "barcode-off"),
                    ("barcode-scan", "barcode-scan"),
                    ("barley", "barley"),
                    ("barley-off", "barley-off"),
                    ("barn", "barn"),
                    ("barrel", "barrel"),
                    ("barrel-outline", "barrel-outline"),
                    ("baseball", "baseball"),
                    ("baseball-bat", "baseball-bat"),
                    ("baseball-diamond", "baseball-diamond"),
                    ("baseball-diamond-outline", "baseball-diamond-outline"),
                    ("baseball-outline", "baseball-outline"),
                    ("basecamp", "basecamp"),
                    ("bash", "bash"),
                    ("basket", "basket"),
                    ("basket-check", "basket-check"),
                    ("basket-check-outline", "basket-check-outline"),
                    ("basket-fill", "basket-fill"),
                    ("basket-minus", "basket-minus"),
                    ("basket-minus-outline", "basket-minus-outline"),
                    ("basket-off", "basket-off"),
                    ("basket-off-outline", "basket-off-outline"),
                    ("basket-outline", "basket-outline"),
                    ("basket-plus", "basket-plus"),
                    ("basket-plus-outline", "basket-plus-outline"),
                    ("basket-remove", "basket-remove"),
                    ("basket-remove-outline", "basket-remove-outline"),
                    ("basket-unfill", "basket-unfill"),
                    ("basketball", "basketball"),
                    ("basketball-hoop", "basketball-hoop"),
                    ("basketball-hoop-outline", "basketball-hoop-outline"),
                    ("bat", "bat"),
                    ("bathtub", "bathtub"),
                    ("bathtub-outline", "bathtub-outline"),
                    ("battery", "battery"),
                    ("battery-10", "battery-10"),
                    ("battery-10-bluetooth", "battery-10-bluetooth"),
                    ("battery-20", "battery-20"),
                    ("battery-20-bluetooth", "battery-20-bluetooth"),
                    ("battery-30", "battery-30"),
                    ("battery-30-bluetooth", "battery-30-bluetooth"),
                    ("battery-40", "battery-40"),
                    ("battery-40-bluetooth", "battery-40-bluetooth"),
                    ("battery-50", "battery-50"),
                    ("battery-50-bluetooth", "battery-50-bluetooth"),
                    ("battery-60", "battery-60"),
                    ("battery-60-bluetooth", "battery-60-bluetooth"),
                    ("battery-70", "battery-70"),
                    ("battery-70-bluetooth", "battery-70-bluetooth"),
                    ("battery-80", "battery-80"),
                    ("battery-80-bluetooth", "battery-80-bluetooth"),
                    ("battery-90", "battery-90"),
                    ("battery-90-bluetooth", "battery-90-bluetooth"),
                    ("battery-alert", "battery-alert"),
                    ("battery-alert-bluetooth", "battery-alert-bluetooth"),
                    ("battery-alert-variant", "battery-alert-variant"),
                    ("battery-alert-variant-outline", "battery-alert-variant-outline"),
                    ("battery-arrow-down", "battery-arrow-down"),
                    ("battery-arrow-down-outline", "battery-arrow-down-outline"),
                    ("battery-arrow-up", "battery-arrow-up"),
                    ("battery-arrow-up-outline", "battery-arrow-up-outline"),
                    ("battery-bluetooth", "battery-bluetooth"),
                    ("battery-bluetooth-variant", "battery-bluetooth-variant"),
                    ("battery-charging", "battery-charging"),
                    ("battery-charging-10", "battery-charging-10"),
                    ("battery-charging-100", "battery-charging-100"),
                    ("battery-charging-20", "battery-charging-20"),
                    ("battery-charging-30", "battery-charging-30"),
                    ("battery-charging-40", "battery-charging-40"),
                    ("battery-charging-50", "battery-charging-50"),
                    ("battery-charging-60", "battery-charging-60"),
                    ("battery-charging-70", "battery-charging-70"),
                    ("battery-charging-80", "battery-charging-80"),
                    ("battery-charging-90", "battery-charging-90"),
                    ("battery-charging-high", "battery-charging-high"),
                    ("battery-charging-low", "battery-charging-low"),
                    ("battery-charging-medium", "battery-charging-medium"),
                    ("battery-charging-outline", "battery-charging-outline"),
                    ("battery-charging-wireless", "battery-charging-wireless"),
                    ("battery-charging-wireless-10", "battery-charging-wireless-10"),
                    ("battery-charging-wireless-20", "battery-charging-wireless-20"),
                    ("battery-charging-wireless-30", "battery-charging-wireless-30"),
                    ("battery-charging-wireless-40", "battery-charging-wireless-40"),
                    ("battery-charging-wireless-50", "battery-charging-wireless-50"),
                    ("battery-charging-wireless-60", "battery-charging-wireless-60"),
                    ("battery-charging-wireless-70", "battery-charging-wireless-70"),
                    ("battery-charging-wireless-80", "battery-charging-wireless-80"),
                    ("battery-charging-wireless-90", "battery-charging-wireless-90"),
                    ("battery-charging-wireless-alert", "battery-charging-wireless-alert"),
                    ("battery-charging-wireless-outline", "battery-charging-wireless-outline"),
                    ("battery-check", "battery-check"),
                    ("battery-check-outline", "battery-check-outline"),
                    ("battery-clock", "battery-clock"),
                    ("battery-clock-outline", "battery-clock-outline"),
                    ("battery-heart", "battery-heart"),
                    ("battery-heart-outline", "battery-heart-outline"),
                    ("battery-heart-variant", "battery-heart-variant"),
                    ("battery-high", "battery-high"),
                    ("battery-lock", "battery-lock"),
                    ("battery-lock-open", "battery-lock-open"),
                    ("battery-low", "battery-low"),
                    ("battery-medium", "battery-medium"),
                    ("battery-minus", "battery-minus"),
                    ("battery-minus-outline", "battery-minus-outline"),
                    ("battery-minus-variant", "battery-minus-variant"),
                    ("battery-negative", "battery-negative"),
                    ("battery-off", "battery-off"),
                    ("battery-off-outline", "battery-off-outline"),
                    ("battery-outline", "battery-outline"),
                    ("battery-plus", "battery-plus"),
                    ("battery-plus-outline", "battery-plus-outline"),
                    ("battery-plus-variant", "battery-plus-variant"),
                    ("battery-positive", "battery-positive"),
                    ("battery-remove", "battery-remove"),
                    ("battery-remove-outline", "battery-remove-outline"),
                    ("battery-standard", "battery-standard"),
                    ("battery-sync", "battery-sync"),
                    ("battery-sync-outline", "battery-sync-outline"),
                    ("battery-unknown", "battery-unknown"),
                    ("battery-unknown-bluetooth", "battery-unknown-bluetooth"),
                    ("battlenet", "battlenet"),
                    ("beach", "beach"),
                    ("beaker", "beaker"),
                    ("beaker-alert", "beaker-alert"),
                    ("beaker-alert-outline", "beaker-alert-outline"),
                    ("beaker-check", "beaker-check"),
                    ("beaker-check-outline", "beaker-check-outline"),
                    ("beaker-minus", "beaker-minus"),
                    ("beaker-minus-outline", "beaker-minus-outline"),
                    ("beaker-outline", "beaker-outline"),
                    ("beaker-plus", "beaker-plus"),
                    ("beaker-plus-outline", "beaker-plus-outline"),
                    ("beaker-question", "beaker-question"),
                    ("beaker-question-outline", "beaker-question-outline"),
                    ("beaker-remove", "beaker-remove"),
                    ("beaker-remove-outline", "beaker-remove-outline"),
                    ("beam", "beam"),
                    ("beats", "beats"),
                    ("bed", "bed"),
                    ("bed-clock", "bed-clock"),
                    ("bed-double", "bed-double"),
                    ("bed-double-outline", "bed-double-outline"),
                    ("bed-empty", "bed-empty"),
                    ("bed-king", "bed-king"),
                    ("bed-king-outline", "bed-king-outline"),
                    ("bed-outline", "bed-outline"),
                    ("bed-queen", "bed-queen"),
                    ("bed-queen-outline", "bed-queen-outline"),
                    ("bed-single", "bed-single"),
                    ("bed-single-outline", "bed-single-outline"),
                    ("bee", "bee"),
                    ("bee-flower", "bee-flower"),
                    ("beehive-off-outline", "beehive-off-outline"),
                    ("beehive-outline", "beehive-outline"),
                    ("beekeeper", "beekeeper"),
                    ("beer", "beer"),
                    ("beer-outline", "beer-outline"),
                    ("behance", "behance"),
                    ("bell", "bell"),
                    ("bell-alert", "bell-alert"),
                    ("bell-alert-outline", "bell-alert-outline"),
                    ("bell-badge", "bell-badge"),
                    ("bell-badge-outline", "bell-badge-outline"),
                    ("bell-cancel", "bell-cancel"),
                    ("bell-cancel-outline", "bell-cancel-outline"),
                    ("bell-check", "bell-check"),
                    ("bell-check-outline", "bell-check-outline"),
                    ("bell-circle", "bell-circle"),
                    ("bell-circle-outline", "bell-circle-outline"),
                    ("bell-cog", "bell-cog"),
                    ("bell-cog-outline", "bell-cog-outline"),
                    ("bell-minus", "bell-minus"),
                    ("bell-minus-outline", "bell-minus-outline"),
                    ("bell-off", "bell-off"),
                    ("bell-off-outline", "bell-off-outline"),
                    ("bell-outline", "bell-outline"),
                    ("bell-plus", "bell-plus"),
                    ("bell-plus-outline", "bell-plus-outline"),
                    ("bell-remove", "bell-remove"),
                    ("bell-remove-outline", "bell-remove-outline"),
                    ("bell-ring", "bell-ring"),
                    ("bell-ring-outline", "bell-ring-outline"),
                    ("bell-sleep", "bell-sleep"),
                    ("bell-sleep-outline", "bell-sleep-outline"),
                    ("bench", "bench"),
                    ("bench-back", "bench-back"),
                    ("beta", "beta"),
                    ("betamax", "betamax"),
                    ("biathlon", "biathlon"),
                    ("bicycle", "bicycle"),
                    ("bicycle-basket", "bicycle-basket"),
                    ("bicycle-cargo", "bicycle-cargo"),
                    ("bicycle-electric", "bicycle-electric"),
                    ("bicycle-penny-farthing", "bicycle-penny-farthing"),
                    ("bike", "bike"),
                    ("bike-fast", "bike-fast"),
                    ("bike-pedal", "bike-pedal"),
                    ("bike-pedal-clipless", "bike-pedal-clipless"),
                    ("bike-pedal-mountain", "bike-pedal-mountain"),
                    ("billboard", "billboard"),
                    ("billiards", "billiards"),
                    ("billiards-rack", "billiards-rack"),
                    ("binoculars", "binoculars"),
                    ("bio", "bio"),
                    ("biohazard", "biohazard"),
                    ("bird", "bird"),
                    ("bitbucket", "bitbucket"),
                    ("bitcoin", "bitcoin"),
                    ("black-mesa", "black-mesa"),
                    ("blackberry", "blackberry"),
                    ("blender", "blender"),
                    ("blender-outline", "blender-outline"),
                    ("blender-software", "blender-software"),
                    ("blinds", "blinds"),
                    ("blinds-horizontal", "blinds-horizontal"),
                    ("blinds-horizontal-closed", "blinds-horizontal-closed"),
                    ("blinds-open", "blinds-open"),
                    ("blinds-vertical", "blinds-vertical"),
                    ("blinds-vertical-closed", "blinds-vertical-closed"),
                    ("block-helper", "block-helper"),
                    ("blogger", "blogger"),
                    ("blood-bag", "blood-bag"),
                    ("bluetooth", "bluetooth"),
                    ("bluetooth-audio", "bluetooth-audio"),
                    ("bluetooth-connect", "bluetooth-connect"),
                    ("bluetooth-off", "bluetooth-off"),
                    ("bluetooth-settings", "bluetooth-settings"),
                    ("bluetooth-transfer", "bluetooth-transfer"),
                    ("blur", "blur"),
                    ("blur-linear", "blur-linear"),
                    ("blur-off", "blur-off"),
                    ("blur-radial", "blur-radial"),
                    ("bolt", "bolt"),
                    ("bomb", "bomb"),
                    ("bomb-off", "bomb-off"),
                    ("bone", "bone"),
                    ("bone-off", "bone-off"),
                    ("book", "book"),
                    ("book-account", "book-account"),
                    ("book-account-outline", "book-account-outline"),
                    ("book-alert", "book-alert"),
                    ("book-alert-outline", "book-alert-outline"),
                    ("book-alphabet", "book-alphabet"),
                    ("book-arrow-down", "book-arrow-down"),
                    ("book-arrow-down-outline", "book-arrow-down-outline"),
                    ("book-arrow-left", "book-arrow-left"),
                    ("book-arrow-left-outline", "book-arrow-left-outline"),
                    ("book-arrow-right", "book-arrow-right"),
                    ("book-arrow-right-outline", "book-arrow-right-outline"),
                    ("book-arrow-up", "book-arrow-up"),
                    ("book-arrow-up-outline", "book-arrow-up-outline"),
                    ("book-cancel", "book-cancel"),
                    ("book-cancel-outline", "book-cancel-outline"),
                    ("book-check", "book-check"),
                    ("book-check-outline", "book-check-outline"),
                    ("book-clock", "book-clock"),
                    ("book-clock-outline", "book-clock-outline"),
                    ("book-cog", "book-cog"),
                    ("book-cog-outline", "book-cog-outline"),
                    ("book-cross", "book-cross"),
                    ("book-edit", "book-edit"),
                    ("book-edit-outline", "book-edit-outline"),
                    ("book-education", "book-education"),
                    ("book-education-outline", "book-education-outline"),
                    ("book-heart", "book-heart"),
                    ("book-heart-outline", "book-heart-outline"),
                    ("book-information-variant", "book-information-variant"),
                    ("book-lock", "book-lock"),
                    ("book-lock-open", "book-lock-open"),
                    ("book-lock-open-outline", "book-lock-open-outline"),
                    ("book-lock-outline", "book-lock-outline"),
                    ("book-marker", "book-marker"),
                    ("book-marker-outline", "book-marker-outline"),
                    ("book-minus", "book-minus"),
                    ("book-minus-multiple", "book-minus-multiple"),
                    ("book-minus-multiple-outline", "book-minus-multiple-outline"),
                    ("book-minus-outline", "book-minus-outline"),
                    ("book-multiple", "book-multiple"),
                    ("book-multiple-minus", "book-multiple-minus"),
                    ("book-multiple-outline", "book-multiple-outline"),
                    ("book-multiple-plus", "book-multiple-plus"),
                    ("book-multiple-remove", "book-multiple-remove"),
                    ("book-multiple-variant", "book-multiple-variant"),
                    ("book-music", "book-music"),
                    ("book-music-outline", "book-music-outline"),
                    ("book-off", "book-off"),
                    ("book-off-outline", "book-off-outline"),
                    ("book-open", "book-open"),
                    ("book-open-blank-variant", "book-open-blank-variant"),
                    ("book-open-outline", "book-open-outline"),
                    ("book-open-page-variant", "book-open-page-variant"),
                    ("book-open-page-variant-outline", "book-open-page-variant-outline"),
                    ("book-open-variant", "book-open-variant"),
                    ("book-outline", "book-outline"),
                    ("book-play", "book-play"),
                    ("book-play-outline", "book-play-outline"),
                    ("book-plus", "book-plus"),
                    ("book-plus-multiple", "book-plus-multiple"),
                    ("book-plus-multiple-outline", "book-plus-multiple-outline"),
                    ("book-plus-outline", "book-plus-outline"),
                    ("book-refresh", "book-refresh"),
                    ("book-refresh-outline", "book-refresh-outline"),
                    ("book-remove", "book-remove"),
                    ("book-remove-multiple", "book-remove-multiple"),
                    ("book-remove-multiple-outline", "book-remove-multiple-outline"),
                    ("book-remove-outline", "book-remove-outline"),
                    ("book-search", "book-search"),
                    ("book-search-outline", "book-search-outline"),
                    ("book-settings", "book-settings"),
                    ("book-settings-outline", "book-settings-outline"),
                    ("book-sync", "book-sync"),
                    ("book-sync-outline", "book-sync-outline"),
                    ("book-variant", "book-variant"),
                    ("book-variant-multiple", "book-variant-multiple"),
                    ("bookmark", "bookmark"),
                    ("bookmark-box", "bookmark-box"),
                    ("bookmark-box-multiple", "bookmark-box-multiple"),
                    ("bookmark-box-multiple-outline", "bookmark-box-multiple-outline"),
                    ("bookmark-box-outline", "bookmark-box-outline"),
                    ("bookmark-check", "bookmark-check"),
                    ("bookmark-check-outline", "bookmark-check-outline"),
                    ("bookmark-minus", "bookmark-minus"),
                    ("bookmark-minus-outline", "bookmark-minus-outline"),
                    ("bookmark-multiple", "bookmark-multiple"),
                    ("bookmark-multiple-outline", "bookmark-multiple-outline"),
                    ("bookmark-music", "bookmark-music"),
                    ("bookmark-music-outline", "bookmark-music-outline"),
                    ("bookmark-off", "bookmark-off"),
                    ("bookmark-off-outline", "bookmark-off-outline"),
                    ("bookmark-outline", "bookmark-outline"),
                    ("bookmark-plus", "bookmark-plus"),
                    ("bookmark-plus-outline", "bookmark-plus-outline"),
                    ("bookmark-remove", "bookmark-remove"),
                    ("bookmark-remove-outline", "bookmark-remove-outline"),
                    ("bookshelf", "bookshelf"),
                    ("boom-gate", "boom-gate"),
                    ("boom-gate-alert", "boom-gate-alert"),
                    ("boom-gate-alert-outline", "boom-gate-alert-outline"),
                    ("boom-gate-arrow-down", "boom-gate-arrow-down"),
                    ("boom-gate-arrow-down-outline", "boom-gate-arrow-down-outline"),
                    ("boom-gate-arrow-up", "boom-gate-arrow-up"),
                    ("boom-gate-arrow-up-outline", "boom-gate-arrow-up-outline"),
                    ("boom-gate-outline", "boom-gate-outline"),
                    ("boom-gate-up", "boom-gate-up"),
                    ("boom-gate-up-outline", "boom-gate-up-outline"),
                    ("boombox", "boombox"),
                    ("boomerang", "boomerang"),
                    ("bootstrap", "bootstrap"),
                    ("border-all", "border-all"),
                    ("border-all-variant", "border-all-variant"),
                    ("border-bottom", "border-bottom"),
                    ("border-bottom-variant", "border-bottom-variant"),
                    ("border-color", "border-color"),
                    ("border-horizontal", "border-horizontal"),
                    ("border-inside", "border-inside"),
                    ("border-left", "border-left"),
                    ("border-left-variant", "border-left-variant"),
                    ("border-none", "border-none"),
                    ("border-none-variant", "border-none-variant"),
                    ("border-outside", "border-outside"),
                    ("border-radius", "border-radius"),
                    ("border-right", "border-right"),
                    ("border-right-variant", "border-right-variant"),
                    ("border-style", "border-style"),
                    ("border-top", "border-top"),
                    ("border-top-variant", "border-top-variant"),
                    ("border-vertical", "border-vertical"),
                    ("bottle-soda", "bottle-soda"),
                    ("bottle-soda-classic", "bottle-soda-classic"),
                    ("bottle-soda-classic-outline", "bottle-soda-classic-outline"),
                    ("bottle-soda-outline", "bottle-soda-outline"),
                    ("bottle-tonic", "bottle-tonic"),
                    ("bottle-tonic-outline", "bottle-tonic-outline"),
                    ("bottle-tonic-plus", "bottle-tonic-plus"),
                    ("bottle-tonic-plus-outline", "bottle-tonic-plus-outline"),
                    ("bottle-tonic-skull", "bottle-tonic-skull"),
                    ("bottle-tonic-skull-outline", "bottle-tonic-skull-outline"),
                    ("bottle-wine", "bottle-wine"),
                    ("bottle-wine-outline", "bottle-wine-outline"),
                    ("bow-arrow", "bow-arrow"),
                    ("bow-tie", "bow-tie"),
                    ("bowl", "bowl"),
                    ("bowl-mix", "bowl-mix"),
                    ("bowl-mix-outline", "bowl-mix-outline"),
                    ("bowl-outline", "bowl-outline"),
                    ("bowling", "bowling"),
                    ("box", "box"),
                    ("box-cutter", "box-cutter"),
                    ("box-cutter-off", "box-cutter-off"),
                    ("box-download", "box-download"),
                    ("box-shadow", "box-shadow"),
                    ("box-upload", "box-upload"),
                    ("boxing-glove", "boxing-glove"),
                    ("boxing-gloves", "boxing-gloves"),
                    ("braille", "braille"),
                    ("brain", "brain"),
                    ("bread-slice", "bread-slice"),
                    ("bread-slice-outline", "bread-slice-outline"),
                    ("bridge", "bridge"),
                    ("briefcase", "briefcase"),
                    ("briefcase-account", "briefcase-account"),
                    ("briefcase-account-outline", "briefcase-account-outline"),
                    ("briefcase-arrow-left-right", "briefcase-arrow-left-right"),
                    ("briefcase-arrow-left-right-outline", "briefcase-arrow-left-right-outline"),
                    ("briefcase-arrow-up-down", "briefcase-arrow-up-down"),
                    ("briefcase-arrow-up-down-outline", "briefcase-arrow-up-down-outline"),
                    ("briefcase-check", "briefcase-check"),
                    ("briefcase-check-outline", "briefcase-check-outline"),
                    ("briefcase-clock", "briefcase-clock"),
                    ("briefcase-clock-outline", "briefcase-clock-outline"),
                    ("briefcase-download", "briefcase-download"),
                    ("briefcase-download-outline", "briefcase-download-outline"),
                    ("briefcase-edit", "briefcase-edit"),
                    ("briefcase-edit-outline", "briefcase-edit-outline"),
                    ("briefcase-eye", "briefcase-eye"),
                    ("briefcase-eye-outline", "briefcase-eye-outline"),
                    ("briefcase-minus", "briefcase-minus"),
                    ("briefcase-minus-outline", "briefcase-minus-outline"),
                    ("briefcase-off", "briefcase-off"),
                    ("briefcase-off-outline", "briefcase-off-outline"),
                    ("briefcase-outline", "briefcase-outline"),
                    ("briefcase-plus", "briefcase-plus"),
                    ("briefcase-plus-outline", "briefcase-plus-outline"),
                    ("briefcase-remove", "briefcase-remove"),
                    ("briefcase-remove-outline", "briefcase-remove-outline"),
                    ("briefcase-search", "briefcase-search"),
                    ("briefcase-search-outline", "briefcase-search-outline"),
                    ("briefcase-upload", "briefcase-upload"),
                    ("briefcase-upload-outline", "briefcase-upload-outline"),
                    ("briefcase-variant", "briefcase-variant"),
                    ("briefcase-variant-off", "briefcase-variant-off"),
                    ("briefcase-variant-off-outline", "briefcase-variant-off-outline"),
                    ("briefcase-variant-outline", "briefcase-variant-outline"),
                    ("brightness", "brightness"),
                    ("brightness-1", "brightness-1"),
                    ("brightness-2", "brightness-2"),
                    ("brightness-3", "brightness-3"),
                    ("brightness-4", "brightness-4"),
                    ("brightness-5", "brightness-5"),
                    ("brightness-6", "brightness-6"),
                    ("brightness-7", "brightness-7"),
                    ("brightness-auto", "brightness-auto"),
                    ("brightness-percent", "brightness-percent"),
                    ("broadcast", "broadcast"),
                    ("broadcast-off", "broadcast-off"),
                    ("broom", "broom"),
                    ("brush", "brush"),
                    ("brush-off", "brush-off"),
                    ("brush-outline", "brush-outline"),
                    ("brush-variant", "brush-variant"),
                    ("bucket", "bucket"),
                    ("bucket-outline", "bucket-outline"),
                    ("buffer", "buffer"),
                    ("buffet", "buffet"),
                    ("bug", "bug"),
                    ("bug-check", "bug-check"),
                    ("bug-check-outline", "bug-check-outline"),
                    ("bug-outline", "bug-outline"),
                    ("bug-pause", "bug-pause"),
                    ("bug-pause-outline", "bug-pause-outline"),
                    ("bug-play", "bug-play"),
                    ("bug-play-outline", "bug-play-outline"),
                    ("bug-stop", "bug-stop"),
                    ("bug-stop-outline", "bug-stop-outline"),
                    ("bugle", "bugle"),
                    ("bulkhead-light", "bulkhead-light"),
                    ("bulldozer", "bulldozer"),
                    ("bullet", "bullet"),
                    ("bulletin-board", "bulletin-board"),
                    ("bullhorn", "bullhorn"),
                    ("bullhorn-outline", "bullhorn-outline"),
                    ("bullhorn-variant", "bullhorn-variant"),
                    ("bullhorn-variant-outline", "bullhorn-variant-outline"),
                    ("bullseye", "bullseye"),
                    ("bullseye-arrow", "bullseye-arrow"),
                    ("bulma", "bulma"),
                    ("bunk-bed", "bunk-bed"),
                    ("bunk-bed-outline", "bunk-bed-outline"),
                    ("bus", "bus"),
                    ("bus-alert", "bus-alert"),
                    ("bus-articulated-end", "bus-articulated-end"),
                    ("bus-articulated-front", "bus-articulated-front"),
                    ("bus-clock", "bus-clock"),
                    ("bus-double-decker", "bus-double-decker"),
                    ("bus-electric", "bus-electric"),
                    ("bus-marker", "bus-marker"),
                    ("bus-multiple", "bus-multiple"),
                    ("bus-school", "bus-school"),
                    ("bus-side", "bus-side"),
                    ("bus-stop", "bus-stop"),
                    ("bus-stop-covered", "bus-stop-covered"),
                    ("bus-stop-uncovered", "bus-stop-uncovered"),
                    ("butterfly", "butterfly"),
                    ("butterfly-outline", "butterfly-outline"),
                    ("button-cursor", "button-cursor"),
                    ("button-pointer", "button-pointer"),
                    ("cabin-a-frame", "cabin-a-frame"),
                    ("cable-data", "cable-data"),
                    ("cached", "cached"),
                    ("cactus", "cactus"),
                    ("cake", "cake"),
                    ("cake-layered", "cake-layered"),
                    ("cake-variant", "cake-variant"),
                    ("cake-variant-outline", "cake-variant-outline"),
                    ("calculator", "calculator"),
                    ("calculator-off", "calculator-off"),
                    ("calculator-variant", "calculator-variant"),
                    ("calculator-variant-outline", "calculator-variant-outline"),
                    ("calendar", "calendar"),
                    ("calendar-account", "calendar-account"),
                    ("calendar-account-outline", "calendar-account-outline"),
                    ("calendar-alert", "calendar-alert"),
                    ("calendar-alert-outline", "calendar-alert-outline"),
                    ("calendar-arrow-left", "calendar-arrow-left"),
                    ("calendar-arrow-right", "calendar-arrow-right"),
                    ("calendar-badge", "calendar-badge"),
                    ("calendar-badge-outline", "calendar-badge-outline"),
                    ("calendar-blank", "calendar-blank"),
                    ("calendar-blank-multiple", "calendar-blank-multiple"),
                    ("calendar-blank-outline", "calendar-blank-outline"),
                    ("calendar-check", "calendar-check"),
                    ("calendar-check-outline", "calendar-check-outline"),
                    ("calendar-clock", "calendar-clock"),
                    ("calendar-clock-outline", "calendar-clock-outline"),
                    ("calendar-collapse-horizontal", "calendar-collapse-horizontal"),
                    (
                        "calendar-collapse-horizontal-outline",
                        "calendar-collapse-horizontal-outline",
                    ),
                    ("calendar-cursor", "calendar-cursor"),
                    ("calendar-cursor-outline", "calendar-cursor-outline"),
                    ("calendar-edit", "calendar-edit"),
                    ("calendar-edit-outline", "calendar-edit-outline"),
                    ("calendar-end", "calendar-end"),
                    ("calendar-end-outline", "calendar-end-outline"),
                    ("calendar-expand-horizontal", "calendar-expand-horizontal"),
                    ("calendar-expand-horizontal-outline", "calendar-expand-horizontal-outline"),
                    ("calendar-export", "calendar-export"),
                    ("calendar-export-outline", "calendar-export-outline"),
                    ("calendar-filter", "calendar-filter"),
                    ("calendar-filter-outline", "calendar-filter-outline"),
                    ("calendar-heart", "calendar-heart"),
                    ("calendar-heart-outline", "calendar-heart-outline"),
                    ("calendar-import", "calendar-import"),
                    ("calendar-import-outline", "calendar-import-outline"),
                    ("calendar-lock", "calendar-lock"),
                    ("calendar-lock-open", "calendar-lock-open"),
                    ("calendar-lock-open-outline", "calendar-lock-open-outline"),
                    ("calendar-lock-outline", "calendar-lock-outline"),
                    ("calendar-minus", "calendar-minus"),
                    ("calendar-minus-outline", "calendar-minus-outline"),
                    ("calendar-month", "calendar-month"),
                    ("calendar-month-outline", "calendar-month-outline"),
                    ("calendar-multiple", "calendar-multiple"),
                    ("calendar-multiple-check", "calendar-multiple-check"),
                    ("calendar-multiselect", "calendar-multiselect"),
                    ("calendar-multiselect-outline", "calendar-multiselect-outline"),
                    ("calendar-outline", "calendar-outline"),
                    ("calendar-plus", "calendar-plus"),
                    ("calendar-plus-outline", "calendar-plus-outline"),
                    ("calendar-question", "calendar-question"),
                    ("calendar-question-outline", "calendar-question-outline"),
                    ("calendar-range", "calendar-range"),
                    ("calendar-range-outline", "calendar-range-outline"),
                    ("calendar-refresh", "calendar-refresh"),
                    ("calendar-refresh-outline", "calendar-refresh-outline"),
                    ("calendar-remove", "calendar-remove"),
                    ("calendar-remove-outline", "calendar-remove-outline"),
                    ("calendar-search", "calendar-search"),
                    ("calendar-search-outline", "calendar-search-outline"),
                    ("calendar-select", "calendar-select"),
                    ("calendar-star", "calendar-star"),
                    ("calendar-star-four-points", "calendar-star-four-points"),
                    ("calendar-star-outline", "calendar-star-outline"),
                    ("calendar-start", "calendar-start"),
                    ("calendar-start-outline", "calendar-start-outline"),
                    ("calendar-sync", "calendar-sync"),
                    ("calendar-sync-outline", "calendar-sync-outline"),
                    ("calendar-text", "calendar-text"),
                    ("calendar-text-outline", "calendar-text-outline"),
                    ("calendar-today", "calendar-today"),
                    ("calendar-today-outline", "calendar-today-outline"),
                    ("calendar-week", "calendar-week"),
                    ("calendar-week-begin", "calendar-week-begin"),
                    ("calendar-week-begin-outline", "calendar-week-begin-outline"),
                    ("calendar-week-end", "calendar-week-end"),
                    ("calendar-week-end-outline", "calendar-week-end-outline"),
                    ("calendar-week-outline", "calendar-week-outline"),
                    ("calendar-weekend", "calendar-weekend"),
                    ("calendar-weekend-outline", "calendar-weekend-outline"),
                    ("call-made", "call-made"),
                    ("call-merge", "call-merge"),
                    ("call-missed", "call-missed"),
                    ("call-received", "call-received"),
                    ("call-split", "call-split"),
                    ("camcorder", "camcorder"),
                    ("camcorder-off", "camcorder-off"),
                    ("camera", "camera"),
                    ("camera-account", "camera-account"),
                    ("camera-burst", "camera-burst"),
                    ("camera-control", "camera-control"),
                    ("camera-document", "camera-document"),
                    ("camera-document-off", "camera-document-off"),
                    ("camera-enhance", "camera-enhance"),
                    ("camera-enhance-outline", "camera-enhance-outline"),
                    ("camera-flip", "camera-flip"),
                    ("camera-flip-outline", "camera-flip-outline"),
                    ("camera-focus", "camera-focus"),
                    ("camera-front", "camera-front"),
                    ("camera-front-variant", "camera-front-variant"),
                    ("camera-gopro", "camera-gopro"),
                    ("camera-image", "camera-image"),
                    ("camera-iris", "camera-iris"),
                    ("camera-lock", "camera-lock"),
                    ("camera-lock-open", "camera-lock-open"),
                    ("camera-lock-open-outline", "camera-lock-open-outline"),
                    ("camera-lock-outline", "camera-lock-outline"),
                    ("camera-marker", "camera-marker"),
                    ("camera-marker-outline", "camera-marker-outline"),
                    ("camera-metering-center", "camera-metering-center"),
                    ("camera-metering-matrix", "camera-metering-matrix"),
                    ("camera-metering-partial", "camera-metering-partial"),
                    ("camera-metering-spot", "camera-metering-spot"),
                    ("camera-off", "camera-off"),
                    ("camera-off-outline", "camera-off-outline"),
                    ("camera-outline", "camera-outline"),
                    ("camera-party-mode", "camera-party-mode"),
                    ("camera-plus", "camera-plus"),
                    ("camera-plus-outline", "camera-plus-outline"),
                    ("camera-rear", "camera-rear"),
                    ("camera-rear-variant", "camera-rear-variant"),
                    ("camera-retake", "camera-retake"),
                    ("camera-retake-outline", "camera-retake-outline"),
                    ("camera-switch", "camera-switch"),
                    ("camera-switch-outline", "camera-switch-outline"),
                    ("camera-timer", "camera-timer"),
                    ("camera-wireless", "camera-wireless"),
                    ("camera-wireless-outline", "camera-wireless-outline"),
                    ("campfire", "campfire"),
                    ("cancel", "cancel"),
                    ("candelabra", "candelabra"),
                    ("candelabra-fire", "candelabra-fire"),
                    ("candle", "candle"),
                    ("candy", "candy"),
                    ("candy-off", "candy-off"),
                    ("candy-off-outline", "candy-off-outline"),
                    ("candy-outline", "candy-outline"),
                    ("candycane", "candycane"),
                    ("cannabis", "cannabis"),
                    ("cannabis-off", "cannabis-off"),
                    ("caps-lock", "caps-lock"),
                    ("car", "car"),
                    ("car-2-plus", "car-2-plus"),
                    ("car-3-plus", "car-3-plus"),
                    ("car-arrow-left", "car-arrow-left"),
                    ("car-arrow-right", "car-arrow-right"),
                    ("car-back", "car-back"),
                    ("car-battery", "car-battery"),
                    ("car-brake-abs", "car-brake-abs"),
                    ("car-brake-alert", "car-brake-alert"),
                    ("car-brake-fluid-level", "car-brake-fluid-level"),
                    ("car-brake-hold", "car-brake-hold"),
                    ("car-brake-low-pressure", "car-brake-low-pressure"),
                    ("car-brake-parking", "car-brake-parking"),
                    ("car-brake-retarder", "car-brake-retarder"),
                    ("car-brake-temperature", "car-brake-temperature"),
                    ("car-brake-worn-linings", "car-brake-worn-linings"),
                    ("car-child-seat", "car-child-seat"),
                    ("car-clock", "car-clock"),
                    ("car-clutch", "car-clutch"),
                    ("car-cog", "car-cog"),
                    ("car-connected", "car-connected"),
                    ("car-convertable", "car-convertable"),
                    ("car-convertible", "car-convertible"),
                    ("car-coolant-level", "car-coolant-level"),
                    ("car-cruise-control", "car-cruise-control"),
                    ("car-defrost-front", "car-defrost-front"),
                    ("car-defrost-rear", "car-defrost-rear"),
                    ("car-door", "car-door"),
                    ("car-door-lock", "car-door-lock"),
                    ("car-door-lock-open", "car-door-lock-open"),
                    ("car-electric", "car-electric"),
                    ("car-electric-outline", "car-electric-outline"),
                    ("car-emergency", "car-emergency"),
                    ("car-esp", "car-esp"),
                    ("car-estate", "car-estate"),
                    ("car-hatchback", "car-hatchback"),
                    ("car-info", "car-info"),
                    ("car-key", "car-key"),
                    ("car-lifted-pickup", "car-lifted-pickup"),
                    ("car-light-alert", "car-light-alert"),
                    ("car-light-dimmed", "car-light-dimmed"),
                    ("car-light-fog", "car-light-fog"),
                    ("car-light-high", "car-light-high"),
                    ("car-limousine", "car-limousine"),
                    ("car-multiple", "car-multiple"),
                    ("car-off", "car-off"),
                    ("car-outline", "car-outline"),
                    ("car-parking-lights", "car-parking-lights"),
                    ("car-pickup", "car-pickup"),
                    ("car-search", "car-search"),
                    ("car-search-outline", "car-search-outline"),
                    ("car-seat", "car-seat"),
                    ("car-seat-cooler", "car-seat-cooler"),
                    ("car-seat-heater", "car-seat-heater"),
                    ("car-select", "car-select"),
                    ("car-settings", "car-settings"),
                    ("car-shift-pattern", "car-shift-pattern"),
                    ("car-side", "car-side"),
                    ("car-speed-limiter", "car-speed-limiter"),
                    ("car-sports", "car-sports"),
                    ("car-tire-alert", "car-tire-alert"),
                    ("car-traction-control", "car-traction-control"),
                    ("car-turbocharger", "car-turbocharger"),
                    ("car-wash", "car-wash"),
                    ("car-windshield", "car-windshield"),
                    ("car-windshield-outline", "car-windshield-outline"),
                    ("car-wireless", "car-wireless"),
                    ("car-wrench", "car-wrench"),
                    ("carabiner", "carabiner"),
                    ("caravan", "caravan"),
                    ("card", "card"),
                    ("card-account-details", "card-account-details"),
                    ("card-account-details-outline", "card-account-details-outline"),
                    ("card-account-details-star", "card-account-details-star"),
                    ("card-account-details-star-outline", "card-account-details-star-outline"),
                    ("card-account-mail", "card-account-mail"),
                    ("card-account-mail-outline", "card-account-mail-outline"),
                    ("card-account-phone", "card-account-phone"),
                    ("card-account-phone-outline", "card-account-phone-outline"),
                    ("card-bulleted", "card-bulleted"),
                    ("card-bulleted-off", "card-bulleted-off"),
                    ("card-bulleted-off-outline", "card-bulleted-off-outline"),
                    ("card-bulleted-outline", "card-bulleted-outline"),
                    ("card-bulleted-settings", "card-bulleted-settings"),
                    ("card-bulleted-settings-outline", "card-bulleted-settings-outline"),
                    ("card-minus", "card-minus"),
                    ("card-minus-outline", "card-minus-outline"),
                    ("card-multiple", "card-multiple"),
                    ("card-multiple-outline", "card-multiple-outline"),
                    ("card-off", "card-off"),
                    ("card-off-outline", "card-off-outline"),
                    ("card-outline", "card-outline"),
                    ("card-plus", "card-plus"),
                    ("card-plus-outline", "card-plus-outline"),
                    ("card-remove", "card-remove"),
                    ("card-remove-outline", "card-remove-outline"),
                    ("card-search", "card-search"),
                    ("card-search-outline", "card-search-outline"),
                    ("card-text", "card-text"),
                    ("card-text-outline", "card-text-outline"),
                    ("cards", "cards"),
                    ("cards-club", "cards-club"),
                    ("cards-club-outline", "cards-club-outline"),
                    ("cards-diamond", "cards-diamond"),
                    ("cards-diamond-outline", "cards-diamond-outline"),
                    ("cards-heart", "cards-heart"),
                    ("cards-heart-outline", "cards-heart-outline"),
                    ("cards-outline", "cards-outline"),
                    ("cards-playing", "cards-playing"),
                    ("cards-playing-club", "cards-playing-club"),
                    ("cards-playing-club-multiple", "cards-playing-club-multiple"),
                    ("cards-playing-club-multiple-outline", "cards-playing-club-multiple-outline"),
                    ("cards-playing-club-outline", "cards-playing-club-outline"),
                    ("cards-playing-diamond", "cards-playing-diamond"),
                    ("cards-playing-diamond-multiple", "cards-playing-diamond-multiple"),
                    (
                        "cards-playing-diamond-multiple-outline",
                        "cards-playing-diamond-multiple-outline",
                    ),
                    ("cards-playing-diamond-outline", "cards-playing-diamond-outline"),
                    ("cards-playing-heart", "cards-playing-heart"),
                    ("cards-playing-heart-multiple", "cards-playing-heart-multiple"),
                    (
                        "cards-playing-heart-multiple-outline",
                        "cards-playing-heart-multiple-outline",
                    ),
                    ("cards-playing-heart-outline", "cards-playing-heart-outline"),
                    ("cards-playing-outline", "cards-playing-outline"),
                    ("cards-playing-spade", "cards-playing-spade"),
                    ("cards-playing-spade-multiple", "cards-playing-spade-multiple"),
                    (
                        "cards-playing-spade-multiple-outline",
                        "cards-playing-spade-multiple-outline",
                    ),
                    ("cards-playing-spade-outline", "cards-playing-spade-outline"),
                    ("cards-spade", "cards-spade"),
                    ("cards-spade-outline", "cards-spade-outline"),
                    ("cards-variant", "cards-variant"),
                    ("carrot", "carrot"),
                    ("cart", "cart"),
                    ("cart-arrow-down", "cart-arrow-down"),
                    ("cart-arrow-right", "cart-arrow-right"),
                    ("cart-arrow-up", "cart-arrow-up"),
                    ("cart-check", "cart-check"),
                    ("cart-heart", "cart-heart"),
                    ("cart-minus", "cart-minus"),
                    ("cart-off", "cart-off"),
                    ("cart-outline", "cart-outline"),
                    ("cart-percent", "cart-percent"),
                    ("cart-plus", "cart-plus"),
                    ("cart-remove", "cart-remove"),
                    ("cart-variant", "cart-variant"),
                    ("case-sensitive-alt", "case-sensitive-alt"),
                    ("cash", "cash"),
                    ("cash-100", "cash-100"),
                    ("cash-check", "cash-check"),
                    ("cash-clock", "cash-clock"),
                    ("cash-fast", "cash-fast"),
                    ("cash-lock", "cash-lock"),
                    ("cash-lock-open", "cash-lock-open"),
                    ("cash-marker", "cash-marker"),
                    ("cash-minus", "cash-minus"),
                    ("cash-multiple", "cash-multiple"),
                    ("cash-off", "cash-off"),
                    ("cash-plus", "cash-plus"),
                    ("cash-refund", "cash-refund"),
                    ("cash-register", "cash-register"),
                    ("cash-remove", "cash-remove"),
                    ("cash-sync", "cash-sync"),
                    ("cash-usd", "cash-usd"),
                    ("cash-usd-outline", "cash-usd-outline"),
                    ("cassette", "cassette"),
                    ("cast", "cast"),
                    ("cast-audio", "cast-audio"),
                    ("cast-audio-variant", "cast-audio-variant"),
                    ("cast-connected", "cast-connected"),
                    ("cast-education", "cast-education"),
                    ("cast-off", "cast-off"),
                    ("cast-variant", "cast-variant"),
                    ("castle", "castle"),
                    ("cat", "cat"),
                    ("cctv", "cctv"),
                    ("cctv-off", "cctv-off"),
                    ("ceiling-fan", "ceiling-fan"),
                    ("ceiling-fan-light", "ceiling-fan-light"),
                    ("ceiling-light", "ceiling-light"),
                    ("ceiling-light-multiple", "ceiling-light-multiple"),
                    ("ceiling-light-multiple-outline", "ceiling-light-multiple-outline"),
                    ("ceiling-light-outline", "ceiling-light-outline"),
                    ("cellphone", "cellphone"),
                    ("cellphone-android", "cellphone-android"),
                    ("cellphone-arrow-down", "cellphone-arrow-down"),
                    ("cellphone-arrow-down-variant", "cellphone-arrow-down-variant"),
                    ("cellphone-basic", "cellphone-basic"),
                    ("cellphone-charging", "cellphone-charging"),
                    ("cellphone-check", "cellphone-check"),
                    ("cellphone-cog", "cellphone-cog"),
                    ("cellphone-dock", "cellphone-dock"),
                    ("cellphone-information", "cellphone-information"),
                    ("cellphone-iphone", "cellphone-iphone"),
                    ("cellphone-key", "cellphone-key"),
                    ("cellphone-link", "cellphone-link"),
                    ("cellphone-link-off", "cellphone-link-off"),
                    ("cellphone-lock", "cellphone-lock"),
                    ("cellphone-marker", "cellphone-marker"),
                    ("cellphone-message", "cellphone-message"),
                    ("cellphone-message-off", "cellphone-message-off"),
                    ("cellphone-nfc", "cellphone-nfc"),
                    ("cellphone-nfc-off", "cellphone-nfc-off"),
                    ("cellphone-off", "cellphone-off"),
                    ("cellphone-play", "cellphone-play"),
                    ("cellphone-remove", "cellphone-remove"),
                    ("cellphone-screenshot", "cellphone-screenshot"),
                    ("cellphone-settings", "cellphone-settings"),
                    ("cellphone-sound", "cellphone-sound"),
                    ("cellphone-text", "cellphone-text"),
                    ("cellphone-wireless", "cellphone-wireless"),
                    ("centos", "centos"),
                    ("certificate", "certificate"),
                    ("certificate-outline", "certificate-outline"),
                    ("chair-rolling", "chair-rolling"),
                    ("chair-school", "chair-school"),
                    ("chandelier", "chandelier"),
                    ("charity", "charity"),
                    ("charity-search", "charity-search"),
                    ("chart-arc", "chart-arc"),
                    ("chart-areaspline", "chart-areaspline"),
                    ("chart-areaspline-variant", "chart-areaspline-variant"),
                    ("chart-bar", "chart-bar"),
                    ("chart-bar-stacked", "chart-bar-stacked"),
                    ("chart-bell-curve", "chart-bell-curve"),
                    ("chart-bell-curve-cumulative", "chart-bell-curve-cumulative"),
                    ("chart-box", "chart-box"),
                    ("chart-box-outline", "chart-box-outline"),
                    ("chart-box-plus-outline", "chart-box-plus-outline"),
                    ("chart-bubble", "chart-bubble"),
                    ("chart-donut", "chart-donut"),
                    ("chart-donut-variant", "chart-donut-variant"),
                    ("chart-gantt", "chart-gantt"),
                    ("chart-histogram", "chart-histogram"),
                    ("chart-line", "chart-line"),
                    ("chart-line-stacked", "chart-line-stacked"),
                    ("chart-line-variant", "chart-line-variant"),
                    ("chart-multiline", "chart-multiline"),
                    ("chart-multiple", "chart-multiple"),
                    ("chart-pie", "chart-pie"),
                    ("chart-pie-outline", "chart-pie-outline"),
                    ("chart-ppf", "chart-ppf"),
                    ("chart-sankey", "chart-sankey"),
                    ("chart-sankey-variant", "chart-sankey-variant"),
                    ("chart-scatter-plot", "chart-scatter-plot"),
                    ("chart-scatter-plot-hexbin", "chart-scatter-plot-hexbin"),
                    ("chart-timeline", "chart-timeline"),
                    ("chart-timeline-variant", "chart-timeline-variant"),
                    ("chart-timeline-variant-shimmer", "chart-timeline-variant-shimmer"),
                    ("chart-tree", "chart-tree"),
                    ("chart-waterfall", "chart-waterfall"),
                    ("chat", "chat"),
                    ("chat-alert", "chat-alert"),
                    ("chat-alert-outline", "chat-alert-outline"),
                    ("chat-minus", "chat-minus"),
                    ("chat-minus-outline", "chat-minus-outline"),
                    ("chat-outline", "chat-outline"),
                    ("chat-plus", "chat-plus"),
                    ("chat-plus-outline", "chat-plus-outline"),
                    ("chat-processing", "chat-processing"),
                    ("chat-processing-outline", "chat-processing-outline"),
                    ("chat-question", "chat-question"),
                    ("chat-question-outline", "chat-question-outline"),
                    ("chat-remove", "chat-remove"),
                    ("chat-remove-outline", "chat-remove-outline"),
                    ("chat-sleep", "chat-sleep"),
                    ("chat-sleep-outline", "chat-sleep-outline"),
                    ("check", "check"),
                    ("check-all", "check-all"),
                    ("check-bold", "check-bold"),
                    ("check-bookmark", "check-bookmark"),
                    ("check-circle", "check-circle"),
                    ("check-circle-outline", "check-circle-outline"),
                    ("check-decagram", "check-decagram"),
                    ("check-decagram-outline", "check-decagram-outline"),
                    ("check-network", "check-network"),
                    ("check-network-outline", "check-network-outline"),
                    ("check-outline", "check-outline"),
                    ("check-underline", "check-underline"),
                    ("check-underline-circle", "check-underline-circle"),
                    ("check-underline-circle-outline", "check-underline-circle-outline"),
                    ("checkbook", "checkbook"),
                    ("checkbook-arrow-left", "checkbook-arrow-left"),
                    ("checkbook-arrow-right", "checkbook-arrow-right"),
                    ("checkbox-blank", "checkbox-blank"),
                    ("checkbox-blank-badge", "checkbox-blank-badge"),
                    ("checkbox-blank-badge-outline", "checkbox-blank-badge-outline"),
                    ("checkbox-blank-circle", "checkbox-blank-circle"),
                    ("checkbox-blank-circle-outline", "checkbox-blank-circle-outline"),
                    ("checkbox-blank-off", "checkbox-blank-off"),
                    ("checkbox-blank-off-outline", "checkbox-blank-off-outline"),
                    ("checkbox-blank-outline", "checkbox-blank-outline"),
                    ("checkbox-intermediate", "checkbox-intermediate"),
                    ("checkbox-intermediate-variant", "checkbox-intermediate-variant"),
                    ("checkbox-marked", "checkbox-marked"),
                    ("checkbox-marked-circle", "checkbox-marked-circle"),
                    ("checkbox-marked-circle-auto-outline", "checkbox-marked-circle-auto-outline"),
                    (
                        "checkbox-marked-circle-minus-outline",
                        "checkbox-marked-circle-minus-outline",
                    ),
                    ("checkbox-marked-circle-outline", "checkbox-marked-circle-outline"),
                    ("checkbox-marked-circle-plus-outline", "checkbox-marked-circle-plus-outline"),
                    ("checkbox-marked-outline", "checkbox-marked-outline"),
                    ("checkbox-multiple-blank", "checkbox-multiple-blank"),
                    ("checkbox-multiple-blank-circle", "checkbox-multiple-blank-circle"),
                    (
                        "checkbox-multiple-blank-circle-outline",
                        "checkbox-multiple-blank-circle-outline",
                    ),
                    ("checkbox-multiple-blank-outline", "checkbox-multiple-blank-outline"),
                    ("checkbox-multiple-marked", "checkbox-multiple-marked"),
                    ("checkbox-multiple-marked-circle", "checkbox-multiple-marked-circle"),
                    (
                        "checkbox-multiple-marked-circle-outline",
                        "checkbox-multiple-marked-circle-outline",
                    ),
                    ("checkbox-multiple-marked-outline", "checkbox-multiple-marked-outline"),
                    ("checkbox-multiple-outline", "checkbox-multiple-outline"),
                    ("checkbox-outline", "checkbox-outline"),
                    ("checkerboard", "checkerboard"),
                    ("checkerboard-minus", "checkerboard-minus"),
                    ("checkerboard-plus", "checkerboard-plus"),
                    ("checkerboard-remove", "checkerboard-remove"),
                    ("cheese", "cheese"),
                    ("cheese-off", "cheese-off"),
                    ("chef-hat", "chef-hat"),
                    ("chemical-weapon", "chemical-weapon"),
                    ("chess-bishop", "chess-bishop"),
                    ("chess-king", "chess-king"),
                    ("chess-knight", "chess-knight"),
                    ("chess-pawn", "chess-pawn"),
                    ("chess-queen", "chess-queen"),
                    ("chess-rook", "chess-rook"),
                    ("chevron-double-down", "chevron-double-down"),
                    ("chevron-double-left", "chevron-double-left"),
                    ("chevron-double-right", "chevron-double-right"),
                    ("chevron-double-up", "chevron-double-up"),
                    ("chevron-down", "chevron-down"),
                    ("chevron-down-box", "chevron-down-box"),
                    ("chevron-down-box-outline", "chevron-down-box-outline"),
                    ("chevron-down-circle", "chevron-down-circle"),
                    ("chevron-down-circle-outline", "chevron-down-circle-outline"),
                    ("chevron-left", "chevron-left"),
                    ("chevron-left-box", "chevron-left-box"),
                    ("chevron-left-box-outline", "chevron-left-box-outline"),
                    ("chevron-left-circle", "chevron-left-circle"),
                    ("chevron-left-circle-outline", "chevron-left-circle-outline"),
                    ("chevron-right", "chevron-right"),
                    ("chevron-right-box", "chevron-right-box"),
                    ("chevron-right-box-outline", "chevron-right-box-outline"),
                    ("chevron-right-circle", "chevron-right-circle"),
                    ("chevron-right-circle-outline", "chevron-right-circle-outline"),
                    ("chevron-triple-down", "chevron-triple-down"),
                    ("chevron-triple-left", "chevron-triple-left"),
                    ("chevron-triple-right", "chevron-triple-right"),
                    ("chevron-triple-up", "chevron-triple-up"),
                    ("chevron-up", "chevron-up"),
                    ("chevron-up-box", "chevron-up-box"),
                    ("chevron-up-box-outline", "chevron-up-box-outline"),
                    ("chevron-up-circle", "chevron-up-circle"),
                    ("chevron-up-circle-outline", "chevron-up-circle-outline"),
                    ("chili-alert", "chili-alert"),
                    ("chili-alert-outline", "chili-alert-outline"),
                    ("chili-hot", "chili-hot"),
                    ("chili-hot-outline", "chili-hot-outline"),
                    ("chili-medium", "chili-medium"),
                    ("chili-medium-outline", "chili-medium-outline"),
                    ("chili-mild", "chili-mild"),
                    ("chili-mild-outline", "chili-mild-outline"),
                    ("chili-off", "chili-off"),
                    ("chili-off-outline", "chili-off-outline"),
                    ("chip", "chip"),
                    ("church", "church"),
                    ("church-outline", "church-outline"),
                    ("cigar", "cigar"),
                    ("cigar-off", "cigar-off"),
                    ("circle", "circle"),
                    ("circle-box", "circle-box"),
                    ("circle-box-outline", "circle-box-outline"),
                    ("circle-double", "circle-double"),
                    ("circle-edit-outline", "circle-edit-outline"),
                    ("circle-expand", "circle-expand"),
                    ("circle-half", "circle-half"),
                    ("circle-half-full", "circle-half-full"),
                    ("circle-medium", "circle-medium"),
                    ("circle-multiple", "circle-multiple"),
                    ("circle-multiple-outline", "circle-multiple-outline"),
                    ("circle-off-outline", "circle-off-outline"),
                    ("circle-opacity", "circle-opacity"),
                    ("circle-outline", "circle-outline"),
                    ("circle-slice-1", "circle-slice-1"),
                    ("circle-slice-2", "circle-slice-2"),
                    ("circle-slice-3", "circle-slice-3"),
                    ("circle-slice-4", "circle-slice-4"),
                    ("circle-slice-5", "circle-slice-5"),
                    ("circle-slice-6", "circle-slice-6"),
                    ("circle-slice-7", "circle-slice-7"),
                    ("circle-slice-8", "circle-slice-8"),
                    ("circle-small", "circle-small"),
                    ("circular-saw", "circular-saw"),
                    ("cisco-webex", "cisco-webex"),
                    ("city", "city"),
                    ("city-switch", "city-switch"),
                    ("city-variant", "city-variant"),
                    ("city-variant-outline", "city-variant-outline"),
                    ("clipboard", "clipboard"),
                    ("clipboard-account", "clipboard-account"),
                    ("clipboard-account-outline", "clipboard-account-outline"),
                    ("clipboard-alert", "clipboard-alert"),
                    ("clipboard-alert-outline", "clipboard-alert-outline"),
                    ("clipboard-arrow-down", "clipboard-arrow-down"),
                    ("clipboard-arrow-down-outline", "clipboard-arrow-down-outline"),
                    ("clipboard-arrow-left", "clipboard-arrow-left"),
                    ("clipboard-arrow-left-outline", "clipboard-arrow-left-outline"),
                    ("clipboard-arrow-right", "clipboard-arrow-right"),
                    ("clipboard-arrow-right-outline", "clipboard-arrow-right-outline"),
                    ("clipboard-arrow-up", "clipboard-arrow-up"),
                    ("clipboard-arrow-up-outline", "clipboard-arrow-up-outline"),
                    ("clipboard-check", "clipboard-check"),
                    ("clipboard-check-multiple", "clipboard-check-multiple"),
                    ("clipboard-check-multiple-outline", "clipboard-check-multiple-outline"),
                    ("clipboard-check-outline", "clipboard-check-outline"),
                    ("clipboard-clock", "clipboard-clock"),
                    ("clipboard-clock-outline", "clipboard-clock-outline"),
                    ("clipboard-edit", "clipboard-edit"),
                    ("clipboard-edit-outline", "clipboard-edit-outline"),
                    ("clipboard-file", "clipboard-file"),
                    ("clipboard-file-outline", "clipboard-file-outline"),
                    ("clipboard-flow", "clipboard-flow"),
                    ("clipboard-flow-outline", "clipboard-flow-outline"),
                    ("clipboard-list", "clipboard-list"),
                    ("clipboard-list-outline", "clipboard-list-outline"),
                    ("clipboard-minus", "clipboard-minus"),
                    ("clipboard-minus-outline", "clipboard-minus-outline"),
                    ("clipboard-multiple", "clipboard-multiple"),
                    ("clipboard-multiple-outline", "clipboard-multiple-outline"),
                    ("clipboard-off", "clipboard-off"),
                    ("clipboard-off-outline", "clipboard-off-outline"),
                    ("clipboard-outline", "clipboard-outline"),
                    ("clipboard-play", "clipboard-play"),
                    ("clipboard-play-multiple", "clipboard-play-multiple"),
                    ("clipboard-play-multiple-outline", "clipboard-play-multiple-outline"),
                    ("clipboard-play-outline", "clipboard-play-outline"),
                    ("clipboard-plus", "clipboard-plus"),
                    ("clipboard-plus-outline", "clipboard-plus-outline"),
                    ("clipboard-pulse", "clipboard-pulse"),
                    ("clipboard-pulse-outline", "clipboard-pulse-outline"),
                    ("clipboard-remove", "clipboard-remove"),
                    ("clipboard-remove-outline", "clipboard-remove-outline"),
                    ("clipboard-search", "clipboard-search"),
                    ("clipboard-search-outline", "clipboard-search-outline"),
                    ("clipboard-text", "clipboard-text"),
                    ("clipboard-text-clock", "clipboard-text-clock"),
                    ("clipboard-text-clock-outline", "clipboard-text-clock-outline"),
                    ("clipboard-text-multiple", "clipboard-text-multiple"),
                    ("clipboard-text-multiple-outline", "clipboard-text-multiple-outline"),
                    ("clipboard-text-off", "clipboard-text-off"),
                    ("clipboard-text-off-outline", "clipboard-text-off-outline"),
                    ("clipboard-text-outline", "clipboard-text-outline"),
                    ("clipboard-text-play", "clipboard-text-play"),
                    ("clipboard-text-play-outline", "clipboard-text-play-outline"),
                    ("clipboard-text-search", "clipboard-text-search"),
                    ("clipboard-text-search-outline", "clipboard-text-search-outline"),
                    ("clippy", "clippy"),
                    ("clock", "clock"),
                    ("clock-alert", "clock-alert"),
                    ("clock-alert-outline", "clock-alert-outline"),
                    ("clock-check", "clock-check"),
                    ("clock-check-outline", "clock-check-outline"),
                    ("clock-digital", "clock-digital"),
                    ("clock-edit", "clock-edit"),
                    ("clock-edit-outline", "clock-edit-outline"),
                    ("clock-end", "clock-end"),
                    ("clock-fast", "clock-fast"),
                    ("clock-in", "clock-in"),
                    ("clock-minus", "clock-minus"),
                    ("clock-minus-outline", "clock-minus-outline"),
                    ("clock-out", "clock-out"),
                    ("clock-outline", "clock-outline"),
                    ("clock-plus", "clock-plus"),
                    ("clock-plus-outline", "clock-plus-outline"),
                    ("clock-remove", "clock-remove"),
                    ("clock-remove-outline", "clock-remove-outline"),
                    ("clock-star-four-points", "clock-star-four-points"),
                    ("clock-star-four-points-outline", "clock-star-four-points-outline"),
                    ("clock-start", "clock-start"),
                    ("clock-time-eight", "clock-time-eight"),
                    ("clock-time-eight-outline", "clock-time-eight-outline"),
                    ("clock-time-eleven", "clock-time-eleven"),
                    ("clock-time-eleven-outline", "clock-time-eleven-outline"),
                    ("clock-time-five", "clock-time-five"),
                    ("clock-time-five-outline", "clock-time-five-outline"),
                    ("clock-time-four", "clock-time-four"),
                    ("clock-time-four-outline", "clock-time-four-outline"),
                    ("clock-time-nine", "clock-time-nine"),
                    ("clock-time-nine-outline", "clock-time-nine-outline"),
                    ("clock-time-one", "clock-time-one"),
                    ("clock-time-one-outline", "clock-time-one-outline"),
                    ("clock-time-seven", "clock-time-seven"),
                    ("clock-time-seven-outline", "clock-time-seven-outline"),
                    ("clock-time-six", "clock-time-six"),
                    ("clock-time-six-outline", "clock-time-six-outline"),
                    ("clock-time-ten", "clock-time-ten"),
                    ("clock-time-ten-outline", "clock-time-ten-outline"),
                    ("clock-time-three", "clock-time-three"),
                    ("clock-time-three-outline", "clock-time-three-outline"),
                    ("clock-time-twelve", "clock-time-twelve"),
                    ("clock-time-twelve-outline", "clock-time-twelve-outline"),
                    ("clock-time-two", "clock-time-two"),
                    ("clock-time-two-outline", "clock-time-two-outline"),
                    ("close", "close"),
                    ("close-box", "close-box"),
                    ("close-box-multiple", "close-box-multiple"),
                    ("close-box-multiple-outline", "close-box-multiple-outline"),
                    ("close-box-outline", "close-box-outline"),
                    ("close-circle", "close-circle"),
                    ("close-circle-multiple", "close-circle-multiple"),
                    ("close-circle-multiple-outline", "close-circle-multiple-outline"),
                    ("close-circle-outline", "close-circle-outline"),
                    ("close-network", "close-network"),
                    ("close-network-outline", "close-network-outline"),
                    ("close-octagon", "close-octagon"),
                    ("close-octagon-outline", "close-octagon-outline"),
                    ("close-outline", "close-outline"),
                    ("close-thick", "close-thick"),
                    ("closed-caption", "closed-caption"),
                    ("closed-caption-outline", "closed-caption-outline"),
                    ("cloud", "cloud"),
                    ("cloud-alert", "cloud-alert"),
                    ("cloud-alert-outline", "cloud-alert-outline"),
                    ("cloud-arrow-down", "cloud-arrow-down"),
                    ("cloud-arrow-down-outline", "cloud-arrow-down-outline"),
                    ("cloud-arrow-left", "cloud-arrow-left"),
                    ("cloud-arrow-left-outline", "cloud-arrow-left-outline"),
                    ("cloud-arrow-right", "cloud-arrow-right"),
                    ("cloud-arrow-right-outline", "cloud-arrow-right-outline"),
                    ("cloud-arrow-up", "cloud-arrow-up"),
                    ("cloud-arrow-up-outline", "cloud-arrow-up-outline"),
                    ("cloud-braces", "cloud-braces"),
                    ("cloud-cancel", "cloud-cancel"),
                    ("cloud-cancel-outline", "cloud-cancel-outline"),
                    ("cloud-check", "cloud-check"),
                    ("cloud-check-outline", "cloud-check-outline"),
                    ("cloud-check-variant", "cloud-check-variant"),
                    ("cloud-check-variant-outline", "cloud-check-variant-outline"),
                    ("cloud-circle", "cloud-circle"),
                    ("cloud-circle-outline", "cloud-circle-outline"),
                    ("cloud-clock", "cloud-clock"),
                    ("cloud-clock-outline", "cloud-clock-outline"),
                    ("cloud-cog", "cloud-cog"),
                    ("cloud-cog-outline", "cloud-cog-outline"),
                    ("cloud-download", "cloud-download"),
                    ("cloud-download-outline", "cloud-download-outline"),
                    ("cloud-key", "cloud-key"),
                    ("cloud-key-outline", "cloud-key-outline"),
                    ("cloud-lock", "cloud-lock"),
                    ("cloud-lock-open", "cloud-lock-open"),
                    ("cloud-lock-open-outline", "cloud-lock-open-outline"),
                    ("cloud-lock-outline", "cloud-lock-outline"),
                    ("cloud-minus", "cloud-minus"),
                    ("cloud-minus-outline", "cloud-minus-outline"),
                    ("cloud-off", "cloud-off"),
                    ("cloud-off-outline", "cloud-off-outline"),
                    ("cloud-outline", "cloud-outline"),
                    ("cloud-percent", "cloud-percent"),
                    ("cloud-percent-outline", "cloud-percent-outline"),
                    ("cloud-plus", "cloud-plus"),
                    ("cloud-plus-outline", "cloud-plus-outline"),
                    ("cloud-print", "cloud-print"),
                    ("cloud-print-outline", "cloud-print-outline"),
                    ("cloud-question", "cloud-question"),
                    ("cloud-question-outline", "cloud-question-outline"),
                    ("cloud-refresh", "cloud-refresh"),
                    ("cloud-refresh-outline", "cloud-refresh-outline"),
                    ("cloud-refresh-variant", "cloud-refresh-variant"),
                    ("cloud-refresh-variant-outline", "cloud-refresh-variant-outline"),
                    ("cloud-remove", "cloud-remove"),
                    ("cloud-remove-outline", "cloud-remove-outline"),
                    ("cloud-search", "cloud-search"),
                    ("cloud-search-outline", "cloud-search-outline"),
                    ("cloud-sync", "cloud-sync"),
                    ("cloud-sync-outline", "cloud-sync-outline"),
                    ("cloud-tags", "cloud-tags"),
                    ("cloud-upload", "cloud-upload"),
                    ("cloud-upload-outline", "cloud-upload-outline"),
                    ("clouds", "clouds"),
                    ("clover", "clover"),
                    ("clover-outline", "clover-outline"),
                    ("coach-lamp", "coach-lamp"),
                    ("coach-lamp-variant", "coach-lamp-variant"),
                    ("coat-rack", "coat-rack"),
                    ("code-array", "code-array"),
                    ("code-block-braces", "code-block-braces"),
                    ("code-block-brackets", "code-block-brackets"),
                    ("code-block-parentheses", "code-block-parentheses"),
                    ("code-block-tags", "code-block-tags"),
                    ("code-braces", "code-braces"),
                    ("code-braces-box", "code-braces-box"),
                    ("code-brackets", "code-brackets"),
                    ("code-equal", "code-equal"),
                    ("code-greater-than", "code-greater-than"),
                    ("code-greater-than-or-equal", "code-greater-than-or-equal"),
                    ("code-json", "code-json"),
                    ("code-less-than", "code-less-than"),
                    ("code-less-than-or-equal", "code-less-than-or-equal"),
                    ("code-not-equal", "code-not-equal"),
                    ("code-not-equal-variant", "code-not-equal-variant"),
                    ("code-parentheses", "code-parentheses"),
                    ("code-parentheses-box", "code-parentheses-box"),
                    ("code-string", "code-string"),
                    ("code-tags", "code-tags"),
                    ("code-tags-check", "code-tags-check"),
                    ("codepen", "codepen"),
                    ("coffee", "coffee"),
                    ("coffee-maker", "coffee-maker"),
                    ("coffee-maker-check", "coffee-maker-check"),
                    ("coffee-maker-check-outline", "coffee-maker-check-outline"),
                    ("coffee-maker-outline", "coffee-maker-outline"),
                    ("coffee-off", "coffee-off"),
                    ("coffee-off-outline", "coffee-off-outline"),
                    ("coffee-outline", "coffee-outline"),
                    ("coffee-to-go", "coffee-to-go"),
                    ("coffee-to-go-outline", "coffee-to-go-outline"),
                    ("coffin", "coffin"),
                    ("cog", "cog"),
                    ("cog-box", "cog-box"),
                    ("cog-clockwise", "cog-clockwise"),
                    ("cog-counterclockwise", "cog-counterclockwise"),
                    ("cog-off", "cog-off"),
                    ("cog-off-outline", "cog-off-outline"),
                    ("cog-outline", "cog-outline"),
                    ("cog-pause", "cog-pause"),
                    ("cog-pause-outline", "cog-pause-outline"),
                    ("cog-play", "cog-play"),
                    ("cog-play-outline", "cog-play-outline"),
                    ("cog-refresh", "cog-refresh"),
                    ("cog-refresh-outline", "cog-refresh-outline"),
                    ("cog-stop", "cog-stop"),
                    ("cog-stop-outline", "cog-stop-outline"),
                    ("cog-sync", "cog-sync"),
                    ("cog-sync-outline", "cog-sync-outline"),
                    ("cog-transfer", "cog-transfer"),
                    ("cog-transfer-outline", "cog-transfer-outline"),
                    ("cogs", "cogs"),
                    ("collage", "collage"),
                    ("collapse-all", "collapse-all"),
                    ("collapse-all-outline", "collapse-all-outline"),
                    ("color-helper", "color-helper"),
                    ("comma", "comma"),
                    ("comma-box", "comma-box"),
                    ("comma-box-outline", "comma-box-outline"),
                    ("comma-circle", "comma-circle"),
                    ("comma-circle-outline", "comma-circle-outline"),
                    ("comment", "comment"),
                    ("comment-account", "comment-account"),
                    ("comment-account-outline", "comment-account-outline"),
                    ("comment-alert", "comment-alert"),
                    ("comment-alert-outline", "comment-alert-outline"),
                    ("comment-arrow-left", "comment-arrow-left"),
                    ("comment-arrow-left-outline", "comment-arrow-left-outline"),
                    ("comment-arrow-right", "comment-arrow-right"),
                    ("comment-arrow-right-outline", "comment-arrow-right-outline"),
                    ("comment-bookmark", "comment-bookmark"),
                    ("comment-bookmark-outline", "comment-bookmark-outline"),
                    ("comment-check", "comment-check"),
                    ("comment-check-outline", "comment-check-outline"),
                    ("comment-edit", "comment-edit"),
                    ("comment-edit-outline", "comment-edit-outline"),
                    ("comment-eye", "comment-eye"),
                    ("comment-eye-outline", "comment-eye-outline"),
                    ("comment-flash", "comment-flash"),
                    ("comment-flash-outline", "comment-flash-outline"),
                    ("comment-minus", "comment-minus"),
                    ("comment-minus-outline", "comment-minus-outline"),
                    ("comment-multiple", "comment-multiple"),
                    ("comment-multiple-outline", "comment-multiple-outline"),
                    ("comment-off", "comment-off"),
                    ("comment-off-outline", "comment-off-outline"),
                    ("comment-outline", "comment-outline"),
                    ("comment-plus", "comment-plus"),
                    ("comment-plus-outline", "comment-plus-outline"),
                    ("comment-processing", "comment-processing"),
                    ("comment-processing-outline", "comment-processing-outline"),
                    ("comment-question", "comment-question"),
                    ("comment-question-outline", "comment-question-outline"),
                    ("comment-quote", "comment-quote"),
                    ("comment-quote-outline", "comment-quote-outline"),
                    ("comment-remove", "comment-remove"),
                    ("comment-remove-outline", "comment-remove-outline"),
                    ("comment-search", "comment-search"),
                    ("comment-search-outline", "comment-search-outline"),
                    ("comment-text", "comment-text"),
                    ("comment-text-multiple", "comment-text-multiple"),
                    ("comment-text-multiple-outline", "comment-text-multiple-outline"),
                    ("comment-text-outline", "comment-text-outline"),
                    ("compare", "compare"),
                    ("compare-horizontal", "compare-horizontal"),
                    ("compare-remove", "compare-remove"),
                    ("compare-vertical", "compare-vertical"),
                    ("compass", "compass"),
                    ("compass-off", "compass-off"),
                    ("compass-off-outline", "compass-off-outline"),
                    ("compass-outline", "compass-outline"),
                    ("compass-rose", "compass-rose"),
                    ("compost", "compost"),
                    ("concourse-ci", "concourse-ci"),
                    ("cone", "cone"),
                    ("cone-off", "cone-off"),
                    ("connection", "connection"),
                    ("console", "console"),
                    ("console-line", "console-line"),
                    ("console-network", "console-network"),
                    ("console-network-outline", "console-network-outline"),
                    ("consolidate", "consolidate"),
                    ("contactless-payment", "contactless-payment"),
                    ("contactless-payment-circle", "contactless-payment-circle"),
                    ("contactless-payment-circle-outline", "contactless-payment-circle-outline"),
                    ("contacts", "contacts"),
                    ("contacts-outline", "contacts-outline"),
                    ("contain", "contain"),
                    ("contain-end", "contain-end"),
                    ("contain-start", "contain-start"),
                    ("content-copy", "content-copy"),
                    ("content-cut", "content-cut"),
                    ("content-duplicate", "content-duplicate"),
                    ("content-paste", "content-paste"),
                    ("content-save", "content-save"),
                    ("content-save-alert", "content-save-alert"),
                    ("content-save-alert-outline", "content-save-alert-outline"),
                    ("content-save-all", "content-save-all"),
                    ("content-save-all-outline", "content-save-all-outline"),
                    ("content-save-check", "content-save-check"),
                    ("content-save-check-outline", "content-save-check-outline"),
                    ("content-save-cog", "content-save-cog"),
                    ("content-save-cog-outline", "content-save-cog-outline"),
                    ("content-save-edit", "content-save-edit"),
                    ("content-save-edit-outline", "content-save-edit-outline"),
                    ("content-save-minus", "content-save-minus"),
                    ("content-save-minus-outline", "content-save-minus-outline"),
                    ("content-save-move", "content-save-move"),
                    ("content-save-move-outline", "content-save-move-outline"),
                    ("content-save-off", "content-save-off"),
                    ("content-save-off-outline", "content-save-off-outline"),
                    ("content-save-outline", "content-save-outline"),
                    ("content-save-plus", "content-save-plus"),
                    ("content-save-plus-outline", "content-save-plus-outline"),
                    ("content-save-settings", "content-save-settings"),
                    ("content-save-settings-outline", "content-save-settings-outline"),
                    ("contrast", "contrast"),
                    ("contrast-box", "contrast-box"),
                    ("contrast-circle", "contrast-circle"),
                    ("controller", "controller"),
                    ("controller-classic", "controller-classic"),
                    ("controller-classic-outline", "controller-classic-outline"),
                    ("controller-off", "controller-off"),
                    ("controller-xbox", "controller-xbox"),
                    ("cookie", "cookie"),
                    ("cookie-alert", "cookie-alert"),
                    ("cookie-alert-outline", "cookie-alert-outline"),
                    ("cookie-check", "cookie-check"),
                    ("cookie-check-outline", "cookie-check-outline"),
                    ("cookie-clock", "cookie-clock"),
                    ("cookie-clock-outline", "cookie-clock-outline"),
                    ("cookie-cog", "cookie-cog"),
                    ("cookie-cog-outline", "cookie-cog-outline"),
                    ("cookie-edit", "cookie-edit"),
                    ("cookie-edit-outline", "cookie-edit-outline"),
                    ("cookie-lock", "cookie-lock"),
                    ("cookie-lock-outline", "cookie-lock-outline"),
                    ("cookie-minus", "cookie-minus"),
                    ("cookie-minus-outline", "cookie-minus-outline"),
                    ("cookie-off", "cookie-off"),
                    ("cookie-off-outline", "cookie-off-outline"),
                    ("cookie-outline", "cookie-outline"),
                    ("cookie-plus", "cookie-plus"),
                    ("cookie-plus-outline", "cookie-plus-outline"),
                    ("cookie-refresh", "cookie-refresh"),
                    ("cookie-refresh-outline", "cookie-refresh-outline"),
                    ("cookie-remove", "cookie-remove"),
                    ("cookie-remove-outline", "cookie-remove-outline"),
                    ("cookie-settings", "cookie-settings"),
                    ("cookie-settings-outline", "cookie-settings-outline"),
                    ("coolant-temperature", "coolant-temperature"),
                    ("copyleft", "copyleft"),
                    ("copyright", "copyright"),
                    ("cordova", "cordova"),
                    ("corn", "corn"),
                    ("corn-off", "corn-off"),
                    ("cosine-wave", "cosine-wave"),
                    ("counter", "counter"),
                    ("countertop", "countertop"),
                    ("countertop-outline", "countertop-outline"),
                    ("cow", "cow"),
                    ("cow-off", "cow-off"),
                    ("cpu-32-bit", "cpu-32-bit"),
                    ("cpu-64-bit", "cpu-64-bit"),
                    ("cradle", "cradle"),
                    ("cradle-outline", "cradle-outline"),
                    ("crane", "crane"),
                    ("creation", "creation"),
                    ("creation-outline", "creation-outline"),
                    ("creative-commons", "creative-commons"),
                    ("credit-card", "credit-card"),
                    ("credit-card-check", "credit-card-check"),
                    ("credit-card-check-outline", "credit-card-check-outline"),
                    ("credit-card-chip", "credit-card-chip"),
                    ("credit-card-chip-outline", "credit-card-chip-outline"),
                    ("credit-card-clock", "credit-card-clock"),
                    ("credit-card-clock-outline", "credit-card-clock-outline"),
                    ("credit-card-edit", "credit-card-edit"),
                    ("credit-card-edit-outline", "credit-card-edit-outline"),
                    ("credit-card-fast", "credit-card-fast"),
                    ("credit-card-fast-outline", "credit-card-fast-outline"),
                    ("credit-card-lock", "credit-card-lock"),
                    ("credit-card-lock-outline", "credit-card-lock-outline"),
                    ("credit-card-marker", "credit-card-marker"),
                    ("credit-card-marker-outline", "credit-card-marker-outline"),
                    ("credit-card-minus", "credit-card-minus"),
                    ("credit-card-minus-outline", "credit-card-minus-outline"),
                    ("credit-card-multiple", "credit-card-multiple"),
                    ("credit-card-multiple-outline", "credit-card-multiple-outline"),
                    ("credit-card-off", "credit-card-off"),
                    ("credit-card-off-outline", "credit-card-off-outline"),
                    ("credit-card-outline", "credit-card-outline"),
                    ("credit-card-plus", "credit-card-plus"),
                    ("credit-card-plus-outline", "credit-card-plus-outline"),
                    ("credit-card-refresh", "credit-card-refresh"),
                    ("credit-card-refresh-outline", "credit-card-refresh-outline"),
                    ("credit-card-refund", "credit-card-refund"),
                    ("credit-card-refund-outline", "credit-card-refund-outline"),
                    ("credit-card-remove", "credit-card-remove"),
                    ("credit-card-remove-outline", "credit-card-remove-outline"),
                    ("credit-card-scan", "credit-card-scan"),
                    ("credit-card-scan-outline", "credit-card-scan-outline"),
                    ("credit-card-search", "credit-card-search"),
                    ("credit-card-search-outline", "credit-card-search-outline"),
                    ("credit-card-settings", "credit-card-settings"),
                    ("credit-card-settings-outline", "credit-card-settings-outline"),
                    ("credit-card-sync", "credit-card-sync"),
                    ("credit-card-sync-outline", "credit-card-sync-outline"),
                    ("credit-card-wireless", "credit-card-wireless"),
                    ("credit-card-wireless-off", "credit-card-wireless-off"),
                    ("credit-card-wireless-off-outline", "credit-card-wireless-off-outline"),
                    ("credit-card-wireless-outline", "credit-card-wireless-outline"),
                    ("cricket", "cricket"),
                    ("crop", "crop"),
                    ("crop-free", "crop-free"),
                    ("crop-landscape", "crop-landscape"),
                    ("crop-portrait", "crop-portrait"),
                    ("crop-rotate", "crop-rotate"),
                    ("crop-square", "crop-square"),
                    ("cross", "cross"),
                    ("cross-bolnisi", "cross-bolnisi"),
                    ("cross-celtic", "cross-celtic"),
                    ("cross-outline", "cross-outline"),
                    ("crosshairs", "crosshairs"),
                    ("crosshairs-gps", "crosshairs-gps"),
                    ("crosshairs-off", "crosshairs-off"),
                    ("crosshairs-question", "crosshairs-question"),
                    ("crowd", "crowd"),
                    ("crown", "crown"),
                    ("crown-circle", "crown-circle"),
                    ("crown-circle-outline", "crown-circle-outline"),
                    ("crown-outline", "crown-outline"),
                    ("cryengine", "cryengine"),
                    ("crystal-ball", "crystal-ball"),
                    ("cube", "cube"),
                    ("cube-off", "cube-off"),
                    ("cube-off-outline", "cube-off-outline"),
                    ("cube-outline", "cube-outline"),
                    ("cube-scan", "cube-scan"),
                    ("cube-send", "cube-send"),
                    ("cube-unfolded", "cube-unfolded"),
                    ("cup", "cup"),
                    ("cup-off", "cup-off"),
                    ("cup-off-outline", "cup-off-outline"),
                    ("cup-outline", "cup-outline"),
                    ("cup-water", "cup-water"),
                    ("cupboard", "cupboard"),
                    ("cupboard-outline", "cupboard-outline"),
                    ("cupcake", "cupcake"),
                    ("curling", "curling"),
                    ("currency-bdt", "currency-bdt"),
                    ("currency-brl", "currency-brl"),
                    ("currency-btc", "currency-btc"),
                    ("currency-chf", "currency-chf"),
                    ("currency-cny", "currency-cny"),
                    ("currency-eth", "currency-eth"),
                    ("currency-eur", "currency-eur"),
                    ("currency-eur-off", "currency-eur-off"),
                    ("currency-fra", "currency-fra"),
                    ("currency-gbp", "currency-gbp"),
                    ("currency-ils", "currency-ils"),
                    ("currency-inr", "currency-inr"),
                    ("currency-jpy", "currency-jpy"),
                    ("currency-krw", "currency-krw"),
                    ("currency-kzt", "currency-kzt"),
                    ("currency-mnt", "currency-mnt"),
                    ("currency-ngn", "currency-ngn"),
                    ("currency-php", "currency-php"),
                    ("currency-rial", "currency-rial"),
                    ("currency-rub", "currency-rub"),
                    ("currency-rupee", "currency-rupee"),
                    ("currency-sign", "currency-sign"),
                    ("currency-thb", "currency-thb"),
                    ("currency-try", "currency-try"),
                    ("currency-twd", "currency-twd"),
                    ("currency-uah", "currency-uah"),
                    ("currency-usd", "currency-usd"),
                    ("currency-usd-circle", "currency-usd-circle"),
                    ("currency-usd-circle-outline", "currency-usd-circle-outline"),
                    ("currency-usd-off", "currency-usd-off"),
                    ("current-ac", "current-ac"),
                    ("current-dc", "current-dc"),
                    ("cursor-default", "cursor-default"),
                    ("cursor-default-click", "cursor-default-click"),
                    ("cursor-default-click-outline", "cursor-default-click-outline"),
                    ("cursor-default-gesture", "cursor-default-gesture"),
                    ("cursor-default-gesture-outline", "cursor-default-gesture-outline"),
                    ("cursor-default-outline", "cursor-default-outline"),
                    ("cursor-move", "cursor-move"),
                    ("cursor-pointer", "cursor-pointer"),
                    ("cursor-text", "cursor-text"),
                    ("curtains", "curtains"),
                    ("curtains-closed", "curtains-closed"),
                    ("cylinder", "cylinder"),
                    ("cylinder-off", "cylinder-off"),
                    ("dance-ballroom", "dance-ballroom"),
                    ("dance-pole", "dance-pole"),
                    ("data", "data"),
                    ("data-matrix", "data-matrix"),
                    ("data-matrix-edit", "data-matrix-edit"),
                    ("data-matrix-minus", "data-matrix-minus"),
                    ("data-matrix-plus", "data-matrix-plus"),
                    ("data-matrix-remove", "data-matrix-remove"),
                    ("data-matrix-scan", "data-matrix-scan"),
                    ("database", "database"),
                    ("database-alert", "database-alert"),
                    ("database-alert-outline", "database-alert-outline"),
                    ("database-arrow-down", "database-arrow-down"),
                    ("database-arrow-down-outline", "database-arrow-down-outline"),
                    ("database-arrow-left", "database-arrow-left"),
                    ("database-arrow-left-outline", "database-arrow-left-outline"),
                    ("database-arrow-right", "database-arrow-right"),
                    ("database-arrow-right-outline", "database-arrow-right-outline"),
                    ("database-arrow-up", "database-arrow-up"),
                    ("database-arrow-up-outline", "database-arrow-up-outline"),
                    ("database-check", "database-check"),
                    ("database-check-outline", "database-check-outline"),
                    ("database-clock", "database-clock"),
                    ("database-clock-outline", "database-clock-outline"),
                    ("database-cog", "database-cog"),
                    ("database-cog-outline", "database-cog-outline"),
                    ("database-edit", "database-edit"),
                    ("database-edit-outline", "database-edit-outline"),
                    ("database-export", "database-export"),
                    ("database-export-outline", "database-export-outline"),
                    ("database-eye", "database-eye"),
                    ("database-eye-off", "database-eye-off"),
                    ("database-eye-off-outline", "database-eye-off-outline"),
                    ("database-eye-outline", "database-eye-outline"),
                    ("database-import", "database-import"),
                    ("database-import-outline", "database-import-outline"),
                    ("database-lock", "database-lock"),
                    ("database-lock-outline", "database-lock-outline"),
                    ("database-marker", "database-marker"),
                    ("database-marker-outline", "database-marker-outline"),
                    ("database-minus", "database-minus"),
                    ("database-minus-outline", "database-minus-outline"),
                    ("database-off", "database-off"),
                    ("database-off-outline", "database-off-outline"),
                    ("database-outline", "database-outline"),
                    ("database-plus", "database-plus"),
                    ("database-plus-outline", "database-plus-outline"),
                    ("database-refresh", "database-refresh"),
                    ("database-refresh-outline", "database-refresh-outline"),
                    ("database-remove", "database-remove"),
                    ("database-remove-outline", "database-remove-outline"),
                    ("database-search", "database-search"),
                    ("database-search-outline", "database-search-outline"),
                    ("database-settings", "database-settings"),
                    ("database-settings-outline", "database-settings-outline"),
                    ("database-sync", "database-sync"),
                    ("database-sync-outline", "database-sync-outline"),
                    ("death-star", "death-star"),
                    ("death-star-variant", "death-star-variant"),
                    ("deathly-hallows", "deathly-hallows"),
                    ("debian", "debian"),
                    ("debug-step-into", "debug-step-into"),
                    ("debug-step-out", "debug-step-out"),
                    ("debug-step-over", "debug-step-over"),
                    ("decagram", "decagram"),
                    ("decagram-outline", "decagram-outline"),
                    ("decimal", "decimal"),
                    ("decimal-comma", "decimal-comma"),
                    ("decimal-comma-decrease", "decimal-comma-decrease"),
                    ("decimal-comma-increase", "decimal-comma-increase"),
                    ("decimal-decrease", "decimal-decrease"),
                    ("decimal-increase", "decimal-increase"),
                    ("delete", "delete"),
                    ("delete-alert", "delete-alert"),
                    ("delete-alert-outline", "delete-alert-outline"),
                    ("delete-circle", "delete-circle"),
                    ("delete-circle-outline", "delete-circle-outline"),
                    ("delete-clock", "delete-clock"),
                    ("delete-clock-outline", "delete-clock-outline"),
                    ("delete-empty", "delete-empty"),
                    ("delete-empty-outline", "delete-empty-outline"),
                    ("delete-forever", "delete-forever"),
                    ("delete-forever-outline", "delete-forever-outline"),
                    ("delete-off", "delete-off"),
                    ("delete-off-outline", "delete-off-outline"),
                    ("delete-outline", "delete-outline"),
                    ("delete-restore", "delete-restore"),
                    ("delete-sweep", "delete-sweep"),
                    ("delete-sweep-outline", "delete-sweep-outline"),
                    ("delete-variant", "delete-variant"),
                    ("delta", "delta"),
                    ("desk", "desk"),
                    ("desk-lamp", "desk-lamp"),
                    ("desk-lamp-off", "desk-lamp-off"),
                    ("desk-lamp-on", "desk-lamp-on"),
                    ("deskphone", "deskphone"),
                    ("desktop-classic", "desktop-classic"),
                    ("desktop-mac", "desktop-mac"),
                    ("desktop-mac-dashboard", "desktop-mac-dashboard"),
                    ("desktop-tower", "desktop-tower"),
                    ("desktop-tower-monitor", "desktop-tower-monitor"),
                    ("details", "details"),
                    ("dev-to", "dev-to"),
                    ("developer-board", "developer-board"),
                    ("deviantart", "deviantart"),
                    ("devices", "devices"),
                    ("dharmachakra", "dharmachakra"),
                    ("diabetes", "diabetes"),
                    ("dialpad", "dialpad"),
                    ("diameter", "diameter"),
                    ("diameter-outline", "diameter-outline"),
                    ("diameter-variant", "diameter-variant"),
                    ("diamond", "diamond"),
                    ("diamond-outline", "diamond-outline"),
                    ("diamond-stone", "diamond-stone"),
                    ("dice", "dice"),
                    ("dice-1", "dice-1"),
                    ("dice-1-outline", "dice-1-outline"),
                    ("dice-2", "dice-2"),
                    ("dice-2-outline", "dice-2-outline"),
                    ("dice-3", "dice-3"),
                    ("dice-3-outline", "dice-3-outline"),
                    ("dice-4", "dice-4"),
                    ("dice-4-outline", "dice-4-outline"),
                    ("dice-5", "dice-5"),
                    ("dice-5-outline", "dice-5-outline"),
                    ("dice-6", "dice-6"),
                    ("dice-6-outline", "dice-6-outline"),
                    ("dice-d10", "dice-d10"),
                    ("dice-d10-outline", "dice-d10-outline"),
                    ("dice-d12", "dice-d12"),
                    ("dice-d12-outline", "dice-d12-outline"),
                    ("dice-d20", "dice-d20"),
                    ("dice-d20-outline", "dice-d20-outline"),
                    ("dice-d4", "dice-d4"),
                    ("dice-d4-outline", "dice-d4-outline"),
                    ("dice-d6", "dice-d6"),
                    ("dice-d6-outline", "dice-d6-outline"),
                    ("dice-d8", "dice-d8"),
                    ("dice-d8-outline", "dice-d8-outline"),
                    ("dice-multiple", "dice-multiple"),
                    ("dice-multiple-outline", "dice-multiple-outline"),
                    ("digital-ocean", "digital-ocean"),
                    ("dip-switch", "dip-switch"),
                    ("directions", "directions"),
                    ("directions-fork", "directions-fork"),
                    ("disc", "disc"),
                    ("disc-alert", "disc-alert"),
                    ("disc-player", "disc-player"),
                    ("discord", "discord"),
                    ("dishwasher", "dishwasher"),
                    ("dishwasher-alert", "dishwasher-alert"),
                    ("dishwasher-off", "dishwasher-off"),
                    ("disk", "disk"),
                    ("disk-alert", "disk-alert"),
                    ("disk-player", "disk-player"),
                    ("disqus", "disqus"),
                    ("disqus-outline", "disqus-outline"),
                    ("distribute-horizontal-center", "distribute-horizontal-center"),
                    ("distribute-horizontal-left", "distribute-horizontal-left"),
                    ("distribute-horizontal-right", "distribute-horizontal-right"),
                    ("distribute-vertical-bottom", "distribute-vertical-bottom"),
                    ("distribute-vertical-center", "distribute-vertical-center"),
                    ("distribute-vertical-top", "distribute-vertical-top"),
                    ("diversify", "diversify"),
                    ("diving", "diving"),
                    ("diving-flippers", "diving-flippers"),
                    ("diving-helmet", "diving-helmet"),
                    ("diving-scuba", "diving-scuba"),
                    ("diving-scuba-flag", "diving-scuba-flag"),
                    ("diving-scuba-mask", "diving-scuba-mask"),
                    ("diving-scuba-tank", "diving-scuba-tank"),
                    ("diving-scuba-tank-multiple", "diving-scuba-tank-multiple"),
                    ("diving-snorkel", "diving-snorkel"),
                    ("division", "division"),
                    ("division-box", "division-box"),
                    ("dlna", "dlna"),
                    ("dna", "dna"),
                    ("dns", "dns"),
                    ("dns-outline", "dns-outline"),
                    ("do-not-disturb", "do-not-disturb"),
                    ("dock-bottom", "dock-bottom"),
                    ("dock-left", "dock-left"),
                    ("dock-right", "dock-right"),
                    ("dock-top", "dock-top"),
                    ("dock-window", "dock-window"),
                    ("docker", "docker"),
                    ("doctor", "doctor"),
                    ("document", "document"),
                    ("dog", "dog"),
                    ("dog-service", "dog-service"),
                    ("dog-side", "dog-side"),
                    ("dog-side-off", "dog-side-off"),
                    ("dolby", "dolby"),
                    ("dolly", "dolly"),
                    ("dolphin", "dolphin"),
                    ("domain", "domain"),
                    ("domain-off", "domain-off"),
                    ("domain-plus", "domain-plus"),
                    ("domain-remove", "domain-remove"),
                    ("domain-switch", "domain-switch"),
                    ("dome-light", "dome-light"),
                    ("domino-mask", "domino-mask"),
                    ("donkey", "donkey"),
                    ("door", "door"),
                    ("door-closed", "door-closed"),
                    ("door-closed-cancel", "door-closed-cancel"),
                    ("door-closed-lock", "door-closed-lock"),
                    ("door-open", "door-open"),
                    ("door-sliding", "door-sliding"),
                    ("door-sliding-lock", "door-sliding-lock"),
                    ("door-sliding-open", "door-sliding-open"),
                    ("doorbell", "doorbell"),
                    ("doorbell-video", "doorbell-video"),
                    ("dot-net", "dot-net"),
                    ("dots-circle", "dots-circle"),
                    ("dots-grid", "dots-grid"),
                    ("dots-hexagon", "dots-hexagon"),
                    ("dots-horizontal", "dots-horizontal"),
                    ("dots-horizontal-circle", "dots-horizontal-circle"),
                    ("dots-horizontal-circle-outline", "dots-horizontal-circle-outline"),
                    ("dots-square", "dots-square"),
                    ("dots-triangle", "dots-triangle"),
                    ("dots-vertical", "dots-vertical"),
                    ("dots-vertical-circle", "dots-vertical-circle"),
                    ("dots-vertical-circle-outline", "dots-vertical-circle-outline"),
                    ("douban", "douban"),
                    ("download", "download"),
                    ("download-box", "download-box"),
                    ("download-box-outline", "download-box-outline"),
                    ("download-circle", "download-circle"),
                    ("download-circle-outline", "download-circle-outline"),
                    ("download-lock", "download-lock"),
                    ("download-lock-outline", "download-lock-outline"),
                    ("download-multiple", "download-multiple"),
                    ("download-network", "download-network"),
                    ("download-network-outline", "download-network-outline"),
                    ("download-off", "download-off"),
                    ("download-off-outline", "download-off-outline"),
                    ("download-outline", "download-outline"),
                    ("drag", "drag"),
                    ("drag-horizontal", "drag-horizontal"),
                    ("drag-horizontal-variant", "drag-horizontal-variant"),
                    ("drag-variant", "drag-variant"),
                    ("drag-vertical", "drag-vertical"),
                    ("drag-vertical-variant", "drag-vertical-variant"),
                    ("drama-masks", "drama-masks"),
                    ("draw", "draw"),
                    ("draw-pen", "draw-pen"),
                    ("drawing", "drawing"),
                    ("drawing-box", "drawing-box"),
                    ("dresser", "dresser"),
                    ("dresser-outline", "dresser-outline"),
                    ("dribbble", "dribbble"),
                    ("dribbble-box", "dribbble-box"),
                    ("drone", "drone"),
                    ("dropbox", "dropbox"),
                    ("drupal", "drupal"),
                    ("duck", "duck"),
                    ("dumbbell", "dumbbell"),
                    ("dump-truck", "dump-truck"),
                    ("ear-hearing", "ear-hearing"),
                    ("ear-hearing-loop", "ear-hearing-loop"),
                    ("ear-hearing-off", "ear-hearing-off"),
                    ("earbuds", "earbuds"),
                    ("earbuds-off", "earbuds-off"),
                    ("earbuds-off-outline", "earbuds-off-outline"),
                    ("earbuds-outline", "earbuds-outline"),
                    ("earth", "earth"),
                    ("earth-arrow-down", "earth-arrow-down"),
                    ("earth-arrow-left", "earth-arrow-left"),
                    ("earth-arrow-right", "earth-arrow-right"),
                    ("earth-arrow-up", "earth-arrow-up"),
                    ("earth-box", "earth-box"),
                    ("earth-box-minus", "earth-box-minus"),
                    ("earth-box-off", "earth-box-off"),
                    ("earth-box-plus", "earth-box-plus"),
                    ("earth-box-remove", "earth-box-remove"),
                    ("earth-minus", "earth-minus"),
                    ("earth-off", "earth-off"),
                    ("earth-plus", "earth-plus"),
                    ("earth-remove", "earth-remove"),
                    ("ebay", "ebay"),
                    ("egg", "egg"),
                    ("egg-easter", "egg-easter"),
                    ("egg-fried", "egg-fried"),
                    ("egg-off", "egg-off"),
                    ("egg-off-outline", "egg-off-outline"),
                    ("egg-outline", "egg-outline"),
                    ("eiffel-tower", "eiffel-tower"),
                    ("eight-track", "eight-track"),
                    ("eject", "eject"),
                    ("eject-circle", "eject-circle"),
                    ("eject-circle-outline", "eject-circle-outline"),
                    ("eject-outline", "eject-outline"),
                    ("electric-switch", "electric-switch"),
                    ("electric-switch-closed", "electric-switch-closed"),
                    ("electron-framework", "electron-framework"),
                    ("elephant", "elephant"),
                    ("elevation-decline", "elevation-decline"),
                    ("elevation-rise", "elevation-rise"),
                    ("elevator", "elevator"),
                    ("elevator-down", "elevator-down"),
                    ("elevator-passenger", "elevator-passenger"),
                    ("elevator-passenger-off", "elevator-passenger-off"),
                    ("elevator-passenger-off-outline", "elevator-passenger-off-outline"),
                    ("elevator-passenger-outline", "elevator-passenger-outline"),
                    ("elevator-up", "elevator-up"),
                    ("ellipse", "ellipse"),
                    ("ellipse-outline", "ellipse-outline"),
                    ("email", "email"),
                    ("email-alert", "email-alert"),
                    ("email-alert-outline", "email-alert-outline"),
                    ("email-arrow-left", "email-arrow-left"),
                    ("email-arrow-left-outline", "email-arrow-left-outline"),
                    ("email-arrow-right", "email-arrow-right"),
                    ("email-arrow-right-outline", "email-arrow-right-outline"),
                    ("email-box", "email-box"),
                    ("email-check", "email-check"),
                    ("email-check-outline", "email-check-outline"),
                    ("email-edit", "email-edit"),
                    ("email-edit-outline", "email-edit-outline"),
                    ("email-fast", "email-fast"),
                    ("email-fast-outline", "email-fast-outline"),
                    ("email-heart-outline", "email-heart-outline"),
                    ("email-lock", "email-lock"),
                    ("email-lock-outline", "email-lock-outline"),
                    ("email-mark-as-unread", "email-mark-as-unread"),
                    ("email-minus", "email-minus"),
                    ("email-minus-outline", "email-minus-outline"),
                    ("email-multiple", "email-multiple"),
                    ("email-multiple-outline", "email-multiple-outline"),
                    ("email-newsletter", "email-newsletter"),
                    ("email-off", "email-off"),
                    ("email-off-outline", "email-off-outline"),
                    ("email-open", "email-open"),
                    ("email-open-heart-outline", "email-open-heart-outline"),
                    ("email-open-multiple", "email-open-multiple"),
                    ("email-open-multiple-outline", "email-open-multiple-outline"),
                    ("email-open-outline", "email-open-outline"),
                    ("email-outline", "email-outline"),
                    ("email-plus", "email-plus"),
                    ("email-plus-outline", "email-plus-outline"),
                    ("email-remove", "email-remove"),
                    ("email-remove-outline", "email-remove-outline"),
                    ("email-seal", "email-seal"),
                    ("email-seal-outline", "email-seal-outline"),
                    ("email-search", "email-search"),
                    ("email-search-outline", "email-search-outline"),
                    ("email-sync", "email-sync"),
                    ("email-sync-outline", "email-sync-outline"),
                    ("email-variant", "email-variant"),
                    ("ember", "ember"),
                    ("emby", "emby"),
                    ("emoticon", "emoticon"),
                    ("emoticon-angry", "emoticon-angry"),
                    ("emoticon-angry-outline", "emoticon-angry-outline"),
                    ("emoticon-confused", "emoticon-confused"),
                    ("emoticon-confused-outline", "emoticon-confused-outline"),
                    ("emoticon-cool", "emoticon-cool"),
                    ("emoticon-cool-outline", "emoticon-cool-outline"),
                    ("emoticon-cry", "emoticon-cry"),
                    ("emoticon-cry-outline", "emoticon-cry-outline"),
                    ("emoticon-dead", "emoticon-dead"),
                    ("emoticon-dead-outline", "emoticon-dead-outline"),
                    ("emoticon-devil", "emoticon-devil"),
                    ("emoticon-devil-outline", "emoticon-devil-outline"),
                    ("emoticon-excited", "emoticon-excited"),
                    ("emoticon-excited-outline", "emoticon-excited-outline"),
                    ("emoticon-frown", "emoticon-frown"),
                    ("emoticon-frown-outline", "emoticon-frown-outline"),
                    ("emoticon-happy", "emoticon-happy"),
                    ("emoticon-happy-outline", "emoticon-happy-outline"),
                    ("emoticon-kiss", "emoticon-kiss"),
                    ("emoticon-kiss-outline", "emoticon-kiss-outline"),
                    ("emoticon-lol", "emoticon-lol"),
                    ("emoticon-lol-outline", "emoticon-lol-outline"),
                    ("emoticon-neutral", "emoticon-neutral"),
                    ("emoticon-neutral-outline", "emoticon-neutral-outline"),
                    ("emoticon-outline", "emoticon-outline"),
                    ("emoticon-poop", "emoticon-poop"),
                    ("emoticon-poop-outline", "emoticon-poop-outline"),
                    ("emoticon-sad", "emoticon-sad"),
                    ("emoticon-sad-outline", "emoticon-sad-outline"),
                    ("emoticon-sick", "emoticon-sick"),
                    ("emoticon-sick-outline", "emoticon-sick-outline"),
                    ("emoticon-tongue", "emoticon-tongue"),
                    ("emoticon-tongue-outline", "emoticon-tongue-outline"),
                    ("emoticon-wink", "emoticon-wink"),
                    ("emoticon-wink-outline", "emoticon-wink-outline"),
                    ("engine", "engine"),
                    ("engine-off", "engine-off"),
                    ("engine-off-outline", "engine-off-outline"),
                    ("engine-outline", "engine-outline"),
                    ("epsilon", "epsilon"),
                    ("equal", "equal"),
                    ("equal-box", "equal-box"),
                    ("equalizer", "equalizer"),
                    ("equalizer-outline", "equalizer-outline"),
                    ("eraser", "eraser"),
                    ("eraser-variant", "eraser-variant"),
                    ("escalator", "escalator"),
                    ("escalator-box", "escalator-box"),
                    ("escalator-down", "escalator-down"),
                    ("escalator-up", "escalator-up"),
                    ("eslint", "eslint"),
                    ("et", "et"),
                    ("ethereum", "ethereum"),
                    ("ethernet", "ethernet"),
                    ("ethernet-cable", "ethernet-cable"),
                    ("ethernet-cable-off", "ethernet-cable-off"),
                    ("etsy", "etsy"),
                    ("ev-plug-ccs1", "ev-plug-ccs1"),
                    ("ev-plug-ccs2", "ev-plug-ccs2"),
                    ("ev-plug-chademo", "ev-plug-chademo"),
                    ("ev-plug-tesla", "ev-plug-tesla"),
                    ("ev-plug-type1", "ev-plug-type1"),
                    ("ev-plug-type2", "ev-plug-type2"),
                    ("ev-station", "ev-station"),
                    ("eventbrite", "eventbrite"),
                    ("evernote", "evernote"),
                    ("excavator", "excavator"),
                    ("exclamation", "exclamation"),
                    ("exclamation-thick", "exclamation-thick"),
                    ("exit-run", "exit-run"),
                    ("exit-to-app", "exit-to-app"),
                    ("expand-all", "expand-all"),
                    ("expand-all-outline", "expand-all-outline"),
                    ("expansion-card", "expansion-card"),
                    ("expansion-card-variant", "expansion-card-variant"),
                    ("exponent", "exponent"),
                    ("exponent-box", "exponent-box"),
                    ("export", "export"),
                    ("export-variant", "export-variant"),
                    ("eye", "eye"),
                    ("eye-arrow-left", "eye-arrow-left"),
                    ("eye-arrow-left-outline", "eye-arrow-left-outline"),
                    ("eye-arrow-right", "eye-arrow-right"),
                    ("eye-arrow-right-outline", "eye-arrow-right-outline"),
                    ("eye-check", "eye-check"),
                    ("eye-check-outline", "eye-check-outline"),
                    ("eye-circle", "eye-circle"),
                    ("eye-circle-outline", "eye-circle-outline"),
                    ("eye-closed", "eye-closed"),
                    ("eye-lock", "eye-lock"),
                    ("eye-lock-open", "eye-lock-open"),
                    ("eye-lock-open-outline", "eye-lock-open-outline"),
                    ("eye-lock-outline", "eye-lock-outline"),
                    ("eye-minus", "eye-minus"),
                    ("eye-minus-outline", "eye-minus-outline"),
                    ("eye-off", "eye-off"),
                    ("eye-off-outline", "eye-off-outline"),
                    ("eye-outline", "eye-outline"),
                    ("eye-plus", "eye-plus"),
                    ("eye-plus-outline", "eye-plus-outline"),
                    ("eye-refresh", "eye-refresh"),
                    ("eye-refresh-outline", "eye-refresh-outline"),
                    ("eye-remove", "eye-remove"),
                    ("eye-remove-outline", "eye-remove-outline"),
                    ("eye-settings", "eye-settings"),
                    ("eye-settings-outline", "eye-settings-outline"),
                    ("eyedropper", "eyedropper"),
                    ("eyedropper-minus", "eyedropper-minus"),
                    ("eyedropper-off", "eyedropper-off"),
                    ("eyedropper-plus", "eyedropper-plus"),
                    ("eyedropper-remove", "eyedropper-remove"),
                    ("eyedropper-variant", "eyedropper-variant"),
                    ("face-agent", "face-agent"),
                    ("face-man", "face-man"),
                    ("face-man-outline", "face-man-outline"),
                    ("face-man-profile", "face-man-profile"),
                    ("face-man-shimmer", "face-man-shimmer"),
                    ("face-man-shimmer-outline", "face-man-shimmer-outline"),
                    ("face-mask", "face-mask"),
                    ("face-mask-outline", "face-mask-outline"),
                    ("face-recognition", "face-recognition"),
                    ("face-woman", "face-woman"),
                    ("face-woman-outline", "face-woman-outline"),
                    ("face-woman-profile", "face-woman-profile"),
                    ("face-woman-shimmer", "face-woman-shimmer"),
                    ("face-woman-shimmer-outline", "face-woman-shimmer-outline"),
                    ("facebook", "facebook"),
                    ("facebook-box", "facebook-box"),
                    ("facebook-gaming", "facebook-gaming"),
                    ("facebook-messenger", "facebook-messenger"),
                    ("facebook-workplace", "facebook-workplace"),
                    ("factory", "factory"),
                    ("family-tree", "family-tree"),
                    ("fan", "fan"),
                    ("fan-alert", "fan-alert"),
                    ("fan-auto", "fan-auto"),
                    ("fan-chevron-down", "fan-chevron-down"),
                    ("fan-chevron-up", "fan-chevron-up"),
                    ("fan-clock", "fan-clock"),
                    ("fan-minus", "fan-minus"),
                    ("fan-off", "fan-off"),
                    ("fan-plus", "fan-plus"),
                    ("fan-remove", "fan-remove"),
                    ("fan-speed-1", "fan-speed-1"),
                    ("fan-speed-2", "fan-speed-2"),
                    ("fan-speed-3", "fan-speed-3"),
                    ("fast-forward", "fast-forward"),
                    ("fast-forward-10", "fast-forward-10"),
                    ("fast-forward-15", "fast-forward-15"),
                    ("fast-forward-30", "fast-forward-30"),
                    ("fast-forward-45", "fast-forward-45"),
                    ("fast-forward-5", "fast-forward-5"),
                    ("fast-forward-60", "fast-forward-60"),
                    ("fast-forward-outline", "fast-forward-outline"),
                    ("faucet", "faucet"),
                    ("faucet-variant", "faucet-variant"),
                    ("fax", "fax"),
                    ("feather", "feather"),
                    ("feature-search", "feature-search"),
                    ("feature-search-outline", "feature-search-outline"),
                    ("fedora", "fedora"),
                    ("fence", "fence"),
                    ("fence-electric", "fence-electric"),
                    ("fencing", "fencing"),
                    ("ferris-wheel", "ferris-wheel"),
                    ("ferry", "ferry"),
                    ("file", "file"),
                    ("file-account", "file-account"),
                    ("file-account-outline", "file-account-outline"),
                    ("file-alert", "file-alert"),
                    ("file-alert-outline", "file-alert-outline"),
                    ("file-arrow-left-right", "file-arrow-left-right"),
                    ("file-arrow-left-right-outline", "file-arrow-left-right-outline"),
                    ("file-arrow-up-down", "file-arrow-up-down"),
                    ("file-arrow-up-down-outline", "file-arrow-up-down-outline"),
                    ("file-cabinet", "file-cabinet"),
                    ("file-cad", "file-cad"),
                    ("file-cad-box", "file-cad-box"),
                    ("file-cancel", "file-cancel"),
                    ("file-cancel-outline", "file-cancel-outline"),
                    ("file-certificate", "file-certificate"),
                    ("file-certificate-outline", "file-certificate-outline"),
                    ("file-chart", "file-chart"),
                    ("file-chart-check", "file-chart-check"),
                    ("file-chart-check-outline", "file-chart-check-outline"),
                    ("file-chart-outline", "file-chart-outline"),
                    ("file-check", "file-check"),
                    ("file-check-outline", "file-check-outline"),
                    ("file-clock", "file-clock"),
                    ("file-clock-outline", "file-clock-outline"),
                    ("file-cloud", "file-cloud"),
                    ("file-cloud-outline", "file-cloud-outline"),
                    ("file-code", "file-code"),
                    ("file-code-outline", "file-code-outline"),
                    ("file-cog", "file-cog"),
                    ("file-cog-outline", "file-cog-outline"),
                    ("file-compare", "file-compare"),
                    ("file-delimited", "file-delimited"),
                    ("file-delimited-outline", "file-delimited-outline"),
                    ("file-document", "file-document"),
                    ("file-document-alert", "file-document-alert"),
                    ("file-document-alert-outline", "file-document-alert-outline"),
                    ("file-document-arrow-right", "file-document-arrow-right"),
                    ("file-document-arrow-right-outline", "file-document-arrow-right-outline"),
                    ("file-document-check", "file-document-check"),
                    ("file-document-check-outline", "file-document-check-outline"),
                    ("file-document-edit", "file-document-edit"),
                    ("file-document-edit-outline", "file-document-edit-outline"),
                    ("file-document-minus", "file-document-minus"),
                    ("file-document-minus-outline", "file-document-minus-outline"),
                    ("file-document-multiple", "file-document-multiple"),
                    ("file-document-multiple-outline", "file-document-multiple-outline"),
                    ("file-document-outline", "file-document-outline"),
                    ("file-document-plus", "file-document-plus"),
                    ("file-document-plus-outline", "file-document-plus-outline"),
                    ("file-document-refresh", "file-document-refresh"),
                    ("file-document-refresh-outline", "file-document-refresh-outline"),
                    ("file-document-remove", "file-document-remove"),
                    ("file-document-remove-outline", "file-document-remove-outline"),
                    ("file-download", "file-download"),
                    ("file-download-outline", "file-download-outline"),
                    ("file-edit", "file-edit"),
                    ("file-edit-outline", "file-edit-outline"),
                    ("file-excel", "file-excel"),
                    ("file-excel-box", "file-excel-box"),
                    ("file-excel-box-outline", "file-excel-box-outline"),
                    ("file-excel-outline", "file-excel-outline"),
                    ("file-export", "file-export"),
                    ("file-export-outline", "file-export-outline"),
                    ("file-eye", "file-eye"),
                    ("file-eye-outline", "file-eye-outline"),
                    ("file-find", "file-find"),
                    ("file-find-outline", "file-find-outline"),
                    ("file-gif-box", "file-gif-box"),
                    ("file-hidden", "file-hidden"),
                    ("file-image", "file-image"),
                    ("file-image-box", "file-image-box"),
                    ("file-image-marker", "file-image-marker"),
                    ("file-image-marker-outline", "file-image-marker-outline"),
                    ("file-image-minus", "file-image-minus"),
                    ("file-image-minus-outline", "file-image-minus-outline"),
                    ("file-image-outline", "file-image-outline"),
                    ("file-image-plus", "file-image-plus"),
                    ("file-image-plus-outline", "file-image-plus-outline"),
                    ("file-image-remove", "file-image-remove"),
                    ("file-image-remove-outline", "file-image-remove-outline"),
                    ("file-import", "file-import"),
                    ("file-import-outline", "file-import-outline"),
                    ("file-jpg-box", "file-jpg-box"),
                    ("file-key", "file-key"),
                    ("file-key-outline", "file-key-outline"),
                    ("file-link", "file-link"),
                    ("file-link-outline", "file-link-outline"),
                    ("file-lock", "file-lock"),
                    ("file-lock-open", "file-lock-open"),
                    ("file-lock-open-outline", "file-lock-open-outline"),
                    ("file-lock-outline", "file-lock-outline"),
                    ("file-marker", "file-marker"),
                    ("file-marker-outline", "file-marker-outline"),
                    ("file-minus", "file-minus"),
                    ("file-minus-outline", "file-minus-outline"),
                    ("file-move", "file-move"),
                    ("file-move-outline", "file-move-outline"),
                    ("file-multiple", "file-multiple"),
                    ("file-multiple-outline", "file-multiple-outline"),
                    ("file-music", "file-music"),
                    ("file-music-outline", "file-music-outline"),
                    ("file-outline", "file-outline"),
                    ("file-pdf", "file-pdf"),
                    ("file-pdf-box", "file-pdf-box"),
                    ("file-pdf-box-outline", "file-pdf-box-outline"),
                    ("file-pdf-outline", "file-pdf-outline"),
                    ("file-percent", "file-percent"),
                    ("file-percent-outline", "file-percent-outline"),
                    ("file-phone", "file-phone"),
                    ("file-phone-outline", "file-phone-outline"),
                    ("file-plus", "file-plus"),
                    ("file-plus-outline", "file-plus-outline"),
                    ("file-png-box", "file-png-box"),
                    ("file-powerpoint", "file-powerpoint"),
                    ("file-powerpoint-box", "file-powerpoint-box"),
                    ("file-powerpoint-box-outline", "file-powerpoint-box-outline"),
                    ("file-powerpoint-outline", "file-powerpoint-outline"),
                    ("file-presentation-box", "file-presentation-box"),
                    ("file-question", "file-question"),
                    ("file-question-outline", "file-question-outline"),
                    ("file-refresh", "file-refresh"),
                    ("file-refresh-outline", "file-refresh-outline"),
                    ("file-remove", "file-remove"),
                    ("file-remove-outline", "file-remove-outline"),
                    ("file-replace", "file-replace"),
                    ("file-replace-outline", "file-replace-outline"),
                    ("file-restore", "file-restore"),
                    ("file-restore-outline", "file-restore-outline"),
                    ("file-rotate-left", "file-rotate-left"),
                    ("file-rotate-left-outline", "file-rotate-left-outline"),
                    ("file-rotate-right", "file-rotate-right"),
                    ("file-rotate-right-outline", "file-rotate-right-outline"),
                    ("file-search", "file-search"),
                    ("file-search-outline", "file-search-outline"),
                    ("file-send", "file-send"),
                    ("file-send-outline", "file-send-outline"),
                    ("file-settings", "file-settings"),
                    ("file-settings-outline", "file-settings-outline"),
                    ("file-sign", "file-sign"),
                    ("file-star", "file-star"),
                    ("file-star-four-points", "file-star-four-points"),
                    ("file-star-four-points-outline", "file-star-four-points-outline"),
                    ("file-star-outline", "file-star-outline"),
                    ("file-swap", "file-swap"),
                    ("file-swap-outline", "file-swap-outline"),
                    ("file-sync", "file-sync"),
                    ("file-sync-outline", "file-sync-outline"),
                    ("file-table", "file-table"),
                    ("file-table-box", "file-table-box"),
                    ("file-table-box-multiple", "file-table-box-multiple"),
                    ("file-table-box-multiple-outline", "file-table-box-multiple-outline"),
                    ("file-table-box-outline", "file-table-box-outline"),
                    ("file-table-outline", "file-table-outline"),
                    ("file-tree", "file-tree"),
                    ("file-tree-outline", "file-tree-outline"),
                    ("file-undo", "file-undo"),
                    ("file-undo-outline", "file-undo-outline"),
                    ("file-upload", "file-upload"),
                    ("file-upload-outline", "file-upload-outline"),
                    ("file-video", "file-video"),
                    ("file-video-outline", "file-video-outline"),
                    ("file-word", "file-word"),
                    ("file-word-box", "file-word-box"),
                    ("file-word-box-outline", "file-word-box-outline"),
                    ("file-word-outline", "file-word-outline"),
                    ("file-xml", "file-xml"),
                    ("file-xml-box", "file-xml-box"),
                    ("fill", "fill"),
                    ("film", "film"),
                    ("filmstrip", "filmstrip"),
                    ("filmstrip-box", "filmstrip-box"),
                    ("filmstrip-box-multiple", "filmstrip-box-multiple"),
                    ("filmstrip-off", "filmstrip-off"),
                    ("filter", "filter"),
                    ("filter-check", "filter-check"),
                    ("filter-check-outline", "filter-check-outline"),
                    ("filter-cog", "filter-cog"),
                    ("filter-cog-outline", "filter-cog-outline"),
                    ("filter-menu", "filter-menu"),
                    ("filter-menu-outline", "filter-menu-outline"),
                    ("filter-minus", "filter-minus"),
                    ("filter-minus-outline", "filter-minus-outline"),
                    ("filter-multiple", "filter-multiple"),
                    ("filter-multiple-outline", "filter-multiple-outline"),
                    ("filter-off", "filter-off"),
                    ("filter-off-outline", "filter-off-outline"),
                    ("filter-outline", "filter-outline"),
                    ("filter-plus", "filter-plus"),
                    ("filter-plus-outline", "filter-plus-outline"),
                    ("filter-remove", "filter-remove"),
                    ("filter-remove-outline", "filter-remove-outline"),
                    ("filter-settings", "filter-settings"),
                    ("filter-settings-outline", "filter-settings-outline"),
                    ("filter-variant", "filter-variant"),
                    ("filter-variant-minus", "filter-variant-minus"),
                    ("filter-variant-plus", "filter-variant-plus"),
                    ("filter-variant-remove", "filter-variant-remove"),
                    ("finance", "finance"),
                    ("find-replace", "find-replace"),
                    ("fingerprint", "fingerprint"),
                    ("fingerprint-off", "fingerprint-off"),
                    ("fire", "fire"),
                    ("fire-alert", "fire-alert"),
                    ("fire-circle", "fire-circle"),
                    ("fire-extinguisher", "fire-extinguisher"),
                    ("fire-hydrant", "fire-hydrant"),
                    ("fire-hydrant-alert", "fire-hydrant-alert"),
                    ("fire-hydrant-off", "fire-hydrant-off"),
                    ("fire-off", "fire-off"),
                    ("fire-truck", "fire-truck"),
                    ("firebase", "firebase"),
                    ("firefox", "firefox"),
                    ("fireplace", "fireplace"),
                    ("fireplace-off", "fireplace-off"),
                    ("firewire", "firewire"),
                    ("firework", "firework"),
                    ("firework-off", "firework-off"),
                    ("fish", "fish"),
                    ("fish-off", "fish-off"),
                    ("fishbowl", "fishbowl"),
                    ("fishbowl-outline", "fishbowl-outline"),
                    ("fit-to-page", "fit-to-page"),
                    ("fit-to-page-outline", "fit-to-page-outline"),
                    ("fit-to-screen", "fit-to-screen"),
                    ("fit-to-screen-outline", "fit-to-screen-outline"),
                    ("flag", "flag"),
                    ("flag-checkered", "flag-checkered"),
                    ("flag-checkered-variant", "flag-checkered-variant"),
                    ("flag-minus", "flag-minus"),
                    ("flag-minus-outline", "flag-minus-outline"),
                    ("flag-off", "flag-off"),
                    ("flag-off-outline", "flag-off-outline"),
                    ("flag-outline", "flag-outline"),
                    ("flag-outline-variant", "flag-outline-variant"),
                    ("flag-plus", "flag-plus"),
                    ("flag-plus-outline", "flag-plus-outline"),
                    ("flag-remove", "flag-remove"),
                    ("flag-remove-outline", "flag-remove-outline"),
                    ("flag-triangle", "flag-triangle"),
                    ("flag-variant", "flag-variant"),
                    ("flag-variant-minus", "flag-variant-minus"),
                    ("flag-variant-minus-outline", "flag-variant-minus-outline"),
                    ("flag-variant-off", "flag-variant-off"),
                    ("flag-variant-off-outline", "flag-variant-off-outline"),
                    ("flag-variant-outline", "flag-variant-outline"),
                    ("flag-variant-plus", "flag-variant-plus"),
                    ("flag-variant-plus-outline", "flag-variant-plus-outline"),
                    ("flag-variant-remove", "flag-variant-remove"),
                    ("flag-variant-remove-outline", "flag-variant-remove-outline"),
                    ("flare", "flare"),
                    ("flash", "flash"),
                    ("flash-alert", "flash-alert"),
                    ("flash-alert-outline", "flash-alert-outline"),
                    ("flash-auto", "flash-auto"),
                    ("flash-off", "flash-off"),
                    ("flash-off-outline", "flash-off-outline"),
                    ("flash-outline", "flash-outline"),
                    ("flash-red-eye", "flash-red-eye"),
                    ("flash-triangle", "flash-triangle"),
                    ("flash-triangle-outline", "flash-triangle-outline"),
                    ("flashlight", "flashlight"),
                    ("flashlight-off", "flashlight-off"),
                    ("flask", "flask"),
                    ("flask-empty", "flask-empty"),
                    ("flask-empty-minus", "flask-empty-minus"),
                    ("flask-empty-minus-outline", "flask-empty-minus-outline"),
                    ("flask-empty-off", "flask-empty-off"),
                    ("flask-empty-off-outline", "flask-empty-off-outline"),
                    ("flask-empty-outline", "flask-empty-outline"),
                    ("flask-empty-plus", "flask-empty-plus"),
                    ("flask-empty-plus-outline", "flask-empty-plus-outline"),
                    ("flask-empty-remove", "flask-empty-remove"),
                    ("flask-empty-remove-outline", "flask-empty-remove-outline"),
                    ("flask-minus", "flask-minus"),
                    ("flask-minus-outline", "flask-minus-outline"),
                    ("flask-off", "flask-off"),
                    ("flask-off-outline", "flask-off-outline"),
                    ("flask-outline", "flask-outline"),
                    ("flask-plus", "flask-plus"),
                    ("flask-plus-outline", "flask-plus-outline"),
                    ("flask-remove", "flask-remove"),
                    ("flask-remove-outline", "flask-remove-outline"),
                    ("flask-round-bottom", "flask-round-bottom"),
                    ("flask-round-bottom-empty", "flask-round-bottom-empty"),
                    ("flask-round-bottom-empty-outline", "flask-round-bottom-empty-outline"),
                    ("flask-round-bottom-outline", "flask-round-bottom-outline"),
                    ("flattr", "flattr"),
                    ("fleur-de-lis", "fleur-de-lis"),
                    ("flickr", "flickr"),
                    ("flickr-after", "flickr-after"),
                    ("flickr-before", "flickr-before"),
                    ("flip-horizontal", "flip-horizontal"),
                    ("flip-to-back", "flip-to-back"),
                    ("flip-to-front", "flip-to-front"),
                    ("flip-vertical", "flip-vertical"),
                    ("floor-1", "floor-1"),
                    ("floor-2", "floor-2"),
                    ("floor-3", "floor-3"),
                    ("floor-a", "floor-a"),
                    ("floor-b", "floor-b"),
                    ("floor-g", "floor-g"),
                    ("floor-l", "floor-l"),
                    ("floor-lamp", "floor-lamp"),
                    ("floor-lamp-dual", "floor-lamp-dual"),
                    ("floor-lamp-dual-outline", "floor-lamp-dual-outline"),
                    ("floor-lamp-outline", "floor-lamp-outline"),
                    ("floor-lamp-torchiere", "floor-lamp-torchiere"),
                    ("floor-lamp-torchiere-outline", "floor-lamp-torchiere-outline"),
                    ("floor-lamp-torchiere-variant", "floor-lamp-torchiere-variant"),
                    (
                        "floor-lamp-torchiere-variant-outline",
                        "floor-lamp-torchiere-variant-outline",
                    ),
                    ("floor-plan", "floor-plan"),
                    ("floppy", "floppy"),
                    ("floppy-variant", "floppy-variant"),
                    ("flower", "flower"),
                    ("flower-outline", "flower-outline"),
                    ("flower-pollen", "flower-pollen"),
                    ("flower-pollen-outline", "flower-pollen-outline"),
                    ("flower-poppy", "flower-poppy"),
                    ("flower-tulip", "flower-tulip"),
                    ("flower-tulip-outline", "flower-tulip-outline"),
                    ("focus-auto", "focus-auto"),
                    ("focus-field", "focus-field"),
                    ("focus-field-horizontal", "focus-field-horizontal"),
                    ("focus-field-vertical", "focus-field-vertical"),
                    ("folder", "folder"),
                    ("folder-account", "folder-account"),
                    ("folder-account-outline", "folder-account-outline"),
                    ("folder-alert", "folder-alert"),
                    ("folder-alert-outline", "folder-alert-outline"),
                    ("folder-arrow-down", "folder-arrow-down"),
                    ("folder-arrow-down-outline", "folder-arrow-down-outline"),
                    ("folder-arrow-left", "folder-arrow-left"),
                    ("folder-arrow-left-outline", "folder-arrow-left-outline"),
                    ("folder-arrow-left-right", "folder-arrow-left-right"),
                    ("folder-arrow-left-right-outline", "folder-arrow-left-right-outline"),
                    ("folder-arrow-right", "folder-arrow-right"),
                    ("folder-arrow-right-outline", "folder-arrow-right-outline"),
                    ("folder-arrow-up", "folder-arrow-up"),
                    ("folder-arrow-up-down", "folder-arrow-up-down"),
                    ("folder-arrow-up-down-outline", "folder-arrow-up-down-outline"),
                    ("folder-arrow-up-outline", "folder-arrow-up-outline"),
                    ("folder-cancel", "folder-cancel"),
                    ("folder-cancel-outline", "folder-cancel-outline"),
                    ("folder-check", "folder-check"),
                    ("folder-check-outline", "folder-check-outline"),
                    ("folder-clock", "folder-clock"),
                    ("folder-clock-outline", "folder-clock-outline"),
                    ("folder-cog", "folder-cog"),
                    ("folder-cog-outline", "folder-cog-outline"),
                    ("folder-download", "folder-download"),
                    ("folder-download-outline", "folder-download-outline"),
                    ("folder-edit", "folder-edit"),
                    ("folder-edit-outline", "folder-edit-outline"),
                    ("folder-eye", "folder-eye"),
                    ("folder-eye-outline", "folder-eye-outline"),
                    ("folder-file", "folder-file"),
                    ("folder-file-outline", "folder-file-outline"),
                    ("folder-google-drive", "folder-google-drive"),
                    ("folder-heart", "folder-heart"),
                    ("folder-heart-outline", "folder-heart-outline"),
                    ("folder-hidden", "folder-hidden"),
                    ("folder-home", "folder-home"),
                    ("folder-home-outline", "folder-home-outline"),
                    ("folder-image", "folder-image"),
                    ("folder-information", "folder-information"),
                    ("folder-information-outline", "folder-information-outline"),
                    ("folder-key", "folder-key"),
                    ("folder-key-network", "folder-key-network"),
                    ("folder-key-network-outline", "folder-key-network-outline"),
                    ("folder-key-outline", "folder-key-outline"),
                    ("folder-lock", "folder-lock"),
                    ("folder-lock-open", "folder-lock-open"),
                    ("folder-lock-open-outline", "folder-lock-open-outline"),
                    ("folder-lock-outline", "folder-lock-outline"),
                    ("folder-marker", "folder-marker"),
                    ("folder-marker-outline", "folder-marker-outline"),
                    ("folder-minus", "folder-minus"),
                    ("folder-minus-outline", "folder-minus-outline"),
                    ("folder-move", "folder-move"),
                    ("folder-move-outline", "folder-move-outline"),
                    ("folder-multiple", "folder-multiple"),
                    ("folder-multiple-image", "folder-multiple-image"),
                    ("folder-multiple-outline", "folder-multiple-outline"),
                    ("folder-multiple-plus", "folder-multiple-plus"),
                    ("folder-multiple-plus-outline", "folder-multiple-plus-outline"),
                    ("folder-music", "folder-music"),
                    ("folder-music-outline", "folder-music-outline"),
                    ("folder-network", "folder-network"),
                    ("folder-network-outline", "folder-network-outline"),
                    ("folder-off", "folder-off"),
                    ("folder-off-outline", "folder-off-outline"),
                    ("folder-open", "folder-open"),
                    ("folder-open-outline", "folder-open-outline"),
                    ("folder-outline", "folder-outline"),
                    ("folder-outline-lock", "folder-outline-lock"),
                    ("folder-play", "folder-play"),
                    ("folder-play-outline", "folder-play-outline"),
                    ("folder-plus", "folder-plus"),
                    ("folder-plus-outline", "folder-plus-outline"),
                    ("folder-pound", "folder-pound"),
                    ("folder-pound-outline", "folder-pound-outline"),
                    ("folder-question", "folder-question"),
                    ("folder-question-outline", "folder-question-outline"),
                    ("folder-refresh", "folder-refresh"),
                    ("folder-refresh-outline", "folder-refresh-outline"),
                    ("folder-remove", "folder-remove"),
                    ("folder-remove-outline", "folder-remove-outline"),
                    ("folder-search", "folder-search"),
                    ("folder-search-outline", "folder-search-outline"),
                    ("folder-settings", "folder-settings"),
                    ("folder-settings-outline", "folder-settings-outline"),
                    ("folder-star", "folder-star"),
                    ("folder-star-multiple", "folder-star-multiple"),
                    ("folder-star-multiple-outline", "folder-star-multiple-outline"),
                    ("folder-star-outline", "folder-star-outline"),
                    ("folder-swap", "folder-swap"),
                    ("folder-swap-outline", "folder-swap-outline"),
                    ("folder-sync", "folder-sync"),
                    ("folder-sync-outline", "folder-sync-outline"),
                    ("folder-table", "folder-table"),
                    ("folder-table-outline", "folder-table-outline"),
                    ("folder-text", "folder-text"),
                    ("folder-text-outline", "folder-text-outline"),
                    ("folder-upload", "folder-upload"),
                    ("folder-upload-outline", "folder-upload-outline"),
                    ("folder-wrench", "folder-wrench"),
                    ("folder-wrench-outline", "folder-wrench-outline"),
                    ("folder-zip", "folder-zip"),
                    ("folder-zip-outline", "folder-zip-outline"),
                    ("font-awesome", "font-awesome"),
                    ("food", "food"),
                    ("food-apple", "food-apple"),
                    ("food-apple-outline", "food-apple-outline"),
                    ("food-croissant", "food-croissant"),
                    ("food-drumstick", "food-drumstick"),
                    ("food-drumstick-off", "food-drumstick-off"),
                    ("food-drumstick-off-outline", "food-drumstick-off-outline"),
                    ("food-drumstick-outline", "food-drumstick-outline"),
                    ("food-fork-drink", "food-fork-drink"),
                    ("food-halal", "food-halal"),
                    ("food-hot-dog", "food-hot-dog"),
                    ("food-kosher", "food-kosher"),
                    ("food-off", "food-off"),
                    ("food-off-outline", "food-off-outline"),
                    ("food-outline", "food-outline"),
                    ("food-steak", "food-steak"),
                    ("food-steak-off", "food-steak-off"),
                    ("food-takeout-box", "food-takeout-box"),
                    ("food-takeout-box-outline", "food-takeout-box-outline"),
                    ("food-turkey", "food-turkey"),
                    ("food-variant", "food-variant"),
                    ("food-variant-off", "food-variant-off"),
                    ("foot-print", "foot-print"),
                    ("football", "football"),
                    ("football-australian", "football-australian"),
                    ("football-helmet", "football-helmet"),
                    ("footer", "footer"),
                    ("forest", "forest"),
                    ("forest-outline", "forest-outline"),
                    ("forklift", "forklift"),
                    ("form-dropdown", "form-dropdown"),
                    ("form-select", "form-select"),
                    ("form-textarea", "form-textarea"),
                    ("form-textbox", "form-textbox"),
                    ("form-textbox-lock", "form-textbox-lock"),
                    ("form-textbox-password", "form-textbox-password"),
                    ("format-align-bottom", "format-align-bottom"),
                    ("format-align-center", "format-align-center"),
                    ("format-align-justify", "format-align-justify"),
                    ("format-align-left", "format-align-left"),
                    ("format-align-middle", "format-align-middle"),
                    ("format-align-right", "format-align-right"),
                    ("format-align-top", "format-align-top"),
                    ("format-annotation-minus", "format-annotation-minus"),
                    ("format-annotation-plus", "format-annotation-plus"),
                    ("format-bold", "format-bold"),
                    ("format-clear", "format-clear"),
                    ("format-color", "format-color"),
                    ("format-color-fill", "format-color-fill"),
                    ("format-color-highlight", "format-color-highlight"),
                    ("format-color-marker-cancel", "format-color-marker-cancel"),
                    ("format-color-text", "format-color-text"),
                    ("format-columns", "format-columns"),
                    ("format-float-center", "format-float-center"),
                    ("format-float-left", "format-float-left"),
                    ("format-float-none", "format-float-none"),
                    ("format-float-right", "format-float-right"),
                    ("format-font", "format-font"),
                    ("format-font-size-decrease", "format-font-size-decrease"),
                    ("format-font-size-increase", "format-font-size-increase"),
                    ("format-header-1", "format-header-1"),
                    ("format-header-2", "format-header-2"),
                    ("format-header-3", "format-header-3"),
                    ("format-header-4", "format-header-4"),
                    ("format-header-5", "format-header-5"),
                    ("format-header-6", "format-header-6"),
                    ("format-header-decrease", "format-header-decrease"),
                    ("format-header-down", "format-header-down"),
                    ("format-header-equal", "format-header-equal"),
                    ("format-header-increase", "format-header-increase"),
                    ("format-header-pound", "format-header-pound"),
                    ("format-header-up", "format-header-up"),
                    ("format-horizontal-align-center", "format-horizontal-align-center"),
                    ("format-horizontal-align-left", "format-horizontal-align-left"),
                    ("format-horizontal-align-right", "format-horizontal-align-right"),
                    ("format-indent-decrease", "format-indent-decrease"),
                    ("format-indent-increase", "format-indent-increase"),
                    ("format-italic", "format-italic"),
                    ("format-letter-case", "format-letter-case"),
                    ("format-letter-case-lower", "format-letter-case-lower"),
                    ("format-letter-case-upper", "format-letter-case-upper"),
                    ("format-letter-ends-with", "format-letter-ends-with"),
                    ("format-letter-matches", "format-letter-matches"),
                    ("format-letter-spacing", "format-letter-spacing"),
                    ("format-letter-spacing-variant", "format-letter-spacing-variant"),
                    ("format-letter-starts-with", "format-letter-starts-with"),
                    ("format-line-height", "format-line-height"),
                    ("format-line-spacing", "format-line-spacing"),
                    ("format-line-style", "format-line-style"),
                    ("format-line-weight", "format-line-weight"),
                    ("format-list-bulleted", "format-list-bulleted"),
                    ("format-list-bulleted-square", "format-list-bulleted-square"),
                    ("format-list-bulleted-triangle", "format-list-bulleted-triangle"),
                    ("format-list-bulleted-type", "format-list-bulleted-type"),
                    ("format-list-checkbox", "format-list-checkbox"),
                    ("format-list-checks", "format-list-checks"),
                    ("format-list-group", "format-list-group"),
                    ("format-list-group-plus", "format-list-group-plus"),
                    ("format-list-numbered", "format-list-numbered"),
                    ("format-list-numbered-rtl", "format-list-numbered-rtl"),
                    ("format-list-text", "format-list-text"),
                    ("format-list-triangle", "format-list-triangle"),
                    ("format-overline", "format-overline"),
                    ("format-page-break", "format-page-break"),
                    ("format-page-split", "format-page-split"),
                    ("format-paint", "format-paint"),
                    ("format-paragraph", "format-paragraph"),
                    ("format-paragraph-spacing", "format-paragraph-spacing"),
                    ("format-pilcrow", "format-pilcrow"),
                    ("format-pilcrow-arrow-left", "format-pilcrow-arrow-left"),
                    ("format-pilcrow-arrow-right", "format-pilcrow-arrow-right"),
                    ("format-quote-close", "format-quote-close"),
                    ("format-quote-close-outline", "format-quote-close-outline"),
                    ("format-quote-open", "format-quote-open"),
                    ("format-quote-open-outline", "format-quote-open-outline"),
                    ("format-rotate-90", "format-rotate-90"),
                    ("format-section", "format-section"),
                    ("format-size", "format-size"),
                    ("format-strikethrough", "format-strikethrough"),
                    ("format-strikethrough-variant", "format-strikethrough-variant"),
                    ("format-subscript", "format-subscript"),
                    ("format-superscript", "format-superscript"),
                    ("format-text", "format-text"),
                    ("format-text-rotation-angle-down", "format-text-rotation-angle-down"),
                    ("format-text-rotation-angle-up", "format-text-rotation-angle-up"),
                    ("format-text-rotation-down", "format-text-rotation-down"),
                    ("format-text-rotation-down-vertical", "format-text-rotation-down-vertical"),
                    ("format-text-rotation-none", "format-text-rotation-none"),
                    ("format-text-rotation-up", "format-text-rotation-up"),
                    ("format-text-rotation-vertical", "format-text-rotation-vertical"),
                    ("format-text-variant", "format-text-variant"),
                    ("format-text-variant-outline", "format-text-variant-outline"),
                    ("format-text-wrapping-clip", "format-text-wrapping-clip"),
                    ("format-text-wrapping-overflow", "format-text-wrapping-overflow"),
                    ("format-text-wrapping-wrap", "format-text-wrapping-wrap"),
                    ("format-textbox", "format-textbox"),
                    ("format-title", "format-title"),
                    ("format-underline", "format-underline"),
                    ("format-underline-wavy", "format-underline-wavy"),
                    ("format-vertical-align-bottom", "format-vertical-align-bottom"),
                    ("format-vertical-align-center", "format-vertical-align-center"),
                    ("format-vertical-align-top", "format-vertical-align-top"),
                    ("format-wrap-inline", "format-wrap-inline"),
                    ("format-wrap-square", "format-wrap-square"),
                    ("format-wrap-tight", "format-wrap-tight"),
                    ("format-wrap-top-bottom", "format-wrap-top-bottom"),
                    ("forum", "forum"),
                    ("forum-minus", "forum-minus"),
                    ("forum-minus-outline", "forum-minus-outline"),
                    ("forum-outline", "forum-outline"),
                    ("forum-plus", "forum-plus"),
                    ("forum-plus-outline", "forum-plus-outline"),
                    ("forum-remove", "forum-remove"),
                    ("forum-remove-outline", "forum-remove-outline"),
                    ("forward", "forward"),
                    ("forwardburger", "forwardburger"),
                    ("fountain", "fountain"),
                    ("fountain-pen", "fountain-pen"),
                    ("fountain-pen-tip", "fountain-pen-tip"),
                    ("foursquare", "foursquare"),
                    ("fraction-one-half", "fraction-one-half"),
                    ("freebsd", "freebsd"),
                    ("french-fries", "french-fries"),
                    ("frequently-asked-questions", "frequently-asked-questions"),
                    ("fridge", "fridge"),
                    ("fridge-alert", "fridge-alert"),
                    ("fridge-alert-outline", "fridge-alert-outline"),
                    ("fridge-bottom", "fridge-bottom"),
                    ("fridge-industrial", "fridge-industrial"),
                    ("fridge-industrial-alert", "fridge-industrial-alert"),
                    ("fridge-industrial-alert-outline", "fridge-industrial-alert-outline"),
                    ("fridge-industrial-off", "fridge-industrial-off"),
                    ("fridge-industrial-off-outline", "fridge-industrial-off-outline"),
                    ("fridge-industrial-outline", "fridge-industrial-outline"),
                    ("fridge-off", "fridge-off"),
                    ("fridge-off-outline", "fridge-off-outline"),
                    ("fridge-outline", "fridge-outline"),
                    ("fridge-top", "fridge-top"),
                    ("fridge-variant", "fridge-variant"),
                    ("fridge-variant-alert", "fridge-variant-alert"),
                    ("fridge-variant-alert-outline", "fridge-variant-alert-outline"),
                    ("fridge-variant-off", "fridge-variant-off"),
                    ("fridge-variant-off-outline", "fridge-variant-off-outline"),
                    ("fridge-variant-outline", "fridge-variant-outline"),
                    ("fruit-cherries", "fruit-cherries"),
                    ("fruit-cherries-off", "fruit-cherries-off"),
                    ("fruit-citrus", "fruit-citrus"),
                    ("fruit-citrus-off", "fruit-citrus-off"),
                    ("fruit-grapes", "fruit-grapes"),
                    ("fruit-grapes-outline", "fruit-grapes-outline"),
                    ("fruit-pear", "fruit-pear"),
                    ("fruit-pineapple", "fruit-pineapple"),
                    ("fruit-watermelon", "fruit-watermelon"),
                    ("fuel", "fuel"),
                    ("fuel-cell", "fuel-cell"),
                    ("fullscreen", "fullscreen"),
                    ("fullscreen-exit", "fullscreen-exit"),
                    ("function", "function"),
                    ("function-variant", "function-variant"),
                    ("furigana-horizontal", "furigana-horizontal"),
                    ("furigana-vertical", "furigana-vertical"),
                    ("fuse", "fuse"),
                    ("fuse-alert", "fuse-alert"),
                    ("fuse-blade", "fuse-blade"),
                    ("fuse-off", "fuse-off"),
                    ("gamepad", "gamepad"),
                    ("gamepad-circle", "gamepad-circle"),
                    ("gamepad-circle-down", "gamepad-circle-down"),
                    ("gamepad-circle-left", "gamepad-circle-left"),
                    ("gamepad-circle-outline", "gamepad-circle-outline"),
                    ("gamepad-circle-right", "gamepad-circle-right"),
                    ("gamepad-circle-up", "gamepad-circle-up"),
                    ("gamepad-down", "gamepad-down"),
                    ("gamepad-left", "gamepad-left"),
                    ("gamepad-outline", "gamepad-outline"),
                    ("gamepad-right", "gamepad-right"),
                    ("gamepad-round", "gamepad-round"),
                    ("gamepad-round-down", "gamepad-round-down"),
                    ("gamepad-round-left", "gamepad-round-left"),
                    ("gamepad-round-outline", "gamepad-round-outline"),
                    ("gamepad-round-right", "gamepad-round-right"),
                    ("gamepad-round-up", "gamepad-round-up"),
                    ("gamepad-square", "gamepad-square"),
                    ("gamepad-square-outline", "gamepad-square-outline"),
                    ("gamepad-up", "gamepad-up"),
                    ("gamepad-variant", "gamepad-variant"),
                    ("gamepad-variant-outline", "gamepad-variant-outline"),
                    ("gamma", "gamma"),
                    ("gantry-crane", "gantry-crane"),
                    ("garage", "garage"),
                    ("garage-alert", "garage-alert"),
                    ("garage-alert-variant", "garage-alert-variant"),
                    ("garage-lock", "garage-lock"),
                    ("garage-open", "garage-open"),
                    ("garage-open-variant", "garage-open-variant"),
                    ("garage-variant", "garage-variant"),
                    ("garage-variant-lock", "garage-variant-lock"),
                    ("gas-burner", "gas-burner"),
                    ("gas-cylinder", "gas-cylinder"),
                    ("gas-station", "gas-station"),
                    ("gas-station-off", "gas-station-off"),
                    ("gas-station-off-outline", "gas-station-off-outline"),
                    ("gas-station-outline", "gas-station-outline"),
                    ("gate", "gate"),
                    ("gate-alert", "gate-alert"),
                    ("gate-and", "gate-and"),
                    ("gate-arrow-left", "gate-arrow-left"),
                    ("gate-arrow-right", "gate-arrow-right"),
                    ("gate-buffer", "gate-buffer"),
                    ("gate-nand", "gate-nand"),
                    ("gate-nor", "gate-nor"),
                    ("gate-not", "gate-not"),
                    ("gate-open", "gate-open"),
                    ("gate-or", "gate-or"),
                    ("gate-xnor", "gate-xnor"),
                    ("gate-xor", "gate-xor"),
                    ("gatsby", "gatsby"),
                    ("gauge", "gauge"),
                    ("gauge-empty", "gauge-empty"),
                    ("gauge-full", "gauge-full"),
                    ("gauge-low", "gauge-low"),
                    ("gavel", "gavel"),
                    ("gender-female", "gender-female"),
                    ("gender-male", "gender-male"),
                    ("gender-male-female", "gender-male-female"),
                    ("gender-male-female-variant", "gender-male-female-variant"),
                    ("gender-non-binary", "gender-non-binary"),
                    ("gender-transgender", "gender-transgender"),
                    ("generator-mobile", "generator-mobile"),
                    ("generator-portable", "generator-portable"),
                    ("generator-stationary", "generator-stationary"),
                    ("gentoo", "gentoo"),
                    ("gesture", "gesture"),
                    ("gesture-double-tap", "gesture-double-tap"),
                    ("gesture-pinch", "gesture-pinch"),
                    ("gesture-spread", "gesture-spread"),
                    ("gesture-swipe", "gesture-swipe"),
                    ("gesture-swipe-down", "gesture-swipe-down"),
                    ("gesture-swipe-horizontal", "gesture-swipe-horizontal"),
                    ("gesture-swipe-left", "gesture-swipe-left"),
                    ("gesture-swipe-right", "gesture-swipe-right"),
                    ("gesture-swipe-up", "gesture-swipe-up"),
                    ("gesture-swipe-vertical", "gesture-swipe-vertical"),
                    ("gesture-tap", "gesture-tap"),
                    ("gesture-tap-box", "gesture-tap-box"),
                    ("gesture-tap-button", "gesture-tap-button"),
                    ("gesture-tap-hold", "gesture-tap-hold"),
                    ("gesture-two-double-tap", "gesture-two-double-tap"),
                    ("gesture-two-tap", "gesture-two-tap"),
                    ("ghost", "ghost"),
                    ("ghost-off", "ghost-off"),
                    ("ghost-off-outline", "ghost-off-outline"),
                    ("ghost-outline", "ghost-outline"),
                    ("gif", "gif"),
                    ("gift", "gift"),
                    ("gift-off", "gift-off"),
                    ("gift-off-outline", "gift-off-outline"),
                    ("gift-open", "gift-open"),
                    ("gift-open-outline", "gift-open-outline"),
                    ("gift-outline", "gift-outline"),
                    ("git", "git"),
                    ("github", "github"),
                    ("github-box", "github-box"),
                    ("github-face", "github-face"),
                    ("gitlab", "gitlab"),
                    ("glass-cocktail", "glass-cocktail"),
                    ("glass-cocktail-off", "glass-cocktail-off"),
                    ("glass-flute", "glass-flute"),
                    ("glass-fragile", "glass-fragile"),
                    ("glass-mug", "glass-mug"),
                    ("glass-mug-off", "glass-mug-off"),
                    ("glass-mug-variant", "glass-mug-variant"),
                    ("glass-mug-variant-off", "glass-mug-variant-off"),
                    ("glass-pint-outline", "glass-pint-outline"),
                    ("glass-stange", "glass-stange"),
                    ("glass-tulip", "glass-tulip"),
                    ("glass-wine", "glass-wine"),
                    ("glassdoor", "glassdoor"),
                    ("glasses", "glasses"),
                    ("globe-light", "globe-light"),
                    ("globe-light-outline", "globe-light-outline"),
                    ("globe-model", "globe-model"),
                    ("gmail", "gmail"),
                    ("gnome", "gnome"),
                    ("go-kart", "go-kart"),
                    ("go-kart-track", "go-kart-track"),
                    ("gog", "gog"),
                    ("gold", "gold"),
                    ("golf", "golf"),
                    ("golf-cart", "golf-cart"),
                    ("golf-tee", "golf-tee"),
                    ("gondola", "gondola"),
                    ("goodreads", "goodreads"),
                    ("google", "google"),
                    ("google-ads", "google-ads"),
                    ("google-allo", "google-allo"),
                    ("google-analytics", "google-analytics"),
                    ("google-assistant", "google-assistant"),
                    ("google-cardboard", "google-cardboard"),
                    ("google-chrome", "google-chrome"),
                    ("google-circles", "google-circles"),
                    ("google-circles-communities", "google-circles-communities"),
                    ("google-circles-extended", "google-circles-extended"),
                    ("google-circles-group", "google-circles-group"),
                    ("google-classroom", "google-classroom"),
                    ("google-cloud", "google-cloud"),
                    ("google-downasaur", "google-downasaur"),
                    ("google-drive", "google-drive"),
                    ("google-earth", "google-earth"),
                    ("google-fit", "google-fit"),
                    ("google-glass", "google-glass"),
                    ("google-hangouts", "google-hangouts"),
                    ("google-home", "google-home"),
                    ("google-keep", "google-keep"),
                    ("google-lens", "google-lens"),
                    ("google-maps", "google-maps"),
                    ("google-my-business", "google-my-business"),
                    ("google-nearby", "google-nearby"),
                    ("google-pages", "google-pages"),
                    ("google-photos", "google-photos"),
                    ("google-physical-web", "google-physical-web"),
                    ("google-play", "google-play"),
                    ("google-plus", "google-plus"),
                    ("google-plus-box", "google-plus-box"),
                    ("google-podcast", "google-podcast"),
                    ("google-spreadsheet", "google-spreadsheet"),
                    ("google-street-view", "google-street-view"),
                    ("google-translate", "google-translate"),
                    ("google-wallet", "google-wallet"),
                    ("gradient-horizontal", "gradient-horizontal"),
                    ("gradient-vertical", "gradient-vertical"),
                    ("grain", "grain"),
                    ("graph", "graph"),
                    ("graph-outline", "graph-outline"),
                    ("graphql", "graphql"),
                    ("grass", "grass"),
                    ("grave-stone", "grave-stone"),
                    ("grease-pencil", "grease-pencil"),
                    ("greater-than", "greater-than"),
                    ("greater-than-or-equal", "greater-than-or-equal"),
                    ("greenhouse", "greenhouse"),
                    ("grid", "grid"),
                    ("grid-large", "grid-large"),
                    ("grid-off", "grid-off"),
                    ("grill", "grill"),
                    ("grill-outline", "grill-outline"),
                    ("group", "group"),
                    ("guitar-acoustic", "guitar-acoustic"),
                    ("guitar-electric", "guitar-electric"),
                    ("guitar-pick", "guitar-pick"),
                    ("guitar-pick-outline", "guitar-pick-outline"),
                    ("guy-fawkes-mask", "guy-fawkes-mask"),
                    ("gymnastics", "gymnastics"),
                    ("hail", "hail"),
                    ("hair-dryer", "hair-dryer"),
                    ("hair-dryer-outline", "hair-dryer-outline"),
                    ("halloween", "halloween"),
                    ("hamburger", "hamburger"),
                    ("hamburger-check", "hamburger-check"),
                    ("hamburger-minus", "hamburger-minus"),
                    ("hamburger-off", "hamburger-off"),
                    ("hamburger-plus", "hamburger-plus"),
                    ("hamburger-remove", "hamburger-remove"),
                    ("hammer", "hammer"),
                    ("hammer-screwdriver", "hammer-screwdriver"),
                    ("hammer-sickle", "hammer-sickle"),
                    ("hammer-wrench", "hammer-wrench"),
                    ("hand-back-left", "hand-back-left"),
                    ("hand-back-left-off", "hand-back-left-off"),
                    ("hand-back-left-off-outline", "hand-back-left-off-outline"),
                    ("hand-back-left-outline", "hand-back-left-outline"),
                    ("hand-back-right", "hand-back-right"),
                    ("hand-back-right-off", "hand-back-right-off"),
                    ("hand-back-right-off-outline", "hand-back-right-off-outline"),
                    ("hand-back-right-outline", "hand-back-right-outline"),
                    ("hand-clap", "hand-clap"),
                    ("hand-clap-off", "hand-clap-off"),
                    ("hand-coin", "hand-coin"),
                    ("hand-coin-outline", "hand-coin-outline"),
                    ("hand-cycle", "hand-cycle"),
                    ("hand-extended", "hand-extended"),
                    ("hand-extended-outline", "hand-extended-outline"),
                    ("hand-front-left", "hand-front-left"),
                    ("hand-front-left-outline", "hand-front-left-outline"),
                    ("hand-front-right", "hand-front-right"),
                    ("hand-front-right-outline", "hand-front-right-outline"),
                    ("hand-heart", "hand-heart"),
                    ("hand-heart-outline", "hand-heart-outline"),
                    ("hand-left", "hand-left"),
                    ("hand-okay", "hand-okay"),
                    ("hand-peace", "hand-peace"),
                    ("hand-peace-variant", "hand-peace-variant"),
                    ("hand-pointing-down", "hand-pointing-down"),
                    ("hand-pointing-left", "hand-pointing-left"),
                    ("hand-pointing-right", "hand-pointing-right"),
                    ("hand-pointing-up", "hand-pointing-up"),
                    ("hand-right", "hand-right"),
                    ("hand-saw", "hand-saw"),
                    ("hand-wash", "hand-wash"),
                    ("hand-wash-outline", "hand-wash-outline"),
                    ("hand-water", "hand-water"),
                    ("hand-wave", "hand-wave"),
                    ("hand-wave-outline", "hand-wave-outline"),
                    ("handball", "handball"),
                    ("handcuffs", "handcuffs"),
                    ("hands-pray", "hands-pray"),
                    ("handshake", "handshake"),
                    ("handshake-outline", "handshake-outline"),
                    ("hanger", "hanger"),
                    ("hangouts", "hangouts"),
                    ("hard-hat", "hard-hat"),
                    ("harddisk", "harddisk"),
                    ("harddisk-plus", "harddisk-plus"),
                    ("harddisk-remove", "harddisk-remove"),
                    ("hat-fedora", "hat-fedora"),
                    ("hazard-lights", "hazard-lights"),
                    ("hdmi-port", "hdmi-port"),
                    ("hdr", "hdr"),
                    ("hdr-off", "hdr-off"),
                    ("head", "head"),
                    ("head-alert", "head-alert"),
                    ("head-alert-outline", "head-alert-outline"),
                    ("head-check", "head-check"),
                    ("head-check-outline", "head-check-outline"),
                    ("head-cog", "head-cog"),
                    ("head-cog-outline", "head-cog-outline"),
                    ("head-dots-horizontal", "head-dots-horizontal"),
                    ("head-dots-horizontal-outline", "head-dots-horizontal-outline"),
                    ("head-flash", "head-flash"),
                    ("head-flash-outline", "head-flash-outline"),
                    ("head-heart", "head-heart"),
                    ("head-heart-outline", "head-heart-outline"),
                    ("head-lightbulb", "head-lightbulb"),
                    ("head-lightbulb-outline", "head-lightbulb-outline"),
                    ("head-minus", "head-minus"),
                    ("head-minus-outline", "head-minus-outline"),
                    ("head-outline", "head-outline"),
                    ("head-plus", "head-plus"),
                    ("head-plus-outline", "head-plus-outline"),
                    ("head-question", "head-question"),
                    ("head-question-outline", "head-question-outline"),
                    ("head-remove", "head-remove"),
                    ("head-remove-outline", "head-remove-outline"),
                    ("head-snowflake", "head-snowflake"),
                    ("head-snowflake-outline", "head-snowflake-outline"),
                    ("head-sync", "head-sync"),
                    ("head-sync-outline", "head-sync-outline"),
                    ("headphones", "headphones"),
                    ("headphones-bluetooth", "headphones-bluetooth"),
                    ("headphones-box", "headphones-box"),
                    ("headphones-off", "headphones-off"),
                    ("headphones-settings", "headphones-settings"),
                    ("headset", "headset"),
                    ("headset-dock", "headset-dock"),
                    ("headset-off", "headset-off"),
                    ("heart", "heart"),
                    ("heart-box", "heart-box"),
                    ("heart-box-outline", "heart-box-outline"),
                    ("heart-broken", "heart-broken"),
                    ("heart-broken-outline", "heart-broken-outline"),
                    ("heart-circle", "heart-circle"),
                    ("heart-circle-outline", "heart-circle-outline"),
                    ("heart-cog", "heart-cog"),
                    ("heart-cog-outline", "heart-cog-outline"),
                    ("heart-flash", "heart-flash"),
                    ("heart-half", "heart-half"),
                    ("heart-half-full", "heart-half-full"),
                    ("heart-half-outline", "heart-half-outline"),
                    ("heart-minus", "heart-minus"),
                    ("heart-minus-outline", "heart-minus-outline"),
                    ("heart-multiple", "heart-multiple"),
                    ("heart-multiple-outline", "heart-multiple-outline"),
                    ("heart-off", "heart-off"),
                    ("heart-off-outline", "heart-off-outline"),
                    ("heart-outline", "heart-outline"),
                    ("heart-plus", "heart-plus"),
                    ("heart-plus-outline", "heart-plus-outline"),
                    ("heart-pulse", "heart-pulse"),
                    ("heart-remove", "heart-remove"),
                    ("heart-remove-outline", "heart-remove-outline"),
                    ("heart-search", "heart-search"),
                    ("heart-settings", "heart-settings"),
                    ("heart-settings-outline", "heart-settings-outline"),
                    ("heat-pump", "heat-pump"),
                    ("heat-pump-outline", "heat-pump-outline"),
                    ("heat-wave", "heat-wave"),
                    ("heating-coil", "heating-coil"),
                    ("helicopter", "helicopter"),
                    ("help", "help"),
                    ("help-box", "help-box"),
                    ("help-box-multiple", "help-box-multiple"),
                    ("help-box-multiple-outline", "help-box-multiple-outline"),
                    ("help-box-outline", "help-box-outline"),
                    ("help-circle", "help-circle"),
                    ("help-circle-outline", "help-circle-outline"),
                    ("help-network", "help-network"),
                    ("help-network-outline", "help-network-outline"),
                    ("help-rhombus", "help-rhombus"),
                    ("help-rhombus-outline", "help-rhombus-outline"),
                    ("hexadecimal", "hexadecimal"),
                    ("hexagon", "hexagon"),
                    ("hexagon-multiple", "hexagon-multiple"),
                    ("hexagon-multiple-outline", "hexagon-multiple-outline"),
                    ("hexagon-outline", "hexagon-outline"),
                    ("hexagon-slice-1", "hexagon-slice-1"),
                    ("hexagon-slice-2", "hexagon-slice-2"),
                    ("hexagon-slice-3", "hexagon-slice-3"),
                    ("hexagon-slice-4", "hexagon-slice-4"),
                    ("hexagon-slice-5", "hexagon-slice-5"),
                    ("hexagon-slice-6", "hexagon-slice-6"),
                    ("hexagram", "hexagram"),
                    ("hexagram-outline", "hexagram-outline"),
                    ("high-definition", "high-definition"),
                    ("high-definition-box", "high-definition-box"),
                    ("highway", "highway"),
                    ("hiking", "hiking"),
                    ("history", "history"),
                    ("hockey-puck", "hockey-puck"),
                    ("hockey-sticks", "hockey-sticks"),
                    ("hololens", "hololens"),
                    ("home", "home"),
                    ("home-account", "home-account"),
                    ("home-alert", "home-alert"),
                    ("home-alert-outline", "home-alert-outline"),
                    ("home-analytics", "home-analytics"),
                    ("home-assistant", "home-assistant"),
                    ("home-automation", "home-automation"),
                    ("home-battery", "home-battery"),
                    ("home-battery-outline", "home-battery-outline"),
                    ("home-circle", "home-circle"),
                    ("home-circle-outline", "home-circle-outline"),
                    ("home-city", "home-city"),
                    ("home-city-outline", "home-city-outline"),
                    ("home-clock", "home-clock"),
                    ("home-clock-outline", "home-clock-outline"),
                    ("home-currency-usd", "home-currency-usd"),
                    ("home-edit", "home-edit"),
                    ("home-edit-outline", "home-edit-outline"),
                    ("home-export-outline", "home-export-outline"),
                    ("home-flood", "home-flood"),
                    ("home-floor-0", "home-floor-0"),
                    ("home-floor-1", "home-floor-1"),
                    ("home-floor-2", "home-floor-2"),
                    ("home-floor-3", "home-floor-3"),
                    ("home-floor-a", "home-floor-a"),
                    ("home-floor-b", "home-floor-b"),
                    ("home-floor-g", "home-floor-g"),
                    ("home-floor-l", "home-floor-l"),
                    ("home-floor-negative-1", "home-floor-negative-1"),
                    ("home-group", "home-group"),
                    ("home-group-minus", "home-group-minus"),
                    ("home-group-plus", "home-group-plus"),
                    ("home-group-remove", "home-group-remove"),
                    ("home-heart", "home-heart"),
                    ("home-import-outline", "home-import-outline"),
                    ("home-lightbulb", "home-lightbulb"),
                    ("home-lightbulb-outline", "home-lightbulb-outline"),
                    ("home-lightning-bolt", "home-lightning-bolt"),
                    ("home-lightning-bolt-outline", "home-lightning-bolt-outline"),
                    ("home-lock", "home-lock"),
                    ("home-lock-open", "home-lock-open"),
                    ("home-map-marker", "home-map-marker"),
                    ("home-minus", "home-minus"),
                    ("home-minus-outline", "home-minus-outline"),
                    ("home-modern", "home-modern"),
                    ("home-off", "home-off"),
                    ("home-off-outline", "home-off-outline"),
                    ("home-outline", "home-outline"),
                    ("home-percent", "home-percent"),
                    ("home-percent-outline", "home-percent-outline"),
                    ("home-plus", "home-plus"),
                    ("home-plus-outline", "home-plus-outline"),
                    ("home-remove", "home-remove"),
                    ("home-remove-outline", "home-remove-outline"),
                    ("home-roof", "home-roof"),
                    ("home-search", "home-search"),
                    ("home-search-outline", "home-search-outline"),
                    ("home-silo", "home-silo"),
                    ("home-silo-outline", "home-silo-outline"),
                    ("home-sound-in", "home-sound-in"),
                    ("home-sound-in-outline", "home-sound-in-outline"),
                    ("home-sound-out", "home-sound-out"),
                    ("home-sound-out-outline", "home-sound-out-outline"),
                    ("home-switch", "home-switch"),
                    ("home-switch-outline", "home-switch-outline"),
                    ("home-thermometer", "home-thermometer"),
                    ("home-thermometer-outline", "home-thermometer-outline"),
                    ("home-variant", "home-variant"),
                    ("home-variant-outline", "home-variant-outline"),
                    ("hook", "hook"),
                    ("hook-off", "hook-off"),
                    ("hoop-house", "hoop-house"),
                    ("hops", "hops"),
                    ("horizontal-rotate-clockwise", "horizontal-rotate-clockwise"),
                    ("horizontal-rotate-counterclockwise", "horizontal-rotate-counterclockwise"),
                    ("horse", "horse"),
                    ("horse-human", "horse-human"),
                    ("horse-variant", "horse-variant"),
                    ("horse-variant-fast", "horse-variant-fast"),
                    ("horseshoe", "horseshoe"),
                    ("hospital", "hospital"),
                    ("hospital-box", "hospital-box"),
                    ("hospital-box-outline", "hospital-box-outline"),
                    ("hospital-building", "hospital-building"),
                    ("hospital-marker", "hospital-marker"),
                    ("hot-tub", "hot-tub"),
                    ("hours-12", "hours-12"),
                    ("hours-24", "hours-24"),
                    ("houzz", "houzz"),
                    ("houzz-box", "houzz-box"),
                    ("hub", "hub"),
                    ("hub-outline", "hub-outline"),
                    ("hubspot", "hubspot"),
                    ("hulu", "hulu"),
                    ("human", "human"),
                    ("human-baby-changing-table", "human-baby-changing-table"),
                    ("human-cane", "human-cane"),
                    ("human-capacity-decrease", "human-capacity-decrease"),
                    ("human-capacity-increase", "human-capacity-increase"),
                    ("human-child", "human-child"),
                    ("human-dolly", "human-dolly"),
                    ("human-edit", "human-edit"),
                    ("human-female", "human-female"),
                    ("human-female-boy", "human-female-boy"),
                    ("human-female-dance", "human-female-dance"),
                    ("human-female-female", "human-female-female"),
                    ("human-female-female-child", "human-female-female-child"),
                    ("human-female-girl", "human-female-girl"),
                    ("human-greeting", "human-greeting"),
                    ("human-greeting-proximity", "human-greeting-proximity"),
                    ("human-greeting-variant", "human-greeting-variant"),
                    ("human-handsdown", "human-handsdown"),
                    ("human-handsup", "human-handsup"),
                    ("human-male", "human-male"),
                    ("human-male-board", "human-male-board"),
                    ("human-male-board-poll", "human-male-board-poll"),
                    ("human-male-boy", "human-male-boy"),
                    ("human-male-child", "human-male-child"),
                    ("human-male-female", "human-male-female"),
                    ("human-male-female-child", "human-male-female-child"),
                    ("human-male-girl", "human-male-girl"),
                    ("human-male-height", "human-male-height"),
                    ("human-male-height-variant", "human-male-height-variant"),
                    ("human-male-male", "human-male-male"),
                    ("human-male-male-child", "human-male-male-child"),
                    ("human-non-binary", "human-non-binary"),
                    ("human-pregnant", "human-pregnant"),
                    ("human-queue", "human-queue"),
                    ("human-scooter", "human-scooter"),
                    ("human-walker", "human-walker"),
                    ("human-wheelchair", "human-wheelchair"),
                    ("human-white-cane", "human-white-cane"),
                    ("humble-bundle", "humble-bundle"),
                    ("hurricane", "hurricane"),
                    ("hvac", "hvac"),
                    ("hvac-off", "hvac-off"),
                    ("hydraulic-oil-level", "hydraulic-oil-level"),
                    ("hydraulic-oil-temperature", "hydraulic-oil-temperature"),
                    ("hydro-power", "hydro-power"),
                    ("hydrogen-station", "hydrogen-station"),
                    ("ice-cream", "ice-cream"),
                    ("ice-cream-off", "ice-cream-off"),
                    ("ice-pop", "ice-pop"),
                    ("id-card", "id-card"),
                    ("identifier", "identifier"),
                    ("ideogram-cjk", "ideogram-cjk"),
                    ("ideogram-cjk-variant", "ideogram-cjk-variant"),
                    ("image", "image"),
                    ("image-album", "image-album"),
                    ("image-area", "image-area"),
                    ("image-area-close", "image-area-close"),
                    ("image-auto-adjust", "image-auto-adjust"),
                    ("image-broken", "image-broken"),
                    ("image-broken-variant", "image-broken-variant"),
                    ("image-check", "image-check"),
                    ("image-check-outline", "image-check-outline"),
                    ("image-edit", "image-edit"),
                    ("image-edit-outline", "image-edit-outline"),
                    ("image-filter-black-white", "image-filter-black-white"),
                    ("image-filter-center-focus", "image-filter-center-focus"),
                    ("image-filter-center-focus-strong", "image-filter-center-focus-strong"),
                    (
                        "image-filter-center-focus-strong-outline",
                        "image-filter-center-focus-strong-outline",
                    ),
                    ("image-filter-center-focus-weak", "image-filter-center-focus-weak"),
                    ("image-filter-drama", "image-filter-drama"),
                    ("image-filter-drama-outline", "image-filter-drama-outline"),
                    ("image-filter-frames", "image-filter-frames"),
                    ("image-filter-hdr", "image-filter-hdr"),
                    ("image-filter-hdr-outline", "image-filter-hdr-outline"),
                    ("image-filter-none", "image-filter-none"),
                    ("image-filter-tilt-shift", "image-filter-tilt-shift"),
                    ("image-filter-vintage", "image-filter-vintage"),
                    ("image-frame", "image-frame"),
                    ("image-lock", "image-lock"),
                    ("image-lock-outline", "image-lock-outline"),
                    ("image-marker", "image-marker"),
                    ("image-marker-outline", "image-marker-outline"),
                    ("image-minus", "image-minus"),
                    ("image-minus-outline", "image-minus-outline"),
                    ("image-move", "image-move"),
                    ("image-multiple", "image-multiple"),
                    ("image-multiple-outline", "image-multiple-outline"),
                    ("image-off", "image-off"),
                    ("image-off-outline", "image-off-outline"),
                    ("image-outline", "image-outline"),
                    ("image-plus", "image-plus"),
                    ("image-plus-outline", "image-plus-outline"),
                    ("image-refresh", "image-refresh"),
                    ("image-refresh-outline", "image-refresh-outline"),
                    ("image-remove", "image-remove"),
                    ("image-remove-outline", "image-remove-outline"),
                    ("image-search", "image-search"),
                    ("image-search-outline", "image-search-outline"),
                    ("image-size-select-actual", "image-size-select-actual"),
                    ("image-size-select-large", "image-size-select-large"),
                    ("image-size-select-small", "image-size-select-small"),
                    ("image-sync", "image-sync"),
                    ("image-sync-outline", "image-sync-outline"),
                    ("image-text", "image-text"),
                    ("import", "import"),
                    ("inbox", "inbox"),
                    ("inbox-arrow-down", "inbox-arrow-down"),
                    ("inbox-arrow-down-outline", "inbox-arrow-down-outline"),
                    ("inbox-arrow-up", "inbox-arrow-up"),
                    ("inbox-arrow-up-outline", "inbox-arrow-up-outline"),
                    ("inbox-full", "inbox-full"),
                    ("inbox-full-outline", "inbox-full-outline"),
                    ("inbox-multiple", "inbox-multiple"),
                    ("inbox-multiple-outline", "inbox-multiple-outline"),
                    ("inbox-outline", "inbox-outline"),
                    ("inbox-remove", "inbox-remove"),
                    ("inbox-remove-outline", "inbox-remove-outline"),
                    ("incognito", "incognito"),
                    ("incognito-circle", "incognito-circle"),
                    ("incognito-circle-off", "incognito-circle-off"),
                    ("incognito-off", "incognito-off"),
                    ("indent", "indent"),
                    ("induction", "induction"),
                    ("infinity", "infinity"),
                    ("information", "information"),
                    ("information-box", "information-box"),
                    ("information-box-outline", "information-box-outline"),
                    ("information-off", "information-off"),
                    ("information-off-outline", "information-off-outline"),
                    ("information-outline", "information-outline"),
                    ("information-slab-box", "information-slab-box"),
                    ("information-slab-box-outline", "information-slab-box-outline"),
                    ("information-slab-circle", "information-slab-circle"),
                    ("information-slab-circle-outline", "information-slab-circle-outline"),
                    ("information-slab-symbol", "information-slab-symbol"),
                    ("information-symbol", "information-symbol"),
                    ("information-variant", "information-variant"),
                    ("information-variant-box", "information-variant-box"),
                    ("information-variant-box-outline", "information-variant-box-outline"),
                    ("information-variant-circle", "information-variant-circle"),
                    ("information-variant-circle-outline", "information-variant-circle-outline"),
                    ("instagram", "instagram"),
                    ("instapaper", "instapaper"),
                    ("instrument-triangle", "instrument-triangle"),
                    ("integrated-circuit-chip", "integrated-circuit-chip"),
                    ("invert-colors", "invert-colors"),
                    ("invert-colors-off", "invert-colors-off"),
                    ("iobroker", "iobroker"),
                    ("ip", "ip"),
                    ("ip-network", "ip-network"),
                    ("ip-network-outline", "ip-network-outline"),
                    ("ip-outline", "ip-outline"),
                    ("ipod", "ipod"),
                    ("iron", "iron"),
                    ("iron-board", "iron-board"),
                    ("iron-outline", "iron-outline"),
                    ("island", "island"),
                    ("itunes", "itunes"),
                    ("iv-bag", "iv-bag"),
                    ("jabber", "jabber"),
                    ("jeepney", "jeepney"),
                    ("jellyfish", "jellyfish"),
                    ("jellyfish-outline", "jellyfish-outline"),
                    ("jira", "jira"),
                    ("jquery", "jquery"),
                    ("jsfiddle", "jsfiddle"),
                    ("jump-rope", "jump-rope"),
                    ("kabaddi", "kabaddi"),
                    ("kangaroo", "kangaroo"),
                    ("karate", "karate"),
                    ("kayaking", "kayaking"),
                    ("keg", "keg"),
                    ("kettle", "kettle"),
                    ("kettle-alert", "kettle-alert"),
                    ("kettle-alert-outline", "kettle-alert-outline"),
                    ("kettle-off", "kettle-off"),
                    ("kettle-off-outline", "kettle-off-outline"),
                    ("kettle-outline", "kettle-outline"),
                    ("kettle-pour-over", "kettle-pour-over"),
                    ("kettle-steam", "kettle-steam"),
                    ("kettle-steam-outline", "kettle-steam-outline"),
                    ("kettlebell", "kettlebell"),
                    ("key", "key"),
                    ("key-alert", "key-alert"),
                    ("key-alert-outline", "key-alert-outline"),
                    ("key-arrow-right", "key-arrow-right"),
                    ("key-chain", "key-chain"),
                    ("key-chain-variant", "key-chain-variant"),
                    ("key-change", "key-change"),
                    ("key-link", "key-link"),
                    ("key-minus", "key-minus"),
                    ("key-outline", "key-outline"),
                    ("key-plus", "key-plus"),
                    ("key-remove", "key-remove"),
                    ("key-star", "key-star"),
                    ("key-variant", "key-variant"),
                    ("key-wireless", "key-wireless"),
                    ("keyboard", "keyboard"),
                    ("keyboard-backspace", "keyboard-backspace"),
                    ("keyboard-caps", "keyboard-caps"),
                    ("keyboard-close", "keyboard-close"),
                    ("keyboard-close-outline", "keyboard-close-outline"),
                    ("keyboard-esc", "keyboard-esc"),
                    ("keyboard-f1", "keyboard-f1"),
                    ("keyboard-f10", "keyboard-f10"),
                    ("keyboard-f11", "keyboard-f11"),
                    ("keyboard-f12", "keyboard-f12"),
                    ("keyboard-f2", "keyboard-f2"),
                    ("keyboard-f3", "keyboard-f3"),
                    ("keyboard-f4", "keyboard-f4"),
                    ("keyboard-f5", "keyboard-f5"),
                    ("keyboard-f6", "keyboard-f6"),
                    ("keyboard-f7", "keyboard-f7"),
                    ("keyboard-f8", "keyboard-f8"),
                    ("keyboard-f9", "keyboard-f9"),
                    ("keyboard-off", "keyboard-off"),
                    ("keyboard-off-outline", "keyboard-off-outline"),
                    ("keyboard-outline", "keyboard-outline"),
                    ("keyboard-return", "keyboard-return"),
                    ("keyboard-settings", "keyboard-settings"),
                    ("keyboard-settings-outline", "keyboard-settings-outline"),
                    ("keyboard-space", "keyboard-space"),
                    ("keyboard-tab", "keyboard-tab"),
                    ("keyboard-tab-reverse", "keyboard-tab-reverse"),
                    ("keyboard-variant", "keyboard-variant"),
                    ("khanda", "khanda"),
                    ("kickstarter", "kickstarter"),
                    ("kite", "kite"),
                    ("kite-outline", "kite-outline"),
                    ("kitesurfing", "kitesurfing"),
                    ("klingon", "klingon"),
                    ("knife", "knife"),
                    ("knife-military", "knife-military"),
                    ("knob", "knob"),
                    ("koala", "koala"),
                    ("kodi", "kodi"),
                    ("kubernetes", "kubernetes"),
                    ("label", "label"),
                    ("label-multiple", "label-multiple"),
                    ("label-multiple-outline", "label-multiple-outline"),
                    ("label-off", "label-off"),
                    ("label-off-outline", "label-off-outline"),
                    ("label-outline", "label-outline"),
                    ("label-percent", "label-percent"),
                    ("label-percent-outline", "label-percent-outline"),
                    ("label-variant", "label-variant"),
                    ("label-variant-outline", "label-variant-outline"),
                    ("ladder", "ladder"),
                    ("ladybug", "ladybug"),
                    ("lambda", "lambda"),
                    ("lamp", "lamp"),
                    ("lamp-outline", "lamp-outline"),
                    ("lamps", "lamps"),
                    ("lamps-outline", "lamps-outline"),
                    ("lan", "lan"),
                    ("lan-check", "lan-check"),
                    ("lan-connect", "lan-connect"),
                    ("lan-disconnect", "lan-disconnect"),
                    ("lan-pending", "lan-pending"),
                    ("land-fields", "land-fields"),
                    ("land-plots", "land-plots"),
                    ("land-plots-circle", "land-plots-circle"),
                    ("land-plots-circle-variant", "land-plots-circle-variant"),
                    ("land-plots-marker", "land-plots-marker"),
                    ("land-rows-horizontal", "land-rows-horizontal"),
                    ("land-rows-vertical", "land-rows-vertical"),
                    ("landslide", "landslide"),
                    ("landslide-outline", "landslide-outline"),
                    ("language-c", "language-c"),
                    ("language-cpp", "language-cpp"),
                    ("language-csharp", "language-csharp"),
                    ("language-css3", "language-css3"),
                    ("language-fortran", "language-fortran"),
                    ("language-go", "language-go"),
                    ("language-haskell", "language-haskell"),
                    ("language-html5", "language-html5"),
                    ("language-java", "language-java"),
                    ("language-javascript", "language-javascript"),
                    ("language-jsx", "language-jsx"),
                    ("language-kotlin", "language-kotlin"),
                    ("language-lua", "language-lua"),
                    ("language-markdown", "language-markdown"),
                    ("language-markdown-outline", "language-markdown-outline"),
                    ("language-php", "language-php"),
                    ("language-python", "language-python"),
                    ("language-python-text", "language-python-text"),
                    ("language-r", "language-r"),
                    ("language-ruby", "language-ruby"),
                    ("language-ruby-on-rails", "language-ruby-on-rails"),
                    ("language-rust", "language-rust"),
                    ("language-swift", "language-swift"),
                    ("language-typescript", "language-typescript"),
                    ("language-xaml", "language-xaml"),
                    ("laptop", "laptop"),
                    ("laptop-account", "laptop-account"),
                    ("laptop-chromebook", "laptop-chromebook"),
                    ("laptop-mac", "laptop-mac"),
                    ("laptop-off", "laptop-off"),
                    ("laptop-windows", "laptop-windows"),
                    ("laravel", "laravel"),
                    ("laser-pointer", "laser-pointer"),
                    ("lasso", "lasso"),
                    ("lastfm", "lastfm"),
                    ("lastpass", "lastpass"),
                    ("latitude", "latitude"),
                    ("launch", "launch"),
                    ("lava-lamp", "lava-lamp"),
                    ("layers", "layers"),
                    ("layers-edit", "layers-edit"),
                    ("layers-minus", "layers-minus"),
                    ("layers-off", "layers-off"),
                    ("layers-off-outline", "layers-off-outline"),
                    ("layers-outline", "layers-outline"),
                    ("layers-plus", "layers-plus"),
                    ("layers-remove", "layers-remove"),
                    ("layers-search", "layers-search"),
                    ("layers-search-outline", "layers-search-outline"),
                    ("layers-triple", "layers-triple"),
                    ("layers-triple-outline", "layers-triple-outline"),
                    ("lead-pencil", "lead-pencil"),
                    ("leaf", "leaf"),
                    ("leaf-circle", "leaf-circle"),
                    ("leaf-circle-outline", "leaf-circle-outline"),
                    ("leaf-maple", "leaf-maple"),
                    ("leaf-maple-off", "leaf-maple-off"),
                    ("leaf-off", "leaf-off"),
                    ("leak", "leak"),
                    ("leak-off", "leak-off"),
                    ("lectern", "lectern"),
                    ("led-off", "led-off"),
                    ("led-on", "led-on"),
                    ("led-outline", "led-outline"),
                    ("led-strip", "led-strip"),
                    ("led-strip-variant", "led-strip-variant"),
                    ("led-strip-variant-off", "led-strip-variant-off"),
                    ("led-variant-off", "led-variant-off"),
                    ("led-variant-on", "led-variant-on"),
                    ("led-variant-outline", "led-variant-outline"),
                    ("leek", "leek"),
                    ("less-than", "less-than"),
                    ("less-than-or-equal", "less-than-or-equal"),
                    ("library", "library"),
                    ("library-books", "library-books"),
                    ("library-outline", "library-outline"),
                    ("library-shelves", "library-shelves"),
                    ("license", "license"),
                    ("lifebuoy", "lifebuoy"),
                    ("light-flood-down", "light-flood-down"),
                    ("light-flood-up", "light-flood-up"),
                    ("light-recessed", "light-recessed"),
                    ("light-switch", "light-switch"),
                    ("light-switch-off", "light-switch-off"),
                    ("lightbulb", "lightbulb"),
                    ("lightbulb-alert", "lightbulb-alert"),
                    ("lightbulb-alert-outline", "lightbulb-alert-outline"),
                    ("lightbulb-auto", "lightbulb-auto"),
                    ("lightbulb-auto-outline", "lightbulb-auto-outline"),
                    ("lightbulb-cfl", "lightbulb-cfl"),
                    ("lightbulb-cfl-off", "lightbulb-cfl-off"),
                    ("lightbulb-cfl-spiral", "lightbulb-cfl-spiral"),
                    ("lightbulb-cfl-spiral-off", "lightbulb-cfl-spiral-off"),
                    ("lightbulb-fluorescent-tube", "lightbulb-fluorescent-tube"),
                    ("lightbulb-fluorescent-tube-outline", "lightbulb-fluorescent-tube-outline"),
                    ("lightbulb-group", "lightbulb-group"),
                    ("lightbulb-group-off", "lightbulb-group-off"),
                    ("lightbulb-group-off-outline", "lightbulb-group-off-outline"),
                    ("lightbulb-group-outline", "lightbulb-group-outline"),
                    ("lightbulb-multiple", "lightbulb-multiple"),
                    ("lightbulb-multiple-off", "lightbulb-multiple-off"),
                    ("lightbulb-multiple-off-outline", "lightbulb-multiple-off-outline"),
                    ("lightbulb-multiple-outline", "lightbulb-multiple-outline"),
                    ("lightbulb-night", "lightbulb-night"),
                    ("lightbulb-night-outline", "lightbulb-night-outline"),
                    ("lightbulb-off", "lightbulb-off"),
                    ("lightbulb-off-outline", "lightbulb-off-outline"),
                    ("lightbulb-on", "lightbulb-on"),
                    ("lightbulb-on-10", "lightbulb-on-10"),
                    ("lightbulb-on-20", "lightbulb-on-20"),
                    ("lightbulb-on-30", "lightbulb-on-30"),
                    ("lightbulb-on-40", "lightbulb-on-40"),
                    ("lightbulb-on-50", "lightbulb-on-50"),
                    ("lightbulb-on-60", "lightbulb-on-60"),
                    ("lightbulb-on-70", "lightbulb-on-70"),
                    ("lightbulb-on-80", "lightbulb-on-80"),
                    ("lightbulb-on-90", "lightbulb-on-90"),
                    ("lightbulb-on-outline", "lightbulb-on-outline"),
                    ("lightbulb-outline", "lightbulb-outline"),
                    ("lightbulb-question", "lightbulb-question"),
                    ("lightbulb-question-outline", "lightbulb-question-outline"),
                    ("lightbulb-spot", "lightbulb-spot"),
                    ("lightbulb-spot-off", "lightbulb-spot-off"),
                    ("lightbulb-variant", "lightbulb-variant"),
                    ("lightbulb-variant-outline", "lightbulb-variant-outline"),
                    ("lighthouse", "lighthouse"),
                    ("lighthouse-on", "lighthouse-on"),
                    ("lightning-bolt", "lightning-bolt"),
                    ("lightning-bolt-circle", "lightning-bolt-circle"),
                    ("lightning-bolt-outline", "lightning-bolt-outline"),
                    ("line-scan", "line-scan"),
                    ("lingerie", "lingerie"),
                    ("link", "link"),
                    ("link-box", "link-box"),
                    ("link-box-outline", "link-box-outline"),
                    ("link-box-variant", "link-box-variant"),
                    ("link-box-variant-outline", "link-box-variant-outline"),
                    ("link-lock", "link-lock"),
                    ("link-off", "link-off"),
                    ("link-plus", "link-plus"),
                    ("link-variant", "link-variant"),
                    ("link-variant-minus", "link-variant-minus"),
                    ("link-variant-off", "link-variant-off"),
                    ("link-variant-plus", "link-variant-plus"),
                    ("link-variant-remove", "link-variant-remove"),
                    ("linkedin", "linkedin"),
                    ("linode", "linode"),
                    ("linux", "linux"),
                    ("linux-mint", "linux-mint"),
                    ("lipstick", "lipstick"),
                    ("liquid-spot", "liquid-spot"),
                    ("liquor", "liquor"),
                    ("list-box", "list-box"),
                    ("list-box-outline", "list-box-outline"),
                    ("list-status", "list-status"),
                    ("litecoin", "litecoin"),
                    ("loading", "loading"),
                    ("location-enter", "location-enter"),
                    ("location-exit", "location-exit"),
                    ("lock", "lock"),
                    ("lock-alert", "lock-alert"),
                    ("lock-alert-outline", "lock-alert-outline"),
                    ("lock-check", "lock-check"),
                    ("lock-check-outline", "lock-check-outline"),
                    ("lock-clock", "lock-clock"),
                    ("lock-minus", "lock-minus"),
                    ("lock-minus-outline", "lock-minus-outline"),
                    ("lock-off", "lock-off"),
                    ("lock-off-outline", "lock-off-outline"),
                    ("lock-open", "lock-open"),
                    ("lock-open-alert", "lock-open-alert"),
                    ("lock-open-alert-outline", "lock-open-alert-outline"),
                    ("lock-open-check", "lock-open-check"),
                    ("lock-open-check-outline", "lock-open-check-outline"),
                    ("lock-open-minus", "lock-open-minus"),
                    ("lock-open-minus-outline", "lock-open-minus-outline"),
                    ("lock-open-outline", "lock-open-outline"),
                    ("lock-open-plus", "lock-open-plus"),
                    ("lock-open-plus-outline", "lock-open-plus-outline"),
                    ("lock-open-remove", "lock-open-remove"),
                    ("lock-open-remove-outline", "lock-open-remove-outline"),
                    ("lock-open-variant", "lock-open-variant"),
                    ("lock-open-variant-outline", "lock-open-variant-outline"),
                    ("lock-outline", "lock-outline"),
                    ("lock-pattern", "lock-pattern"),
                    ("lock-percent", "lock-percent"),
                    ("lock-percent-open", "lock-percent-open"),
                    ("lock-percent-open-outline", "lock-percent-open-outline"),
                    ("lock-percent-open-variant", "lock-percent-open-variant"),
                    ("lock-percent-open-variant-outline", "lock-percent-open-variant-outline"),
                    ("lock-percent-outline", "lock-percent-outline"),
                    ("lock-plus", "lock-plus"),
                    ("lock-plus-outline", "lock-plus-outline"),
                    ("lock-question", "lock-question"),
                    ("lock-remove", "lock-remove"),
                    ("lock-remove-outline", "lock-remove-outline"),
                    ("lock-reset", "lock-reset"),
                    ("lock-smart", "lock-smart"),
                    ("locker", "locker"),
                    ("locker-multiple", "locker-multiple"),
                    ("login", "login"),
                    ("login-variant", "login-variant"),
                    ("logout", "logout"),
                    ("logout-variant", "logout-variant"),
                    ("longitude", "longitude"),
                    ("looks", "looks"),
                    ("lotion", "lotion"),
                    ("lotion-outline", "lotion-outline"),
                    ("lotion-plus", "lotion-plus"),
                    ("lotion-plus-outline", "lotion-plus-outline"),
                    ("loupe", "loupe"),
                    ("lumx", "lumx"),
                    ("lungs", "lungs"),
                    ("lyft", "lyft"),
                    ("mace", "mace"),
                    ("magazine-pistol", "magazine-pistol"),
                    ("magazine-rifle", "magazine-rifle"),
                    ("magic-staff", "magic-staff"),
                    ("magnet", "magnet"),
                    ("magnet-on", "magnet-on"),
                    ("magnify", "magnify"),
                    ("magnify-close", "magnify-close"),
                    ("magnify-expand", "magnify-expand"),
                    ("magnify-minus", "magnify-minus"),
                    ("magnify-minus-cursor", "magnify-minus-cursor"),
                    ("magnify-minus-outline", "magnify-minus-outline"),
                    ("magnify-plus", "magnify-plus"),
                    ("magnify-plus-cursor", "magnify-plus-cursor"),
                    ("magnify-plus-outline", "magnify-plus-outline"),
                    ("magnify-remove-cursor", "magnify-remove-cursor"),
                    ("magnify-remove-outline", "magnify-remove-outline"),
                    ("magnify-scan", "magnify-scan"),
                    ("mail", "mail"),
                    ("mail-ru", "mail-ru"),
                    ("mailbox", "mailbox"),
                    ("mailbox-open", "mailbox-open"),
                    ("mailbox-open-outline", "mailbox-open-outline"),
                    ("mailbox-open-up", "mailbox-open-up"),
                    ("mailbox-open-up-outline", "mailbox-open-up-outline"),
                    ("mailbox-outline", "mailbox-outline"),
                    ("mailbox-up", "mailbox-up"),
                    ("mailbox-up-outline", "mailbox-up-outline"),
                    ("manjaro", "manjaro"),
                    ("map", "map"),
                    ("map-check", "map-check"),
                    ("map-check-outline", "map-check-outline"),
                    ("map-clock", "map-clock"),
                    ("map-clock-outline", "map-clock-outline"),
                    ("map-legend", "map-legend"),
                    ("map-marker", "map-marker"),
                    ("map-marker-account", "map-marker-account"),
                    ("map-marker-account-outline", "map-marker-account-outline"),
                    ("map-marker-alert", "map-marker-alert"),
                    ("map-marker-alert-outline", "map-marker-alert-outline"),
                    ("map-marker-check", "map-marker-check"),
                    ("map-marker-check-outline", "map-marker-check-outline"),
                    ("map-marker-circle", "map-marker-circle"),
                    ("map-marker-distance", "map-marker-distance"),
                    ("map-marker-down", "map-marker-down"),
                    ("map-marker-left", "map-marker-left"),
                    ("map-marker-left-outline", "map-marker-left-outline"),
                    ("map-marker-minus", "map-marker-minus"),
                    ("map-marker-minus-outline", "map-marker-minus-outline"),
                    ("map-marker-multiple", "map-marker-multiple"),
                    ("map-marker-multiple-outline", "map-marker-multiple-outline"),
                    ("map-marker-off", "map-marker-off"),
                    ("map-marker-off-outline", "map-marker-off-outline"),
                    ("map-marker-outline", "map-marker-outline"),
                    ("map-marker-path", "map-marker-path"),
                    ("map-marker-plus", "map-marker-plus"),
                    ("map-marker-plus-outline", "map-marker-plus-outline"),
                    ("map-marker-question", "map-marker-question"),
                    ("map-marker-question-outline", "map-marker-question-outline"),
                    ("map-marker-radius", "map-marker-radius"),
                    ("map-marker-radius-outline", "map-marker-radius-outline"),
                    ("map-marker-remove", "map-marker-remove"),
                    ("map-marker-remove-outline", "map-marker-remove-outline"),
                    ("map-marker-remove-variant", "map-marker-remove-variant"),
                    ("map-marker-right", "map-marker-right"),
                    ("map-marker-right-outline", "map-marker-right-outline"),
                    ("map-marker-star", "map-marker-star"),
                    ("map-marker-star-outline", "map-marker-star-outline"),
                    ("map-marker-up", "map-marker-up"),
                    ("map-minus", "map-minus"),
                    ("map-outline", "map-outline"),
                    ("map-plus", "map-plus"),
                    ("map-search", "map-search"),
                    ("map-search-outline", "map-search-outline"),
                    ("mapbox", "mapbox"),
                    ("margin", "margin"),
                    ("marker", "marker"),
                    ("marker-cancel", "marker-cancel"),
                    ("marker-check", "marker-check"),
                    ("mastodon", "mastodon"),
                    ("mastodon-variant", "mastodon-variant"),
                    ("material-design", "material-design"),
                    ("material-ui", "material-ui"),
                    ("math-compass", "math-compass"),
                    ("math-cos", "math-cos"),
                    ("math-integral", "math-integral"),
                    ("math-integral-box", "math-integral-box"),
                    ("math-log", "math-log"),
                    ("math-norm", "math-norm"),
                    ("math-norm-box", "math-norm-box"),
                    ("math-sin", "math-sin"),
                    ("math-tan", "math-tan"),
                    ("matrix", "matrix"),
                    ("maxcdn", "maxcdn"),
                    ("medal", "medal"),
                    ("medal-outline", "medal-outline"),
                    ("medical-bag", "medical-bag"),
                    ("medical-cotton-swab", "medical-cotton-swab"),
                    ("medication", "medication"),
                    ("medication-outline", "medication-outline"),
                    ("meditation", "meditation"),
                    ("medium", "medium"),
                    ("meetup", "meetup"),
                    ("memory", "memory"),
                    ("memory-arrow-down", "memory-arrow-down"),
                    ("menorah", "menorah"),
                    ("menorah-fire", "menorah-fire"),
                    ("menu", "menu"),
                    ("menu-close", "menu-close"),
                    ("menu-down", "menu-down"),
                    ("menu-down-outline", "menu-down-outline"),
                    ("menu-left", "menu-left"),
                    ("menu-left-outline", "menu-left-outline"),
                    ("menu-open", "menu-open"),
                    ("menu-right", "menu-right"),
                    ("menu-right-outline", "menu-right-outline"),
                    ("menu-swap", "menu-swap"),
                    ("menu-swap-outline", "menu-swap-outline"),
                    ("menu-up", "menu-up"),
                    ("menu-up-outline", "menu-up-outline"),
                    ("merge", "merge"),
                    ("message", "message"),
                    ("message-alert", "message-alert"),
                    ("message-alert-outline", "message-alert-outline"),
                    ("message-arrow-left", "message-arrow-left"),
                    ("message-arrow-left-outline", "message-arrow-left-outline"),
                    ("message-arrow-right", "message-arrow-right"),
                    ("message-arrow-right-outline", "message-arrow-right-outline"),
                    ("message-badge", "message-badge"),
                    ("message-badge-outline", "message-badge-outline"),
                    ("message-bookmark", "message-bookmark"),
                    ("message-bookmark-outline", "message-bookmark-outline"),
                    ("message-bulleted", "message-bulleted"),
                    ("message-bulleted-off", "message-bulleted-off"),
                    ("message-check", "message-check"),
                    ("message-check-outline", "message-check-outline"),
                    ("message-cog", "message-cog"),
                    ("message-cog-outline", "message-cog-outline"),
                    ("message-draw", "message-draw"),
                    ("message-fast", "message-fast"),
                    ("message-fast-outline", "message-fast-outline"),
                    ("message-flash", "message-flash"),
                    ("message-flash-outline", "message-flash-outline"),
                    ("message-image", "message-image"),
                    ("message-image-outline", "message-image-outline"),
                    ("message-lock", "message-lock"),
                    ("message-lock-outline", "message-lock-outline"),
                    ("message-minus", "message-minus"),
                    ("message-minus-outline", "message-minus-outline"),
                    ("message-off", "message-off"),
                    ("message-off-outline", "message-off-outline"),
                    ("message-outline", "message-outline"),
                    ("message-plus", "message-plus"),
                    ("message-plus-outline", "message-plus-outline"),
                    ("message-processing", "message-processing"),
                    ("message-processing-outline", "message-processing-outline"),
                    ("message-question", "message-question"),
                    ("message-question-outline", "message-question-outline"),
                    ("message-reply", "message-reply"),
                    ("message-reply-outline", "message-reply-outline"),
                    ("message-reply-text", "message-reply-text"),
                    ("message-reply-text-outline", "message-reply-text-outline"),
                    ("message-settings", "message-settings"),
                    ("message-settings-outline", "message-settings-outline"),
                    ("message-star", "message-star"),
                    ("message-star-outline", "message-star-outline"),
                    ("message-text", "message-text"),
                    ("message-text-clock", "message-text-clock"),
                    ("message-text-clock-outline", "message-text-clock-outline"),
                    ("message-text-fast", "message-text-fast"),
                    ("message-text-fast-outline", "message-text-fast-outline"),
                    ("message-text-lock", "message-text-lock"),
                    ("message-text-lock-outline", "message-text-lock-outline"),
                    ("message-text-outline", "message-text-outline"),
                    ("message-video", "message-video"),
                    ("meteor", "meteor"),
                    ("meter-electric", "meter-electric"),
                    ("meter-electric-outline", "meter-electric-outline"),
                    ("meter-gas", "meter-gas"),
                    ("meter-gas-outline", "meter-gas-outline"),
                    ("metronome", "metronome"),
                    ("metronome-tick", "metronome-tick"),
                    ("micro-sd", "micro-sd"),
                    ("microphone", "microphone"),
                    ("microphone-message", "microphone-message"),
                    ("microphone-message-off", "microphone-message-off"),
                    ("microphone-minus", "microphone-minus"),
                    ("microphone-off", "microphone-off"),
                    ("microphone-outline", "microphone-outline"),
                    ("microphone-plus", "microphone-plus"),
                    ("microphone-question", "microphone-question"),
                    ("microphone-question-outline", "microphone-question-outline"),
                    ("microphone-settings", "microphone-settings"),
                    ("microphone-variant", "microphone-variant"),
                    ("microphone-variant-off", "microphone-variant-off"),
                    ("microscope", "microscope"),
                    ("microsoft", "microsoft"),
                    ("microsoft-access", "microsoft-access"),
                    ("microsoft-azure", "microsoft-azure"),
                    ("microsoft-azure-devops", "microsoft-azure-devops"),
                    ("microsoft-bing", "microsoft-bing"),
                    ("microsoft-dynamics-365", "microsoft-dynamics-365"),
                    ("microsoft-edge", "microsoft-edge"),
                    ("microsoft-edge-legacy", "microsoft-edge-legacy"),
                    ("microsoft-excel", "microsoft-excel"),
                    ("microsoft-internet-explorer", "microsoft-internet-explorer"),
                    ("microsoft-office", "microsoft-office"),
                    ("microsoft-onedrive", "microsoft-onedrive"),
                    ("microsoft-onenote", "microsoft-onenote"),
                    ("microsoft-outlook", "microsoft-outlook"),
                    ("microsoft-powerpoint", "microsoft-powerpoint"),
                    ("microsoft-sharepoint", "microsoft-sharepoint"),
                    ("microsoft-teams", "microsoft-teams"),
                    ("microsoft-visual-studio", "microsoft-visual-studio"),
                    ("microsoft-visual-studio-code", "microsoft-visual-studio-code"),
                    ("microsoft-windows", "microsoft-windows"),
                    ("microsoft-windows-classic", "microsoft-windows-classic"),
                    ("microsoft-word", "microsoft-word"),
                    ("microsoft-xbox", "microsoft-xbox"),
                    ("microsoft-xbox-controller", "microsoft-xbox-controller"),
                    (
                        "microsoft-xbox-controller-battery-alert",
                        "microsoft-xbox-controller-battery-alert",
                    ),
                    (
                        "microsoft-xbox-controller-battery-charging",
                        "microsoft-xbox-controller-battery-charging",
                    ),
                    (
                        "microsoft-xbox-controller-battery-empty",
                        "microsoft-xbox-controller-battery-empty",
                    ),
                    (
                        "microsoft-xbox-controller-battery-full",
                        "microsoft-xbox-controller-battery-full",
                    ),
                    (
                        "microsoft-xbox-controller-battery-low",
                        "microsoft-xbox-controller-battery-low",
                    ),
                    (
                        "microsoft-xbox-controller-battery-medium",
                        "microsoft-xbox-controller-battery-medium",
                    ),
                    (
                        "microsoft-xbox-controller-battery-unknown",
                        "microsoft-xbox-controller-battery-unknown",
                    ),
                    ("microsoft-xbox-controller-menu", "microsoft-xbox-controller-menu"),
                    ("microsoft-xbox-controller-off", "microsoft-xbox-controller-off"),
                    ("microsoft-xbox-controller-view", "microsoft-xbox-controller-view"),
                    ("microsoft-yammer", "microsoft-yammer"),
                    ("microwave", "microwave"),
                    ("microwave-off", "microwave-off"),
                    ("middleware", "middleware"),
                    ("middleware-outline", "middleware-outline"),
                    ("midi", "midi"),
                    ("midi-input", "midi-input"),
                    ("midi-port", "midi-port"),
                    ("mine", "mine"),
                    ("minecraft", "minecraft"),
                    ("mini-sd", "mini-sd"),
                    ("minidisc", "minidisc"),
                    ("minus", "minus"),
                    ("minus-box", "minus-box"),
                    ("minus-box-multiple", "minus-box-multiple"),
                    ("minus-box-multiple-outline", "minus-box-multiple-outline"),
                    ("minus-box-outline", "minus-box-outline"),
                    ("minus-circle", "minus-circle"),
                    ("minus-circle-multiple", "minus-circle-multiple"),
                    ("minus-circle-multiple-outline", "minus-circle-multiple-outline"),
                    ("minus-circle-off", "minus-circle-off"),
                    ("minus-circle-off-outline", "minus-circle-off-outline"),
                    ("minus-circle-outline", "minus-circle-outline"),
                    ("minus-network", "minus-network"),
                    ("minus-network-outline", "minus-network-outline"),
                    ("minus-thick", "minus-thick"),
                    ("mirror", "mirror"),
                    ("mirror-rectangle", "mirror-rectangle"),
                    ("mirror-variant", "mirror-variant"),
                    ("mixcloud", "mixcloud"),
                    ("mixed-martial-arts", "mixed-martial-arts"),
                    ("mixed-reality", "mixed-reality"),
                    ("mixer", "mixer"),
                    ("molecule", "molecule"),
                    ("molecule-co", "molecule-co"),
                    ("molecule-co2", "molecule-co2"),
                    ("monitor", "monitor"),
                    ("monitor-account", "monitor-account"),
                    ("monitor-arrow-down", "monitor-arrow-down"),
                    ("monitor-arrow-down-variant", "monitor-arrow-down-variant"),
                    ("monitor-cellphone", "monitor-cellphone"),
                    ("monitor-cellphone-star", "monitor-cellphone-star"),
                    ("monitor-dashboard", "monitor-dashboard"),
                    ("monitor-edit", "monitor-edit"),
                    ("monitor-eye", "monitor-eye"),
                    ("monitor-lock", "monitor-lock"),
                    ("monitor-multiple", "monitor-multiple"),
                    ("monitor-off", "monitor-off"),
                    ("monitor-screenshot", "monitor-screenshot"),
                    ("monitor-share", "monitor-share"),
                    ("monitor-shimmer", "monitor-shimmer"),
                    ("monitor-small", "monitor-small"),
                    ("monitor-speaker", "monitor-speaker"),
                    ("monitor-speaker-off", "monitor-speaker-off"),
                    ("monitor-star", "monitor-star"),
                    ("monitor-vertical", "monitor-vertical"),
                    ("moon-first-quarter", "moon-first-quarter"),
                    ("moon-full", "moon-full"),
                    ("moon-last-quarter", "moon-last-quarter"),
                    ("moon-new", "moon-new"),
                    ("moon-waning-crescent", "moon-waning-crescent"),
                    ("moon-waning-gibbous", "moon-waning-gibbous"),
                    ("moon-waxing-crescent", "moon-waxing-crescent"),
                    ("moon-waxing-gibbous", "moon-waxing-gibbous"),
                    ("moped", "moped"),
                    ("moped-electric", "moped-electric"),
                    ("moped-electric-outline", "moped-electric-outline"),
                    ("moped-outline", "moped-outline"),
                    ("more", "more"),
                    ("mortar-pestle", "mortar-pestle"),
                    ("mortar-pestle-plus", "mortar-pestle-plus"),
                    ("mosque", "mosque"),
                    ("mosque-outline", "mosque-outline"),
                    ("mother-heart", "mother-heart"),
                    ("mother-nurse", "mother-nurse"),
                    ("motion", "motion"),
                    ("motion-outline", "motion-outline"),
                    ("motion-pause", "motion-pause"),
                    ("motion-pause-outline", "motion-pause-outline"),
                    ("motion-play", "motion-play"),
                    ("motion-play-outline", "motion-play-outline"),
                    ("motion-sensor", "motion-sensor"),
                    ("motion-sensor-off", "motion-sensor-off"),
                    ("motorbike", "motorbike"),
                    ("motorbike-electric", "motorbike-electric"),
                    ("motorbike-off", "motorbike-off"),
                    ("mouse", "mouse"),
                    ("mouse-bluetooth", "mouse-bluetooth"),
                    ("mouse-move-down", "mouse-move-down"),
                    ("mouse-move-up", "mouse-move-up"),
                    ("mouse-move-vertical", "mouse-move-vertical"),
                    ("mouse-off", "mouse-off"),
                    ("mouse-variant", "mouse-variant"),
                    ("mouse-variant-off", "mouse-variant-off"),
                    ("move-resize", "move-resize"),
                    ("move-resize-variant", "move-resize-variant"),
                    ("movie", "movie"),
                    ("movie-check", "movie-check"),
                    ("movie-check-outline", "movie-check-outline"),
                    ("movie-cog", "movie-cog"),
                    ("movie-cog-outline", "movie-cog-outline"),
                    ("movie-edit", "movie-edit"),
                    ("movie-edit-outline", "movie-edit-outline"),
                    ("movie-filter", "movie-filter"),
                    ("movie-filter-outline", "movie-filter-outline"),
                    ("movie-minus", "movie-minus"),
                    ("movie-minus-outline", "movie-minus-outline"),
                    ("movie-off", "movie-off"),
                    ("movie-off-outline", "movie-off-outline"),
                    ("movie-open", "movie-open"),
                    ("movie-open-check", "movie-open-check"),
                    ("movie-open-check-outline", "movie-open-check-outline"),
                    ("movie-open-cog", "movie-open-cog"),
                    ("movie-open-cog-outline", "movie-open-cog-outline"),
                    ("movie-open-edit", "movie-open-edit"),
                    ("movie-open-edit-outline", "movie-open-edit-outline"),
                    ("movie-open-minus", "movie-open-minus"),
                    ("movie-open-minus-outline", "movie-open-minus-outline"),
                    ("movie-open-off", "movie-open-off"),
                    ("movie-open-off-outline", "movie-open-off-outline"),
                    ("movie-open-outline", "movie-open-outline"),
                    ("movie-open-play", "movie-open-play"),
                    ("movie-open-play-outline", "movie-open-play-outline"),
                    ("movie-open-plus", "movie-open-plus"),
                    ("movie-open-plus-outline", "movie-open-plus-outline"),
                    ("movie-open-remove", "movie-open-remove"),
                    ("movie-open-remove-outline", "movie-open-remove-outline"),
                    ("movie-open-settings", "movie-open-settings"),
                    ("movie-open-settings-outline", "movie-open-settings-outline"),
                    ("movie-open-star", "movie-open-star"),
                    ("movie-open-star-outline", "movie-open-star-outline"),
                    ("movie-outline", "movie-outline"),
                    ("movie-play", "movie-play"),
                    ("movie-play-outline", "movie-play-outline"),
                    ("movie-plus", "movie-plus"),
                    ("movie-plus-outline", "movie-plus-outline"),
                    ("movie-remove", "movie-remove"),
                    ("movie-remove-outline", "movie-remove-outline"),
                    ("movie-roll", "movie-roll"),
                    ("movie-search", "movie-search"),
                    ("movie-search-outline", "movie-search-outline"),
                    ("movie-settings", "movie-settings"),
                    ("movie-settings-outline", "movie-settings-outline"),
                    ("movie-star", "movie-star"),
                    ("movie-star-outline", "movie-star-outline"),
                    ("mower", "mower"),
                    ("mower-bag", "mower-bag"),
                    ("mower-bag-on", "mower-bag-on"),
                    ("mower-on", "mower-on"),
                    ("muffin", "muffin"),
                    ("multicast", "multicast"),
                    ("multimedia", "multimedia"),
                    ("multiplication", "multiplication"),
                    ("multiplication-box", "multiplication-box"),
                    ("mushroom", "mushroom"),
                    ("mushroom-off", "mushroom-off"),
                    ("mushroom-off-outline", "mushroom-off-outline"),
                    ("mushroom-outline", "mushroom-outline"),
                    ("music", "music"),
                    ("music-accidental-double-flat", "music-accidental-double-flat"),
                    ("music-accidental-double-sharp", "music-accidental-double-sharp"),
                    ("music-accidental-flat", "music-accidental-flat"),
                    ("music-accidental-natural", "music-accidental-natural"),
                    ("music-accidental-sharp", "music-accidental-sharp"),
                    ("music-box", "music-box"),
                    ("music-box-multiple", "music-box-multiple"),
                    ("music-box-multiple-outline", "music-box-multiple-outline"),
                    ("music-box-outline", "music-box-outline"),
                    ("music-circle", "music-circle"),
                    ("music-circle-outline", "music-circle-outline"),
                    ("music-clef-alto", "music-clef-alto"),
                    ("music-clef-bass", "music-clef-bass"),
                    ("music-clef-treble", "music-clef-treble"),
                    ("music-note", "music-note"),
                    ("music-note-bluetooth", "music-note-bluetooth"),
                    ("music-note-bluetooth-off", "music-note-bluetooth-off"),
                    ("music-note-eighth", "music-note-eighth"),
                    ("music-note-eighth-dotted", "music-note-eighth-dotted"),
                    ("music-note-half", "music-note-half"),
                    ("music-note-half-dotted", "music-note-half-dotted"),
                    ("music-note-minus", "music-note-minus"),
                    ("music-note-off", "music-note-off"),
                    ("music-note-off-outline", "music-note-off-outline"),
                    ("music-note-outline", "music-note-outline"),
                    ("music-note-plus", "music-note-plus"),
                    ("music-note-quarter", "music-note-quarter"),
                    ("music-note-quarter-dotted", "music-note-quarter-dotted"),
                    ("music-note-sixteenth", "music-note-sixteenth"),
                    ("music-note-sixteenth-dotted", "music-note-sixteenth-dotted"),
                    ("music-note-whole", "music-note-whole"),
                    ("music-note-whole-dotted", "music-note-whole-dotted"),
                    ("music-off", "music-off"),
                    ("music-rest-eighth", "music-rest-eighth"),
                    ("music-rest-half", "music-rest-half"),
                    ("music-rest-quarter", "music-rest-quarter"),
                    ("music-rest-sixteenth", "music-rest-sixteenth"),
                    ("music-rest-whole", "music-rest-whole"),
                    ("mustache", "mustache"),
                    ("nail", "nail"),
                    ("nas", "nas"),
                    ("nativescript", "nativescript"),
                    ("nature", "nature"),
                    ("nature-outline", "nature-outline"),
                    ("nature-people", "nature-people"),
                    ("nature-people-outline", "nature-people-outline"),
                    ("navigation", "navigation"),
                    ("navigation-outline", "navigation-outline"),
                    ("navigation-variant", "navigation-variant"),
                    ("navigation-variant-outline", "navigation-variant-outline"),
                    ("near-me", "near-me"),
                    ("necklace", "necklace"),
                    ("needle", "needle"),
                    ("needle-off", "needle-off"),
                    ("nest-thermostat", "nest-thermostat"),
                    ("netflix", "netflix"),
                    ("network", "network"),
                    ("network-off", "network-off"),
                    ("network-off-outline", "network-off-outline"),
                    ("network-outline", "network-outline"),
                    ("network-pos", "network-pos"),
                    ("network-strength-1", "network-strength-1"),
                    ("network-strength-1-alert", "network-strength-1-alert"),
                    ("network-strength-2", "network-strength-2"),
                    ("network-strength-2-alert", "network-strength-2-alert"),
                    ("network-strength-3", "network-strength-3"),
                    ("network-strength-3-alert", "network-strength-3-alert"),
                    ("network-strength-4", "network-strength-4"),
                    ("network-strength-4-alert", "network-strength-4-alert"),
                    ("network-strength-4-cog", "network-strength-4-cog"),
                    ("network-strength-alert", "network-strength-alert"),
                    ("network-strength-alert-outline", "network-strength-alert-outline"),
                    ("network-strength-off", "network-strength-off"),
                    ("network-strength-off-outline", "network-strength-off-outline"),
                    ("network-strength-outline", "network-strength-outline"),
                    ("new-box", "new-box"),
                    ("newspaper", "newspaper"),
                    ("newspaper-check", "newspaper-check"),
                    ("newspaper-minus", "newspaper-minus"),
                    ("newspaper-plus", "newspaper-plus"),
                    ("newspaper-remove", "newspaper-remove"),
                    ("newspaper-variant", "newspaper-variant"),
                    ("newspaper-variant-multiple", "newspaper-variant-multiple"),
                    ("newspaper-variant-multiple-outline", "newspaper-variant-multiple-outline"),
                    ("newspaper-variant-outline", "newspaper-variant-outline"),
                    ("nfc", "nfc"),
                    ("nfc-off", "nfc-off"),
                    ("nfc-search-variant", "nfc-search-variant"),
                    ("nfc-tap", "nfc-tap"),
                    ("nfc-variant", "nfc-variant"),
                    ("nfc-variant-off", "nfc-variant-off"),
                    ("ninja", "ninja"),
                    ("nintendo-game-boy", "nintendo-game-boy"),
                    ("nintendo-switch", "nintendo-switch"),
                    ("nintendo-wii", "nintendo-wii"),
                    ("nintendo-wiiu", "nintendo-wiiu"),
                    ("nix", "nix"),
                    ("nodejs", "nodejs"),
                    ("noodles", "noodles"),
                    ("not-equal", "not-equal"),
                    ("not-equal-variant", "not-equal-variant"),
                    ("note", "note"),
                    ("note-alert", "note-alert"),
                    ("note-alert-outline", "note-alert-outline"),
                    ("note-check", "note-check"),
                    ("note-check-outline", "note-check-outline"),
                    ("note-edit", "note-edit"),
                    ("note-edit-outline", "note-edit-outline"),
                    ("note-minus", "note-minus"),
                    ("note-minus-outline", "note-minus-outline"),
                    ("note-multiple", "note-multiple"),
                    ("note-multiple-outline", "note-multiple-outline"),
                    ("note-off", "note-off"),
                    ("note-off-outline", "note-off-outline"),
                    ("note-outline", "note-outline"),
                    ("note-plus", "note-plus"),
                    ("note-plus-outline", "note-plus-outline"),
                    ("note-remove", "note-remove"),
                    ("note-remove-outline", "note-remove-outline"),
                    ("note-search", "note-search"),
                    ("note-search-outline", "note-search-outline"),
                    ("note-text", "note-text"),
                    ("note-text-outline", "note-text-outline"),
                    ("notebook", "notebook"),
                    ("notebook-check", "notebook-check"),
                    ("notebook-check-outline", "notebook-check-outline"),
                    ("notebook-edit", "notebook-edit"),
                    ("notebook-edit-outline", "notebook-edit-outline"),
                    ("notebook-heart", "notebook-heart"),
                    ("notebook-heart-outline", "notebook-heart-outline"),
                    ("notebook-minus", "notebook-minus"),
                    ("notebook-minus-outline", "notebook-minus-outline"),
                    ("notebook-multiple", "notebook-multiple"),
                    ("notebook-outline", "notebook-outline"),
                    ("notebook-plus", "notebook-plus"),
                    ("notebook-plus-outline", "notebook-plus-outline"),
                    ("notebook-remove", "notebook-remove"),
                    ("notebook-remove-outline", "notebook-remove-outline"),
                    ("notification-clear-all", "notification-clear-all"),
                    ("npm", "npm"),
                    ("npm-variant", "npm-variant"),
                    ("npm-variant-outline", "npm-variant-outline"),
                    ("nuke", "nuke"),
                    ("null", "null"),
                    ("numeric", "numeric"),
                    ("numeric-0", "numeric-0"),
                    ("numeric-0-box", "numeric-0-box"),
                    ("numeric-0-box-multiple", "numeric-0-box-multiple"),
                    ("numeric-0-box-multiple-outline", "numeric-0-box-multiple-outline"),
                    ("numeric-0-box-outline", "numeric-0-box-outline"),
                    ("numeric-0-circle", "numeric-0-circle"),
                    ("numeric-0-circle-outline", "numeric-0-circle-outline"),
                    ("numeric-1", "numeric-1"),
                    ("numeric-1-box", "numeric-1-box"),
                    ("numeric-1-box-multiple", "numeric-1-box-multiple"),
                    ("numeric-1-box-multiple-outline", "numeric-1-box-multiple-outline"),
                    ("numeric-1-box-outline", "numeric-1-box-outline"),
                    ("numeric-1-circle", "numeric-1-circle"),
                    ("numeric-1-circle-outline", "numeric-1-circle-outline"),
                    ("numeric-10", "numeric-10"),
                    ("numeric-10-box", "numeric-10-box"),
                    ("numeric-10-box-multiple", "numeric-10-box-multiple"),
                    ("numeric-10-box-multiple-outline", "numeric-10-box-multiple-outline"),
                    ("numeric-10-box-outline", "numeric-10-box-outline"),
                    ("numeric-10-circle", "numeric-10-circle"),
                    ("numeric-10-circle-outline", "numeric-10-circle-outline"),
                    ("numeric-2", "numeric-2"),
                    ("numeric-2-box", "numeric-2-box"),
                    ("numeric-2-box-multiple", "numeric-2-box-multiple"),
                    ("numeric-2-box-multiple-outline", "numeric-2-box-multiple-outline"),
                    ("numeric-2-box-outline", "numeric-2-box-outline"),
                    ("numeric-2-circle", "numeric-2-circle"),
                    ("numeric-2-circle-outline", "numeric-2-circle-outline"),
                    ("numeric-3", "numeric-3"),
                    ("numeric-3-box", "numeric-3-box"),
                    ("numeric-3-box-multiple", "numeric-3-box-multiple"),
                    ("numeric-3-box-multiple-outline", "numeric-3-box-multiple-outline"),
                    ("numeric-3-box-outline", "numeric-3-box-outline"),
                    ("numeric-3-circle", "numeric-3-circle"),
                    ("numeric-3-circle-outline", "numeric-3-circle-outline"),
                    ("numeric-4", "numeric-4"),
                    ("numeric-4-box", "numeric-4-box"),
                    ("numeric-4-box-multiple", "numeric-4-box-multiple"),
                    ("numeric-4-box-multiple-outline", "numeric-4-box-multiple-outline"),
                    ("numeric-4-box-outline", "numeric-4-box-outline"),
                    ("numeric-4-circle", "numeric-4-circle"),
                    ("numeric-4-circle-outline", "numeric-4-circle-outline"),
                    ("numeric-5", "numeric-5"),
                    ("numeric-5-box", "numeric-5-box"),
                    ("numeric-5-box-multiple", "numeric-5-box-multiple"),
                    ("numeric-5-box-multiple-outline", "numeric-5-box-multiple-outline"),
                    ("numeric-5-box-outline", "numeric-5-box-outline"),
                    ("numeric-5-circle", "numeric-5-circle"),
                    ("numeric-5-circle-outline", "numeric-5-circle-outline"),
                    ("numeric-6", "numeric-6"),
                    ("numeric-6-box", "numeric-6-box"),
                    ("numeric-6-box-multiple", "numeric-6-box-multiple"),
                    ("numeric-6-box-multiple-outline", "numeric-6-box-multiple-outline"),
                    ("numeric-6-box-outline", "numeric-6-box-outline"),
                    ("numeric-6-circle", "numeric-6-circle"),
                    ("numeric-6-circle-outline", "numeric-6-circle-outline"),
                    ("numeric-7", "numeric-7"),
                    ("numeric-7-box", "numeric-7-box"),
                    ("numeric-7-box-multiple", "numeric-7-box-multiple"),
                    ("numeric-7-box-multiple-outline", "numeric-7-box-multiple-outline"),
                    ("numeric-7-box-outline", "numeric-7-box-outline"),
                    ("numeric-7-circle", "numeric-7-circle"),
                    ("numeric-7-circle-outline", "numeric-7-circle-outline"),
                    ("numeric-8", "numeric-8"),
                    ("numeric-8-box", "numeric-8-box"),
                    ("numeric-8-box-multiple", "numeric-8-box-multiple"),
                    ("numeric-8-box-multiple-outline", "numeric-8-box-multiple-outline"),
                    ("numeric-8-box-outline", "numeric-8-box-outline"),
                    ("numeric-8-circle", "numeric-8-circle"),
                    ("numeric-8-circle-outline", "numeric-8-circle-outline"),
                    ("numeric-9", "numeric-9"),
                    ("numeric-9-box", "numeric-9-box"),
                    ("numeric-9-box-multiple", "numeric-9-box-multiple"),
                    ("numeric-9-box-multiple-outline", "numeric-9-box-multiple-outline"),
                    ("numeric-9-box-outline", "numeric-9-box-outline"),
                    ("numeric-9-circle", "numeric-9-circle"),
                    ("numeric-9-circle-outline", "numeric-9-circle-outline"),
                    ("numeric-9-plus", "numeric-9-plus"),
                    ("numeric-9-plus-box", "numeric-9-plus-box"),
                    ("numeric-9-plus-box-multiple", "numeric-9-plus-box-multiple"),
                    ("numeric-9-plus-box-multiple-outline", "numeric-9-plus-box-multiple-outline"),
                    ("numeric-9-plus-box-outline", "numeric-9-plus-box-outline"),
                    ("numeric-9-plus-circle", "numeric-9-plus-circle"),
                    ("numeric-9-plus-circle-outline", "numeric-9-plus-circle-outline"),
                    ("numeric-negative-1", "numeric-negative-1"),
                    ("numeric-off", "numeric-off"),
                    ("numeric-positive-1", "numeric-positive-1"),
                    ("nut", "nut"),
                    ("nutrition", "nutrition"),
                    ("nuxt", "nuxt"),
                    ("oar", "oar"),
                    ("ocarina", "ocarina"),
                    ("oci", "oci"),
                    ("ocr", "ocr"),
                    ("octagon", "octagon"),
                    ("octagon-outline", "octagon-outline"),
                    ("octagram", "octagram"),
                    ("octagram-edit", "octagram-edit"),
                    ("octagram-edit-outline", "octagram-edit-outline"),
                    ("octagram-minus", "octagram-minus"),
                    ("octagram-minus-outline", "octagram-minus-outline"),
                    ("octagram-outline", "octagram-outline"),
                    ("octagram-plus", "octagram-plus"),
                    ("octagram-plus-outline", "octagram-plus-outline"),
                    ("octahedron", "octahedron"),
                    ("octahedron-off", "octahedron-off"),
                    ("odnoklassniki", "odnoklassniki"),
                    ("offer", "offer"),
                    ("office-building", "office-building"),
                    ("office-building-cog", "office-building-cog"),
                    ("office-building-cog-outline", "office-building-cog-outline"),
                    ("office-building-marker", "office-building-marker"),
                    ("office-building-marker-outline", "office-building-marker-outline"),
                    ("office-building-minus", "office-building-minus"),
                    ("office-building-minus-outline", "office-building-minus-outline"),
                    ("office-building-outline", "office-building-outline"),
                    ("office-building-plus", "office-building-plus"),
                    ("office-building-plus-outline", "office-building-plus-outline"),
                    ("office-building-remove", "office-building-remove"),
                    ("office-building-remove-outline", "office-building-remove-outline"),
                    ("oil", "oil"),
                    ("oil-lamp", "oil-lamp"),
                    ("oil-level", "oil-level"),
                    ("oil-temperature", "oil-temperature"),
                    ("om", "om"),
                    ("omega", "omega"),
                    ("one-up", "one-up"),
                    ("onedrive", "onedrive"),
                    ("onenote", "onenote"),
                    ("onepassword", "onepassword"),
                    ("opacity", "opacity"),
                    ("open-in-app", "open-in-app"),
                    ("open-in-new", "open-in-new"),
                    ("open-source-initiative", "open-source-initiative"),
                    ("openid", "openid"),
                    ("opera", "opera"),
                    ("orbit", "orbit"),
                    ("orbit-variant", "orbit-variant"),
                    ("order-alphabetical-ascending", "order-alphabetical-ascending"),
                    ("order-alphabetical-descending", "order-alphabetical-descending"),
                    ("order-bool-ascending", "order-bool-ascending"),
                    ("order-bool-ascending-variant", "order-bool-ascending-variant"),
                    ("order-bool-descending", "order-bool-descending"),
                    ("order-bool-descending-variant", "order-bool-descending-variant"),
                    ("order-numeric-ascending", "order-numeric-ascending"),
                    ("order-numeric-descending", "order-numeric-descending"),
                    ("origin", "origin"),
                    ("ornament", "ornament"),
                    ("ornament-variant", "ornament-variant"),
                    ("outbox", "outbox"),
                    ("outdent", "outdent"),
                    ("outdoor-lamp", "outdoor-lamp"),
                    ("outlook", "outlook"),
                    ("overscan", "overscan"),
                    ("owl", "owl"),
                    ("pac-man", "pac-man"),
                    ("package", "package"),
                    ("package-check", "package-check"),
                    ("package-down", "package-down"),
                    ("package-up", "package-up"),
                    ("package-variant", "package-variant"),
                    ("package-variant-closed", "package-variant-closed"),
                    ("package-variant-closed-check", "package-variant-closed-check"),
                    ("package-variant-closed-minus", "package-variant-closed-minus"),
                    ("package-variant-closed-plus", "package-variant-closed-plus"),
                    ("package-variant-closed-remove", "package-variant-closed-remove"),
                    ("package-variant-minus", "package-variant-minus"),
                    ("package-variant-plus", "package-variant-plus"),
                    ("package-variant-remove", "package-variant-remove"),
                    ("page-first", "page-first"),
                    ("page-last", "page-last"),
                    ("page-layout-body", "page-layout-body"),
                    ("page-layout-footer", "page-layout-footer"),
                    ("page-layout-header", "page-layout-header"),
                    ("page-layout-header-footer", "page-layout-header-footer"),
                    ("page-layout-sidebar-left", "page-layout-sidebar-left"),
                    ("page-layout-sidebar-right", "page-layout-sidebar-right"),
                    ("page-next", "page-next"),
                    ("page-next-outline", "page-next-outline"),
                    ("page-previous", "page-previous"),
                    ("page-previous-outline", "page-previous-outline"),
                    ("pail", "pail"),
                    ("pail-minus", "pail-minus"),
                    ("pail-minus-outline", "pail-minus-outline"),
                    ("pail-off", "pail-off"),
                    ("pail-off-outline", "pail-off-outline"),
                    ("pail-outline", "pail-outline"),
                    ("pail-plus", "pail-plus"),
                    ("pail-plus-outline", "pail-plus-outline"),
                    ("pail-remove", "pail-remove"),
                    ("pail-remove-outline", "pail-remove-outline"),
                    ("palette", "palette"),
                    ("palette-advanced", "palette-advanced"),
                    ("palette-outline", "palette-outline"),
                    ("palette-swatch", "palette-swatch"),
                    ("palette-swatch-outline", "palette-swatch-outline"),
                    ("palette-swatch-variant", "palette-swatch-variant"),
                    ("palm-tree", "palm-tree"),
                    ("pan", "pan"),
                    ("pan-bottom-left", "pan-bottom-left"),
                    ("pan-bottom-right", "pan-bottom-right"),
                    ("pan-down", "pan-down"),
                    ("pan-horizontal", "pan-horizontal"),
                    ("pan-left", "pan-left"),
                    ("pan-right", "pan-right"),
                    ("pan-top-left", "pan-top-left"),
                    ("pan-top-right", "pan-top-right"),
                    ("pan-up", "pan-up"),
                    ("pan-vertical", "pan-vertical"),
                    ("panda", "panda"),
                    ("pandora", "pandora"),
                    ("panorama", "panorama"),
                    ("panorama-fisheye", "panorama-fisheye"),
                    ("panorama-horizontal", "panorama-horizontal"),
                    ("panorama-horizontal-outline", "panorama-horizontal-outline"),
                    ("panorama-outline", "panorama-outline"),
                    ("panorama-sphere", "panorama-sphere"),
                    ("panorama-sphere-outline", "panorama-sphere-outline"),
                    ("panorama-variant", "panorama-variant"),
                    ("panorama-variant-outline", "panorama-variant-outline"),
                    ("panorama-vertical", "panorama-vertical"),
                    ("panorama-vertical-outline", "panorama-vertical-outline"),
                    ("panorama-wide-angle", "panorama-wide-angle"),
                    ("panorama-wide-angle-outline", "panorama-wide-angle-outline"),
                    ("paper-cut-vertical", "paper-cut-vertical"),
                    ("paper-roll", "paper-roll"),
                    ("paper-roll-outline", "paper-roll-outline"),
                    ("paperclip", "paperclip"),
                    ("paperclip-check", "paperclip-check"),
                    ("paperclip-lock", "paperclip-lock"),
                    ("paperclip-minus", "paperclip-minus"),
                    ("paperclip-off", "paperclip-off"),
                    ("paperclip-plus", "paperclip-plus"),
                    ("paperclip-remove", "paperclip-remove"),
                    ("parachute", "parachute"),
                    ("parachute-outline", "parachute-outline"),
                    ("paragliding", "paragliding"),
                    ("parking", "parking"),
                    ("party-popper", "party-popper"),
                    ("passport", "passport"),
                    ("passport-biometric", "passport-biometric"),
                    ("pasta", "pasta"),
                    ("patio-heater", "patio-heater"),
                    ("patreon", "patreon"),
                    ("pause", "pause"),
                    ("pause-box", "pause-box"),
                    ("pause-box-outline", "pause-box-outline"),
                    ("pause-circle", "pause-circle"),
                    ("pause-circle-outline", "pause-circle-outline"),
                    ("pause-octagon", "pause-octagon"),
                    ("pause-octagon-outline", "pause-octagon-outline"),
                    ("paw", "paw"),
                    ("paw-off", "paw-off"),
                    ("paw-off-outline", "paw-off-outline"),
                    ("paw-outline", "paw-outline"),
                    ("paypal", "paypal"),
                    ("peace", "peace"),
                    ("peanut", "peanut"),
                    ("peanut-off", "peanut-off"),
                    ("peanut-off-outline", "peanut-off-outline"),
                    ("peanut-outline", "peanut-outline"),
                    ("pen", "pen"),
                    ("pen-lock", "pen-lock"),
                    ("pen-minus", "pen-minus"),
                    ("pen-off", "pen-off"),
                    ("pen-plus", "pen-plus"),
                    ("pen-remove", "pen-remove"),
                    ("pencil", "pencil"),
                    ("pencil-box", "pencil-box"),
                    ("pencil-box-multiple", "pencil-box-multiple"),
                    ("pencil-box-multiple-outline", "pencil-box-multiple-outline"),
                    ("pencil-box-outline", "pencil-box-outline"),
                    ("pencil-circle", "pencil-circle"),
                    ("pencil-circle-outline", "pencil-circle-outline"),
                    ("pencil-lock", "pencil-lock"),
                    ("pencil-lock-outline", "pencil-lock-outline"),
                    ("pencil-minus", "pencil-minus"),
                    ("pencil-minus-outline", "pencil-minus-outline"),
                    ("pencil-off", "pencil-off"),
                    ("pencil-off-outline", "pencil-off-outline"),
                    ("pencil-outline", "pencil-outline"),
                    ("pencil-plus", "pencil-plus"),
                    ("pencil-plus-outline", "pencil-plus-outline"),
                    ("pencil-remove", "pencil-remove"),
                    ("pencil-remove-outline", "pencil-remove-outline"),
                    ("pencil-ruler", "pencil-ruler"),
                    ("pencil-ruler-outline", "pencil-ruler-outline"),
                    ("penguin", "penguin"),
                    ("pentagon", "pentagon"),
                    ("pentagon-outline", "pentagon-outline"),
                    ("pentagram", "pentagram"),
                    ("percent", "percent"),
                    ("percent-box", "percent-box"),
                    ("percent-box-outline", "percent-box-outline"),
                    ("percent-circle", "percent-circle"),
                    ("percent-circle-outline", "percent-circle-outline"),
                    ("percent-outline", "percent-outline"),
                    ("periodic-table", "periodic-table"),
                    ("periscope", "periscope"),
                    ("perspective-less", "perspective-less"),
                    ("perspective-more", "perspective-more"),
                    ("ph", "ph"),
                    ("phone", "phone"),
                    ("phone-alert", "phone-alert"),
                    ("phone-alert-outline", "phone-alert-outline"),
                    ("phone-bluetooth", "phone-bluetooth"),
                    ("phone-bluetooth-outline", "phone-bluetooth-outline"),
                    ("phone-cancel", "phone-cancel"),
                    ("phone-cancel-outline", "phone-cancel-outline"),
                    ("phone-check", "phone-check"),
                    ("phone-check-outline", "phone-check-outline"),
                    ("phone-classic", "phone-classic"),
                    ("phone-classic-off", "phone-classic-off"),
                    ("phone-clock", "phone-clock"),
                    ("phone-dial", "phone-dial"),
                    ("phone-dial-outline", "phone-dial-outline"),
                    ("phone-forward", "phone-forward"),
                    ("phone-forward-outline", "phone-forward-outline"),
                    ("phone-hangup", "phone-hangup"),
                    ("phone-hangup-outline", "phone-hangup-outline"),
                    ("phone-in-talk", "phone-in-talk"),
                    ("phone-in-talk-outline", "phone-in-talk-outline"),
                    ("phone-incoming", "phone-incoming"),
                    ("phone-incoming-outgoing", "phone-incoming-outgoing"),
                    ("phone-incoming-outgoing-outline", "phone-incoming-outgoing-outline"),
                    ("phone-incoming-outline", "phone-incoming-outline"),
                    ("phone-lock", "phone-lock"),
                    ("phone-lock-outline", "phone-lock-outline"),
                    ("phone-log", "phone-log"),
                    ("phone-log-outline", "phone-log-outline"),
                    ("phone-message", "phone-message"),
                    ("phone-message-outline", "phone-message-outline"),
                    ("phone-minus", "phone-minus"),
                    ("phone-minus-outline", "phone-minus-outline"),
                    ("phone-missed", "phone-missed"),
                    ("phone-missed-outline", "phone-missed-outline"),
                    ("phone-off", "phone-off"),
                    ("phone-off-outline", "phone-off-outline"),
                    ("phone-outgoing", "phone-outgoing"),
                    ("phone-outgoing-outline", "phone-outgoing-outline"),
                    ("phone-outline", "phone-outline"),
                    ("phone-paused", "phone-paused"),
                    ("phone-paused-outline", "phone-paused-outline"),
                    ("phone-plus", "phone-plus"),
                    ("phone-plus-outline", "phone-plus-outline"),
                    ("phone-refresh", "phone-refresh"),
                    ("phone-refresh-outline", "phone-refresh-outline"),
                    ("phone-remove", "phone-remove"),
                    ("phone-remove-outline", "phone-remove-outline"),
                    ("phone-return", "phone-return"),
                    ("phone-return-outline", "phone-return-outline"),
                    ("phone-ring", "phone-ring"),
                    ("phone-ring-outline", "phone-ring-outline"),
                    ("phone-rotate-landscape", "phone-rotate-landscape"),
                    ("phone-rotate-portrait", "phone-rotate-portrait"),
                    ("phone-settings", "phone-settings"),
                    ("phone-settings-outline", "phone-settings-outline"),
                    ("phone-sync", "phone-sync"),
                    ("phone-sync-outline", "phone-sync-outline"),
                    ("phone-voip", "phone-voip"),
                    ("pi", "pi"),
                    ("pi-box", "pi-box"),
                    ("pi-hole", "pi-hole"),
                    ("piano", "piano"),
                    ("piano-off", "piano-off"),
                    ("pickaxe", "pickaxe"),
                    ("picture-in-picture-bottom-right", "picture-in-picture-bottom-right"),
                    (
                        "picture-in-picture-bottom-right-outline",
                        "picture-in-picture-bottom-right-outline",
                    ),
                    ("picture-in-picture-top-right", "picture-in-picture-top-right"),
                    (
                        "picture-in-picture-top-right-outline",
                        "picture-in-picture-top-right-outline",
                    ),
                    ("pier", "pier"),
                    ("pier-crane", "pier-crane"),
                    ("pig", "pig"),
                    ("pig-variant", "pig-variant"),
                    ("pig-variant-outline", "pig-variant-outline"),
                    ("piggy-bank", "piggy-bank"),
                    ("piggy-bank-outline", "piggy-bank-outline"),
                    ("pill", "pill"),
                    ("pill-multiple", "pill-multiple"),
                    ("pill-off", "pill-off"),
                    ("pillar", "pillar"),
                    ("pin", "pin"),
                    ("pin-off", "pin-off"),
                    ("pin-off-outline", "pin-off-outline"),
                    ("pin-outline", "pin-outline"),
                    ("pine-tree", "pine-tree"),
                    ("pine-tree-box", "pine-tree-box"),
                    ("pine-tree-fire", "pine-tree-fire"),
                    ("pine-tree-variant", "pine-tree-variant"),
                    ("pine-tree-variant-outline", "pine-tree-variant-outline"),
                    ("pinterest", "pinterest"),
                    ("pinterest-box", "pinterest-box"),
                    ("pinwheel", "pinwheel"),
                    ("pinwheel-outline", "pinwheel-outline"),
                    ("pipe", "pipe"),
                    ("pipe-disconnected", "pipe-disconnected"),
                    ("pipe-leak", "pipe-leak"),
                    ("pipe-valve", "pipe-valve"),
                    ("pipe-wrench", "pipe-wrench"),
                    ("pirate", "pirate"),
                    ("pistol", "pistol"),
                    ("piston", "piston"),
                    ("pitchfork", "pitchfork"),
                    ("pizza", "pizza"),
                    ("plane-car", "plane-car"),
                    ("plane-train", "plane-train"),
                    ("play", "play"),
                    ("play-box", "play-box"),
                    ("play-box-edit-outline", "play-box-edit-outline"),
                    ("play-box-lock", "play-box-lock"),
                    ("play-box-lock-open", "play-box-lock-open"),
                    ("play-box-lock-open-outline", "play-box-lock-open-outline"),
                    ("play-box-lock-outline", "play-box-lock-outline"),
                    ("play-box-multiple", "play-box-multiple"),
                    ("play-box-multiple-outline", "play-box-multiple-outline"),
                    ("play-box-outline", "play-box-outline"),
                    ("play-circle", "play-circle"),
                    ("play-circle-outline", "play-circle-outline"),
                    ("play-network", "play-network"),
                    ("play-network-outline", "play-network-outline"),
                    ("play-outline", "play-outline"),
                    ("play-pause", "play-pause"),
                    ("play-protected-content", "play-protected-content"),
                    ("play-speed", "play-speed"),
                    ("playlist-check", "playlist-check"),
                    ("playlist-edit", "playlist-edit"),
                    ("playlist-minus", "playlist-minus"),
                    ("playlist-music", "playlist-music"),
                    ("playlist-music-outline", "playlist-music-outline"),
                    ("playlist-play", "playlist-play"),
                    ("playlist-plus", "playlist-plus"),
                    ("playlist-remove", "playlist-remove"),
                    ("playlist-star", "playlist-star"),
                    ("plex", "plex"),
                    ("pliers", "pliers"),
                    ("plus", "plus"),
                    ("plus-box", "plus-box"),
                    ("plus-box-multiple", "plus-box-multiple"),
                    ("plus-box-multiple-outline", "plus-box-multiple-outline"),
                    ("plus-box-outline", "plus-box-outline"),
                    ("plus-circle", "plus-circle"),
                    ("plus-circle-multiple", "plus-circle-multiple"),
                    ("plus-circle-multiple-outline", "plus-circle-multiple-outline"),
                    ("plus-circle-outline", "plus-circle-outline"),
                    ("plus-lock", "plus-lock"),
                    ("plus-lock-open", "plus-lock-open"),
                    ("plus-minus", "plus-minus"),
                    ("plus-minus-box", "plus-minus-box"),
                    ("plus-minus-variant", "plus-minus-variant"),
                    ("plus-network", "plus-network"),
                    ("plus-network-outline", "plus-network-outline"),
                    ("plus-outline", "plus-outline"),
                    ("plus-thick", "plus-thick"),
                    ("pocket", "pocket"),
                    ("podcast", "podcast"),
                    ("podium", "podium"),
                    ("podium-bronze", "podium-bronze"),
                    ("podium-gold", "podium-gold"),
                    ("podium-silver", "podium-silver"),
                    ("point-of-sale", "point-of-sale"),
                    ("pokeball", "pokeball"),
                    ("pokemon-go", "pokemon-go"),
                    ("poker-chip", "poker-chip"),
                    ("polaroid", "polaroid"),
                    ("police-badge", "police-badge"),
                    ("police-badge-outline", "police-badge-outline"),
                    ("police-station", "police-station"),
                    ("poll", "poll"),
                    ("polo", "polo"),
                    ("polymer", "polymer"),
                    ("pool", "pool"),
                    ("pool-thermometer", "pool-thermometer"),
                    ("popcorn", "popcorn"),
                    ("post", "post"),
                    ("post-lamp", "post-lamp"),
                    ("post-outline", "post-outline"),
                    ("postage-stamp", "postage-stamp"),
                    ("pot", "pot"),
                    ("pot-mix", "pot-mix"),
                    ("pot-mix-outline", "pot-mix-outline"),
                    ("pot-outline", "pot-outline"),
                    ("pot-steam", "pot-steam"),
                    ("pot-steam-outline", "pot-steam-outline"),
                    ("pound", "pound"),
                    ("pound-box", "pound-box"),
                    ("pound-box-outline", "pound-box-outline"),
                    ("power", "power"),
                    ("power-cycle", "power-cycle"),
                    ("power-off", "power-off"),
                    ("power-on", "power-on"),
                    ("power-plug", "power-plug"),
                    ("power-plug-battery", "power-plug-battery"),
                    ("power-plug-battery-outline", "power-plug-battery-outline"),
                    ("power-plug-off", "power-plug-off"),
                    ("power-plug-off-outline", "power-plug-off-outline"),
                    ("power-plug-outline", "power-plug-outline"),
                    ("power-settings", "power-settings"),
                    ("power-sleep", "power-sleep"),
                    ("power-socket", "power-socket"),
                    ("power-socket-au", "power-socket-au"),
                    ("power-socket-ch", "power-socket-ch"),
                    ("power-socket-de", "power-socket-de"),
                    ("power-socket-eu", "power-socket-eu"),
                    ("power-socket-fr", "power-socket-fr"),
                    ("power-socket-it", "power-socket-it"),
                    ("power-socket-jp", "power-socket-jp"),
                    ("power-socket-uk", "power-socket-uk"),
                    ("power-socket-us", "power-socket-us"),
                    ("power-standby", "power-standby"),
                    ("powershell", "powershell"),
                    ("prescription", "prescription"),
                    ("presentation", "presentation"),
                    ("presentation-play", "presentation-play"),
                    ("pretzel", "pretzel"),
                    ("prezi", "prezi"),
                    ("printer", "printer"),
                    ("printer-3d", "printer-3d"),
                    ("printer-3d-nozzle", "printer-3d-nozzle"),
                    ("printer-3d-nozzle-alert", "printer-3d-nozzle-alert"),
                    ("printer-3d-nozzle-alert-outline", "printer-3d-nozzle-alert-outline"),
                    ("printer-3d-nozzle-heat", "printer-3d-nozzle-heat"),
                    ("printer-3d-nozzle-heat-outline", "printer-3d-nozzle-heat-outline"),
                    ("printer-3d-nozzle-off", "printer-3d-nozzle-off"),
                    ("printer-3d-nozzle-off-outline", "printer-3d-nozzle-off-outline"),
                    ("printer-3d-nozzle-outline", "printer-3d-nozzle-outline"),
                    ("printer-3d-off", "printer-3d-off"),
                    ("printer-alert", "printer-alert"),
                    ("printer-check", "printer-check"),
                    ("printer-eye", "printer-eye"),
                    ("printer-off", "printer-off"),
                    ("printer-off-outline", "printer-off-outline"),
                    ("printer-outline", "printer-outline"),
                    ("printer-pos", "printer-pos"),
                    ("printer-pos-alert", "printer-pos-alert"),
                    ("printer-pos-alert-outline", "printer-pos-alert-outline"),
                    ("printer-pos-cancel", "printer-pos-cancel"),
                    ("printer-pos-cancel-outline", "printer-pos-cancel-outline"),
                    ("printer-pos-check", "printer-pos-check"),
                    ("printer-pos-check-outline", "printer-pos-check-outline"),
                    ("printer-pos-cog", "printer-pos-cog"),
                    ("printer-pos-cog-outline", "printer-pos-cog-outline"),
                    ("printer-pos-edit", "printer-pos-edit"),
                    ("printer-pos-edit-outline", "printer-pos-edit-outline"),
                    ("printer-pos-minus", "printer-pos-minus"),
                    ("printer-pos-minus-outline", "printer-pos-minus-outline"),
                    ("printer-pos-network", "printer-pos-network"),
                    ("printer-pos-network-outline", "printer-pos-network-outline"),
                    ("printer-pos-off", "printer-pos-off"),
                    ("printer-pos-off-outline", "printer-pos-off-outline"),
                    ("printer-pos-outline", "printer-pos-outline"),
                    ("printer-pos-pause", "printer-pos-pause"),
                    ("printer-pos-pause-outline", "printer-pos-pause-outline"),
                    ("printer-pos-play", "printer-pos-play"),
                    ("printer-pos-play-outline", "printer-pos-play-outline"),
                    ("printer-pos-plus", "printer-pos-plus"),
                    ("printer-pos-plus-outline", "printer-pos-plus-outline"),
                    ("printer-pos-refresh", "printer-pos-refresh"),
                    ("printer-pos-refresh-outline", "printer-pos-refresh-outline"),
                    ("printer-pos-remove", "printer-pos-remove"),
                    ("printer-pos-remove-outline", "printer-pos-remove-outline"),
                    ("printer-pos-star", "printer-pos-star"),
                    ("printer-pos-star-outline", "printer-pos-star-outline"),
                    ("printer-pos-stop", "printer-pos-stop"),
                    ("printer-pos-stop-outline", "printer-pos-stop-outline"),
                    ("printer-pos-sync", "printer-pos-sync"),
                    ("printer-pos-sync-outline", "printer-pos-sync-outline"),
                    ("printer-pos-wrench", "printer-pos-wrench"),
                    ("printer-pos-wrench-outline", "printer-pos-wrench-outline"),
                    ("printer-search", "printer-search"),
                    ("printer-settings", "printer-settings"),
                    ("printer-wireless", "printer-wireless"),
                    ("priority-high", "priority-high"),
                    ("priority-low", "priority-low"),
                    ("professional-hexagon", "professional-hexagon"),
                    ("progress-alert", "progress-alert"),
                    ("progress-check", "progress-check"),
                    ("progress-clock", "progress-clock"),
                    ("progress-close", "progress-close"),
                    ("progress-download", "progress-download"),
                    ("progress-helper", "progress-helper"),
                    ("progress-pencil", "progress-pencil"),
                    ("progress-question", "progress-question"),
                    ("progress-star", "progress-star"),
                    ("progress-star-four-points", "progress-star-four-points"),
                    ("progress-upload", "progress-upload"),
                    ("progress-wrench", "progress-wrench"),
                    ("projector", "projector"),
                    ("projector-off", "projector-off"),
                    ("projector-screen", "projector-screen"),
                    ("projector-screen-off", "projector-screen-off"),
                    ("projector-screen-off-outline", "projector-screen-off-outline"),
                    ("projector-screen-outline", "projector-screen-outline"),
                    ("projector-screen-variant", "projector-screen-variant"),
                    ("projector-screen-variant-off", "projector-screen-variant-off"),
                    (
                        "projector-screen-variant-off-outline",
                        "projector-screen-variant-off-outline",
                    ),
                    ("projector-screen-variant-outline", "projector-screen-variant-outline"),
                    ("propane-tank", "propane-tank"),
                    ("propane-tank-outline", "propane-tank-outline"),
                    ("protocol", "protocol"),
                    ("publish", "publish"),
                    ("publish-off", "publish-off"),
                    ("pulse", "pulse"),
                    ("pump", "pump"),
                    ("pump-off", "pump-off"),
                    ("pumpkin", "pumpkin"),
                    ("purse", "purse"),
                    ("purse-outline", "purse-outline"),
                    ("puzzle", "puzzle"),
                    ("puzzle-check", "puzzle-check"),
                    ("puzzle-check-outline", "puzzle-check-outline"),
                    ("puzzle-edit", "puzzle-edit"),
                    ("puzzle-edit-outline", "puzzle-edit-outline"),
                    ("puzzle-heart", "puzzle-heart"),
                    ("puzzle-heart-outline", "puzzle-heart-outline"),
                    ("puzzle-minus", "puzzle-minus"),
                    ("puzzle-minus-outline", "puzzle-minus-outline"),
                    ("puzzle-outline", "puzzle-outline"),
                    ("puzzle-plus", "puzzle-plus"),
                    ("puzzle-plus-outline", "puzzle-plus-outline"),
                    ("puzzle-remove", "puzzle-remove"),
                    ("puzzle-remove-outline", "puzzle-remove-outline"),
                    ("puzzle-star", "puzzle-star"),
                    ("puzzle-star-outline", "puzzle-star-outline"),
                    ("pyramid", "pyramid"),
                    ("pyramid-off", "pyramid-off"),
                    ("qi", "qi"),
                    ("qqchat", "qqchat"),
                    ("qrcode", "qrcode"),
                    ("qrcode-edit", "qrcode-edit"),
                    ("qrcode-minus", "qrcode-minus"),
                    ("qrcode-plus", "qrcode-plus"),
                    ("qrcode-remove", "qrcode-remove"),
                    ("qrcode-scan", "qrcode-scan"),
                    ("quadcopter", "quadcopter"),
                    ("quality-high", "quality-high"),
                    ("quality-low", "quality-low"),
                    ("quality-medium", "quality-medium"),
                    ("quick-reply", "quick-reply"),
                    ("quicktime", "quicktime"),
                    ("quora", "quora"),
                    ("rabbit", "rabbit"),
                    ("rabbit-variant", "rabbit-variant"),
                    ("rabbit-variant-outline", "rabbit-variant-outline"),
                    ("racing-helmet", "racing-helmet"),
                    ("racquetball", "racquetball"),
                    ("radar", "radar"),
                    ("radiator", "radiator"),
                    ("radiator-disabled", "radiator-disabled"),
                    ("radiator-off", "radiator-off"),
                    ("radio", "radio"),
                    ("radio-am", "radio-am"),
                    ("radio-fm", "radio-fm"),
                    ("radio-handheld", "radio-handheld"),
                    ("radio-off", "radio-off"),
                    ("radio-tower", "radio-tower"),
                    ("radioactive", "radioactive"),
                    ("radioactive-circle", "radioactive-circle"),
                    ("radioactive-circle-outline", "radioactive-circle-outline"),
                    ("radioactive-off", "radioactive-off"),
                    ("radiobox-blank", "radiobox-blank"),
                    ("radiobox-indeterminate-variant", "radiobox-indeterminate-variant"),
                    ("radiobox-marked", "radiobox-marked"),
                    ("radiology-box", "radiology-box"),
                    ("radiology-box-outline", "radiology-box-outline"),
                    ("radius", "radius"),
                    ("radius-outline", "radius-outline"),
                    ("railroad-light", "railroad-light"),
                    ("rake", "rake"),
                    ("raspberry-pi", "raspberry-pi"),
                    ("raw", "raw"),
                    ("raw-off", "raw-off"),
                    ("ray-end", "ray-end"),
                    ("ray-end-arrow", "ray-end-arrow"),
                    ("ray-start", "ray-start"),
                    ("ray-start-arrow", "ray-start-arrow"),
                    ("ray-start-end", "ray-start-end"),
                    ("ray-start-vertex-end", "ray-start-vertex-end"),
                    ("ray-vertex", "ray-vertex"),
                    ("razor-double-edge", "razor-double-edge"),
                    ("razor-single-edge", "razor-single-edge"),
                    ("rdio", "rdio"),
                    ("react", "react"),
                    ("read", "read"),
                    ("receipt", "receipt"),
                    ("receipt-clock", "receipt-clock"),
                    ("receipt-clock-outline", "receipt-clock-outline"),
                    ("receipt-outline", "receipt-outline"),
                    ("receipt-send", "receipt-send"),
                    ("receipt-send-outline", "receipt-send-outline"),
                    ("receipt-text", "receipt-text"),
                    ("receipt-text-arrow-left", "receipt-text-arrow-left"),
                    ("receipt-text-arrow-left-outline", "receipt-text-arrow-left-outline"),
                    ("receipt-text-arrow-right", "receipt-text-arrow-right"),
                    ("receipt-text-arrow-right-outline", "receipt-text-arrow-right-outline"),
                    ("receipt-text-check", "receipt-text-check"),
                    ("receipt-text-check-outline", "receipt-text-check-outline"),
                    ("receipt-text-clock", "receipt-text-clock"),
                    ("receipt-text-clock-outline", "receipt-text-clock-outline"),
                    ("receipt-text-edit", "receipt-text-edit"),
                    ("receipt-text-edit-outline", "receipt-text-edit-outline"),
                    ("receipt-text-minus", "receipt-text-minus"),
                    ("receipt-text-minus-outline", "receipt-text-minus-outline"),
                    ("receipt-text-outline", "receipt-text-outline"),
                    ("receipt-text-plus", "receipt-text-plus"),
                    ("receipt-text-plus-outline", "receipt-text-plus-outline"),
                    ("receipt-text-remove", "receipt-text-remove"),
                    ("receipt-text-remove-outline", "receipt-text-remove-outline"),
                    ("receipt-text-send", "receipt-text-send"),
                    ("receipt-text-send-outline", "receipt-text-send-outline"),
                    ("record", "record"),
                    ("record-circle", "record-circle"),
                    ("record-circle-outline", "record-circle-outline"),
                    ("record-player", "record-player"),
                    ("record-rec", "record-rec"),
                    ("rectangle", "rectangle"),
                    ("rectangle-outline", "rectangle-outline"),
                    ("recycle", "recycle"),
                    ("recycle-variant", "recycle-variant"),
                    ("reddit", "reddit"),
                    ("redhat", "redhat"),
                    ("redo", "redo"),
                    ("redo-variant", "redo-variant"),
                    ("reflect-horizontal", "reflect-horizontal"),
                    ("reflect-vertical", "reflect-vertical"),
                    ("refresh", "refresh"),
                    ("refresh-auto", "refresh-auto"),
                    ("refresh-circle", "refresh-circle"),
                    ("regex", "regex"),
                    ("registered-trademark", "registered-trademark"),
                    ("reiterate", "reiterate"),
                    ("relation-many-to-many", "relation-many-to-many"),
                    ("relation-many-to-one", "relation-many-to-one"),
                    ("relation-many-to-one-or-many", "relation-many-to-one-or-many"),
                    ("relation-many-to-only-one", "relation-many-to-only-one"),
                    ("relation-many-to-zero-or-many", "relation-many-to-zero-or-many"),
                    ("relation-many-to-zero-or-one", "relation-many-to-zero-or-one"),
                    ("relation-one-or-many-to-many", "relation-one-or-many-to-many"),
                    ("relation-one-or-many-to-one", "relation-one-or-many-to-one"),
                    ("relation-one-or-many-to-one-or-many", "relation-one-or-many-to-one-or-many"),
                    ("relation-one-or-many-to-only-one", "relation-one-or-many-to-only-one"),
                    (
                        "relation-one-or-many-to-zero-or-many",
                        "relation-one-or-many-to-zero-or-many",
                    ),
                    ("relation-one-or-many-to-zero-or-one", "relation-one-or-many-to-zero-or-one"),
                    ("relation-one-to-many", "relation-one-to-many"),
                    ("relation-one-to-one", "relation-one-to-one"),
                    ("relation-one-to-one-or-many", "relation-one-to-one-or-many"),
                    ("relation-one-to-only-one", "relation-one-to-only-one"),
                    ("relation-one-to-zero-or-many", "relation-one-to-zero-or-many"),
                    ("relation-one-to-zero-or-one", "relation-one-to-zero-or-one"),
                    ("relation-only-one-to-many", "relation-only-one-to-many"),
                    ("relation-only-one-to-one", "relation-only-one-to-one"),
                    ("relation-only-one-to-one-or-many", "relation-only-one-to-one-or-many"),
                    ("relation-only-one-to-only-one", "relation-only-one-to-only-one"),
                    ("relation-only-one-to-zero-or-many", "relation-only-one-to-zero-or-many"),
                    ("relation-only-one-to-zero-or-one", "relation-only-one-to-zero-or-one"),
                    ("relation-zero-or-many-to-many", "relation-zero-or-many-to-many"),
                    ("relation-zero-or-many-to-one", "relation-zero-or-many-to-one"),
                    (
                        "relation-zero-or-many-to-one-or-many",
                        "relation-zero-or-many-to-one-or-many",
                    ),
                    ("relation-zero-or-many-to-only-one", "relation-zero-or-many-to-only-one"),
                    (
                        "relation-zero-or-many-to-zero-or-many",
                        "relation-zero-or-many-to-zero-or-many",
                    ),
                    (
                        "relation-zero-or-many-to-zero-or-one",
                        "relation-zero-or-many-to-zero-or-one",
                    ),
                    ("relation-zero-or-one-to-many", "relation-zero-or-one-to-many"),
                    ("relation-zero-or-one-to-one", "relation-zero-or-one-to-one"),
                    ("relation-zero-or-one-to-one-or-many", "relation-zero-or-one-to-one-or-many"),
                    ("relation-zero-or-one-to-only-one", "relation-zero-or-one-to-only-one"),
                    (
                        "relation-zero-or-one-to-zero-or-many",
                        "relation-zero-or-one-to-zero-or-many",
                    ),
                    ("relation-zero-or-one-to-zero-or-one", "relation-zero-or-one-to-zero-or-one"),
                    ("relative-scale", "relative-scale"),
                    ("reload", "reload"),
                    ("reload-alert", "reload-alert"),
                    ("reminder", "reminder"),
                    ("remote", "remote"),
                    ("remote-desktop", "remote-desktop"),
                    ("remote-off", "remote-off"),
                    ("remote-tv", "remote-tv"),
                    ("remote-tv-off", "remote-tv-off"),
                    ("rename", "rename"),
                    ("rename-box", "rename-box"),
                    ("rename-box-outline", "rename-box-outline"),
                    ("rename-outline", "rename-outline"),
                    ("reorder-horizontal", "reorder-horizontal"),
                    ("reorder-vertical", "reorder-vertical"),
                    ("repeat", "repeat"),
                    ("repeat-off", "repeat-off"),
                    ("repeat-once", "repeat-once"),
                    ("repeat-variant", "repeat-variant"),
                    ("replay", "replay"),
                    ("reply", "reply"),
                    ("reply-all", "reply-all"),
                    ("reply-all-outline", "reply-all-outline"),
                    ("reply-circle", "reply-circle"),
                    ("reply-outline", "reply-outline"),
                    ("reproduction", "reproduction"),
                    ("resistor", "resistor"),
                    ("resistor-nodes", "resistor-nodes"),
                    ("resize", "resize"),
                    ("resize-bottom-right", "resize-bottom-right"),
                    ("responsive", "responsive"),
                    ("restart", "restart"),
                    ("restart-alert", "restart-alert"),
                    ("restart-off", "restart-off"),
                    ("restore", "restore"),
                    ("restore-alert", "restore-alert"),
                    ("rewind", "rewind"),
                    ("rewind-10", "rewind-10"),
                    ("rewind-15", "rewind-15"),
                    ("rewind-30", "rewind-30"),
                    ("rewind-45", "rewind-45"),
                    ("rewind-5", "rewind-5"),
                    ("rewind-60", "rewind-60"),
                    ("rewind-outline", "rewind-outline"),
                    ("rhombus", "rhombus"),
                    ("rhombus-medium", "rhombus-medium"),
                    ("rhombus-medium-outline", "rhombus-medium-outline"),
                    ("rhombus-outline", "rhombus-outline"),
                    ("rhombus-split", "rhombus-split"),
                    ("rhombus-split-outline", "rhombus-split-outline"),
                    ("ribbon", "ribbon"),
                    ("rice", "rice"),
                    ("rickshaw", "rickshaw"),
                    ("rickshaw-electric", "rickshaw-electric"),
                    ("ring", "ring"),
                    ("rivet", "rivet"),
                    ("road", "road"),
                    ("road-variant", "road-variant"),
                    ("robber", "robber"),
                    ("robot", "robot"),
                    ("robot-angry", "robot-angry"),
                    ("robot-angry-outline", "robot-angry-outline"),
                    ("robot-confused", "robot-confused"),
                    ("robot-confused-outline", "robot-confused-outline"),
                    ("robot-dead", "robot-dead"),
                    ("robot-dead-outline", "robot-dead-outline"),
                    ("robot-excited", "robot-excited"),
                    ("robot-excited-outline", "robot-excited-outline"),
                    ("robot-happy", "robot-happy"),
                    ("robot-happy-outline", "robot-happy-outline"),
                    ("robot-industrial", "robot-industrial"),
                    ("robot-industrial-outline", "robot-industrial-outline"),
                    ("robot-love", "robot-love"),
                    ("robot-love-outline", "robot-love-outline"),
                    ("robot-mower", "robot-mower"),
                    ("robot-mower-outline", "robot-mower-outline"),
                    ("robot-off", "robot-off"),
                    ("robot-off-outline", "robot-off-outline"),
                    ("robot-outline", "robot-outline"),
                    ("robot-vacuum", "robot-vacuum"),
                    ("robot-vacuum-alert", "robot-vacuum-alert"),
                    ("robot-vacuum-off", "robot-vacuum-off"),
                    ("robot-vacuum-variant", "robot-vacuum-variant"),
                    ("robot-vacuum-variant-alert", "robot-vacuum-variant-alert"),
                    ("robot-vacuum-variant-off", "robot-vacuum-variant-off"),
                    ("rocket", "rocket"),
                    ("rocket-launch", "rocket-launch"),
                    ("rocket-launch-outline", "rocket-launch-outline"),
                    ("rocket-outline", "rocket-outline"),
                    ("rodent", "rodent"),
                    ("roller-shade", "roller-shade"),
                    ("roller-shade-closed", "roller-shade-closed"),
                    ("roller-skate", "roller-skate"),
                    ("roller-skate-off", "roller-skate-off"),
                    ("rollerblade", "rollerblade"),
                    ("rollerblade-off", "rollerblade-off"),
                    ("rollupjs", "rollupjs"),
                    ("rolodex", "rolodex"),
                    ("rolodex-outline", "rolodex-outline"),
                    ("roman-numeral-1", "roman-numeral-1"),
                    ("roman-numeral-10", "roman-numeral-10"),
                    ("roman-numeral-2", "roman-numeral-2"),
                    ("roman-numeral-3", "roman-numeral-3"),
                    ("roman-numeral-4", "roman-numeral-4"),
                    ("roman-numeral-5", "roman-numeral-5"),
                    ("roman-numeral-6", "roman-numeral-6"),
                    ("roman-numeral-7", "roman-numeral-7"),
                    ("roman-numeral-8", "roman-numeral-8"),
                    ("roman-numeral-9", "roman-numeral-9"),
                    ("room-service", "room-service"),
                    ("room-service-outline", "room-service-outline"),
                    ("rotate-360", "rotate-360"),
                    ("rotate-3d", "rotate-3d"),
                    ("rotate-3d-variant", "rotate-3d-variant"),
                    ("rotate-left", "rotate-left"),
                    ("rotate-left-variant", "rotate-left-variant"),
                    ("rotate-orbit", "rotate-orbit"),
                    ("rotate-right", "rotate-right"),
                    ("rotate-right-variant", "rotate-right-variant"),
                    ("rounded-corner", "rounded-corner"),
                    ("router", "router"),
                    ("router-network", "router-network"),
                    ("router-network-wireless", "router-network-wireless"),
                    ("router-wireless", "router-wireless"),
                    ("router-wireless-off", "router-wireless-off"),
                    ("router-wireless-settings", "router-wireless-settings"),
                    ("routes", "routes"),
                    ("routes-clock", "routes-clock"),
                    ("rowing", "rowing"),
                    ("rss", "rss"),
                    ("rss-box", "rss-box"),
                    ("rss-off", "rss-off"),
                    ("rug", "rug"),
                    ("rugby", "rugby"),
                    ("ruler", "ruler"),
                    ("ruler-square", "ruler-square"),
                    ("ruler-square-compass", "ruler-square-compass"),
                    ("run", "run"),
                    ("run-fast", "run-fast"),
                    ("rv-truck", "rv-truck"),
                    ("sack", "sack"),
                    ("sack-outline", "sack-outline"),
                    ("sack-percent", "sack-percent"),
                    ("safe", "safe"),
                    ("safe-square", "safe-square"),
                    ("safe-square-outline", "safe-square-outline"),
                    ("safety-goggles", "safety-goggles"),
                    ("safety-googles", "safety-googles"),
                    ("sail-boat", "sail-boat"),
                    ("sail-boat-sink", "sail-boat-sink"),
                    ("sale", "sale"),
                    ("sale-outline", "sale-outline"),
                    ("salesforce", "salesforce"),
                    ("sass", "sass"),
                    ("satellite", "satellite"),
                    ("satellite-uplink", "satellite-uplink"),
                    ("satellite-variant", "satellite-variant"),
                    ("sausage", "sausage"),
                    ("sausage-off", "sausage-off"),
                    ("saw-blade", "saw-blade"),
                    ("sawtooth-wave", "sawtooth-wave"),
                    ("saxophone", "saxophone"),
                    ("scale", "scale"),
                    ("scale-balance", "scale-balance"),
                    ("scale-bathroom", "scale-bathroom"),
                    ("scale-off", "scale-off"),
                    ("scale-unbalanced", "scale-unbalanced"),
                    ("scan-helper", "scan-helper"),
                    ("scanner", "scanner"),
                    ("scanner-off", "scanner-off"),
                    ("scatter-plot", "scatter-plot"),
                    ("scatter-plot-outline", "scatter-plot-outline"),
                    ("scent", "scent"),
                    ("scent-off", "scent-off"),
                    ("school", "school"),
                    ("school-outline", "school-outline"),
                    ("scissors-cutting", "scissors-cutting"),
                    ("scooter", "scooter"),
                    ("scooter-electric", "scooter-electric"),
                    ("scoreboard", "scoreboard"),
                    ("scoreboard-outline", "scoreboard-outline"),
                    ("screen-rotation", "screen-rotation"),
                    ("screen-rotation-lock", "screen-rotation-lock"),
                    ("screw-flat-top", "screw-flat-top"),
                    ("screw-lag", "screw-lag"),
                    ("screw-machine-flat-top", "screw-machine-flat-top"),
                    ("screw-machine-round-top", "screw-machine-round-top"),
                    ("screw-round-top", "screw-round-top"),
                    ("screwdriver", "screwdriver"),
                    ("script", "script"),
                    ("script-outline", "script-outline"),
                    ("script-text", "script-text"),
                    ("script-text-key", "script-text-key"),
                    ("script-text-key-outline", "script-text-key-outline"),
                    ("script-text-outline", "script-text-outline"),
                    ("script-text-play", "script-text-play"),
                    ("script-text-play-outline", "script-text-play-outline"),
                    ("sd", "sd"),
                    ("seal", "seal"),
                    ("seal-variant", "seal-variant"),
                    ("search-web", "search-web"),
                    ("seat", "seat"),
                    ("seat-flat", "seat-flat"),
                    ("seat-flat-angled", "seat-flat-angled"),
                    ("seat-individual-suite", "seat-individual-suite"),
                    ("seat-legroom-extra", "seat-legroom-extra"),
                    ("seat-legroom-normal", "seat-legroom-normal"),
                    ("seat-legroom-reduced", "seat-legroom-reduced"),
                    ("seat-outline", "seat-outline"),
                    ("seat-passenger", "seat-passenger"),
                    ("seat-recline-extra", "seat-recline-extra"),
                    ("seat-recline-normal", "seat-recline-normal"),
                    ("seatbelt", "seatbelt"),
                    ("security", "security"),
                    ("security-close", "security-close"),
                    ("security-network", "security-network"),
                    ("seed", "seed"),
                    ("seed-off", "seed-off"),
                    ("seed-off-outline", "seed-off-outline"),
                    ("seed-outline", "seed-outline"),
                    ("seed-plus", "seed-plus"),
                    ("seed-plus-outline", "seed-plus-outline"),
                    ("seesaw", "seesaw"),
                    ("segment", "segment"),
                    ("select", "select"),
                    ("select-all", "select-all"),
                    ("select-arrow-down", "select-arrow-down"),
                    ("select-arrow-up", "select-arrow-up"),
                    ("select-color", "select-color"),
                    ("select-compare", "select-compare"),
                    ("select-drag", "select-drag"),
                    ("select-group", "select-group"),
                    ("select-inverse", "select-inverse"),
                    ("select-marker", "select-marker"),
                    ("select-multiple", "select-multiple"),
                    ("select-multiple-marker", "select-multiple-marker"),
                    ("select-off", "select-off"),
                    ("select-place", "select-place"),
                    ("select-remove", "select-remove"),
                    ("select-search", "select-search"),
                    ("selection", "selection"),
                    ("selection-drag", "selection-drag"),
                    ("selection-ellipse", "selection-ellipse"),
                    ("selection-ellipse-arrow-inside", "selection-ellipse-arrow-inside"),
                    ("selection-ellipse-remove", "selection-ellipse-remove"),
                    ("selection-lasso", "selection-lasso"),
                    ("selection-marker", "selection-marker"),
                    ("selection-multiple", "selection-multiple"),
                    ("selection-multiple-marker", "selection-multiple-marker"),
                    ("selection-off", "selection-off"),
                    ("selection-remove", "selection-remove"),
                    ("selection-search", "selection-search"),
                    ("semantic-web", "semantic-web"),
                    ("send", "send"),
                    ("send-check", "send-check"),
                    ("send-check-outline", "send-check-outline"),
                    ("send-circle", "send-circle"),
                    ("send-circle-outline", "send-circle-outline"),
                    ("send-clock", "send-clock"),
                    ("send-clock-outline", "send-clock-outline"),
                    ("send-lock", "send-lock"),
                    ("send-lock-outline", "send-lock-outline"),
                    ("send-outline", "send-outline"),
                    ("send-variant", "send-variant"),
                    ("send-variant-clock", "send-variant-clock"),
                    ("send-variant-clock-outline", "send-variant-clock-outline"),
                    ("send-variant-outline", "send-variant-outline"),
                    ("serial-port", "serial-port"),
                    ("server", "server"),
                    ("server-minus", "server-minus"),
                    ("server-minus-outline", "server-minus-outline"),
                    ("server-network", "server-network"),
                    ("server-network-off", "server-network-off"),
                    ("server-network-outline", "server-network-outline"),
                    ("server-off", "server-off"),
                    ("server-outline", "server-outline"),
                    ("server-plus", "server-plus"),
                    ("server-plus-outline", "server-plus-outline"),
                    ("server-remove", "server-remove"),
                    ("server-security", "server-security"),
                    ("set-all", "set-all"),
                    ("set-center", "set-center"),
                    ("set-center-right", "set-center-right"),
                    ("set-left", "set-left"),
                    ("set-left-center", "set-left-center"),
                    ("set-left-right", "set-left-right"),
                    ("set-merge", "set-merge"),
                    ("set-none", "set-none"),
                    ("set-right", "set-right"),
                    ("set-split", "set-split"),
                    ("set-square", "set-square"),
                    ("set-top-box", "set-top-box"),
                    ("settings-helper", "settings-helper"),
                    ("shaker", "shaker"),
                    ("shaker-outline", "shaker-outline"),
                    ("shape", "shape"),
                    ("shape-circle-plus", "shape-circle-plus"),
                    ("shape-outline", "shape-outline"),
                    ("shape-oval-plus", "shape-oval-plus"),
                    ("shape-plus", "shape-plus"),
                    ("shape-plus-outline", "shape-plus-outline"),
                    ("shape-polygon-plus", "shape-polygon-plus"),
                    ("shape-rectangle-plus", "shape-rectangle-plus"),
                    ("shape-square-plus", "shape-square-plus"),
                    ("shape-square-rounded-plus", "shape-square-rounded-plus"),
                    ("share", "share"),
                    ("share-all", "share-all"),
                    ("share-all-outline", "share-all-outline"),
                    ("share-circle", "share-circle"),
                    ("share-off", "share-off"),
                    ("share-off-outline", "share-off-outline"),
                    ("share-outline", "share-outline"),
                    ("share-variant", "share-variant"),
                    ("share-variant-outline", "share-variant-outline"),
                    ("shark", "shark"),
                    ("shark-fin", "shark-fin"),
                    ("shark-fin-outline", "shark-fin-outline"),
                    ("shark-off", "shark-off"),
                    ("sheep", "sheep"),
                    ("shield", "shield"),
                    ("shield-account", "shield-account"),
                    ("shield-account-outline", "shield-account-outline"),
                    ("shield-account-variant", "shield-account-variant"),
                    ("shield-account-variant-outline", "shield-account-variant-outline"),
                    ("shield-airplane", "shield-airplane"),
                    ("shield-airplane-outline", "shield-airplane-outline"),
                    ("shield-alert", "shield-alert"),
                    ("shield-alert-outline", "shield-alert-outline"),
                    ("shield-bug", "shield-bug"),
                    ("shield-bug-outline", "shield-bug-outline"),
                    ("shield-car", "shield-car"),
                    ("shield-check", "shield-check"),
                    ("shield-check-outline", "shield-check-outline"),
                    ("shield-cross", "shield-cross"),
                    ("shield-cross-outline", "shield-cross-outline"),
                    ("shield-crown", "shield-crown"),
                    ("shield-crown-outline", "shield-crown-outline"),
                    ("shield-edit", "shield-edit"),
                    ("shield-edit-outline", "shield-edit-outline"),
                    ("shield-half", "shield-half"),
                    ("shield-half-full", "shield-half-full"),
                    ("shield-home", "shield-home"),
                    ("shield-home-outline", "shield-home-outline"),
                    ("shield-key", "shield-key"),
                    ("shield-key-outline", "shield-key-outline"),
                    ("shield-link-variant", "shield-link-variant"),
                    ("shield-link-variant-outline", "shield-link-variant-outline"),
                    ("shield-lock", "shield-lock"),
                    ("shield-lock-open", "shield-lock-open"),
                    ("shield-lock-open-outline", "shield-lock-open-outline"),
                    ("shield-lock-outline", "shield-lock-outline"),
                    ("shield-moon", "shield-moon"),
                    ("shield-moon-outline", "shield-moon-outline"),
                    ("shield-off", "shield-off"),
                    ("shield-off-outline", "shield-off-outline"),
                    ("shield-outline", "shield-outline"),
                    ("shield-plus", "shield-plus"),
                    ("shield-plus-outline", "shield-plus-outline"),
                    ("shield-refresh", "shield-refresh"),
                    ("shield-refresh-outline", "shield-refresh-outline"),
                    ("shield-remove", "shield-remove"),
                    ("shield-remove-outline", "shield-remove-outline"),
                    ("shield-search", "shield-search"),
                    ("shield-star", "shield-star"),
                    ("shield-star-outline", "shield-star-outline"),
                    ("shield-sun", "shield-sun"),
                    ("shield-sun-outline", "shield-sun-outline"),
                    ("shield-sword", "shield-sword"),
                    ("shield-sword-outline", "shield-sword-outline"),
                    ("shield-sync", "shield-sync"),
                    ("shield-sync-outline", "shield-sync-outline"),
                    ("shimmer", "shimmer"),
                    ("ship-wheel", "ship-wheel"),
                    ("shipping-pallet", "shipping-pallet"),
                    ("shoe-ballet", "shoe-ballet"),
                    ("shoe-cleat", "shoe-cleat"),
                    ("shoe-formal", "shoe-formal"),
                    ("shoe-heel", "shoe-heel"),
                    ("shoe-print", "shoe-print"),
                    ("shoe-sneaker", "shoe-sneaker"),
                    ("shopify", "shopify"),
                    ("shopping", "shopping"),
                    ("shopping-music", "shopping-music"),
                    ("shopping-outline", "shopping-outline"),
                    ("shopping-search", "shopping-search"),
                    ("shopping-search-outline", "shopping-search-outline"),
                    ("shore", "shore"),
                    ("shovel", "shovel"),
                    ("shovel-off", "shovel-off"),
                    ("shower", "shower"),
                    ("shower-head", "shower-head"),
                    ("shredder", "shredder"),
                    ("shuffle", "shuffle"),
                    ("shuffle-disabled", "shuffle-disabled"),
                    ("shuffle-variant", "shuffle-variant"),
                    ("shuriken", "shuriken"),
                    ("sickle", "sickle"),
                    ("sigma", "sigma"),
                    ("sigma-lower", "sigma-lower"),
                    ("sign-caution", "sign-caution"),
                    ("sign-direction", "sign-direction"),
                    ("sign-direction-minus", "sign-direction-minus"),
                    ("sign-direction-plus", "sign-direction-plus"),
                    ("sign-direction-remove", "sign-direction-remove"),
                    ("sign-language", "sign-language"),
                    ("sign-language-outline", "sign-language-outline"),
                    ("sign-pole", "sign-pole"),
                    ("sign-real-estate", "sign-real-estate"),
                    ("sign-text", "sign-text"),
                    ("sign-yield", "sign-yield"),
                    ("signal", "signal"),
                    ("signal-2g", "signal-2g"),
                    ("signal-3g", "signal-3g"),
                    ("signal-4g", "signal-4g"),
                    ("signal-5g", "signal-5g"),
                    ("signal-cellular-1", "signal-cellular-1"),
                    ("signal-cellular-2", "signal-cellular-2"),
                    ("signal-cellular-3", "signal-cellular-3"),
                    ("signal-cellular-outline", "signal-cellular-outline"),
                    ("signal-distance-variant", "signal-distance-variant"),
                    ("signal-hspa", "signal-hspa"),
                    ("signal-hspa-plus", "signal-hspa-plus"),
                    ("signal-off", "signal-off"),
                    ("signal-variant", "signal-variant"),
                    ("signature", "signature"),
                    ("signature-freehand", "signature-freehand"),
                    ("signature-image", "signature-image"),
                    ("signature-text", "signature-text"),
                    ("silo", "silo"),
                    ("silo-outline", "silo-outline"),
                    ("silverware", "silverware"),
                    ("silverware-clean", "silverware-clean"),
                    ("silverware-fork", "silverware-fork"),
                    ("silverware-fork-knife", "silverware-fork-knife"),
                    ("silverware-spoon", "silverware-spoon"),
                    ("silverware-variant", "silverware-variant"),
                    ("sim", "sim"),
                    ("sim-alert", "sim-alert"),
                    ("sim-alert-outline", "sim-alert-outline"),
                    ("sim-off", "sim-off"),
                    ("sim-off-outline", "sim-off-outline"),
                    ("sim-outline", "sim-outline"),
                    ("simple-icons", "simple-icons"),
                    ("sina-weibo", "sina-weibo"),
                    ("sine-wave", "sine-wave"),
                    ("sitemap", "sitemap"),
                    ("sitemap-outline", "sitemap-outline"),
                    ("size-l", "size-l"),
                    ("size-m", "size-m"),
                    ("size-s", "size-s"),
                    ("size-xl", "size-xl"),
                    ("size-xs", "size-xs"),
                    ("size-xxl", "size-xxl"),
                    ("size-xxs", "size-xxs"),
                    ("size-xxxl", "size-xxxl"),
                    ("skate", "skate"),
                    ("skate-off", "skate-off"),
                    ("skateboard", "skateboard"),
                    ("skateboarding", "skateboarding"),
                    ("skew-less", "skew-less"),
                    ("skew-more", "skew-more"),
                    ("ski", "ski"),
                    ("ski-cross-country", "ski-cross-country"),
                    ("ski-water", "ski-water"),
                    ("skip-backward", "skip-backward"),
                    ("skip-backward-outline", "skip-backward-outline"),
                    ("skip-forward", "skip-forward"),
                    ("skip-forward-outline", "skip-forward-outline"),
                    ("skip-next", "skip-next"),
                    ("skip-next-circle", "skip-next-circle"),
                    ("skip-next-circle-outline", "skip-next-circle-outline"),
                    ("skip-next-outline", "skip-next-outline"),
                    ("skip-previous", "skip-previous"),
                    ("skip-previous-circle", "skip-previous-circle"),
                    ("skip-previous-circle-outline", "skip-previous-circle-outline"),
                    ("skip-previous-outline", "skip-previous-outline"),
                    ("skull", "skull"),
                    ("skull-crossbones", "skull-crossbones"),
                    ("skull-crossbones-outline", "skull-crossbones-outline"),
                    ("skull-outline", "skull-outline"),
                    ("skull-scan", "skull-scan"),
                    ("skull-scan-outline", "skull-scan-outline"),
                    ("skype", "skype"),
                    ("skype-business", "skype-business"),
                    ("slack", "slack"),
                    ("slackware", "slackware"),
                    ("slash-forward", "slash-forward"),
                    ("slash-forward-box", "slash-forward-box"),
                    ("sledding", "sledding"),
                    ("sleep", "sleep"),
                    ("sleep-off", "sleep-off"),
                    ("slide", "slide"),
                    ("slope-downhill", "slope-downhill"),
                    ("slope-uphill", "slope-uphill"),
                    ("slot-machine", "slot-machine"),
                    ("slot-machine-outline", "slot-machine-outline"),
                    ("smart-card", "smart-card"),
                    ("smart-card-off", "smart-card-off"),
                    ("smart-card-off-outline", "smart-card-off-outline"),
                    ("smart-card-outline", "smart-card-outline"),
                    ("smart-card-reader", "smart-card-reader"),
                    ("smart-card-reader-outline", "smart-card-reader-outline"),
                    ("smog", "smog"),
                    ("smoke", "smoke"),
                    ("smoke-detector", "smoke-detector"),
                    ("smoke-detector-alert", "smoke-detector-alert"),
                    ("smoke-detector-alert-outline", "smoke-detector-alert-outline"),
                    ("smoke-detector-off", "smoke-detector-off"),
                    ("smoke-detector-off-outline", "smoke-detector-off-outline"),
                    ("smoke-detector-outline", "smoke-detector-outline"),
                    ("smoke-detector-variant", "smoke-detector-variant"),
                    ("smoke-detector-variant-alert", "smoke-detector-variant-alert"),
                    ("smoke-detector-variant-off", "smoke-detector-variant-off"),
                    ("smoking", "smoking"),
                    ("smoking-off", "smoking-off"),
                    ("smoking-pipe", "smoking-pipe"),
                    ("smoking-pipe-off", "smoking-pipe-off"),
                    ("snail", "snail"),
                    ("snake", "snake"),
                    ("snapchat", "snapchat"),
                    ("snowboard", "snowboard"),
                    ("snowflake", "snowflake"),
                    ("snowflake-alert", "snowflake-alert"),
                    ("snowflake-check", "snowflake-check"),
                    ("snowflake-melt", "snowflake-melt"),
                    ("snowflake-off", "snowflake-off"),
                    ("snowflake-thermometer", "snowflake-thermometer"),
                    ("snowflake-variant", "snowflake-variant"),
                    ("snowman", "snowman"),
                    ("snowmobile", "snowmobile"),
                    ("snowshoeing", "snowshoeing"),
                    ("soccer", "soccer"),
                    ("soccer-field", "soccer-field"),
                    ("social-distance-2-meters", "social-distance-2-meters"),
                    ("social-distance-6-feet", "social-distance-6-feet"),
                    ("sofa", "sofa"),
                    ("sofa-outline", "sofa-outline"),
                    ("sofa-single", "sofa-single"),
                    ("sofa-single-outline", "sofa-single-outline"),
                    ("solar-panel", "solar-panel"),
                    ("solar-panel-large", "solar-panel-large"),
                    ("solar-power", "solar-power"),
                    ("solar-power-variant", "solar-power-variant"),
                    ("solar-power-variant-outline", "solar-power-variant-outline"),
                    ("soldering-iron", "soldering-iron"),
                    ("solid", "solid"),
                    ("sony-playstation", "sony-playstation"),
                    ("sort", "sort"),
                    ("sort-alphabetical-ascending", "sort-alphabetical-ascending"),
                    ("sort-alphabetical-ascending-variant", "sort-alphabetical-ascending-variant"),
                    ("sort-alphabetical-descending", "sort-alphabetical-descending"),
                    (
                        "sort-alphabetical-descending-variant",
                        "sort-alphabetical-descending-variant",
                    ),
                    ("sort-alphabetical-variant", "sort-alphabetical-variant"),
                    ("sort-ascending", "sort-ascending"),
                    ("sort-bool-ascending", "sort-bool-ascending"),
                    ("sort-bool-ascending-variant", "sort-bool-ascending-variant"),
                    ("sort-bool-descending", "sort-bool-descending"),
                    ("sort-bool-descending-variant", "sort-bool-descending-variant"),
                    ("sort-calendar-ascending", "sort-calendar-ascending"),
                    ("sort-calendar-descending", "sort-calendar-descending"),
                    ("sort-clock-ascending", "sort-clock-ascending"),
                    ("sort-clock-ascending-outline", "sort-clock-ascending-outline"),
                    ("sort-clock-descending", "sort-clock-descending"),
                    ("sort-clock-descending-outline", "sort-clock-descending-outline"),
                    ("sort-descending", "sort-descending"),
                    ("sort-numeric-ascending", "sort-numeric-ascending"),
                    ("sort-numeric-ascending-variant", "sort-numeric-ascending-variant"),
                    ("sort-numeric-descending", "sort-numeric-descending"),
                    ("sort-numeric-descending-variant", "sort-numeric-descending-variant"),
                    ("sort-numeric-variant", "sort-numeric-variant"),
                    ("sort-reverse-variant", "sort-reverse-variant"),
                    ("sort-variant", "sort-variant"),
                    ("sort-variant-lock", "sort-variant-lock"),
                    ("sort-variant-lock-open", "sort-variant-lock-open"),
                    ("sort-variant-off", "sort-variant-off"),
                    ("sort-variant-remove", "sort-variant-remove"),
                    ("soundbar", "soundbar"),
                    ("soundcloud", "soundcloud"),
                    ("source-branch", "source-branch"),
                    ("source-branch-check", "source-branch-check"),
                    ("source-branch-minus", "source-branch-minus"),
                    ("source-branch-plus", "source-branch-plus"),
                    ("source-branch-refresh", "source-branch-refresh"),
                    ("source-branch-remove", "source-branch-remove"),
                    ("source-branch-sync", "source-branch-sync"),
                    ("source-commit", "source-commit"),
                    ("source-commit-end", "source-commit-end"),
                    ("source-commit-end-local", "source-commit-end-local"),
                    ("source-commit-local", "source-commit-local"),
                    ("source-commit-next-local", "source-commit-next-local"),
                    ("source-commit-start", "source-commit-start"),
                    ("source-commit-start-next-local", "source-commit-start-next-local"),
                    ("source-fork", "source-fork"),
                    ("source-merge", "source-merge"),
                    ("source-pull", "source-pull"),
                    ("source-repository", "source-repository"),
                    ("source-repository-multiple", "source-repository-multiple"),
                    ("soy-sauce", "soy-sauce"),
                    ("soy-sauce-off", "soy-sauce-off"),
                    ("spa", "spa"),
                    ("spa-outline", "spa-outline"),
                    ("space-invaders", "space-invaders"),
                    ("space-station", "space-station"),
                    ("spade", "spade"),
                    ("speaker", "speaker"),
                    ("speaker-bluetooth", "speaker-bluetooth"),
                    ("speaker-message", "speaker-message"),
                    ("speaker-multiple", "speaker-multiple"),
                    ("speaker-off", "speaker-off"),
                    ("speaker-pause", "speaker-pause"),
                    ("speaker-play", "speaker-play"),
                    ("speaker-stop", "speaker-stop"),
                    ("speaker-wireless", "speaker-wireless"),
                    ("spear", "spear"),
                    ("speedometer", "speedometer"),
                    ("speedometer-medium", "speedometer-medium"),
                    ("speedometer-slow", "speedometer-slow"),
                    ("spellcheck", "spellcheck"),
                    ("sphere", "sphere"),
                    ("sphere-off", "sphere-off"),
                    ("spider", "spider"),
                    ("spider-outline", "spider-outline"),
                    ("spider-thread", "spider-thread"),
                    ("spider-web", "spider-web"),
                    ("spirit-level", "spirit-level"),
                    ("split-horizontal", "split-horizontal"),
                    ("split-vertical", "split-vertical"),
                    ("spoon-sugar", "spoon-sugar"),
                    ("spotify", "spotify"),
                    ("spotlight", "spotlight"),
                    ("spotlight-beam", "spotlight-beam"),
                    ("spray", "spray"),
                    ("spray-bottle", "spray-bottle"),
                    ("spreadsheet", "spreadsheet"),
                    ("sprinkler", "sprinkler"),
                    ("sprinkler-fire", "sprinkler-fire"),
                    ("sprinkler-variant", "sprinkler-variant"),
                    ("sprout", "sprout"),
                    ("sprout-outline", "sprout-outline"),
                    ("square", "square"),
                    ("square-circle", "square-circle"),
                    ("square-circle-outline", "square-circle-outline"),
                    ("square-edit-outline", "square-edit-outline"),
                    ("square-inc", "square-inc"),
                    ("square-inc-cash", "square-inc-cash"),
                    ("square-medium", "square-medium"),
                    ("square-medium-outline", "square-medium-outline"),
                    ("square-off", "square-off"),
                    ("square-off-outline", "square-off-outline"),
                    ("square-opacity", "square-opacity"),
                    ("square-outline", "square-outline"),
                    ("square-root", "square-root"),
                    ("square-root-box", "square-root-box"),
                    ("square-rounded", "square-rounded"),
                    ("square-rounded-badge", "square-rounded-badge"),
                    ("square-rounded-badge-outline", "square-rounded-badge-outline"),
                    ("square-rounded-outline", "square-rounded-outline"),
                    ("square-small", "square-small"),
                    ("square-wave", "square-wave"),
                    ("squeegee", "squeegee"),
                    ("ssh", "ssh"),
                    ("stack-exchange", "stack-exchange"),
                    ("stack-overflow", "stack-overflow"),
                    ("stackpath", "stackpath"),
                    ("stadium", "stadium"),
                    ("stadium-outline", "stadium-outline"),
                    ("stadium-variant", "stadium-variant"),
                    ("stairs", "stairs"),
                    ("stairs-box", "stairs-box"),
                    ("stairs-down", "stairs-down"),
                    ("stairs-up", "stairs-up"),
                    ("stamper", "stamper"),
                    ("standard-definition", "standard-definition"),
                    ("star", "star"),
                    ("star-box", "star-box"),
                    ("star-box-multiple", "star-box-multiple"),
                    ("star-box-multiple-outline", "star-box-multiple-outline"),
                    ("star-box-outline", "star-box-outline"),
                    ("star-check", "star-check"),
                    ("star-check-outline", "star-check-outline"),
                    ("star-circle", "star-circle"),
                    ("star-circle-outline", "star-circle-outline"),
                    ("star-cog", "star-cog"),
                    ("star-cog-outline", "star-cog-outline"),
                    ("star-crescent", "star-crescent"),
                    ("star-david", "star-david"),
                    ("star-face", "star-face"),
                    ("star-four-points", "star-four-points"),
                    ("star-four-points-box", "star-four-points-box"),
                    ("star-four-points-box-outline", "star-four-points-box-outline"),
                    ("star-four-points-circle", "star-four-points-circle"),
                    ("star-four-points-circle-outline", "star-four-points-circle-outline"),
                    ("star-four-points-outline", "star-four-points-outline"),
                    ("star-four-points-small", "star-four-points-small"),
                    ("star-half", "star-half"),
                    ("star-half-full", "star-half-full"),
                    ("star-minus", "star-minus"),
                    ("star-minus-outline", "star-minus-outline"),
                    ("star-off", "star-off"),
                    ("star-off-outline", "star-off-outline"),
                    ("star-outline", "star-outline"),
                    ("star-plus", "star-plus"),
                    ("star-plus-outline", "star-plus-outline"),
                    ("star-remove", "star-remove"),
                    ("star-remove-outline", "star-remove-outline"),
                    ("star-settings", "star-settings"),
                    ("star-settings-outline", "star-settings-outline"),
                    ("star-shooting", "star-shooting"),
                    ("star-shooting-outline", "star-shooting-outline"),
                    ("star-three-points", "star-three-points"),
                    ("star-three-points-outline", "star-three-points-outline"),
                    ("state-machine", "state-machine"),
                    ("steam", "steam"),
                    ("steam-box", "steam-box"),
                    ("steering", "steering"),
                    ("steering-off", "steering-off"),
                    ("step-backward", "step-backward"),
                    ("step-backward-2", "step-backward-2"),
                    ("step-forward", "step-forward"),
                    ("step-forward-2", "step-forward-2"),
                    ("stethoscope", "stethoscope"),
                    ("sticker", "sticker"),
                    ("sticker-alert", "sticker-alert"),
                    ("sticker-alert-outline", "sticker-alert-outline"),
                    ("sticker-check", "sticker-check"),
                    ("sticker-check-outline", "sticker-check-outline"),
                    ("sticker-circle-outline", "sticker-circle-outline"),
                    ("sticker-emoji", "sticker-emoji"),
                    ("sticker-minus", "sticker-minus"),
                    ("sticker-minus-outline", "sticker-minus-outline"),
                    ("sticker-outline", "sticker-outline"),
                    ("sticker-plus", "sticker-plus"),
                    ("sticker-plus-outline", "sticker-plus-outline"),
                    ("sticker-remove", "sticker-remove"),
                    ("sticker-remove-outline", "sticker-remove-outline"),
                    ("sticker-text", "sticker-text"),
                    ("sticker-text-outline", "sticker-text-outline"),
                    ("stocking", "stocking"),
                    ("stomach", "stomach"),
                    ("stool", "stool"),
                    ("stool-outline", "stool-outline"),
                    ("stop", "stop"),
                    ("stop-circle", "stop-circle"),
                    ("stop-circle-outline", "stop-circle-outline"),
                    ("storage-tank", "storage-tank"),
                    ("storage-tank-outline", "storage-tank-outline"),
                    ("store", "store"),
                    ("store-24-hour", "store-24-hour"),
                    ("store-alert", "store-alert"),
                    ("store-alert-outline", "store-alert-outline"),
                    ("store-check", "store-check"),
                    ("store-check-outline", "store-check-outline"),
                    ("store-clock", "store-clock"),
                    ("store-clock-outline", "store-clock-outline"),
                    ("store-cog", "store-cog"),
                    ("store-cog-outline", "store-cog-outline"),
                    ("store-edit", "store-edit"),
                    ("store-edit-outline", "store-edit-outline"),
                    ("store-marker", "store-marker"),
                    ("store-marker-outline", "store-marker-outline"),
                    ("store-minus", "store-minus"),
                    ("store-minus-outline", "store-minus-outline"),
                    ("store-off", "store-off"),
                    ("store-off-outline", "store-off-outline"),
                    ("store-outline", "store-outline"),
                    ("store-plus", "store-plus"),
                    ("store-plus-outline", "store-plus-outline"),
                    ("store-remove", "store-remove"),
                    ("store-remove-outline", "store-remove-outline"),
                    ("store-search", "store-search"),
                    ("store-search-outline", "store-search-outline"),
                    ("store-settings", "store-settings"),
                    ("store-settings-outline", "store-settings-outline"),
                    ("storefront", "storefront"),
                    ("storefront-check", "storefront-check"),
                    ("storefront-check-outline", "storefront-check-outline"),
                    ("storefront-edit", "storefront-edit"),
                    ("storefront-edit-outline", "storefront-edit-outline"),
                    ("storefront-minus", "storefront-minus"),
                    ("storefront-minus-outline", "storefront-minus-outline"),
                    ("storefront-outline", "storefront-outline"),
                    ("storefront-plus", "storefront-plus"),
                    ("storefront-plus-outline", "storefront-plus-outline"),
                    ("storefront-remove", "storefront-remove"),
                    ("storefront-remove-outline", "storefront-remove-outline"),
                    ("stove", "stove"),
                    ("strategy", "strategy"),
                    ("strava", "strava"),
                    ("stretch-to-page", "stretch-to-page"),
                    ("stretch-to-page-outline", "stretch-to-page-outline"),
                    ("string-lights", "string-lights"),
                    ("string-lights-off", "string-lights-off"),
                    ("subdirectory-arrow-left", "subdirectory-arrow-left"),
                    ("subdirectory-arrow-right", "subdirectory-arrow-right"),
                    ("submarine", "submarine"),
                    ("subtitles", "subtitles"),
                    ("subtitles-outline", "subtitles-outline"),
                    ("subway", "subway"),
                    ("subway-alert-variant", "subway-alert-variant"),
                    ("subway-variant", "subway-variant"),
                    ("summit", "summit"),
                    ("sun-angle", "sun-angle"),
                    ("sun-angle-outline", "sun-angle-outline"),
                    ("sun-clock", "sun-clock"),
                    ("sun-clock-outline", "sun-clock-outline"),
                    ("sun-compass", "sun-compass"),
                    ("sun-snowflake", "sun-snowflake"),
                    ("sun-snowflake-variant", "sun-snowflake-variant"),
                    ("sun-thermometer", "sun-thermometer"),
                    ("sun-thermometer-outline", "sun-thermometer-outline"),
                    ("sun-wireless", "sun-wireless"),
                    ("sun-wireless-outline", "sun-wireless-outline"),
                    ("sunglasses", "sunglasses"),
                    ("surfing", "surfing"),
                    ("surround-sound", "surround-sound"),
                    ("surround-sound-2-0", "surround-sound-2-0"),
                    ("surround-sound-2-1", "surround-sound-2-1"),
                    ("surround-sound-3-1", "surround-sound-3-1"),
                    ("surround-sound-5-1", "surround-sound-5-1"),
                    ("surround-sound-5-1-2", "surround-sound-5-1-2"),
                    ("surround-sound-7-1", "surround-sound-7-1"),
                    ("svg", "svg"),
                    ("swap-horizontal", "swap-horizontal"),
                    ("swap-horizontal-bold", "swap-horizontal-bold"),
                    ("swap-horizontal-circle", "swap-horizontal-circle"),
                    ("swap-horizontal-circle-outline", "swap-horizontal-circle-outline"),
                    ("swap-horizontal-variant", "swap-horizontal-variant"),
                    ("swap-vertical", "swap-vertical"),
                    ("swap-vertical-bold", "swap-vertical-bold"),
                    ("swap-vertical-circle", "swap-vertical-circle"),
                    ("swap-vertical-circle-outline", "swap-vertical-circle-outline"),
                    ("swap-vertical-variant", "swap-vertical-variant"),
                    ("swim", "swim"),
                    ("switch", "switch"),
                    ("sword", "sword"),
                    ("sword-cross", "sword-cross"),
                    ("syllabary-hangul", "syllabary-hangul"),
                    ("syllabary-hiragana", "syllabary-hiragana"),
                    ("syllabary-katakana", "syllabary-katakana"),
                    ("syllabary-katakana-halfwidth", "syllabary-katakana-halfwidth"),
                    ("symbol", "symbol"),
                    ("symfony", "symfony"),
                    ("synagogue", "synagogue"),
                    ("synagogue-outline", "synagogue-outline"),
                    ("sync", "sync"),
                    ("sync-alert", "sync-alert"),
                    ("sync-circle", "sync-circle"),
                    ("sync-off", "sync-off"),
                    ("tab", "tab"),
                    ("tab-minus", "tab-minus"),
                    ("tab-plus", "tab-plus"),
                    ("tab-remove", "tab-remove"),
                    ("tab-search", "tab-search"),
                    ("tab-unselected", "tab-unselected"),
                    ("table", "table"),
                    ("table-account", "table-account"),
                    ("table-alert", "table-alert"),
                    ("table-arrow-down", "table-arrow-down"),
                    ("table-arrow-left", "table-arrow-left"),
                    ("table-arrow-right", "table-arrow-right"),
                    ("table-arrow-up", "table-arrow-up"),
                    ("table-border", "table-border"),
                    ("table-cancel", "table-cancel"),
                    ("table-chair", "table-chair"),
                    ("table-check", "table-check"),
                    ("table-clock", "table-clock"),
                    ("table-cog", "table-cog"),
                    ("table-column", "table-column"),
                    ("table-column-plus-after", "table-column-plus-after"),
                    ("table-column-plus-before", "table-column-plus-before"),
                    ("table-column-remove", "table-column-remove"),
                    ("table-column-width", "table-column-width"),
                    ("table-edit", "table-edit"),
                    ("table-eye", "table-eye"),
                    ("table-eye-off", "table-eye-off"),
                    ("table-filter", "table-filter"),
                    ("table-furniture", "table-furniture"),
                    ("table-headers-eye", "table-headers-eye"),
                    ("table-headers-eye-off", "table-headers-eye-off"),
                    ("table-heart", "table-heart"),
                    ("table-key", "table-key"),
                    ("table-large", "table-large"),
                    ("table-large-plus", "table-large-plus"),
                    ("table-large-remove", "table-large-remove"),
                    ("table-lock", "table-lock"),
                    ("table-merge-cells", "table-merge-cells"),
                    ("table-minus", "table-minus"),
                    ("table-multiple", "table-multiple"),
                    ("table-network", "table-network"),
                    ("table-of-contents", "table-of-contents"),
                    ("table-off", "table-off"),
                    ("table-picnic", "table-picnic"),
                    ("table-pivot", "table-pivot"),
                    ("table-plus", "table-plus"),
                    ("table-question", "table-question"),
                    ("table-refresh", "table-refresh"),
                    ("table-remove", "table-remove"),
                    ("table-row", "table-row"),
                    ("table-row-height", "table-row-height"),
                    ("table-row-plus-after", "table-row-plus-after"),
                    ("table-row-plus-before", "table-row-plus-before"),
                    ("table-row-remove", "table-row-remove"),
                    ("table-search", "table-search"),
                    ("table-settings", "table-settings"),
                    ("table-split-cell", "table-split-cell"),
                    ("table-star", "table-star"),
                    ("table-sync", "table-sync"),
                    ("table-tennis", "table-tennis"),
                    ("tablet", "tablet"),
                    ("tablet-android", "tablet-android"),
                    ("tablet-cellphone", "tablet-cellphone"),
                    ("tablet-dashboard", "tablet-dashboard"),
                    ("tablet-ipad", "tablet-ipad"),
                    ("taco", "taco"),
                    ("tag", "tag"),
                    ("tag-arrow-down", "tag-arrow-down"),
                    ("tag-arrow-down-outline", "tag-arrow-down-outline"),
                    ("tag-arrow-left", "tag-arrow-left"),
                    ("tag-arrow-left-outline", "tag-arrow-left-outline"),
                    ("tag-arrow-right", "tag-arrow-right"),
                    ("tag-arrow-right-outline", "tag-arrow-right-outline"),
                    ("tag-arrow-up", "tag-arrow-up"),
                    ("tag-arrow-up-outline", "tag-arrow-up-outline"),
                    ("tag-check", "tag-check"),
                    ("tag-check-outline", "tag-check-outline"),
                    ("tag-edit", "tag-edit"),
                    ("tag-edit-outline", "tag-edit-outline"),
                    ("tag-faces", "tag-faces"),
                    ("tag-heart", "tag-heart"),
                    ("tag-heart-outline", "tag-heart-outline"),
                    ("tag-hidden", "tag-hidden"),
                    ("tag-minus", "tag-minus"),
                    ("tag-minus-outline", "tag-minus-outline"),
                    ("tag-multiple", "tag-multiple"),
                    ("tag-multiple-outline", "tag-multiple-outline"),
                    ("tag-off", "tag-off"),
                    ("tag-off-outline", "tag-off-outline"),
                    ("tag-outline", "tag-outline"),
                    ("tag-plus", "tag-plus"),
                    ("tag-plus-outline", "tag-plus-outline"),
                    ("tag-remove", "tag-remove"),
                    ("tag-remove-outline", "tag-remove-outline"),
                    ("tag-search", "tag-search"),
                    ("tag-search-outline", "tag-search-outline"),
                    ("tag-text", "tag-text"),
                    ("tag-text-outline", "tag-text-outline"),
                    ("tailwind", "tailwind"),
                    ("tally-mark-1", "tally-mark-1"),
                    ("tally-mark-2", "tally-mark-2"),
                    ("tally-mark-3", "tally-mark-3"),
                    ("tally-mark-4", "tally-mark-4"),
                    ("tally-mark-5", "tally-mark-5"),
                    ("tangram", "tangram"),
                    ("tank", "tank"),
                    ("tanker-truck", "tanker-truck"),
                    ("tape-drive", "tape-drive"),
                    ("tape-measure", "tape-measure"),
                    ("target", "target"),
                    ("target-account", "target-account"),
                    ("target-variant", "target-variant"),
                    ("taxi", "taxi"),
                    ("tea", "tea"),
                    ("tea-outline", "tea-outline"),
                    ("teamspeak", "teamspeak"),
                    ("teamviewer", "teamviewer"),
                    ("teddy-bear", "teddy-bear"),
                    ("telegram", "telegram"),
                    ("telescope", "telescope"),
                    ("television", "television"),
                    ("television-ambient-light", "television-ambient-light"),
                    ("television-box", "television-box"),
                    ("television-classic", "television-classic"),
                    ("television-classic-off", "television-classic-off"),
                    ("television-guide", "television-guide"),
                    ("television-off", "television-off"),
                    ("television-pause", "television-pause"),
                    ("television-play", "television-play"),
                    ("television-shimmer", "television-shimmer"),
                    ("television-speaker", "television-speaker"),
                    ("television-speaker-off", "television-speaker-off"),
                    ("television-stop", "television-stop"),
                    ("temperature-celsius", "temperature-celsius"),
                    ("temperature-fahrenheit", "temperature-fahrenheit"),
                    ("temperature-kelvin", "temperature-kelvin"),
                    ("temple-buddhist", "temple-buddhist"),
                    ("temple-buddhist-outline", "temple-buddhist-outline"),
                    ("temple-hindu", "temple-hindu"),
                    ("temple-hindu-outline", "temple-hindu-outline"),
                    ("tennis", "tennis"),
                    ("tennis-ball", "tennis-ball"),
                    ("tennis-ball-outline", "tennis-ball-outline"),
                    ("tent", "tent"),
                    ("terraform", "terraform"),
                    ("terrain", "terrain"),
                    ("test-tube", "test-tube"),
                    ("test-tube-empty", "test-tube-empty"),
                    ("test-tube-off", "test-tube-off"),
                    ("text", "text"),
                    ("text-account", "text-account"),
                    ("text-box", "text-box"),
                    ("text-box-check", "text-box-check"),
                    ("text-box-check-outline", "text-box-check-outline"),
                    ("text-box-edit", "text-box-edit"),
                    ("text-box-edit-outline", "text-box-edit-outline"),
                    ("text-box-minus", "text-box-minus"),
                    ("text-box-minus-outline", "text-box-minus-outline"),
                    ("text-box-multiple", "text-box-multiple"),
                    ("text-box-multiple-outline", "text-box-multiple-outline"),
                    ("text-box-outline", "text-box-outline"),
                    ("text-box-plus", "text-box-plus"),
                    ("text-box-plus-outline", "text-box-plus-outline"),
                    ("text-box-remove", "text-box-remove"),
                    ("text-box-remove-outline", "text-box-remove-outline"),
                    ("text-box-search", "text-box-search"),
                    ("text-box-search-outline", "text-box-search-outline"),
                    ("text-long", "text-long"),
                    ("text-recognition", "text-recognition"),
                    ("text-search", "text-search"),
                    ("text-search-variant", "text-search-variant"),
                    ("text-shadow", "text-shadow"),
                    ("text-short", "text-short"),
                    ("texture", "texture"),
                    ("texture-box", "texture-box"),
                    ("theater", "theater"),
                    ("theme-light-dark", "theme-light-dark"),
                    ("thermometer", "thermometer"),
                    ("thermometer-alert", "thermometer-alert"),
                    ("thermometer-auto", "thermometer-auto"),
                    ("thermometer-bluetooth", "thermometer-bluetooth"),
                    ("thermometer-check", "thermometer-check"),
                    ("thermometer-chevron-down", "thermometer-chevron-down"),
                    ("thermometer-chevron-up", "thermometer-chevron-up"),
                    ("thermometer-high", "thermometer-high"),
                    ("thermometer-lines", "thermometer-lines"),
                    ("thermometer-low", "thermometer-low"),
                    ("thermometer-minus", "thermometer-minus"),
                    ("thermometer-off", "thermometer-off"),
                    ("thermometer-plus", "thermometer-plus"),
                    ("thermometer-probe", "thermometer-probe"),
                    ("thermometer-probe-off", "thermometer-probe-off"),
                    ("thermometer-water", "thermometer-water"),
                    ("thermostat", "thermostat"),
                    ("thermostat-auto", "thermostat-auto"),
                    ("thermostat-box", "thermostat-box"),
                    ("thermostat-box-auto", "thermostat-box-auto"),
                    ("thermostat-cog", "thermostat-cog"),
                    ("thought-bubble", "thought-bubble"),
                    ("thought-bubble-outline", "thought-bubble-outline"),
                    ("thumb-down", "thumb-down"),
                    ("thumb-down-outline", "thumb-down-outline"),
                    ("thumb-up", "thumb-up"),
                    ("thumb-up-outline", "thumb-up-outline"),
                    ("thumbs-up-down", "thumbs-up-down"),
                    ("thumbs-up-down-outline", "thumbs-up-down-outline"),
                    ("ticket", "ticket"),
                    ("ticket-account", "ticket-account"),
                    ("ticket-confirmation", "ticket-confirmation"),
                    ("ticket-confirmation-outline", "ticket-confirmation-outline"),
                    ("ticket-outline", "ticket-outline"),
                    ("ticket-percent", "ticket-percent"),
                    ("ticket-percent-outline", "ticket-percent-outline"),
                    ("tie", "tie"),
                    ("tilde", "tilde"),
                    ("tilde-off", "tilde-off"),
                    ("timelapse", "timelapse"),
                    ("timeline", "timeline"),
                    ("timeline-alert", "timeline-alert"),
                    ("timeline-alert-outline", "timeline-alert-outline"),
                    ("timeline-check", "timeline-check"),
                    ("timeline-check-outline", "timeline-check-outline"),
                    ("timeline-clock", "timeline-clock"),
                    ("timeline-clock-outline", "timeline-clock-outline"),
                    ("timeline-minus", "timeline-minus"),
                    ("timeline-minus-outline", "timeline-minus-outline"),
                    ("timeline-outline", "timeline-outline"),
                    ("timeline-plus", "timeline-plus"),
                    ("timeline-plus-outline", "timeline-plus-outline"),
                    ("timeline-question", "timeline-question"),
                    ("timeline-question-outline", "timeline-question-outline"),
                    ("timeline-remove", "timeline-remove"),
                    ("timeline-remove-outline", "timeline-remove-outline"),
                    ("timeline-text", "timeline-text"),
                    ("timeline-text-outline", "timeline-text-outline"),
                    ("timer", "timer"),
                    ("timer-10", "timer-10"),
                    ("timer-3", "timer-3"),
                    ("timer-alert", "timer-alert"),
                    ("timer-alert-outline", "timer-alert-outline"),
                    ("timer-cancel", "timer-cancel"),
                    ("timer-cancel-outline", "timer-cancel-outline"),
                    ("timer-check", "timer-check"),
                    ("timer-check-outline", "timer-check-outline"),
                    ("timer-cog", "timer-cog"),
                    ("timer-cog-outline", "timer-cog-outline"),
                    ("timer-edit", "timer-edit"),
                    ("timer-edit-outline", "timer-edit-outline"),
                    ("timer-lock", "timer-lock"),
                    ("timer-lock-open", "timer-lock-open"),
                    ("timer-lock-open-outline", "timer-lock-open-outline"),
                    ("timer-lock-outline", "timer-lock-outline"),
                    ("timer-marker", "timer-marker"),
                    ("timer-marker-outline", "timer-marker-outline"),
                    ("timer-minus", "timer-minus"),
                    ("timer-minus-outline", "timer-minus-outline"),
                    ("timer-music", "timer-music"),
                    ("timer-music-outline", "timer-music-outline"),
                    ("timer-off", "timer-off"),
                    ("timer-off-outline", "timer-off-outline"),
                    ("timer-outline", "timer-outline"),
                    ("timer-pause", "timer-pause"),
                    ("timer-pause-outline", "timer-pause-outline"),
                    ("timer-play", "timer-play"),
                    ("timer-play-outline", "timer-play-outline"),
                    ("timer-plus", "timer-plus"),
                    ("timer-plus-outline", "timer-plus-outline"),
                    ("timer-refresh", "timer-refresh"),
                    ("timer-refresh-outline", "timer-refresh-outline"),
                    ("timer-remove", "timer-remove"),
                    ("timer-remove-outline", "timer-remove-outline"),
                    ("timer-sand", "timer-sand"),
                    ("timer-sand-complete", "timer-sand-complete"),
                    ("timer-sand-empty", "timer-sand-empty"),
                    ("timer-sand-full", "timer-sand-full"),
                    ("timer-sand-paused", "timer-sand-paused"),
                    ("timer-settings", "timer-settings"),
                    ("timer-settings-outline", "timer-settings-outline"),
                    ("timer-star", "timer-star"),
                    ("timer-star-outline", "timer-star-outline"),
                    ("timer-stop", "timer-stop"),
                    ("timer-stop-outline", "timer-stop-outline"),
                    ("timer-sync", "timer-sync"),
                    ("timer-sync-outline", "timer-sync-outline"),
                    ("timetable", "timetable"),
                    ("tire", "tire"),
                    ("toaster", "toaster"),
                    ("toaster-off", "toaster-off"),
                    ("toaster-oven", "toaster-oven"),
                    ("toggle-switch", "toggle-switch"),
                    ("toggle-switch-off", "toggle-switch-off"),
                    ("toggle-switch-off-outline", "toggle-switch-off-outline"),
                    ("toggle-switch-outline", "toggle-switch-outline"),
                    ("toggle-switch-variant", "toggle-switch-variant"),
                    ("toggle-switch-variant-off", "toggle-switch-variant-off"),
                    ("toilet", "toilet"),
                    ("toolbox", "toolbox"),
                    ("toolbox-outline", "toolbox-outline"),
                    ("tools", "tools"),
                    ("tooltip", "tooltip"),
                    ("tooltip-account", "tooltip-account"),
                    ("tooltip-cellphone", "tooltip-cellphone"),
                    ("tooltip-check", "tooltip-check"),
                    ("tooltip-check-outline", "tooltip-check-outline"),
                    ("tooltip-edit", "tooltip-edit"),
                    ("tooltip-edit-outline", "tooltip-edit-outline"),
                    ("tooltip-image", "tooltip-image"),
                    ("tooltip-image-outline", "tooltip-image-outline"),
                    ("tooltip-minus", "tooltip-minus"),
                    ("tooltip-minus-outline", "tooltip-minus-outline"),
                    ("tooltip-outline", "tooltip-outline"),
                    ("tooltip-plus", "tooltip-plus"),
                    ("tooltip-plus-outline", "tooltip-plus-outline"),
                    ("tooltip-question", "tooltip-question"),
                    ("tooltip-question-outline", "tooltip-question-outline"),
                    ("tooltip-remove", "tooltip-remove"),
                    ("tooltip-remove-outline", "tooltip-remove-outline"),
                    ("tooltip-text", "tooltip-text"),
                    ("tooltip-text-outline", "tooltip-text-outline"),
                    ("tooth", "tooth"),
                    ("tooth-outline", "tooth-outline"),
                    ("toothbrush", "toothbrush"),
                    ("toothbrush-electric", "toothbrush-electric"),
                    ("toothbrush-paste", "toothbrush-paste"),
                    ("tor", "tor"),
                    ("torch", "torch"),
                    ("tortoise", "tortoise"),
                    ("toslink", "toslink"),
                    ("touch-text-outline", "touch-text-outline"),
                    ("tournament", "tournament"),
                    ("tow-truck", "tow-truck"),
                    ("tower-beach", "tower-beach"),
                    ("tower-fire", "tower-fire"),
                    ("town-hall", "town-hall"),
                    ("toy-brick", "toy-brick"),
                    ("toy-brick-marker", "toy-brick-marker"),
                    ("toy-brick-marker-outline", "toy-brick-marker-outline"),
                    ("toy-brick-minus", "toy-brick-minus"),
                    ("toy-brick-minus-outline", "toy-brick-minus-outline"),
                    ("toy-brick-outline", "toy-brick-outline"),
                    ("toy-brick-plus", "toy-brick-plus"),
                    ("toy-brick-plus-outline", "toy-brick-plus-outline"),
                    ("toy-brick-remove", "toy-brick-remove"),
                    ("toy-brick-remove-outline", "toy-brick-remove-outline"),
                    ("toy-brick-search", "toy-brick-search"),
                    ("toy-brick-search-outline", "toy-brick-search-outline"),
                    ("track-light", "track-light"),
                    ("track-light-off", "track-light-off"),
                    ("trackpad", "trackpad"),
                    ("trackpad-lock", "trackpad-lock"),
                    ("tractor", "tractor"),
                    ("tractor-variant", "tractor-variant"),
                    ("trademark", "trademark"),
                    ("traffic-cone", "traffic-cone"),
                    ("traffic-light", "traffic-light"),
                    ("traffic-light-outline", "traffic-light-outline"),
                    ("train", "train"),
                    ("train-car", "train-car"),
                    ("train-car-autorack", "train-car-autorack"),
                    ("train-car-box", "train-car-box"),
                    ("train-car-box-full", "train-car-box-full"),
                    ("train-car-box-open", "train-car-box-open"),
                    ("train-car-caboose", "train-car-caboose"),
                    ("train-car-centerbeam", "train-car-centerbeam"),
                    ("train-car-centerbeam-full", "train-car-centerbeam-full"),
                    ("train-car-container", "train-car-container"),
                    ("train-car-flatbed", "train-car-flatbed"),
                    ("train-car-flatbed-car", "train-car-flatbed-car"),
                    ("train-car-flatbed-tank", "train-car-flatbed-tank"),
                    ("train-car-gondola", "train-car-gondola"),
                    ("train-car-gondola-full", "train-car-gondola-full"),
                    ("train-car-hopper", "train-car-hopper"),
                    ("train-car-hopper-covered", "train-car-hopper-covered"),
                    ("train-car-hopper-full", "train-car-hopper-full"),
                    ("train-car-intermodal", "train-car-intermodal"),
                    ("train-car-passenger", "train-car-passenger"),
                    ("train-car-passenger-door", "train-car-passenger-door"),
                    ("train-car-passenger-door-open", "train-car-passenger-door-open"),
                    ("train-car-passenger-variant", "train-car-passenger-variant"),
                    ("train-car-tank", "train-car-tank"),
                    ("train-variant", "train-variant"),
                    ("tram", "tram"),
                    ("tram-side", "tram-side"),
                    ("transcribe", "transcribe"),
                    ("transcribe-close", "transcribe-close"),
                    ("transfer", "transfer"),
                    ("transfer-down", "transfer-down"),
                    ("transfer-left", "transfer-left"),
                    ("transfer-right", "transfer-right"),
                    ("transfer-up", "transfer-up"),
                    ("transit-connection", "transit-connection"),
                    ("transit-connection-horizontal", "transit-connection-horizontal"),
                    ("transit-connection-variant", "transit-connection-variant"),
                    ("transit-detour", "transit-detour"),
                    ("transit-skip", "transit-skip"),
                    ("transit-transfer", "transit-transfer"),
                    ("transition", "transition"),
                    ("transition-masked", "transition-masked"),
                    ("translate", "translate"),
                    ("translate-off", "translate-off"),
                    ("translate-variant", "translate-variant"),
                    ("transmission-tower", "transmission-tower"),
                    ("transmission-tower-export", "transmission-tower-export"),
                    ("transmission-tower-import", "transmission-tower-import"),
                    ("transmission-tower-off", "transmission-tower-off"),
                    ("trash-can", "trash-can"),
                    ("trash-can-outline", "trash-can-outline"),
                    ("tray", "tray"),
                    ("tray-alert", "tray-alert"),
                    ("tray-arrow-down", "tray-arrow-down"),
                    ("tray-arrow-up", "tray-arrow-up"),
                    ("tray-full", "tray-full"),
                    ("tray-minus", "tray-minus"),
                    ("tray-plus", "tray-plus"),
                    ("tray-remove", "tray-remove"),
                    ("treasure-chest", "treasure-chest"),
                    ("treasure-chest-outline", "treasure-chest-outline"),
                    ("tree", "tree"),
                    ("tree-outline", "tree-outline"),
                    ("trello", "trello"),
                    ("trending-down", "trending-down"),
                    ("trending-neutral", "trending-neutral"),
                    ("trending-up", "trending-up"),
                    ("triangle", "triangle"),
                    ("triangle-down", "triangle-down"),
                    ("triangle-down-outline", "triangle-down-outline"),
                    ("triangle-outline", "triangle-outline"),
                    ("triangle-small-down", "triangle-small-down"),
                    ("triangle-small-up", "triangle-small-up"),
                    ("triangle-wave", "triangle-wave"),
                    ("triforce", "triforce"),
                    ("trophy", "trophy"),
                    ("trophy-award", "trophy-award"),
                    ("trophy-broken", "trophy-broken"),
                    ("trophy-outline", "trophy-outline"),
                    ("trophy-variant", "trophy-variant"),
                    ("trophy-variant-outline", "trophy-variant-outline"),
                    ("truck", "truck"),
                    ("truck-alert", "truck-alert"),
                    ("truck-alert-outline", "truck-alert-outline"),
                    ("truck-cargo-container", "truck-cargo-container"),
                    ("truck-check", "truck-check"),
                    ("truck-check-outline", "truck-check-outline"),
                    ("truck-delivery", "truck-delivery"),
                    ("truck-delivery-outline", "truck-delivery-outline"),
                    ("truck-fast", "truck-fast"),
                    ("truck-fast-outline", "truck-fast-outline"),
                    ("truck-flatbed", "truck-flatbed"),
                    ("truck-minus", "truck-minus"),
                    ("truck-minus-outline", "truck-minus-outline"),
                    ("truck-off-road", "truck-off-road"),
                    ("truck-off-road-off", "truck-off-road-off"),
                    ("truck-outline", "truck-outline"),
                    ("truck-plus", "truck-plus"),
                    ("truck-plus-outline", "truck-plus-outline"),
                    ("truck-remove", "truck-remove"),
                    ("truck-remove-outline", "truck-remove-outline"),
                    ("truck-snowflake", "truck-snowflake"),
                    ("truck-trailer", "truck-trailer"),
                    ("trumpet", "trumpet"),
                    ("tshirt-crew", "tshirt-crew"),
                    ("tshirt-crew-outline", "tshirt-crew-outline"),
                    ("tshirt-v", "tshirt-v"),
                    ("tshirt-v-outline", "tshirt-v-outline"),
                    ("tsunami", "tsunami"),
                    ("tumble-dryer", "tumble-dryer"),
                    ("tumble-dryer-alert", "tumble-dryer-alert"),
                    ("tumble-dryer-off", "tumble-dryer-off"),
                    ("tumblr", "tumblr"),
                    ("tumblr-box", "tumblr-box"),
                    ("tumblr-reblog", "tumblr-reblog"),
                    ("tune", "tune"),
                    ("tune-variant", "tune-variant"),
                    ("tune-vertical", "tune-vertical"),
                    ("tune-vertical-variant", "tune-vertical-variant"),
                    ("tunnel", "tunnel"),
                    ("tunnel-outline", "tunnel-outline"),
                    ("turbine", "turbine"),
                    ("turkey", "turkey"),
                    ("turnstile", "turnstile"),
                    ("turnstile-outline", "turnstile-outline"),
                    ("turtle", "turtle"),
                    ("twitch", "twitch"),
                    ("twitter", "twitter"),
                    ("twitter-box", "twitter-box"),
                    ("twitter-circle", "twitter-circle"),
                    ("two-factor-authentication", "two-factor-authentication"),
                    ("typewriter", "typewriter"),
                    ("uber", "uber"),
                    ("ubisoft", "ubisoft"),
                    ("ubuntu", "ubuntu"),
                    ("ufo", "ufo"),
                    ("ufo-outline", "ufo-outline"),
                    ("ultra-high-definition", "ultra-high-definition"),
                    ("umbraco", "umbraco"),
                    ("umbrella", "umbrella"),
                    ("umbrella-beach", "umbrella-beach"),
                    ("umbrella-beach-outline", "umbrella-beach-outline"),
                    ("umbrella-closed", "umbrella-closed"),
                    ("umbrella-closed-outline", "umbrella-closed-outline"),
                    ("umbrella-closed-variant", "umbrella-closed-variant"),
                    ("umbrella-outline", "umbrella-outline"),
                    ("undo", "undo"),
                    ("undo-variant", "undo-variant"),
                    ("unfold-less-horizontal", "unfold-less-horizontal"),
                    ("unfold-less-vertical", "unfold-less-vertical"),
                    ("unfold-more-horizontal", "unfold-more-horizontal"),
                    ("unfold-more-vertical", "unfold-more-vertical"),
                    ("ungroup", "ungroup"),
                    ("unicode", "unicode"),
                    ("unicorn", "unicorn"),
                    ("unicorn-variant", "unicorn-variant"),
                    ("unicycle", "unicycle"),
                    ("unity", "unity"),
                    ("unreal", "unreal"),
                    ("untappd", "untappd"),
                    ("update", "update"),
                    ("upload", "upload"),
                    ("upload-lock", "upload-lock"),
                    ("upload-lock-outline", "upload-lock-outline"),
                    ("upload-multiple", "upload-multiple"),
                    ("upload-network", "upload-network"),
                    ("upload-network-outline", "upload-network-outline"),
                    ("upload-off", "upload-off"),
                    ("upload-off-outline", "upload-off-outline"),
                    ("upload-outline", "upload-outline"),
                    ("usb", "usb"),
                    ("usb-flash-drive", "usb-flash-drive"),
                    ("usb-flash-drive-outline", "usb-flash-drive-outline"),
                    ("usb-port", "usb-port"),
                    ("vacuum", "vacuum"),
                    ("vacuum-outline", "vacuum-outline"),
                    ("valve", "valve"),
                    ("valve-closed", "valve-closed"),
                    ("valve-open", "valve-open"),
                    ("van-passenger", "van-passenger"),
                    ("van-utility", "van-utility"),
                    ("vanish", "vanish"),
                    ("vanish-quarter", "vanish-quarter"),
                    ("vanity-light", "vanity-light"),
                    ("variable", "variable"),
                    ("variable-box", "variable-box"),
                    ("vector-arrange-above", "vector-arrange-above"),
                    ("vector-arrange-below", "vector-arrange-below"),
                    ("vector-bezier", "vector-bezier"),
                    ("vector-circle", "vector-circle"),
                    ("vector-circle-variant", "vector-circle-variant"),
                    ("vector-combine", "vector-combine"),
                    ("vector-curve", "vector-curve"),
                    ("vector-difference", "vector-difference"),
                    ("vector-difference-ab", "vector-difference-ab"),
                    ("vector-difference-ba", "vector-difference-ba"),
                    ("vector-ellipse", "vector-ellipse"),
                    ("vector-intersection", "vector-intersection"),
                    ("vector-line", "vector-line"),
                    ("vector-link", "vector-link"),
                    ("vector-point", "vector-point"),
                    ("vector-point-edit", "vector-point-edit"),
                    ("vector-point-minus", "vector-point-minus"),
                    ("vector-point-plus", "vector-point-plus"),
                    ("vector-point-select", "vector-point-select"),
                    ("vector-polygon", "vector-polygon"),
                    ("vector-polygon-variant", "vector-polygon-variant"),
                    ("vector-polyline", "vector-polyline"),
                    ("vector-polyline-edit", "vector-polyline-edit"),
                    ("vector-polyline-minus", "vector-polyline-minus"),
                    ("vector-polyline-plus", "vector-polyline-plus"),
                    ("vector-polyline-remove", "vector-polyline-remove"),
                    ("vector-radius", "vector-radius"),
                    ("vector-rectangle", "vector-rectangle"),
                    ("vector-selection", "vector-selection"),
                    ("vector-square", "vector-square"),
                    ("vector-square-close", "vector-square-close"),
                    ("vector-square-edit", "vector-square-edit"),
                    ("vector-square-minus", "vector-square-minus"),
                    ("vector-square-open", "vector-square-open"),
                    ("vector-square-plus", "vector-square-plus"),
                    ("vector-square-remove", "vector-square-remove"),
                    ("vector-triangle", "vector-triangle"),
                    ("vector-union", "vector-union"),
                    ("venmo", "venmo"),
                    ("vhs", "vhs"),
                    ("vibrate", "vibrate"),
                    ("vibrate-off", "vibrate-off"),
                    ("video", "video"),
                    ("video-2d", "video-2d"),
                    ("video-3d", "video-3d"),
                    ("video-3d-off", "video-3d-off"),
                    ("video-3d-variant", "video-3d-variant"),
                    ("video-4k-box", "video-4k-box"),
                    ("video-account", "video-account"),
                    ("video-box", "video-box"),
                    ("video-box-off", "video-box-off"),
                    ("video-check", "video-check"),
                    ("video-check-outline", "video-check-outline"),
                    ("video-high-definition", "video-high-definition"),
                    ("video-image", "video-image"),
                    ("video-input-antenna", "video-input-antenna"),
                    ("video-input-component", "video-input-component"),
                    ("video-input-hdmi", "video-input-hdmi"),
                    ("video-input-scart", "video-input-scart"),
                    ("video-input-svideo", "video-input-svideo"),
                    ("video-marker", "video-marker"),
                    ("video-marker-outline", "video-marker-outline"),
                    ("video-minus", "video-minus"),
                    ("video-minus-outline", "video-minus-outline"),
                    ("video-off", "video-off"),
                    ("video-off-outline", "video-off-outline"),
                    ("video-outline", "video-outline"),
                    ("video-plus", "video-plus"),
                    ("video-plus-outline", "video-plus-outline"),
                    ("video-stabilization", "video-stabilization"),
                    ("video-standard-definition", "video-standard-definition"),
                    ("video-switch", "video-switch"),
                    ("video-switch-outline", "video-switch-outline"),
                    ("video-vintage", "video-vintage"),
                    ("video-wireless", "video-wireless"),
                    ("video-wireless-outline", "video-wireless-outline"),
                    ("view-agenda", "view-agenda"),
                    ("view-agenda-outline", "view-agenda-outline"),
                    ("view-array", "view-array"),
                    ("view-array-outline", "view-array-outline"),
                    ("view-carousel", "view-carousel"),
                    ("view-carousel-outline", "view-carousel-outline"),
                    ("view-column", "view-column"),
                    ("view-column-outline", "view-column-outline"),
                    ("view-comfy", "view-comfy"),
                    ("view-comfy-outline", "view-comfy-outline"),
                    ("view-compact", "view-compact"),
                    ("view-compact-outline", "view-compact-outline"),
                    ("view-dashboard", "view-dashboard"),
                    ("view-dashboard-edit", "view-dashboard-edit"),
                    ("view-dashboard-edit-outline", "view-dashboard-edit-outline"),
                    ("view-dashboard-outline", "view-dashboard-outline"),
                    ("view-dashboard-variant", "view-dashboard-variant"),
                    ("view-dashboard-variant-outline", "view-dashboard-variant-outline"),
                    ("view-day", "view-day"),
                    ("view-day-outline", "view-day-outline"),
                    ("view-gallery", "view-gallery"),
                    ("view-gallery-outline", "view-gallery-outline"),
                    ("view-grid", "view-grid"),
                    ("view-grid-compact", "view-grid-compact"),
                    ("view-grid-outline", "view-grid-outline"),
                    ("view-grid-plus", "view-grid-plus"),
                    ("view-grid-plus-outline", "view-grid-plus-outline"),
                    ("view-headline", "view-headline"),
                    ("view-list", "view-list"),
                    ("view-list-outline", "view-list-outline"),
                    ("view-module", "view-module"),
                    ("view-module-outline", "view-module-outline"),
                    ("view-parallel", "view-parallel"),
                    ("view-parallel-outline", "view-parallel-outline"),
                    ("view-quilt", "view-quilt"),
                    ("view-quilt-outline", "view-quilt-outline"),
                    ("view-sequential", "view-sequential"),
                    ("view-sequential-outline", "view-sequential-outline"),
                    ("view-split-horizontal", "view-split-horizontal"),
                    ("view-split-vertical", "view-split-vertical"),
                    ("view-stream", "view-stream"),
                    ("view-stream-outline", "view-stream-outline"),
                    ("view-week", "view-week"),
                    ("view-week-outline", "view-week-outline"),
                    ("vimeo", "vimeo"),
                    ("vine", "vine"),
                    ("violin", "violin"),
                    ("virtual-reality", "virtual-reality"),
                    ("virus", "virus"),
                    ("virus-off", "virus-off"),
                    ("virus-off-outline", "virus-off-outline"),
                    ("virus-outline", "virus-outline"),
                    ("vk", "vk"),
                    ("vk-box", "vk-box"),
                    ("vk-circle", "vk-circle"),
                    ("vlc", "vlc"),
                    ("voicemail", "voicemail"),
                    ("volcano", "volcano"),
                    ("volcano-outline", "volcano-outline"),
                    ("volleyball", "volleyball"),
                    ("volume", "volume"),
                    ("volume-equal", "volume-equal"),
                    ("volume-high", "volume-high"),
                    ("volume-low", "volume-low"),
                    ("volume-medium", "volume-medium"),
                    ("volume-minus", "volume-minus"),
                    ("volume-mute", "volume-mute"),
                    ("volume-off", "volume-off"),
                    ("volume-plus", "volume-plus"),
                    ("volume-source", "volume-source"),
                    ("volume-variant-off", "volume-variant-off"),
                    ("volume-vibrate", "volume-vibrate"),
                    ("vote", "vote"),
                    ("vote-outline", "vote-outline"),
                    ("vpn", "vpn"),
                    ("vuejs", "vuejs"),
                    ("vuetify", "vuetify"),
                    ("walk", "walk"),
                    ("wall", "wall"),
                    ("wall-fire", "wall-fire"),
                    ("wall-sconce", "wall-sconce"),
                    ("wall-sconce-flat", "wall-sconce-flat"),
                    ("wall-sconce-flat-outline", "wall-sconce-flat-outline"),
                    ("wall-sconce-flat-variant", "wall-sconce-flat-variant"),
                    ("wall-sconce-flat-variant-outline", "wall-sconce-flat-variant-outline"),
                    ("wall-sconce-outline", "wall-sconce-outline"),
                    ("wall-sconce-round", "wall-sconce-round"),
                    ("wall-sconce-round-outline", "wall-sconce-round-outline"),
                    ("wall-sconce-round-variant", "wall-sconce-round-variant"),
                    ("wall-sconce-round-variant-outline", "wall-sconce-round-variant-outline"),
                    ("wall-sconce-variant", "wall-sconce-variant"),
                    ("wallet", "wallet"),
                    ("wallet-bifold", "wallet-bifold"),
                    ("wallet-bifold-outline", "wallet-bifold-outline"),
                    ("wallet-giftcard", "wallet-giftcard"),
                    ("wallet-membership", "wallet-membership"),
                    ("wallet-outline", "wallet-outline"),
                    ("wallet-plus", "wallet-plus"),
                    ("wallet-plus-outline", "wallet-plus-outline"),
                    ("wallet-travel", "wallet-travel"),
                    ("wallpaper", "wallpaper"),
                    ("wan", "wan"),
                    ("wardrobe", "wardrobe"),
                    ("wardrobe-outline", "wardrobe-outline"),
                    ("warehouse", "warehouse"),
                    ("washing-machine", "washing-machine"),
                    ("washing-machine-alert", "washing-machine-alert"),
                    ("washing-machine-off", "washing-machine-off"),
                    ("watch", "watch"),
                    ("watch-export", "watch-export"),
                    ("watch-export-variant", "watch-export-variant"),
                    ("watch-import", "watch-import"),
                    ("watch-import-variant", "watch-import-variant"),
                    ("watch-variant", "watch-variant"),
                    ("watch-vibrate", "watch-vibrate"),
                    ("watch-vibrate-off", "watch-vibrate-off"),
                    ("water", "water"),
                    ("water-alert", "water-alert"),
                    ("water-alert-outline", "water-alert-outline"),
                    ("water-boiler", "water-boiler"),
                    ("water-boiler-alert", "water-boiler-alert"),
                    ("water-boiler-auto", "water-boiler-auto"),
                    ("water-boiler-off", "water-boiler-off"),
                    ("water-check", "water-check"),
                    ("water-check-outline", "water-check-outline"),
                    ("water-circle", "water-circle"),
                    ("water-minus", "water-minus"),
                    ("water-minus-outline", "water-minus-outline"),
                    ("water-off", "water-off"),
                    ("water-off-outline", "water-off-outline"),
                    ("water-opacity", "water-opacity"),
                    ("water-outline", "water-outline"),
                    ("water-percent", "water-percent"),
                    ("water-percent-alert", "water-percent-alert"),
                    ("water-plus", "water-plus"),
                    ("water-plus-outline", "water-plus-outline"),
                    ("water-polo", "water-polo"),
                    ("water-pump", "water-pump"),
                    ("water-pump-off", "water-pump-off"),
                    ("water-remove", "water-remove"),
                    ("water-remove-outline", "water-remove-outline"),
                    ("water-sync", "water-sync"),
                    ("water-thermometer", "water-thermometer"),
                    ("water-thermometer-outline", "water-thermometer-outline"),
                    ("water-well", "water-well"),
                    ("water-well-outline", "water-well-outline"),
                    ("waterfall", "waterfall"),
                    ("watering-can", "watering-can"),
                    ("watering-can-outline", "watering-can-outline"),
                    ("watermark", "watermark"),
                    ("wave", "wave"),
                    ("waveform", "waveform"),
                    ("waves", "waves"),
                    ("waves-arrow-left", "waves-arrow-left"),
                    ("waves-arrow-right", "waves-arrow-right"),
                    ("waves-arrow-up", "waves-arrow-up"),
                    ("waze", "waze"),
                    ("weather-cloudy", "weather-cloudy"),
                    ("weather-cloudy-alert", "weather-cloudy-alert"),
                    ("weather-cloudy-arrow-right", "weather-cloudy-arrow-right"),
                    ("weather-cloudy-clock", "weather-cloudy-clock"),
                    ("weather-dust", "weather-dust"),
                    ("weather-fog", "weather-fog"),
                    ("weather-hail", "weather-hail"),
                    ("weather-hazy", "weather-hazy"),
                    ("weather-hurricane", "weather-hurricane"),
                    ("weather-hurricane-outline", "weather-hurricane-outline"),
                    ("weather-lightning", "weather-lightning"),
                    ("weather-lightning-rainy", "weather-lightning-rainy"),
                    ("weather-night", "weather-night"),
                    ("weather-night-partly-cloudy", "weather-night-partly-cloudy"),
                    ("weather-partly-cloudy", "weather-partly-cloudy"),
                    ("weather-partly-lightning", "weather-partly-lightning"),
                    ("weather-partly-rainy", "weather-partly-rainy"),
                    ("weather-partly-snowy", "weather-partly-snowy"),
                    ("weather-partly-snowy-rainy", "weather-partly-snowy-rainy"),
                    ("weather-pouring", "weather-pouring"),
                    ("weather-rainy", "weather-rainy"),
                    ("weather-snowy", "weather-snowy"),
                    ("weather-snowy-heavy", "weather-snowy-heavy"),
                    ("weather-snowy-rainy", "weather-snowy-rainy"),
                    ("weather-sunny", "weather-sunny"),
                    ("weather-sunny-alert", "weather-sunny-alert"),
                    ("weather-sunny-off", "weather-sunny-off"),
                    ("weather-sunset", "weather-sunset"),
                    ("weather-sunset-down", "weather-sunset-down"),
                    ("weather-sunset-up", "weather-sunset-up"),
                    ("weather-tornado", "weather-tornado"),
                    ("weather-windy", "weather-windy"),
                    ("weather-windy-variant", "weather-windy-variant"),
                    ("web", "web"),
                    ("web-box", "web-box"),
                    ("web-cancel", "web-cancel"),
                    ("web-check", "web-check"),
                    ("web-clock", "web-clock"),
                    ("web-minus", "web-minus"),
                    ("web-off", "web-off"),
                    ("web-plus", "web-plus"),
                    ("web-refresh", "web-refresh"),
                    ("web-remove", "web-remove"),
                    ("web-sync", "web-sync"),
                    ("webcam", "webcam"),
                    ("webcam-off", "webcam-off"),
                    ("webhook", "webhook"),
                    ("webpack", "webpack"),
                    ("webrtc", "webrtc"),
                    ("wechat", "wechat"),
                    ("weight", "weight"),
                    ("weight-gram", "weight-gram"),
                    ("weight-kilogram", "weight-kilogram"),
                    ("weight-lifter", "weight-lifter"),
                    ("weight-pound", "weight-pound"),
                    ("whatsapp", "whatsapp"),
                    ("wheel-barrow", "wheel-barrow"),
                    ("wheelchair", "wheelchair"),
                    ("wheelchair-accessibility", "wheelchair-accessibility"),
                    ("whistle", "whistle"),
                    ("whistle-outline", "whistle-outline"),
                    ("white-balance-auto", "white-balance-auto"),
                    ("white-balance-incandescent", "white-balance-incandescent"),
                    ("white-balance-iridescent", "white-balance-iridescent"),
                    ("white-balance-sunny", "white-balance-sunny"),
                    ("widgets", "widgets"),
                    ("widgets-outline", "widgets-outline"),
                    ("wifi", "wifi"),
                    ("wifi-alert", "wifi-alert"),
                    ("wifi-arrow-down", "wifi-arrow-down"),
                    ("wifi-arrow-left", "wifi-arrow-left"),
                    ("wifi-arrow-left-right", "wifi-arrow-left-right"),
                    ("wifi-arrow-right", "wifi-arrow-right"),
                    ("wifi-arrow-up", "wifi-arrow-up"),
                    ("wifi-arrow-up-down", "wifi-arrow-up-down"),
                    ("wifi-cancel", "wifi-cancel"),
                    ("wifi-check", "wifi-check"),
                    ("wifi-cog", "wifi-cog"),
                    ("wifi-lock", "wifi-lock"),
                    ("wifi-lock-open", "wifi-lock-open"),
                    ("wifi-marker", "wifi-marker"),
                    ("wifi-minus", "wifi-minus"),
                    ("wifi-off", "wifi-off"),
                    ("wifi-plus", "wifi-plus"),
                    ("wifi-refresh", "wifi-refresh"),
                    ("wifi-remove", "wifi-remove"),
                    ("wifi-settings", "wifi-settings"),
                    ("wifi-star", "wifi-star"),
                    ("wifi-strength-1", "wifi-strength-1"),
                    ("wifi-strength-1-alert", "wifi-strength-1-alert"),
                    ("wifi-strength-1-lock", "wifi-strength-1-lock"),
                    ("wifi-strength-1-lock-open", "wifi-strength-1-lock-open"),
                    ("wifi-strength-2", "wifi-strength-2"),
                    ("wifi-strength-2-alert", "wifi-strength-2-alert"),
                    ("wifi-strength-2-lock", "wifi-strength-2-lock"),
                    ("wifi-strength-2-lock-open", "wifi-strength-2-lock-open"),
                    ("wifi-strength-3", "wifi-strength-3"),
                    ("wifi-strength-3-alert", "wifi-strength-3-alert"),
                    ("wifi-strength-3-lock", "wifi-strength-3-lock"),
                    ("wifi-strength-3-lock-open", "wifi-strength-3-lock-open"),
                    ("wifi-strength-4", "wifi-strength-4"),
                    ("wifi-strength-4-alert", "wifi-strength-4-alert"),
                    ("wifi-strength-4-lock", "wifi-strength-4-lock"),
                    ("wifi-strength-4-lock-open", "wifi-strength-4-lock-open"),
                    ("wifi-strength-alert-outline", "wifi-strength-alert-outline"),
                    ("wifi-strength-lock-open-outline", "wifi-strength-lock-open-outline"),
                    ("wifi-strength-lock-outline", "wifi-strength-lock-outline"),
                    ("wifi-strength-off", "wifi-strength-off"),
                    ("wifi-strength-off-outline", "wifi-strength-off-outline"),
                    ("wifi-strength-outline", "wifi-strength-outline"),
                    ("wifi-sync", "wifi-sync"),
                    ("wikipedia", "wikipedia"),
                    ("wind-power", "wind-power"),
                    ("wind-power-outline", "wind-power-outline"),
                    ("wind-turbine", "wind-turbine"),
                    ("wind-turbine-alert", "wind-turbine-alert"),
                    ("wind-turbine-check", "wind-turbine-check"),
                    ("window-close", "window-close"),
                    ("window-closed", "window-closed"),
                    ("window-closed-variant", "window-closed-variant"),
                    ("window-maximize", "window-maximize"),
                    ("window-minimize", "window-minimize"),
                    ("window-open", "window-open"),
                    ("window-open-variant", "window-open-variant"),
                    ("window-restore", "window-restore"),
                    ("window-shutter", "window-shutter"),
                    ("window-shutter-alert", "window-shutter-alert"),
                    ("window-shutter-auto", "window-shutter-auto"),
                    ("window-shutter-cog", "window-shutter-cog"),
                    ("window-shutter-open", "window-shutter-open"),
                    ("window-shutter-settings", "window-shutter-settings"),
                    ("windsock", "windsock"),
                    ("wiper", "wiper"),
                    ("wiper-wash", "wiper-wash"),
                    ("wiper-wash-alert", "wiper-wash-alert"),
                    ("wizard-hat", "wizard-hat"),
                    ("wordpress", "wordpress"),
                    ("wrap", "wrap"),
                    ("wrap-disabled", "wrap-disabled"),
                    ("wrench", "wrench"),
                    ("wrench-check", "wrench-check"),
                    ("wrench-check-outline", "wrench-check-outline"),
                    ("wrench-clock", "wrench-clock"),
                    ("wrench-clock-outline", "wrench-clock-outline"),
                    ("wrench-cog", "wrench-cog"),
                    ("wrench-cog-outline", "wrench-cog-outline"),
                    ("wrench-outline", "wrench-outline"),
                    ("wunderlist", "wunderlist"),
                    ("xamarin", "xamarin"),
                    ("xamarin-outline", "xamarin-outline"),
                    ("xda", "xda"),
                    ("xing", "xing"),
                    ("xing-circle", "xing-circle"),
                    ("xml", "xml"),
                    ("xmpp", "xmpp"),
                    ("y-combinator", "y-combinator"),
                    ("yahoo", "yahoo"),
                    ("yammer", "yammer"),
                    ("yeast", "yeast"),
                    ("yelp", "yelp"),
                    ("yin-yang", "yin-yang"),
                    ("yoga", "yoga"),
                    ("youtube", "youtube"),
                    ("youtube-gaming", "youtube-gaming"),
                    ("youtube-studio", "youtube-studio"),
                    ("youtube-subscription", "youtube-subscription"),
                    ("youtube-tv", "youtube-tv"),
                    ("yurt", "yurt"),
                    ("z-wave", "z-wave"),
                    ("zend", "zend"),
                    ("zigbee", "zigbee"),
                    ("zip-box", "zip-box"),
                    ("zip-box-outline", "zip-box-outline"),
                    ("zip-disk", "zip-disk"),
                    ("zodiac-aquarius", "zodiac-aquarius"),
                    ("zodiac-aries", "zodiac-aries"),
                    ("zodiac-cancer", "zodiac-cancer"),
                    ("zodiac-capricorn", "zodiac-capricorn"),
                    ("zodiac-gemini", "zodiac-gemini"),
                    ("zodiac-leo", "zodiac-leo"),
                    ("zodiac-libra", "zodiac-libra"),
                    ("zodiac-pisces", "zodiac-pisces"),
                    ("zodiac-sagittarius", "zodiac-sagittarius"),
                    ("zodiac-scorpio", "zodiac-scorpio"),
                    ("zodiac-taurus", "zodiac-taurus"),
                    ("zodiac-virgo", "zodiac-virgo"),
                ],
                max_length=50,
                verbose_name="Icon",
            ),
        ),
        migrations.AlterField(
            model_name="notification",
            name="icon",
            field=models.CharField(
                choices=[
                    ("ab-testing", "ab-testing"),
                    ("abacus", "abacus"),
                    ("abjad-arabic", "abjad-arabic"),
                    ("abjad-hebrew", "abjad-hebrew"),
                    ("abugida-devanagari", "abugida-devanagari"),
                    ("abugida-thai", "abugida-thai"),
                    ("access-point", "access-point"),
                    ("access-point-check", "access-point-check"),
                    ("access-point-minus", "access-point-minus"),
                    ("access-point-network", "access-point-network"),
                    ("access-point-network-off", "access-point-network-off"),
                    ("access-point-off", "access-point-off"),
                    ("access-point-plus", "access-point-plus"),
                    ("access-point-remove", "access-point-remove"),
                    ("account", "account"),
                    ("account-alert", "account-alert"),
                    ("account-alert-outline", "account-alert-outline"),
                    ("account-arrow-down", "account-arrow-down"),
                    ("account-arrow-down-outline", "account-arrow-down-outline"),
                    ("account-arrow-left", "account-arrow-left"),
                    ("account-arrow-left-outline", "account-arrow-left-outline"),
                    ("account-arrow-right", "account-arrow-right"),
                    ("account-arrow-right-outline", "account-arrow-right-outline"),
                    ("account-arrow-up", "account-arrow-up"),
                    ("account-arrow-up-outline", "account-arrow-up-outline"),
                    ("account-badge", "account-badge"),
                    ("account-badge-outline", "account-badge-outline"),
                    ("account-box", "account-box"),
                    ("account-box-multiple", "account-box-multiple"),
                    ("account-box-multiple-outline", "account-box-multiple-outline"),
                    ("account-box-outline", "account-box-outline"),
                    ("account-cancel", "account-cancel"),
                    ("account-cancel-outline", "account-cancel-outline"),
                    ("account-card", "account-card"),
                    ("account-card-outline", "account-card-outline"),
                    ("account-cash", "account-cash"),
                    ("account-cash-outline", "account-cash-outline"),
                    ("account-check", "account-check"),
                    ("account-check-outline", "account-check-outline"),
                    ("account-child", "account-child"),
                    ("account-child-circle", "account-child-circle"),
                    ("account-child-outline", "account-child-outline"),
                    ("account-circle", "account-circle"),
                    ("account-circle-outline", "account-circle-outline"),
                    ("account-clock", "account-clock"),
                    ("account-clock-outline", "account-clock-outline"),
                    ("account-cog", "account-cog"),
                    ("account-cog-outline", "account-cog-outline"),
                    ("account-convert", "account-convert"),
                    ("account-convert-outline", "account-convert-outline"),
                    ("account-cowboy-hat", "account-cowboy-hat"),
                    ("account-cowboy-hat-outline", "account-cowboy-hat-outline"),
                    ("account-credit-card", "account-credit-card"),
                    ("account-credit-card-outline", "account-credit-card-outline"),
                    ("account-details", "account-details"),
                    ("account-details-outline", "account-details-outline"),
                    ("account-edit", "account-edit"),
                    ("account-edit-outline", "account-edit-outline"),
                    ("account-eye", "account-eye"),
                    ("account-eye-outline", "account-eye-outline"),
                    ("account-filter", "account-filter"),
                    ("account-filter-outline", "account-filter-outline"),
                    ("account-group", "account-group"),
                    ("account-group-outline", "account-group-outline"),
                    ("account-hard-hat", "account-hard-hat"),
                    ("account-hard-hat-outline", "account-hard-hat-outline"),
                    ("account-heart", "account-heart"),
                    ("account-heart-outline", "account-heart-outline"),
                    ("account-injury", "account-injury"),
                    ("account-injury-outline", "account-injury-outline"),
                    ("account-key", "account-key"),
                    ("account-key-outline", "account-key-outline"),
                    ("account-lock", "account-lock"),
                    ("account-lock-open", "account-lock-open"),
                    ("account-lock-open-outline", "account-lock-open-outline"),
                    ("account-lock-outline", "account-lock-outline"),
                    ("account-minus", "account-minus"),
                    ("account-minus-outline", "account-minus-outline"),
                    ("account-multiple", "account-multiple"),
                    ("account-multiple-check", "account-multiple-check"),
                    ("account-multiple-check-outline", "account-multiple-check-outline"),
                    ("account-multiple-minus", "account-multiple-minus"),
                    ("account-multiple-minus-outline", "account-multiple-minus-outline"),
                    ("account-multiple-outline", "account-multiple-outline"),
                    ("account-multiple-plus", "account-multiple-plus"),
                    ("account-multiple-plus-outline", "account-multiple-plus-outline"),
                    ("account-multiple-remove", "account-multiple-remove"),
                    ("account-multiple-remove-outline", "account-multiple-remove-outline"),
                    ("account-music", "account-music"),
                    ("account-music-outline", "account-music-outline"),
                    ("account-network", "account-network"),
                    ("account-network-off", "account-network-off"),
                    ("account-network-off-outline", "account-network-off-outline"),
                    ("account-network-outline", "account-network-outline"),
                    ("account-off", "account-off"),
                    ("account-off-outline", "account-off-outline"),
                    ("account-outline", "account-outline"),
                    ("account-plus", "account-plus"),
                    ("account-plus-outline", "account-plus-outline"),
                    ("account-question", "account-question"),
                    ("account-question-outline", "account-question-outline"),
                    ("account-reactivate", "account-reactivate"),
                    ("account-reactivate-outline", "account-reactivate-outline"),
                    ("account-remove", "account-remove"),
                    ("account-remove-outline", "account-remove-outline"),
                    ("account-school", "account-school"),
                    ("account-school-outline", "account-school-outline"),
                    ("account-search", "account-search"),
                    ("account-search-outline", "account-search-outline"),
                    ("account-settings", "account-settings"),
                    ("account-settings-outline", "account-settings-outline"),
                    ("account-settings-variant", "account-settings-variant"),
                    ("account-star", "account-star"),
                    ("account-star-outline", "account-star-outline"),
                    ("account-supervisor", "account-supervisor"),
                    ("account-supervisor-circle", "account-supervisor-circle"),
                    ("account-supervisor-circle-outline", "account-supervisor-circle-outline"),
                    ("account-supervisor-outline", "account-supervisor-outline"),
                    ("account-switch", "account-switch"),
                    ("account-switch-outline", "account-switch-outline"),
                    ("account-sync", "account-sync"),
                    ("account-sync-outline", "account-sync-outline"),
                    ("account-tag", "account-tag"),
                    ("account-tag-outline", "account-tag-outline"),
                    ("account-tie", "account-tie"),
                    ("account-tie-hat", "account-tie-hat"),
                    ("account-tie-hat-outline", "account-tie-hat-outline"),
                    ("account-tie-outline", "account-tie-outline"),
                    ("account-tie-voice", "account-tie-voice"),
                    ("account-tie-voice-off", "account-tie-voice-off"),
                    ("account-tie-voice-off-outline", "account-tie-voice-off-outline"),
                    ("account-tie-voice-outline", "account-tie-voice-outline"),
                    ("account-tie-woman", "account-tie-woman"),
                    ("account-voice", "account-voice"),
                    ("account-voice-off", "account-voice-off"),
                    ("account-wrench", "account-wrench"),
                    ("account-wrench-outline", "account-wrench-outline"),
                    ("accusoft", "accusoft"),
                    ("ad-choices", "ad-choices"),
                    ("adchoices", "adchoices"),
                    ("adjust", "adjust"),
                    ("adobe", "adobe"),
                    ("advertisements", "advertisements"),
                    ("advertisements-off", "advertisements-off"),
                    ("air-conditioner", "air-conditioner"),
                    ("air-filter", "air-filter"),
                    ("air-horn", "air-horn"),
                    ("air-humidifier", "air-humidifier"),
                    ("air-humidifier-off", "air-humidifier-off"),
                    ("air-purifier", "air-purifier"),
                    ("air-purifier-off", "air-purifier-off"),
                    ("airbag", "airbag"),
                    ("airballoon", "airballoon"),
                    ("airballoon-outline", "airballoon-outline"),
                    ("airplane", "airplane"),
                    ("airplane-alert", "airplane-alert"),
                    ("airplane-check", "airplane-check"),
                    ("airplane-clock", "airplane-clock"),
                    ("airplane-cog", "airplane-cog"),
                    ("airplane-edit", "airplane-edit"),
                    ("airplane-landing", "airplane-landing"),
                    ("airplane-marker", "airplane-marker"),
                    ("airplane-minus", "airplane-minus"),
                    ("airplane-off", "airplane-off"),
                    ("airplane-plus", "airplane-plus"),
                    ("airplane-remove", "airplane-remove"),
                    ("airplane-search", "airplane-search"),
                    ("airplane-settings", "airplane-settings"),
                    ("airplane-takeoff", "airplane-takeoff"),
                    ("airport", "airport"),
                    ("alarm", "alarm"),
                    ("alarm-bell", "alarm-bell"),
                    ("alarm-check", "alarm-check"),
                    ("alarm-light", "alarm-light"),
                    ("alarm-light-off", "alarm-light-off"),
                    ("alarm-light-off-outline", "alarm-light-off-outline"),
                    ("alarm-light-outline", "alarm-light-outline"),
                    ("alarm-multiple", "alarm-multiple"),
                    ("alarm-note", "alarm-note"),
                    ("alarm-note-off", "alarm-note-off"),
                    ("alarm-off", "alarm-off"),
                    ("alarm-panel", "alarm-panel"),
                    ("alarm-panel-outline", "alarm-panel-outline"),
                    ("alarm-plus", "alarm-plus"),
                    ("alarm-snooze", "alarm-snooze"),
                    ("album", "album"),
                    ("alert", "alert"),
                    ("alert-box", "alert-box"),
                    ("alert-box-outline", "alert-box-outline"),
                    ("alert-circle", "alert-circle"),
                    ("alert-circle-check", "alert-circle-check"),
                    ("alert-circle-check-outline", "alert-circle-check-outline"),
                    ("alert-circle-outline", "alert-circle-outline"),
                    ("alert-decagram", "alert-decagram"),
                    ("alert-decagram-outline", "alert-decagram-outline"),
                    ("alert-minus", "alert-minus"),
                    ("alert-minus-outline", "alert-minus-outline"),
                    ("alert-octagon", "alert-octagon"),
                    ("alert-octagon-outline", "alert-octagon-outline"),
                    ("alert-octagram", "alert-octagram"),
                    ("alert-octagram-outline", "alert-octagram-outline"),
                    ("alert-outline", "alert-outline"),
                    ("alert-plus", "alert-plus"),
                    ("alert-plus-outline", "alert-plus-outline"),
                    ("alert-remove", "alert-remove"),
                    ("alert-remove-outline", "alert-remove-outline"),
                    ("alert-rhombus", "alert-rhombus"),
                    ("alert-rhombus-outline", "alert-rhombus-outline"),
                    ("alien", "alien"),
                    ("alien-outline", "alien-outline"),
                    ("align-horizontal-center", "align-horizontal-center"),
                    ("align-horizontal-distribute", "align-horizontal-distribute"),
                    ("align-horizontal-left", "align-horizontal-left"),
                    ("align-horizontal-right", "align-horizontal-right"),
                    ("align-vertical-bottom", "align-vertical-bottom"),
                    ("align-vertical-center", "align-vertical-center"),
                    ("align-vertical-distribute", "align-vertical-distribute"),
                    ("align-vertical-top", "align-vertical-top"),
                    ("all-inclusive", "all-inclusive"),
                    ("all-inclusive-box", "all-inclusive-box"),
                    ("all-inclusive-box-outline", "all-inclusive-box-outline"),
                    ("allergy", "allergy"),
                    ("allo", "allo"),
                    ("alpha", "alpha"),
                    ("alpha-a", "alpha-a"),
                    ("alpha-a-box", "alpha-a-box"),
                    ("alpha-a-box-outline", "alpha-a-box-outline"),
                    ("alpha-a-circle", "alpha-a-circle"),
                    ("alpha-a-circle-outline", "alpha-a-circle-outline"),
                    ("alpha-b", "alpha-b"),
                    ("alpha-b-box", "alpha-b-box"),
                    ("alpha-b-box-outline", "alpha-b-box-outline"),
                    ("alpha-b-circle", "alpha-b-circle"),
                    ("alpha-b-circle-outline", "alpha-b-circle-outline"),
                    ("alpha-c", "alpha-c"),
                    ("alpha-c-box", "alpha-c-box"),
                    ("alpha-c-box-outline", "alpha-c-box-outline"),
                    ("alpha-c-circle", "alpha-c-circle"),
                    ("alpha-c-circle-outline", "alpha-c-circle-outline"),
                    ("alpha-d", "alpha-d"),
                    ("alpha-d-box", "alpha-d-box"),
                    ("alpha-d-box-outline", "alpha-d-box-outline"),
                    ("alpha-d-circle", "alpha-d-circle"),
                    ("alpha-d-circle-outline", "alpha-d-circle-outline"),
                    ("alpha-e", "alpha-e"),
                    ("alpha-e-box", "alpha-e-box"),
                    ("alpha-e-box-outline", "alpha-e-box-outline"),
                    ("alpha-e-circle", "alpha-e-circle"),
                    ("alpha-e-circle-outline", "alpha-e-circle-outline"),
                    ("alpha-f", "alpha-f"),
                    ("alpha-f-box", "alpha-f-box"),
                    ("alpha-f-box-outline", "alpha-f-box-outline"),
                    ("alpha-f-circle", "alpha-f-circle"),
                    ("alpha-f-circle-outline", "alpha-f-circle-outline"),
                    ("alpha-g", "alpha-g"),
                    ("alpha-g-box", "alpha-g-box"),
                    ("alpha-g-box-outline", "alpha-g-box-outline"),
                    ("alpha-g-circle", "alpha-g-circle"),
                    ("alpha-g-circle-outline", "alpha-g-circle-outline"),
                    ("alpha-h", "alpha-h"),
                    ("alpha-h-box", "alpha-h-box"),
                    ("alpha-h-box-outline", "alpha-h-box-outline"),
                    ("alpha-h-circle", "alpha-h-circle"),
                    ("alpha-h-circle-outline", "alpha-h-circle-outline"),
                    ("alpha-i", "alpha-i"),
                    ("alpha-i-box", "alpha-i-box"),
                    ("alpha-i-box-outline", "alpha-i-box-outline"),
                    ("alpha-i-circle", "alpha-i-circle"),
                    ("alpha-i-circle-outline", "alpha-i-circle-outline"),
                    ("alpha-j", "alpha-j"),
                    ("alpha-j-box", "alpha-j-box"),
                    ("alpha-j-box-outline", "alpha-j-box-outline"),
                    ("alpha-j-circle", "alpha-j-circle"),
                    ("alpha-j-circle-outline", "alpha-j-circle-outline"),
                    ("alpha-k", "alpha-k"),
                    ("alpha-k-box", "alpha-k-box"),
                    ("alpha-k-box-outline", "alpha-k-box-outline"),
                    ("alpha-k-circle", "alpha-k-circle"),
                    ("alpha-k-circle-outline", "alpha-k-circle-outline"),
                    ("alpha-l", "alpha-l"),
                    ("alpha-l-box", "alpha-l-box"),
                    ("alpha-l-box-outline", "alpha-l-box-outline"),
                    ("alpha-l-circle", "alpha-l-circle"),
                    ("alpha-l-circle-outline", "alpha-l-circle-outline"),
                    ("alpha-m", "alpha-m"),
                    ("alpha-m-box", "alpha-m-box"),
                    ("alpha-m-box-outline", "alpha-m-box-outline"),
                    ("alpha-m-circle", "alpha-m-circle"),
                    ("alpha-m-circle-outline", "alpha-m-circle-outline"),
                    ("alpha-n", "alpha-n"),
                    ("alpha-n-box", "alpha-n-box"),
                    ("alpha-n-box-outline", "alpha-n-box-outline"),
                    ("alpha-n-circle", "alpha-n-circle"),
                    ("alpha-n-circle-outline", "alpha-n-circle-outline"),
                    ("alpha-o", "alpha-o"),
                    ("alpha-o-box", "alpha-o-box"),
                    ("alpha-o-box-outline", "alpha-o-box-outline"),
                    ("alpha-o-circle", "alpha-o-circle"),
                    ("alpha-o-circle-outline", "alpha-o-circle-outline"),
                    ("alpha-p", "alpha-p"),
                    ("alpha-p-box", "alpha-p-box"),
                    ("alpha-p-box-outline", "alpha-p-box-outline"),
                    ("alpha-p-circle", "alpha-p-circle"),
                    ("alpha-p-circle-outline", "alpha-p-circle-outline"),
                    ("alpha-q", "alpha-q"),
                    ("alpha-q-box", "alpha-q-box"),
                    ("alpha-q-box-outline", "alpha-q-box-outline"),
                    ("alpha-q-circle", "alpha-q-circle"),
                    ("alpha-q-circle-outline", "alpha-q-circle-outline"),
                    ("alpha-r", "alpha-r"),
                    ("alpha-r-box", "alpha-r-box"),
                    ("alpha-r-box-outline", "alpha-r-box-outline"),
                    ("alpha-r-circle", "alpha-r-circle"),
                    ("alpha-r-circle-outline", "alpha-r-circle-outline"),
                    ("alpha-s", "alpha-s"),
                    ("alpha-s-box", "alpha-s-box"),
                    ("alpha-s-box-outline", "alpha-s-box-outline"),
                    ("alpha-s-circle", "alpha-s-circle"),
                    ("alpha-s-circle-outline", "alpha-s-circle-outline"),
                    ("alpha-t", "alpha-t"),
                    ("alpha-t-box", "alpha-t-box"),
                    ("alpha-t-box-outline", "alpha-t-box-outline"),
                    ("alpha-t-circle", "alpha-t-circle"),
                    ("alpha-t-circle-outline", "alpha-t-circle-outline"),
                    ("alpha-u", "alpha-u"),
                    ("alpha-u-box", "alpha-u-box"),
                    ("alpha-u-box-outline", "alpha-u-box-outline"),
                    ("alpha-u-circle", "alpha-u-circle"),
                    ("alpha-u-circle-outline", "alpha-u-circle-outline"),
                    ("alpha-v", "alpha-v"),
                    ("alpha-v-box", "alpha-v-box"),
                    ("alpha-v-box-outline", "alpha-v-box-outline"),
                    ("alpha-v-circle", "alpha-v-circle"),
                    ("alpha-v-circle-outline", "alpha-v-circle-outline"),
                    ("alpha-w", "alpha-w"),
                    ("alpha-w-box", "alpha-w-box"),
                    ("alpha-w-box-outline", "alpha-w-box-outline"),
                    ("alpha-w-circle", "alpha-w-circle"),
                    ("alpha-w-circle-outline", "alpha-w-circle-outline"),
                    ("alpha-x", "alpha-x"),
                    ("alpha-x-box", "alpha-x-box"),
                    ("alpha-x-box-outline", "alpha-x-box-outline"),
                    ("alpha-x-circle", "alpha-x-circle"),
                    ("alpha-x-circle-outline", "alpha-x-circle-outline"),
                    ("alpha-y", "alpha-y"),
                    ("alpha-y-box", "alpha-y-box"),
                    ("alpha-y-box-outline", "alpha-y-box-outline"),
                    ("alpha-y-circle", "alpha-y-circle"),
                    ("alpha-y-circle-outline", "alpha-y-circle-outline"),
                    ("alpha-z", "alpha-z"),
                    ("alpha-z-box", "alpha-z-box"),
                    ("alpha-z-box-outline", "alpha-z-box-outline"),
                    ("alpha-z-circle", "alpha-z-circle"),
                    ("alpha-z-circle-outline", "alpha-z-circle-outline"),
                    ("alphabet-aurebesh", "alphabet-aurebesh"),
                    ("alphabet-cyrillic", "alphabet-cyrillic"),
                    ("alphabet-greek", "alphabet-greek"),
                    ("alphabet-latin", "alphabet-latin"),
                    ("alphabet-piqad", "alphabet-piqad"),
                    ("alphabet-tengwar", "alphabet-tengwar"),
                    ("alphabetical", "alphabetical"),
                    ("alphabetical-off", "alphabetical-off"),
                    ("alphabetical-variant", "alphabetical-variant"),
                    ("alphabetical-variant-off", "alphabetical-variant-off"),
                    ("altimeter", "altimeter"),
                    ("amazon", "amazon"),
                    ("amazon-alexa", "amazon-alexa"),
                    ("amazon-drive", "amazon-drive"),
                    ("ambulance", "ambulance"),
                    ("ammunition", "ammunition"),
                    ("ampersand", "ampersand"),
                    ("amplifier", "amplifier"),
                    ("amplifier-off", "amplifier-off"),
                    ("anchor", "anchor"),
                    ("android", "android"),
                    ("android-auto", "android-auto"),
                    ("android-debug-bridge", "android-debug-bridge"),
                    ("android-head", "android-head"),
                    ("android-messages", "android-messages"),
                    ("android-studio", "android-studio"),
                    ("angle-acute", "angle-acute"),
                    ("angle-obtuse", "angle-obtuse"),
                    ("angle-right", "angle-right"),
                    ("angular", "angular"),
                    ("angularjs", "angularjs"),
                    ("animation", "animation"),
                    ("animation-outline", "animation-outline"),
                    ("animation-play", "animation-play"),
                    ("animation-play-outline", "animation-play-outline"),
                    ("ansible", "ansible"),
                    ("antenna", "antenna"),
                    ("anvil", "anvil"),
                    ("apache-kafka", "apache-kafka"),
                    ("api", "api"),
                    ("api-off", "api-off"),
                    ("apple", "apple"),
                    ("apple-finder", "apple-finder"),
                    ("apple-icloud", "apple-icloud"),
                    ("apple-ios", "apple-ios"),
                    ("apple-keyboard-caps", "apple-keyboard-caps"),
                    ("apple-keyboard-command", "apple-keyboard-command"),
                    ("apple-keyboard-control", "apple-keyboard-control"),
                    ("apple-keyboard-option", "apple-keyboard-option"),
                    ("apple-keyboard-shift", "apple-keyboard-shift"),
                    ("apple-safari", "apple-safari"),
                    ("application", "application"),
                    ("application-array", "application-array"),
                    ("application-array-outline", "application-array-outline"),
                    ("application-braces", "application-braces"),
                    ("application-braces-outline", "application-braces-outline"),
                    ("application-brackets", "application-brackets"),
                    ("application-brackets-outline", "application-brackets-outline"),
                    ("application-cog", "application-cog"),
                    ("application-cog-outline", "application-cog-outline"),
                    ("application-edit", "application-edit"),
                    ("application-edit-outline", "application-edit-outline"),
                    ("application-export", "application-export"),
                    ("application-import", "application-import"),
                    ("application-outline", "application-outline"),
                    ("application-parentheses", "application-parentheses"),
                    ("application-parentheses-outline", "application-parentheses-outline"),
                    ("application-settings", "application-settings"),
                    ("application-settings-outline", "application-settings-outline"),
                    ("application-variable", "application-variable"),
                    ("application-variable-outline", "application-variable-outline"),
                    ("appnet", "appnet"),
                    ("approximately-equal", "approximately-equal"),
                    ("approximately-equal-box", "approximately-equal-box"),
                    ("apps", "apps"),
                    ("apps-box", "apps-box"),
                    ("arch", "arch"),
                    ("archive", "archive"),
                    ("archive-alert", "archive-alert"),
                    ("archive-alert-outline", "archive-alert-outline"),
                    ("archive-arrow-down", "archive-arrow-down"),
                    ("archive-arrow-down-outline", "archive-arrow-down-outline"),
                    ("archive-arrow-up", "archive-arrow-up"),
                    ("archive-arrow-up-outline", "archive-arrow-up-outline"),
                    ("archive-cancel", "archive-cancel"),
                    ("archive-cancel-outline", "archive-cancel-outline"),
                    ("archive-check", "archive-check"),
                    ("archive-check-outline", "archive-check-outline"),
                    ("archive-clock", "archive-clock"),
                    ("archive-clock-outline", "archive-clock-outline"),
                    ("archive-cog", "archive-cog"),
                    ("archive-cog-outline", "archive-cog-outline"),
                    ("archive-edit", "archive-edit"),
                    ("archive-edit-outline", "archive-edit-outline"),
                    ("archive-eye", "archive-eye"),
                    ("archive-eye-outline", "archive-eye-outline"),
                    ("archive-lock", "archive-lock"),
                    ("archive-lock-open", "archive-lock-open"),
                    ("archive-lock-open-outline", "archive-lock-open-outline"),
                    ("archive-lock-outline", "archive-lock-outline"),
                    ("archive-marker", "archive-marker"),
                    ("archive-marker-outline", "archive-marker-outline"),
                    ("archive-minus", "archive-minus"),
                    ("archive-minus-outline", "archive-minus-outline"),
                    ("archive-music", "archive-music"),
                    ("archive-music-outline", "archive-music-outline"),
                    ("archive-off", "archive-off"),
                    ("archive-off-outline", "archive-off-outline"),
                    ("archive-outline", "archive-outline"),
                    ("archive-plus", "archive-plus"),
                    ("archive-plus-outline", "archive-plus-outline"),
                    ("archive-refresh", "archive-refresh"),
                    ("archive-refresh-outline", "archive-refresh-outline"),
                    ("archive-remove", "archive-remove"),
                    ("archive-remove-outline", "archive-remove-outline"),
                    ("archive-search", "archive-search"),
                    ("archive-search-outline", "archive-search-outline"),
                    ("archive-settings", "archive-settings"),
                    ("archive-settings-outline", "archive-settings-outline"),
                    ("archive-star", "archive-star"),
                    ("archive-star-outline", "archive-star-outline"),
                    ("archive-sync", "archive-sync"),
                    ("archive-sync-outline", "archive-sync-outline"),
                    ("arm-flex", "arm-flex"),
                    ("arm-flex-outline", "arm-flex-outline"),
                    ("arrange-bring-forward", "arrange-bring-forward"),
                    ("arrange-bring-to-front", "arrange-bring-to-front"),
                    ("arrange-send-backward", "arrange-send-backward"),
                    ("arrange-send-to-back", "arrange-send-to-back"),
                    ("arrow-all", "arrow-all"),
                    ("arrow-bottom-left", "arrow-bottom-left"),
                    ("arrow-bottom-left-bold-box", "arrow-bottom-left-bold-box"),
                    ("arrow-bottom-left-bold-box-outline", "arrow-bottom-left-bold-box-outline"),
                    ("arrow-bottom-left-bold-outline", "arrow-bottom-left-bold-outline"),
                    ("arrow-bottom-left-thick", "arrow-bottom-left-thick"),
                    ("arrow-bottom-left-thin", "arrow-bottom-left-thin"),
                    (
                        "arrow-bottom-left-thin-circle-outline",
                        "arrow-bottom-left-thin-circle-outline",
                    ),
                    ("arrow-bottom-right", "arrow-bottom-right"),
                    ("arrow-bottom-right-bold-box", "arrow-bottom-right-bold-box"),
                    ("arrow-bottom-right-bold-box-outline", "arrow-bottom-right-bold-box-outline"),
                    ("arrow-bottom-right-bold-outline", "arrow-bottom-right-bold-outline"),
                    ("arrow-bottom-right-thick", "arrow-bottom-right-thick"),
                    ("arrow-bottom-right-thin", "arrow-bottom-right-thin"),
                    (
                        "arrow-bottom-right-thin-circle-outline",
                        "arrow-bottom-right-thin-circle-outline",
                    ),
                    ("arrow-collapse", "arrow-collapse"),
                    ("arrow-collapse-all", "arrow-collapse-all"),
                    ("arrow-collapse-down", "arrow-collapse-down"),
                    ("arrow-collapse-horizontal", "arrow-collapse-horizontal"),
                    ("arrow-collapse-left", "arrow-collapse-left"),
                    ("arrow-collapse-right", "arrow-collapse-right"),
                    ("arrow-collapse-up", "arrow-collapse-up"),
                    ("arrow-collapse-vertical", "arrow-collapse-vertical"),
                    ("arrow-decision", "arrow-decision"),
                    ("arrow-decision-auto", "arrow-decision-auto"),
                    ("arrow-decision-auto-outline", "arrow-decision-auto-outline"),
                    ("arrow-decision-outline", "arrow-decision-outline"),
                    ("arrow-down", "arrow-down"),
                    ("arrow-down-bold", "arrow-down-bold"),
                    ("arrow-down-bold-box", "arrow-down-bold-box"),
                    ("arrow-down-bold-box-outline", "arrow-down-bold-box-outline"),
                    ("arrow-down-bold-circle", "arrow-down-bold-circle"),
                    ("arrow-down-bold-circle-outline", "arrow-down-bold-circle-outline"),
                    ("arrow-down-bold-hexagon-outline", "arrow-down-bold-hexagon-outline"),
                    ("arrow-down-bold-outline", "arrow-down-bold-outline"),
                    ("arrow-down-box", "arrow-down-box"),
                    ("arrow-down-circle", "arrow-down-circle"),
                    ("arrow-down-circle-outline", "arrow-down-circle-outline"),
                    ("arrow-down-drop-circle", "arrow-down-drop-circle"),
                    ("arrow-down-drop-circle-outline", "arrow-down-drop-circle-outline"),
                    ("arrow-down-left", "arrow-down-left"),
                    ("arrow-down-left-bold", "arrow-down-left-bold"),
                    ("arrow-down-right", "arrow-down-right"),
                    ("arrow-down-right-bold", "arrow-down-right-bold"),
                    ("arrow-down-thick", "arrow-down-thick"),
                    ("arrow-down-thin", "arrow-down-thin"),
                    ("arrow-down-thin-circle-outline", "arrow-down-thin-circle-outline"),
                    ("arrow-expand", "arrow-expand"),
                    ("arrow-expand-all", "arrow-expand-all"),
                    ("arrow-expand-down", "arrow-expand-down"),
                    ("arrow-expand-horizontal", "arrow-expand-horizontal"),
                    ("arrow-expand-left", "arrow-expand-left"),
                    ("arrow-expand-right", "arrow-expand-right"),
                    ("arrow-expand-up", "arrow-expand-up"),
                    ("arrow-expand-vertical", "arrow-expand-vertical"),
                    ("arrow-horizontal-lock", "arrow-horizontal-lock"),
                    ("arrow-left", "arrow-left"),
                    ("arrow-left-bold", "arrow-left-bold"),
                    ("arrow-left-bold-box", "arrow-left-bold-box"),
                    ("arrow-left-bold-box-outline", "arrow-left-bold-box-outline"),
                    ("arrow-left-bold-circle", "arrow-left-bold-circle"),
                    ("arrow-left-bold-circle-outline", "arrow-left-bold-circle-outline"),
                    ("arrow-left-bold-hexagon-outline", "arrow-left-bold-hexagon-outline"),
                    ("arrow-left-bold-outline", "arrow-left-bold-outline"),
                    ("arrow-left-bottom", "arrow-left-bottom"),
                    ("arrow-left-bottom-bold", "arrow-left-bottom-bold"),
                    ("arrow-left-box", "arrow-left-box"),
                    ("arrow-left-circle", "arrow-left-circle"),
                    ("arrow-left-circle-outline", "arrow-left-circle-outline"),
                    ("arrow-left-drop-circle", "arrow-left-drop-circle"),
                    ("arrow-left-drop-circle-outline", "arrow-left-drop-circle-outline"),
                    ("arrow-left-right", "arrow-left-right"),
                    ("arrow-left-right-bold", "arrow-left-right-bold"),
                    ("arrow-left-right-bold-outline", "arrow-left-right-bold-outline"),
                    ("arrow-left-thick", "arrow-left-thick"),
                    ("arrow-left-thin", "arrow-left-thin"),
                    ("arrow-left-thin-circle-outline", "arrow-left-thin-circle-outline"),
                    ("arrow-left-top", "arrow-left-top"),
                    ("arrow-left-top-bold", "arrow-left-top-bold"),
                    ("arrow-oscillating", "arrow-oscillating"),
                    ("arrow-oscillating-off", "arrow-oscillating-off"),
                    ("arrow-projectile", "arrow-projectile"),
                    ("arrow-projectile-multiple", "arrow-projectile-multiple"),
                    ("arrow-right", "arrow-right"),
                    ("arrow-right-bold", "arrow-right-bold"),
                    ("arrow-right-bold-box", "arrow-right-bold-box"),
                    ("arrow-right-bold-box-outline", "arrow-right-bold-box-outline"),
                    ("arrow-right-bold-circle", "arrow-right-bold-circle"),
                    ("arrow-right-bold-circle-outline", "arrow-right-bold-circle-outline"),
                    ("arrow-right-bold-hexagon-outline", "arrow-right-bold-hexagon-outline"),
                    ("arrow-right-bold-outline", "arrow-right-bold-outline"),
                    ("arrow-right-bottom", "arrow-right-bottom"),
                    ("arrow-right-bottom-bold", "arrow-right-bottom-bold"),
                    ("arrow-right-box", "arrow-right-box"),
                    ("arrow-right-circle", "arrow-right-circle"),
                    ("arrow-right-circle-outline", "arrow-right-circle-outline"),
                    ("arrow-right-drop-circle", "arrow-right-drop-circle"),
                    ("arrow-right-drop-circle-outline", "arrow-right-drop-circle-outline"),
                    ("arrow-right-thick", "arrow-right-thick"),
                    ("arrow-right-thin", "arrow-right-thin"),
                    ("arrow-right-thin-circle-outline", "arrow-right-thin-circle-outline"),
                    ("arrow-right-top", "arrow-right-top"),
                    ("arrow-right-top-bold", "arrow-right-top-bold"),
                    ("arrow-split-horizontal", "arrow-split-horizontal"),
                    ("arrow-split-vertical", "arrow-split-vertical"),
                    ("arrow-top-left", "arrow-top-left"),
                    ("arrow-top-left-bold-box", "arrow-top-left-bold-box"),
                    ("arrow-top-left-bold-box-outline", "arrow-top-left-bold-box-outline"),
                    ("arrow-top-left-bold-outline", "arrow-top-left-bold-outline"),
                    ("arrow-top-left-bottom-right", "arrow-top-left-bottom-right"),
                    ("arrow-top-left-bottom-right-bold", "arrow-top-left-bottom-right-bold"),
                    ("arrow-top-left-thick", "arrow-top-left-thick"),
                    ("arrow-top-left-thin", "arrow-top-left-thin"),
                    ("arrow-top-left-thin-circle-outline", "arrow-top-left-thin-circle-outline"),
                    ("arrow-top-right", "arrow-top-right"),
                    ("arrow-top-right-bold-box", "arrow-top-right-bold-box"),
                    ("arrow-top-right-bold-box-outline", "arrow-top-right-bold-box-outline"),
                    ("arrow-top-right-bold-outline", "arrow-top-right-bold-outline"),
                    ("arrow-top-right-bottom-left", "arrow-top-right-bottom-left"),
                    ("arrow-top-right-bottom-left-bold", "arrow-top-right-bottom-left-bold"),
                    ("arrow-top-right-thick", "arrow-top-right-thick"),
                    ("arrow-top-right-thin", "arrow-top-right-thin"),
                    ("arrow-top-right-thin-circle-outline", "arrow-top-right-thin-circle-outline"),
                    ("arrow-u-down-left", "arrow-u-down-left"),
                    ("arrow-u-down-left-bold", "arrow-u-down-left-bold"),
                    ("arrow-u-down-right", "arrow-u-down-right"),
                    ("arrow-u-down-right-bold", "arrow-u-down-right-bold"),
                    ("arrow-u-left-bottom", "arrow-u-left-bottom"),
                    ("arrow-u-left-bottom-bold", "arrow-u-left-bottom-bold"),
                    ("arrow-u-left-top", "arrow-u-left-top"),
                    ("arrow-u-left-top-bold", "arrow-u-left-top-bold"),
                    ("arrow-u-right-bottom", "arrow-u-right-bottom"),
                    ("arrow-u-right-bottom-bold", "arrow-u-right-bottom-bold"),
                    ("arrow-u-right-top", "arrow-u-right-top"),
                    ("arrow-u-right-top-bold", "arrow-u-right-top-bold"),
                    ("arrow-u-up-left", "arrow-u-up-left"),
                    ("arrow-u-up-left-bold", "arrow-u-up-left-bold"),
                    ("arrow-u-up-right", "arrow-u-up-right"),
                    ("arrow-u-up-right-bold", "arrow-u-up-right-bold"),
                    ("arrow-up", "arrow-up"),
                    ("arrow-up-bold", "arrow-up-bold"),
                    ("arrow-up-bold-box", "arrow-up-bold-box"),
                    ("arrow-up-bold-box-outline", "arrow-up-bold-box-outline"),
                    ("arrow-up-bold-circle", "arrow-up-bold-circle"),
                    ("arrow-up-bold-circle-outline", "arrow-up-bold-circle-outline"),
                    ("arrow-up-bold-hexagon-outline", "arrow-up-bold-hexagon-outline"),
                    ("arrow-up-bold-outline", "arrow-up-bold-outline"),
                    ("arrow-up-box", "arrow-up-box"),
                    ("arrow-up-circle", "arrow-up-circle"),
                    ("arrow-up-circle-outline", "arrow-up-circle-outline"),
                    ("arrow-up-down", "arrow-up-down"),
                    ("arrow-up-down-bold", "arrow-up-down-bold"),
                    ("arrow-up-down-bold-outline", "arrow-up-down-bold-outline"),
                    ("arrow-up-drop-circle", "arrow-up-drop-circle"),
                    ("arrow-up-drop-circle-outline", "arrow-up-drop-circle-outline"),
                    ("arrow-up-left", "arrow-up-left"),
                    ("arrow-up-left-bold", "arrow-up-left-bold"),
                    ("arrow-up-right", "arrow-up-right"),
                    ("arrow-up-right-bold", "arrow-up-right-bold"),
                    ("arrow-up-thick", "arrow-up-thick"),
                    ("arrow-up-thin", "arrow-up-thin"),
                    ("arrow-up-thin-circle-outline", "arrow-up-thin-circle-outline"),
                    ("arrow-vertical-lock", "arrow-vertical-lock"),
                    ("artboard", "artboard"),
                    ("artstation", "artstation"),
                    ("aspect-ratio", "aspect-ratio"),
                    ("assistant", "assistant"),
                    ("asterisk", "asterisk"),
                    ("asterisk-circle-outline", "asterisk-circle-outline"),
                    ("at", "at"),
                    ("atlassian", "atlassian"),
                    ("atm", "atm"),
                    ("atom", "atom"),
                    ("atom-variant", "atom-variant"),
                    ("attachment", "attachment"),
                    ("attachment-check", "attachment-check"),
                    ("attachment-lock", "attachment-lock"),
                    ("attachment-minus", "attachment-minus"),
                    ("attachment-off", "attachment-off"),
                    ("attachment-plus", "attachment-plus"),
                    ("attachment-remove", "attachment-remove"),
                    ("atv", "atv"),
                    ("audio-input-rca", "audio-input-rca"),
                    ("audio-input-stereo-minijack", "audio-input-stereo-minijack"),
                    ("audio-input-xlr", "audio-input-xlr"),
                    ("audio-video", "audio-video"),
                    ("audio-video-off", "audio-video-off"),
                    ("augmented-reality", "augmented-reality"),
                    ("aurora", "aurora"),
                    ("auto-download", "auto-download"),
                    ("auto-fix", "auto-fix"),
                    ("auto-mode", "auto-mode"),
                    ("auto-upload", "auto-upload"),
                    ("autorenew", "autorenew"),
                    ("autorenew-off", "autorenew-off"),
                    ("av-timer", "av-timer"),
                    ("awning", "awning"),
                    ("awning-outline", "awning-outline"),
                    ("aws", "aws"),
                    ("axe", "axe"),
                    ("axe-battle", "axe-battle"),
                    ("axis", "axis"),
                    ("axis-arrow", "axis-arrow"),
                    ("axis-arrow-info", "axis-arrow-info"),
                    ("axis-arrow-lock", "axis-arrow-lock"),
                    ("axis-lock", "axis-lock"),
                    ("axis-x-arrow", "axis-x-arrow"),
                    ("axis-x-arrow-lock", "axis-x-arrow-lock"),
                    ("axis-x-rotate-clockwise", "axis-x-rotate-clockwise"),
                    ("axis-x-rotate-counterclockwise", "axis-x-rotate-counterclockwise"),
                    ("axis-x-y-arrow-lock", "axis-x-y-arrow-lock"),
                    ("axis-y-arrow", "axis-y-arrow"),
                    ("axis-y-arrow-lock", "axis-y-arrow-lock"),
                    ("axis-y-rotate-clockwise", "axis-y-rotate-clockwise"),
                    ("axis-y-rotate-counterclockwise", "axis-y-rotate-counterclockwise"),
                    ("axis-z-arrow", "axis-z-arrow"),
                    ("axis-z-arrow-lock", "axis-z-arrow-lock"),
                    ("axis-z-rotate-clockwise", "axis-z-rotate-clockwise"),
                    ("axis-z-rotate-counterclockwise", "axis-z-rotate-counterclockwise"),
                    ("babel", "babel"),
                    ("baby", "baby"),
                    ("baby-bottle", "baby-bottle"),
                    ("baby-bottle-outline", "baby-bottle-outline"),
                    ("baby-buggy", "baby-buggy"),
                    ("baby-buggy-off", "baby-buggy-off"),
                    ("baby-carriage", "baby-carriage"),
                    ("baby-carriage-off", "baby-carriage-off"),
                    ("baby-face", "baby-face"),
                    ("baby-face-outline", "baby-face-outline"),
                    ("backburger", "backburger"),
                    ("backspace", "backspace"),
                    ("backspace-outline", "backspace-outline"),
                    ("backspace-reverse", "backspace-reverse"),
                    ("backspace-reverse-outline", "backspace-reverse-outline"),
                    ("backup-restore", "backup-restore"),
                    ("bacteria", "bacteria"),
                    ("bacteria-outline", "bacteria-outline"),
                    ("badge-account", "badge-account"),
                    ("badge-account-alert", "badge-account-alert"),
                    ("badge-account-alert-outline", "badge-account-alert-outline"),
                    ("badge-account-horizontal", "badge-account-horizontal"),
                    ("badge-account-horizontal-outline", "badge-account-horizontal-outline"),
                    ("badge-account-outline", "badge-account-outline"),
                    ("badminton", "badminton"),
                    ("bag-carry-on", "bag-carry-on"),
                    ("bag-carry-on-check", "bag-carry-on-check"),
                    ("bag-carry-on-off", "bag-carry-on-off"),
                    ("bag-checked", "bag-checked"),
                    ("bag-personal", "bag-personal"),
                    ("bag-personal-off", "bag-personal-off"),
                    ("bag-personal-off-outline", "bag-personal-off-outline"),
                    ("bag-personal-outline", "bag-personal-outline"),
                    ("bag-personal-plus", "bag-personal-plus"),
                    ("bag-personal-plus-outline", "bag-personal-plus-outline"),
                    ("bag-personal-tag", "bag-personal-tag"),
                    ("bag-personal-tag-outline", "bag-personal-tag-outline"),
                    ("bag-suitcase", "bag-suitcase"),
                    ("bag-suitcase-off", "bag-suitcase-off"),
                    ("bag-suitcase-off-outline", "bag-suitcase-off-outline"),
                    ("bag-suitcase-outline", "bag-suitcase-outline"),
                    ("baguette", "baguette"),
                    ("balcony", "balcony"),
                    ("balloon", "balloon"),
                    ("ballot", "ballot"),
                    ("ballot-outline", "ballot-outline"),
                    ("ballot-recount", "ballot-recount"),
                    ("ballot-recount-outline", "ballot-recount-outline"),
                    ("bandage", "bandage"),
                    ("bandcamp", "bandcamp"),
                    ("bank", "bank"),
                    ("bank-check", "bank-check"),
                    ("bank-circle", "bank-circle"),
                    ("bank-circle-outline", "bank-circle-outline"),
                    ("bank-minus", "bank-minus"),
                    ("bank-off", "bank-off"),
                    ("bank-off-outline", "bank-off-outline"),
                    ("bank-outline", "bank-outline"),
                    ("bank-plus", "bank-plus"),
                    ("bank-remove", "bank-remove"),
                    ("bank-transfer", "bank-transfer"),
                    ("bank-transfer-in", "bank-transfer-in"),
                    ("bank-transfer-out", "bank-transfer-out"),
                    ("barcode", "barcode"),
                    ("barcode-off", "barcode-off"),
                    ("barcode-scan", "barcode-scan"),
                    ("barley", "barley"),
                    ("barley-off", "barley-off"),
                    ("barn", "barn"),
                    ("barrel", "barrel"),
                    ("barrel-outline", "barrel-outline"),
                    ("baseball", "baseball"),
                    ("baseball-bat", "baseball-bat"),
                    ("baseball-diamond", "baseball-diamond"),
                    ("baseball-diamond-outline", "baseball-diamond-outline"),
                    ("baseball-outline", "baseball-outline"),
                    ("basecamp", "basecamp"),
                    ("bash", "bash"),
                    ("basket", "basket"),
                    ("basket-check", "basket-check"),
                    ("basket-check-outline", "basket-check-outline"),
                    ("basket-fill", "basket-fill"),
                    ("basket-minus", "basket-minus"),
                    ("basket-minus-outline", "basket-minus-outline"),
                    ("basket-off", "basket-off"),
                    ("basket-off-outline", "basket-off-outline"),
                    ("basket-outline", "basket-outline"),
                    ("basket-plus", "basket-plus"),
                    ("basket-plus-outline", "basket-plus-outline"),
                    ("basket-remove", "basket-remove"),
                    ("basket-remove-outline", "basket-remove-outline"),
                    ("basket-unfill", "basket-unfill"),
                    ("basketball", "basketball"),
                    ("basketball-hoop", "basketball-hoop"),
                    ("basketball-hoop-outline", "basketball-hoop-outline"),
                    ("bat", "bat"),
                    ("bathtub", "bathtub"),
                    ("bathtub-outline", "bathtub-outline"),
                    ("battery", "battery"),
                    ("battery-10", "battery-10"),
                    ("battery-10-bluetooth", "battery-10-bluetooth"),
                    ("battery-20", "battery-20"),
                    ("battery-20-bluetooth", "battery-20-bluetooth"),
                    ("battery-30", "battery-30"),
                    ("battery-30-bluetooth", "battery-30-bluetooth"),
                    ("battery-40", "battery-40"),
                    ("battery-40-bluetooth", "battery-40-bluetooth"),
                    ("battery-50", "battery-50"),
                    ("battery-50-bluetooth", "battery-50-bluetooth"),
                    ("battery-60", "battery-60"),
                    ("battery-60-bluetooth", "battery-60-bluetooth"),
                    ("battery-70", "battery-70"),
                    ("battery-70-bluetooth", "battery-70-bluetooth"),
                    ("battery-80", "battery-80"),
                    ("battery-80-bluetooth", "battery-80-bluetooth"),
                    ("battery-90", "battery-90"),
                    ("battery-90-bluetooth", "battery-90-bluetooth"),
                    ("battery-alert", "battery-alert"),
                    ("battery-alert-bluetooth", "battery-alert-bluetooth"),
                    ("battery-alert-variant", "battery-alert-variant"),
                    ("battery-alert-variant-outline", "battery-alert-variant-outline"),
                    ("battery-arrow-down", "battery-arrow-down"),
                    ("battery-arrow-down-outline", "battery-arrow-down-outline"),
                    ("battery-arrow-up", "battery-arrow-up"),
                    ("battery-arrow-up-outline", "battery-arrow-up-outline"),
                    ("battery-bluetooth", "battery-bluetooth"),
                    ("battery-bluetooth-variant", "battery-bluetooth-variant"),
                    ("battery-charging", "battery-charging"),
                    ("battery-charging-10", "battery-charging-10"),
                    ("battery-charging-100", "battery-charging-100"),
                    ("battery-charging-20", "battery-charging-20"),
                    ("battery-charging-30", "battery-charging-30"),
                    ("battery-charging-40", "battery-charging-40"),
                    ("battery-charging-50", "battery-charging-50"),
                    ("battery-charging-60", "battery-charging-60"),
                    ("battery-charging-70", "battery-charging-70"),
                    ("battery-charging-80", "battery-charging-80"),
                    ("battery-charging-90", "battery-charging-90"),
                    ("battery-charging-high", "battery-charging-high"),
                    ("battery-charging-low", "battery-charging-low"),
                    ("battery-charging-medium", "battery-charging-medium"),
                    ("battery-charging-outline", "battery-charging-outline"),
                    ("battery-charging-wireless", "battery-charging-wireless"),
                    ("battery-charging-wireless-10", "battery-charging-wireless-10"),
                    ("battery-charging-wireless-20", "battery-charging-wireless-20"),
                    ("battery-charging-wireless-30", "battery-charging-wireless-30"),
                    ("battery-charging-wireless-40", "battery-charging-wireless-40"),
                    ("battery-charging-wireless-50", "battery-charging-wireless-50"),
                    ("battery-charging-wireless-60", "battery-charging-wireless-60"),
                    ("battery-charging-wireless-70", "battery-charging-wireless-70"),
                    ("battery-charging-wireless-80", "battery-charging-wireless-80"),
                    ("battery-charging-wireless-90", "battery-charging-wireless-90"),
                    ("battery-charging-wireless-alert", "battery-charging-wireless-alert"),
                    ("battery-charging-wireless-outline", "battery-charging-wireless-outline"),
                    ("battery-check", "battery-check"),
                    ("battery-check-outline", "battery-check-outline"),
                    ("battery-clock", "battery-clock"),
                    ("battery-clock-outline", "battery-clock-outline"),
                    ("battery-heart", "battery-heart"),
                    ("battery-heart-outline", "battery-heart-outline"),
                    ("battery-heart-variant", "battery-heart-variant"),
                    ("battery-high", "battery-high"),
                    ("battery-lock", "battery-lock"),
                    ("battery-lock-open", "battery-lock-open"),
                    ("battery-low", "battery-low"),
                    ("battery-medium", "battery-medium"),
                    ("battery-minus", "battery-minus"),
                    ("battery-minus-outline", "battery-minus-outline"),
                    ("battery-minus-variant", "battery-minus-variant"),
                    ("battery-negative", "battery-negative"),
                    ("battery-off", "battery-off"),
                    ("battery-off-outline", "battery-off-outline"),
                    ("battery-outline", "battery-outline"),
                    ("battery-plus", "battery-plus"),
                    ("battery-plus-outline", "battery-plus-outline"),
                    ("battery-plus-variant", "battery-plus-variant"),
                    ("battery-positive", "battery-positive"),
                    ("battery-remove", "battery-remove"),
                    ("battery-remove-outline", "battery-remove-outline"),
                    ("battery-standard", "battery-standard"),
                    ("battery-sync", "battery-sync"),
                    ("battery-sync-outline", "battery-sync-outline"),
                    ("battery-unknown", "battery-unknown"),
                    ("battery-unknown-bluetooth", "battery-unknown-bluetooth"),
                    ("battlenet", "battlenet"),
                    ("beach", "beach"),
                    ("beaker", "beaker"),
                    ("beaker-alert", "beaker-alert"),
                    ("beaker-alert-outline", "beaker-alert-outline"),
                    ("beaker-check", "beaker-check"),
                    ("beaker-check-outline", "beaker-check-outline"),
                    ("beaker-minus", "beaker-minus"),
                    ("beaker-minus-outline", "beaker-minus-outline"),
                    ("beaker-outline", "beaker-outline"),
                    ("beaker-plus", "beaker-plus"),
                    ("beaker-plus-outline", "beaker-plus-outline"),
                    ("beaker-question", "beaker-question"),
                    ("beaker-question-outline", "beaker-question-outline"),
                    ("beaker-remove", "beaker-remove"),
                    ("beaker-remove-outline", "beaker-remove-outline"),
                    ("beam", "beam"),
                    ("beats", "beats"),
                    ("bed", "bed"),
                    ("bed-clock", "bed-clock"),
                    ("bed-double", "bed-double"),
                    ("bed-double-outline", "bed-double-outline"),
                    ("bed-empty", "bed-empty"),
                    ("bed-king", "bed-king"),
                    ("bed-king-outline", "bed-king-outline"),
                    ("bed-outline", "bed-outline"),
                    ("bed-queen", "bed-queen"),
                    ("bed-queen-outline", "bed-queen-outline"),
                    ("bed-single", "bed-single"),
                    ("bed-single-outline", "bed-single-outline"),
                    ("bee", "bee"),
                    ("bee-flower", "bee-flower"),
                    ("beehive-off-outline", "beehive-off-outline"),
                    ("beehive-outline", "beehive-outline"),
                    ("beekeeper", "beekeeper"),
                    ("beer", "beer"),
                    ("beer-outline", "beer-outline"),
                    ("behance", "behance"),
                    ("bell", "bell"),
                    ("bell-alert", "bell-alert"),
                    ("bell-alert-outline", "bell-alert-outline"),
                    ("bell-badge", "bell-badge"),
                    ("bell-badge-outline", "bell-badge-outline"),
                    ("bell-cancel", "bell-cancel"),
                    ("bell-cancel-outline", "bell-cancel-outline"),
                    ("bell-check", "bell-check"),
                    ("bell-check-outline", "bell-check-outline"),
                    ("bell-circle", "bell-circle"),
                    ("bell-circle-outline", "bell-circle-outline"),
                    ("bell-cog", "bell-cog"),
                    ("bell-cog-outline", "bell-cog-outline"),
                    ("bell-minus", "bell-minus"),
                    ("bell-minus-outline", "bell-minus-outline"),
                    ("bell-off", "bell-off"),
                    ("bell-off-outline", "bell-off-outline"),
                    ("bell-outline", "bell-outline"),
                    ("bell-plus", "bell-plus"),
                    ("bell-plus-outline", "bell-plus-outline"),
                    ("bell-remove", "bell-remove"),
                    ("bell-remove-outline", "bell-remove-outline"),
                    ("bell-ring", "bell-ring"),
                    ("bell-ring-outline", "bell-ring-outline"),
                    ("bell-sleep", "bell-sleep"),
                    ("bell-sleep-outline", "bell-sleep-outline"),
                    ("bench", "bench"),
                    ("bench-back", "bench-back"),
                    ("beta", "beta"),
                    ("betamax", "betamax"),
                    ("biathlon", "biathlon"),
                    ("bicycle", "bicycle"),
                    ("bicycle-basket", "bicycle-basket"),
                    ("bicycle-cargo", "bicycle-cargo"),
                    ("bicycle-electric", "bicycle-electric"),
                    ("bicycle-penny-farthing", "bicycle-penny-farthing"),
                    ("bike", "bike"),
                    ("bike-fast", "bike-fast"),
                    ("bike-pedal", "bike-pedal"),
                    ("bike-pedal-clipless", "bike-pedal-clipless"),
                    ("bike-pedal-mountain", "bike-pedal-mountain"),
                    ("billboard", "billboard"),
                    ("billiards", "billiards"),
                    ("billiards-rack", "billiards-rack"),
                    ("binoculars", "binoculars"),
                    ("bio", "bio"),
                    ("biohazard", "biohazard"),
                    ("bird", "bird"),
                    ("bitbucket", "bitbucket"),
                    ("bitcoin", "bitcoin"),
                    ("black-mesa", "black-mesa"),
                    ("blackberry", "blackberry"),
                    ("blender", "blender"),
                    ("blender-outline", "blender-outline"),
                    ("blender-software", "blender-software"),
                    ("blinds", "blinds"),
                    ("blinds-horizontal", "blinds-horizontal"),
                    ("blinds-horizontal-closed", "blinds-horizontal-closed"),
                    ("blinds-open", "blinds-open"),
                    ("blinds-vertical", "blinds-vertical"),
                    ("blinds-vertical-closed", "blinds-vertical-closed"),
                    ("block-helper", "block-helper"),
                    ("blogger", "blogger"),
                    ("blood-bag", "blood-bag"),
                    ("bluetooth", "bluetooth"),
                    ("bluetooth-audio", "bluetooth-audio"),
                    ("bluetooth-connect", "bluetooth-connect"),
                    ("bluetooth-off", "bluetooth-off"),
                    ("bluetooth-settings", "bluetooth-settings"),
                    ("bluetooth-transfer", "bluetooth-transfer"),
                    ("blur", "blur"),
                    ("blur-linear", "blur-linear"),
                    ("blur-off", "blur-off"),
                    ("blur-radial", "blur-radial"),
                    ("bolt", "bolt"),
                    ("bomb", "bomb"),
                    ("bomb-off", "bomb-off"),
                    ("bone", "bone"),
                    ("bone-off", "bone-off"),
                    ("book", "book"),
                    ("book-account", "book-account"),
                    ("book-account-outline", "book-account-outline"),
                    ("book-alert", "book-alert"),
                    ("book-alert-outline", "book-alert-outline"),
                    ("book-alphabet", "book-alphabet"),
                    ("book-arrow-down", "book-arrow-down"),
                    ("book-arrow-down-outline", "book-arrow-down-outline"),
                    ("book-arrow-left", "book-arrow-left"),
                    ("book-arrow-left-outline", "book-arrow-left-outline"),
                    ("book-arrow-right", "book-arrow-right"),
                    ("book-arrow-right-outline", "book-arrow-right-outline"),
                    ("book-arrow-up", "book-arrow-up"),
                    ("book-arrow-up-outline", "book-arrow-up-outline"),
                    ("book-cancel", "book-cancel"),
                    ("book-cancel-outline", "book-cancel-outline"),
                    ("book-check", "book-check"),
                    ("book-check-outline", "book-check-outline"),
                    ("book-clock", "book-clock"),
                    ("book-clock-outline", "book-clock-outline"),
                    ("book-cog", "book-cog"),
                    ("book-cog-outline", "book-cog-outline"),
                    ("book-cross", "book-cross"),
                    ("book-edit", "book-edit"),
                    ("book-edit-outline", "book-edit-outline"),
                    ("book-education", "book-education"),
                    ("book-education-outline", "book-education-outline"),
                    ("book-heart", "book-heart"),
                    ("book-heart-outline", "book-heart-outline"),
                    ("book-information-variant", "book-information-variant"),
                    ("book-lock", "book-lock"),
                    ("book-lock-open", "book-lock-open"),
                    ("book-lock-open-outline", "book-lock-open-outline"),
                    ("book-lock-outline", "book-lock-outline"),
                    ("book-marker", "book-marker"),
                    ("book-marker-outline", "book-marker-outline"),
                    ("book-minus", "book-minus"),
                    ("book-minus-multiple", "book-minus-multiple"),
                    ("book-minus-multiple-outline", "book-minus-multiple-outline"),
                    ("book-minus-outline", "book-minus-outline"),
                    ("book-multiple", "book-multiple"),
                    ("book-multiple-minus", "book-multiple-minus"),
                    ("book-multiple-outline", "book-multiple-outline"),
                    ("book-multiple-plus", "book-multiple-plus"),
                    ("book-multiple-remove", "book-multiple-remove"),
                    ("book-multiple-variant", "book-multiple-variant"),
                    ("book-music", "book-music"),
                    ("book-music-outline", "book-music-outline"),
                    ("book-off", "book-off"),
                    ("book-off-outline", "book-off-outline"),
                    ("book-open", "book-open"),
                    ("book-open-blank-variant", "book-open-blank-variant"),
                    ("book-open-outline", "book-open-outline"),
                    ("book-open-page-variant", "book-open-page-variant"),
                    ("book-open-page-variant-outline", "book-open-page-variant-outline"),
                    ("book-open-variant", "book-open-variant"),
                    ("book-outline", "book-outline"),
                    ("book-play", "book-play"),
                    ("book-play-outline", "book-play-outline"),
                    ("book-plus", "book-plus"),
                    ("book-plus-multiple", "book-plus-multiple"),
                    ("book-plus-multiple-outline", "book-plus-multiple-outline"),
                    ("book-plus-outline", "book-plus-outline"),
                    ("book-refresh", "book-refresh"),
                    ("book-refresh-outline", "book-refresh-outline"),
                    ("book-remove", "book-remove"),
                    ("book-remove-multiple", "book-remove-multiple"),
                    ("book-remove-multiple-outline", "book-remove-multiple-outline"),
                    ("book-remove-outline", "book-remove-outline"),
                    ("book-search", "book-search"),
                    ("book-search-outline", "book-search-outline"),
                    ("book-settings", "book-settings"),
                    ("book-settings-outline", "book-settings-outline"),
                    ("book-sync", "book-sync"),
                    ("book-sync-outline", "book-sync-outline"),
                    ("book-variant", "book-variant"),
                    ("book-variant-multiple", "book-variant-multiple"),
                    ("bookmark", "bookmark"),
                    ("bookmark-box", "bookmark-box"),
                    ("bookmark-box-multiple", "bookmark-box-multiple"),
                    ("bookmark-box-multiple-outline", "bookmark-box-multiple-outline"),
                    ("bookmark-box-outline", "bookmark-box-outline"),
                    ("bookmark-check", "bookmark-check"),
                    ("bookmark-check-outline", "bookmark-check-outline"),
                    ("bookmark-minus", "bookmark-minus"),
                    ("bookmark-minus-outline", "bookmark-minus-outline"),
                    ("bookmark-multiple", "bookmark-multiple"),
                    ("bookmark-multiple-outline", "bookmark-multiple-outline"),
                    ("bookmark-music", "bookmark-music"),
                    ("bookmark-music-outline", "bookmark-music-outline"),
                    ("bookmark-off", "bookmark-off"),
                    ("bookmark-off-outline", "bookmark-off-outline"),
                    ("bookmark-outline", "bookmark-outline"),
                    ("bookmark-plus", "bookmark-plus"),
                    ("bookmark-plus-outline", "bookmark-plus-outline"),
                    ("bookmark-remove", "bookmark-remove"),
                    ("bookmark-remove-outline", "bookmark-remove-outline"),
                    ("bookshelf", "bookshelf"),
                    ("boom-gate", "boom-gate"),
                    ("boom-gate-alert", "boom-gate-alert"),
                    ("boom-gate-alert-outline", "boom-gate-alert-outline"),
                    ("boom-gate-arrow-down", "boom-gate-arrow-down"),
                    ("boom-gate-arrow-down-outline", "boom-gate-arrow-down-outline"),
                    ("boom-gate-arrow-up", "boom-gate-arrow-up"),
                    ("boom-gate-arrow-up-outline", "boom-gate-arrow-up-outline"),
                    ("boom-gate-outline", "boom-gate-outline"),
                    ("boom-gate-up", "boom-gate-up"),
                    ("boom-gate-up-outline", "boom-gate-up-outline"),
                    ("boombox", "boombox"),
                    ("boomerang", "boomerang"),
                    ("bootstrap", "bootstrap"),
                    ("border-all", "border-all"),
                    ("border-all-variant", "border-all-variant"),
                    ("border-bottom", "border-bottom"),
                    ("border-bottom-variant", "border-bottom-variant"),
                    ("border-color", "border-color"),
                    ("border-horizontal", "border-horizontal"),
                    ("border-inside", "border-inside"),
                    ("border-left", "border-left"),
                    ("border-left-variant", "border-left-variant"),
                    ("border-none", "border-none"),
                    ("border-none-variant", "border-none-variant"),
                    ("border-outside", "border-outside"),
                    ("border-radius", "border-radius"),
                    ("border-right", "border-right"),
                    ("border-right-variant", "border-right-variant"),
                    ("border-style", "border-style"),
                    ("border-top", "border-top"),
                    ("border-top-variant", "border-top-variant"),
                    ("border-vertical", "border-vertical"),
                    ("bottle-soda", "bottle-soda"),
                    ("bottle-soda-classic", "bottle-soda-classic"),
                    ("bottle-soda-classic-outline", "bottle-soda-classic-outline"),
                    ("bottle-soda-outline", "bottle-soda-outline"),
                    ("bottle-tonic", "bottle-tonic"),
                    ("bottle-tonic-outline", "bottle-tonic-outline"),
                    ("bottle-tonic-plus", "bottle-tonic-plus"),
                    ("bottle-tonic-plus-outline", "bottle-tonic-plus-outline"),
                    ("bottle-tonic-skull", "bottle-tonic-skull"),
                    ("bottle-tonic-skull-outline", "bottle-tonic-skull-outline"),
                    ("bottle-wine", "bottle-wine"),
                    ("bottle-wine-outline", "bottle-wine-outline"),
                    ("bow-arrow", "bow-arrow"),
                    ("bow-tie", "bow-tie"),
                    ("bowl", "bowl"),
                    ("bowl-mix", "bowl-mix"),
                    ("bowl-mix-outline", "bowl-mix-outline"),
                    ("bowl-outline", "bowl-outline"),
                    ("bowling", "bowling"),
                    ("box", "box"),
                    ("box-cutter", "box-cutter"),
                    ("box-cutter-off", "box-cutter-off"),
                    ("box-download", "box-download"),
                    ("box-shadow", "box-shadow"),
                    ("box-upload", "box-upload"),
                    ("boxing-glove", "boxing-glove"),
                    ("boxing-gloves", "boxing-gloves"),
                    ("braille", "braille"),
                    ("brain", "brain"),
                    ("bread-slice", "bread-slice"),
                    ("bread-slice-outline", "bread-slice-outline"),
                    ("bridge", "bridge"),
                    ("briefcase", "briefcase"),
                    ("briefcase-account", "briefcase-account"),
                    ("briefcase-account-outline", "briefcase-account-outline"),
                    ("briefcase-arrow-left-right", "briefcase-arrow-left-right"),
                    ("briefcase-arrow-left-right-outline", "briefcase-arrow-left-right-outline"),
                    ("briefcase-arrow-up-down", "briefcase-arrow-up-down"),
                    ("briefcase-arrow-up-down-outline", "briefcase-arrow-up-down-outline"),
                    ("briefcase-check", "briefcase-check"),
                    ("briefcase-check-outline", "briefcase-check-outline"),
                    ("briefcase-clock", "briefcase-clock"),
                    ("briefcase-clock-outline", "briefcase-clock-outline"),
                    ("briefcase-download", "briefcase-download"),
                    ("briefcase-download-outline", "briefcase-download-outline"),
                    ("briefcase-edit", "briefcase-edit"),
                    ("briefcase-edit-outline", "briefcase-edit-outline"),
                    ("briefcase-eye", "briefcase-eye"),
                    ("briefcase-eye-outline", "briefcase-eye-outline"),
                    ("briefcase-minus", "briefcase-minus"),
                    ("briefcase-minus-outline", "briefcase-minus-outline"),
                    ("briefcase-off", "briefcase-off"),
                    ("briefcase-off-outline", "briefcase-off-outline"),
                    ("briefcase-outline", "briefcase-outline"),
                    ("briefcase-plus", "briefcase-plus"),
                    ("briefcase-plus-outline", "briefcase-plus-outline"),
                    ("briefcase-remove", "briefcase-remove"),
                    ("briefcase-remove-outline", "briefcase-remove-outline"),
                    ("briefcase-search", "briefcase-search"),
                    ("briefcase-search-outline", "briefcase-search-outline"),
                    ("briefcase-upload", "briefcase-upload"),
                    ("briefcase-upload-outline", "briefcase-upload-outline"),
                    ("briefcase-variant", "briefcase-variant"),
                    ("briefcase-variant-off", "briefcase-variant-off"),
                    ("briefcase-variant-off-outline", "briefcase-variant-off-outline"),
                    ("briefcase-variant-outline", "briefcase-variant-outline"),
                    ("brightness", "brightness"),
                    ("brightness-1", "brightness-1"),
                    ("brightness-2", "brightness-2"),
                    ("brightness-3", "brightness-3"),
                    ("brightness-4", "brightness-4"),
                    ("brightness-5", "brightness-5"),
                    ("brightness-6", "brightness-6"),
                    ("brightness-7", "brightness-7"),
                    ("brightness-auto", "brightness-auto"),
                    ("brightness-percent", "brightness-percent"),
                    ("broadcast", "broadcast"),
                    ("broadcast-off", "broadcast-off"),
                    ("broom", "broom"),
                    ("brush", "brush"),
                    ("brush-off", "brush-off"),
                    ("brush-outline", "brush-outline"),
                    ("brush-variant", "brush-variant"),
                    ("bucket", "bucket"),
                    ("bucket-outline", "bucket-outline"),
                    ("buffer", "buffer"),
                    ("buffet", "buffet"),
                    ("bug", "bug"),
                    ("bug-check", "bug-check"),
                    ("bug-check-outline", "bug-check-outline"),
                    ("bug-outline", "bug-outline"),
                    ("bug-pause", "bug-pause"),
                    ("bug-pause-outline", "bug-pause-outline"),
                    ("bug-play", "bug-play"),
                    ("bug-play-outline", "bug-play-outline"),
                    ("bug-stop", "bug-stop"),
                    ("bug-stop-outline", "bug-stop-outline"),
                    ("bugle", "bugle"),
                    ("bulkhead-light", "bulkhead-light"),
                    ("bulldozer", "bulldozer"),
                    ("bullet", "bullet"),
                    ("bulletin-board", "bulletin-board"),
                    ("bullhorn", "bullhorn"),
                    ("bullhorn-outline", "bullhorn-outline"),
                    ("bullhorn-variant", "bullhorn-variant"),
                    ("bullhorn-variant-outline", "bullhorn-variant-outline"),
                    ("bullseye", "bullseye"),
                    ("bullseye-arrow", "bullseye-arrow"),
                    ("bulma", "bulma"),
                    ("bunk-bed", "bunk-bed"),
                    ("bunk-bed-outline", "bunk-bed-outline"),
                    ("bus", "bus"),
                    ("bus-alert", "bus-alert"),
                    ("bus-articulated-end", "bus-articulated-end"),
                    ("bus-articulated-front", "bus-articulated-front"),
                    ("bus-clock", "bus-clock"),
                    ("bus-double-decker", "bus-double-decker"),
                    ("bus-electric", "bus-electric"),
                    ("bus-marker", "bus-marker"),
                    ("bus-multiple", "bus-multiple"),
                    ("bus-school", "bus-school"),
                    ("bus-side", "bus-side"),
                    ("bus-stop", "bus-stop"),
                    ("bus-stop-covered", "bus-stop-covered"),
                    ("bus-stop-uncovered", "bus-stop-uncovered"),
                    ("butterfly", "butterfly"),
                    ("butterfly-outline", "butterfly-outline"),
                    ("button-cursor", "button-cursor"),
                    ("button-pointer", "button-pointer"),
                    ("cabin-a-frame", "cabin-a-frame"),
                    ("cable-data", "cable-data"),
                    ("cached", "cached"),
                    ("cactus", "cactus"),
                    ("cake", "cake"),
                    ("cake-layered", "cake-layered"),
                    ("cake-variant", "cake-variant"),
                    ("cake-variant-outline", "cake-variant-outline"),
                    ("calculator", "calculator"),
                    ("calculator-off", "calculator-off"),
                    ("calculator-variant", "calculator-variant"),
                    ("calculator-variant-outline", "calculator-variant-outline"),
                    ("calendar", "calendar"),
                    ("calendar-account", "calendar-account"),
                    ("calendar-account-outline", "calendar-account-outline"),
                    ("calendar-alert", "calendar-alert"),
                    ("calendar-alert-outline", "calendar-alert-outline"),
                    ("calendar-arrow-left", "calendar-arrow-left"),
                    ("calendar-arrow-right", "calendar-arrow-right"),
                    ("calendar-badge", "calendar-badge"),
                    ("calendar-badge-outline", "calendar-badge-outline"),
                    ("calendar-blank", "calendar-blank"),
                    ("calendar-blank-multiple", "calendar-blank-multiple"),
                    ("calendar-blank-outline", "calendar-blank-outline"),
                    ("calendar-check", "calendar-check"),
                    ("calendar-check-outline", "calendar-check-outline"),
                    ("calendar-clock", "calendar-clock"),
                    ("calendar-clock-outline", "calendar-clock-outline"),
                    ("calendar-collapse-horizontal", "calendar-collapse-horizontal"),
                    (
                        "calendar-collapse-horizontal-outline",
                        "calendar-collapse-horizontal-outline",
                    ),
                    ("calendar-cursor", "calendar-cursor"),
                    ("calendar-cursor-outline", "calendar-cursor-outline"),
                    ("calendar-edit", "calendar-edit"),
                    ("calendar-edit-outline", "calendar-edit-outline"),
                    ("calendar-end", "calendar-end"),
                    ("calendar-end-outline", "calendar-end-outline"),
                    ("calendar-expand-horizontal", "calendar-expand-horizontal"),
                    ("calendar-expand-horizontal-outline", "calendar-expand-horizontal-outline"),
                    ("calendar-export", "calendar-export"),
                    ("calendar-export-outline", "calendar-export-outline"),
                    ("calendar-filter", "calendar-filter"),
                    ("calendar-filter-outline", "calendar-filter-outline"),
                    ("calendar-heart", "calendar-heart"),
                    ("calendar-heart-outline", "calendar-heart-outline"),
                    ("calendar-import", "calendar-import"),
                    ("calendar-import-outline", "calendar-import-outline"),
                    ("calendar-lock", "calendar-lock"),
                    ("calendar-lock-open", "calendar-lock-open"),
                    ("calendar-lock-open-outline", "calendar-lock-open-outline"),
                    ("calendar-lock-outline", "calendar-lock-outline"),
                    ("calendar-minus", "calendar-minus"),
                    ("calendar-minus-outline", "calendar-minus-outline"),
                    ("calendar-month", "calendar-month"),
                    ("calendar-month-outline", "calendar-month-outline"),
                    ("calendar-multiple", "calendar-multiple"),
                    ("calendar-multiple-check", "calendar-multiple-check"),
                    ("calendar-multiselect", "calendar-multiselect"),
                    ("calendar-multiselect-outline", "calendar-multiselect-outline"),
                    ("calendar-outline", "calendar-outline"),
                    ("calendar-plus", "calendar-plus"),
                    ("calendar-plus-outline", "calendar-plus-outline"),
                    ("calendar-question", "calendar-question"),
                    ("calendar-question-outline", "calendar-question-outline"),
                    ("calendar-range", "calendar-range"),
                    ("calendar-range-outline", "calendar-range-outline"),
                    ("calendar-refresh", "calendar-refresh"),
                    ("calendar-refresh-outline", "calendar-refresh-outline"),
                    ("calendar-remove", "calendar-remove"),
                    ("calendar-remove-outline", "calendar-remove-outline"),
                    ("calendar-search", "calendar-search"),
                    ("calendar-search-outline", "calendar-search-outline"),
                    ("calendar-select", "calendar-select"),
                    ("calendar-star", "calendar-star"),
                    ("calendar-star-four-points", "calendar-star-four-points"),
                    ("calendar-star-outline", "calendar-star-outline"),
                    ("calendar-start", "calendar-start"),
                    ("calendar-start-outline", "calendar-start-outline"),
                    ("calendar-sync", "calendar-sync"),
                    ("calendar-sync-outline", "calendar-sync-outline"),
                    ("calendar-text", "calendar-text"),
                    ("calendar-text-outline", "calendar-text-outline"),
                    ("calendar-today", "calendar-today"),
                    ("calendar-today-outline", "calendar-today-outline"),
                    ("calendar-week", "calendar-week"),
                    ("calendar-week-begin", "calendar-week-begin"),
                    ("calendar-week-begin-outline", "calendar-week-begin-outline"),
                    ("calendar-week-end", "calendar-week-end"),
                    ("calendar-week-end-outline", "calendar-week-end-outline"),
                    ("calendar-week-outline", "calendar-week-outline"),
                    ("calendar-weekend", "calendar-weekend"),
                    ("calendar-weekend-outline", "calendar-weekend-outline"),
                    ("call-made", "call-made"),
                    ("call-merge", "call-merge"),
                    ("call-missed", "call-missed"),
                    ("call-received", "call-received"),
                    ("call-split", "call-split"),
                    ("camcorder", "camcorder"),
                    ("camcorder-off", "camcorder-off"),
                    ("camera", "camera"),
                    ("camera-account", "camera-account"),
                    ("camera-burst", "camera-burst"),
                    ("camera-control", "camera-control"),
                    ("camera-document", "camera-document"),
                    ("camera-document-off", "camera-document-off"),
                    ("camera-enhance", "camera-enhance"),
                    ("camera-enhance-outline", "camera-enhance-outline"),
                    ("camera-flip", "camera-flip"),
                    ("camera-flip-outline", "camera-flip-outline"),
                    ("camera-focus", "camera-focus"),
                    ("camera-front", "camera-front"),
                    ("camera-front-variant", "camera-front-variant"),
                    ("camera-gopro", "camera-gopro"),
                    ("camera-image", "camera-image"),
                    ("camera-iris", "camera-iris"),
                    ("camera-lock", "camera-lock"),
                    ("camera-lock-open", "camera-lock-open"),
                    ("camera-lock-open-outline", "camera-lock-open-outline"),
                    ("camera-lock-outline", "camera-lock-outline"),
                    ("camera-marker", "camera-marker"),
                    ("camera-marker-outline", "camera-marker-outline"),
                    ("camera-metering-center", "camera-metering-center"),
                    ("camera-metering-matrix", "camera-metering-matrix"),
                    ("camera-metering-partial", "camera-metering-partial"),
                    ("camera-metering-spot", "camera-metering-spot"),
                    ("camera-off", "camera-off"),
                    ("camera-off-outline", "camera-off-outline"),
                    ("camera-outline", "camera-outline"),
                    ("camera-party-mode", "camera-party-mode"),
                    ("camera-plus", "camera-plus"),
                    ("camera-plus-outline", "camera-plus-outline"),
                    ("camera-rear", "camera-rear"),
                    ("camera-rear-variant", "camera-rear-variant"),
                    ("camera-retake", "camera-retake"),
                    ("camera-retake-outline", "camera-retake-outline"),
                    ("camera-switch", "camera-switch"),
                    ("camera-switch-outline", "camera-switch-outline"),
                    ("camera-timer", "camera-timer"),
                    ("camera-wireless", "camera-wireless"),
                    ("camera-wireless-outline", "camera-wireless-outline"),
                    ("campfire", "campfire"),
                    ("cancel", "cancel"),
                    ("candelabra", "candelabra"),
                    ("candelabra-fire", "candelabra-fire"),
                    ("candle", "candle"),
                    ("candy", "candy"),
                    ("candy-off", "candy-off"),
                    ("candy-off-outline", "candy-off-outline"),
                    ("candy-outline", "candy-outline"),
                    ("candycane", "candycane"),
                    ("cannabis", "cannabis"),
                    ("cannabis-off", "cannabis-off"),
                    ("caps-lock", "caps-lock"),
                    ("car", "car"),
                    ("car-2-plus", "car-2-plus"),
                    ("car-3-plus", "car-3-plus"),
                    ("car-arrow-left", "car-arrow-left"),
                    ("car-arrow-right", "car-arrow-right"),
                    ("car-back", "car-back"),
                    ("car-battery", "car-battery"),
                    ("car-brake-abs", "car-brake-abs"),
                    ("car-brake-alert", "car-brake-alert"),
                    ("car-brake-fluid-level", "car-brake-fluid-level"),
                    ("car-brake-hold", "car-brake-hold"),
                    ("car-brake-low-pressure", "car-brake-low-pressure"),
                    ("car-brake-parking", "car-brake-parking"),
                    ("car-brake-retarder", "car-brake-retarder"),
                    ("car-brake-temperature", "car-brake-temperature"),
                    ("car-brake-worn-linings", "car-brake-worn-linings"),
                    ("car-child-seat", "car-child-seat"),
                    ("car-clock", "car-clock"),
                    ("car-clutch", "car-clutch"),
                    ("car-cog", "car-cog"),
                    ("car-connected", "car-connected"),
                    ("car-convertable", "car-convertable"),
                    ("car-convertible", "car-convertible"),
                    ("car-coolant-level", "car-coolant-level"),
                    ("car-cruise-control", "car-cruise-control"),
                    ("car-defrost-front", "car-defrost-front"),
                    ("car-defrost-rear", "car-defrost-rear"),
                    ("car-door", "car-door"),
                    ("car-door-lock", "car-door-lock"),
                    ("car-door-lock-open", "car-door-lock-open"),
                    ("car-electric", "car-electric"),
                    ("car-electric-outline", "car-electric-outline"),
                    ("car-emergency", "car-emergency"),
                    ("car-esp", "car-esp"),
                    ("car-estate", "car-estate"),
                    ("car-hatchback", "car-hatchback"),
                    ("car-info", "car-info"),
                    ("car-key", "car-key"),
                    ("car-lifted-pickup", "car-lifted-pickup"),
                    ("car-light-alert", "car-light-alert"),
                    ("car-light-dimmed", "car-light-dimmed"),
                    ("car-light-fog", "car-light-fog"),
                    ("car-light-high", "car-light-high"),
                    ("car-limousine", "car-limousine"),
                    ("car-multiple", "car-multiple"),
                    ("car-off", "car-off"),
                    ("car-outline", "car-outline"),
                    ("car-parking-lights", "car-parking-lights"),
                    ("car-pickup", "car-pickup"),
                    ("car-search", "car-search"),
                    ("car-search-outline", "car-search-outline"),
                    ("car-seat", "car-seat"),
                    ("car-seat-cooler", "car-seat-cooler"),
                    ("car-seat-heater", "car-seat-heater"),
                    ("car-select", "car-select"),
                    ("car-settings", "car-settings"),
                    ("car-shift-pattern", "car-shift-pattern"),
                    ("car-side", "car-side"),
                    ("car-speed-limiter", "car-speed-limiter"),
                    ("car-sports", "car-sports"),
                    ("car-tire-alert", "car-tire-alert"),
                    ("car-traction-control", "car-traction-control"),
                    ("car-turbocharger", "car-turbocharger"),
                    ("car-wash", "car-wash"),
                    ("car-windshield", "car-windshield"),
                    ("car-windshield-outline", "car-windshield-outline"),
                    ("car-wireless", "car-wireless"),
                    ("car-wrench", "car-wrench"),
                    ("carabiner", "carabiner"),
                    ("caravan", "caravan"),
                    ("card", "card"),
                    ("card-account-details", "card-account-details"),
                    ("card-account-details-outline", "card-account-details-outline"),
                    ("card-account-details-star", "card-account-details-star"),
                    ("card-account-details-star-outline", "card-account-details-star-outline"),
                    ("card-account-mail", "card-account-mail"),
                    ("card-account-mail-outline", "card-account-mail-outline"),
                    ("card-account-phone", "card-account-phone"),
                    ("card-account-phone-outline", "card-account-phone-outline"),
                    ("card-bulleted", "card-bulleted"),
                    ("card-bulleted-off", "card-bulleted-off"),
                    ("card-bulleted-off-outline", "card-bulleted-off-outline"),
                    ("card-bulleted-outline", "card-bulleted-outline"),
                    ("card-bulleted-settings", "card-bulleted-settings"),
                    ("card-bulleted-settings-outline", "card-bulleted-settings-outline"),
                    ("card-minus", "card-minus"),
                    ("card-minus-outline", "card-minus-outline"),
                    ("card-multiple", "card-multiple"),
                    ("card-multiple-outline", "card-multiple-outline"),
                    ("card-off", "card-off"),
                    ("card-off-outline", "card-off-outline"),
                    ("card-outline", "card-outline"),
                    ("card-plus", "card-plus"),
                    ("card-plus-outline", "card-plus-outline"),
                    ("card-remove", "card-remove"),
                    ("card-remove-outline", "card-remove-outline"),
                    ("card-search", "card-search"),
                    ("card-search-outline", "card-search-outline"),
                    ("card-text", "card-text"),
                    ("card-text-outline", "card-text-outline"),
                    ("cards", "cards"),
                    ("cards-club", "cards-club"),
                    ("cards-club-outline", "cards-club-outline"),
                    ("cards-diamond", "cards-diamond"),
                    ("cards-diamond-outline", "cards-diamond-outline"),
                    ("cards-heart", "cards-heart"),
                    ("cards-heart-outline", "cards-heart-outline"),
                    ("cards-outline", "cards-outline"),
                    ("cards-playing", "cards-playing"),
                    ("cards-playing-club", "cards-playing-club"),
                    ("cards-playing-club-multiple", "cards-playing-club-multiple"),
                    ("cards-playing-club-multiple-outline", "cards-playing-club-multiple-outline"),
                    ("cards-playing-club-outline", "cards-playing-club-outline"),
                    ("cards-playing-diamond", "cards-playing-diamond"),
                    ("cards-playing-diamond-multiple", "cards-playing-diamond-multiple"),
                    (
                        "cards-playing-diamond-multiple-outline",
                        "cards-playing-diamond-multiple-outline",
                    ),
                    ("cards-playing-diamond-outline", "cards-playing-diamond-outline"),
                    ("cards-playing-heart", "cards-playing-heart"),
                    ("cards-playing-heart-multiple", "cards-playing-heart-multiple"),
                    (
                        "cards-playing-heart-multiple-outline",
                        "cards-playing-heart-multiple-outline",
                    ),
                    ("cards-playing-heart-outline", "cards-playing-heart-outline"),
                    ("cards-playing-outline", "cards-playing-outline"),
                    ("cards-playing-spade", "cards-playing-spade"),
                    ("cards-playing-spade-multiple", "cards-playing-spade-multiple"),
                    (
                        "cards-playing-spade-multiple-outline",
                        "cards-playing-spade-multiple-outline",
                    ),
                    ("cards-playing-spade-outline", "cards-playing-spade-outline"),
                    ("cards-spade", "cards-spade"),
                    ("cards-spade-outline", "cards-spade-outline"),
                    ("cards-variant", "cards-variant"),
                    ("carrot", "carrot"),
                    ("cart", "cart"),
                    ("cart-arrow-down", "cart-arrow-down"),
                    ("cart-arrow-right", "cart-arrow-right"),
                    ("cart-arrow-up", "cart-arrow-up"),
                    ("cart-check", "cart-check"),
                    ("cart-heart", "cart-heart"),
                    ("cart-minus", "cart-minus"),
                    ("cart-off", "cart-off"),
                    ("cart-outline", "cart-outline"),
                    ("cart-percent", "cart-percent"),
                    ("cart-plus", "cart-plus"),
                    ("cart-remove", "cart-remove"),
                    ("cart-variant", "cart-variant"),
                    ("case-sensitive-alt", "case-sensitive-alt"),
                    ("cash", "cash"),
                    ("cash-100", "cash-100"),
                    ("cash-check", "cash-check"),
                    ("cash-clock", "cash-clock"),
                    ("cash-fast", "cash-fast"),
                    ("cash-lock", "cash-lock"),
                    ("cash-lock-open", "cash-lock-open"),
                    ("cash-marker", "cash-marker"),
                    ("cash-minus", "cash-minus"),
                    ("cash-multiple", "cash-multiple"),
                    ("cash-off", "cash-off"),
                    ("cash-plus", "cash-plus"),
                    ("cash-refund", "cash-refund"),
                    ("cash-register", "cash-register"),
                    ("cash-remove", "cash-remove"),
                    ("cash-sync", "cash-sync"),
                    ("cash-usd", "cash-usd"),
                    ("cash-usd-outline", "cash-usd-outline"),
                    ("cassette", "cassette"),
                    ("cast", "cast"),
                    ("cast-audio", "cast-audio"),
                    ("cast-audio-variant", "cast-audio-variant"),
                    ("cast-connected", "cast-connected"),
                    ("cast-education", "cast-education"),
                    ("cast-off", "cast-off"),
                    ("cast-variant", "cast-variant"),
                    ("castle", "castle"),
                    ("cat", "cat"),
                    ("cctv", "cctv"),
                    ("cctv-off", "cctv-off"),
                    ("ceiling-fan", "ceiling-fan"),
                    ("ceiling-fan-light", "ceiling-fan-light"),
                    ("ceiling-light", "ceiling-light"),
                    ("ceiling-light-multiple", "ceiling-light-multiple"),
                    ("ceiling-light-multiple-outline", "ceiling-light-multiple-outline"),
                    ("ceiling-light-outline", "ceiling-light-outline"),
                    ("cellphone", "cellphone"),
                    ("cellphone-android", "cellphone-android"),
                    ("cellphone-arrow-down", "cellphone-arrow-down"),
                    ("cellphone-arrow-down-variant", "cellphone-arrow-down-variant"),
                    ("cellphone-basic", "cellphone-basic"),
                    ("cellphone-charging", "cellphone-charging"),
                    ("cellphone-check", "cellphone-check"),
                    ("cellphone-cog", "cellphone-cog"),
                    ("cellphone-dock", "cellphone-dock"),
                    ("cellphone-information", "cellphone-information"),
                    ("cellphone-iphone", "cellphone-iphone"),
                    ("cellphone-key", "cellphone-key"),
                    ("cellphone-link", "cellphone-link"),
                    ("cellphone-link-off", "cellphone-link-off"),
                    ("cellphone-lock", "cellphone-lock"),
                    ("cellphone-marker", "cellphone-marker"),
                    ("cellphone-message", "cellphone-message"),
                    ("cellphone-message-off", "cellphone-message-off"),
                    ("cellphone-nfc", "cellphone-nfc"),
                    ("cellphone-nfc-off", "cellphone-nfc-off"),
                    ("cellphone-off", "cellphone-off"),
                    ("cellphone-play", "cellphone-play"),
                    ("cellphone-remove", "cellphone-remove"),
                    ("cellphone-screenshot", "cellphone-screenshot"),
                    ("cellphone-settings", "cellphone-settings"),
                    ("cellphone-sound", "cellphone-sound"),
                    ("cellphone-text", "cellphone-text"),
                    ("cellphone-wireless", "cellphone-wireless"),
                    ("centos", "centos"),
                    ("certificate", "certificate"),
                    ("certificate-outline", "certificate-outline"),
                    ("chair-rolling", "chair-rolling"),
                    ("chair-school", "chair-school"),
                    ("chandelier", "chandelier"),
                    ("charity", "charity"),
                    ("charity-search", "charity-search"),
                    ("chart-arc", "chart-arc"),
                    ("chart-areaspline", "chart-areaspline"),
                    ("chart-areaspline-variant", "chart-areaspline-variant"),
                    ("chart-bar", "chart-bar"),
                    ("chart-bar-stacked", "chart-bar-stacked"),
                    ("chart-bell-curve", "chart-bell-curve"),
                    ("chart-bell-curve-cumulative", "chart-bell-curve-cumulative"),
                    ("chart-box", "chart-box"),
                    ("chart-box-outline", "chart-box-outline"),
                    ("chart-box-plus-outline", "chart-box-plus-outline"),
                    ("chart-bubble", "chart-bubble"),
                    ("chart-donut", "chart-donut"),
                    ("chart-donut-variant", "chart-donut-variant"),
                    ("chart-gantt", "chart-gantt"),
                    ("chart-histogram", "chart-histogram"),
                    ("chart-line", "chart-line"),
                    ("chart-line-stacked", "chart-line-stacked"),
                    ("chart-line-variant", "chart-line-variant"),
                    ("chart-multiline", "chart-multiline"),
                    ("chart-multiple", "chart-multiple"),
                    ("chart-pie", "chart-pie"),
                    ("chart-pie-outline", "chart-pie-outline"),
                    ("chart-ppf", "chart-ppf"),
                    ("chart-sankey", "chart-sankey"),
                    ("chart-sankey-variant", "chart-sankey-variant"),
                    ("chart-scatter-plot", "chart-scatter-plot"),
                    ("chart-scatter-plot-hexbin", "chart-scatter-plot-hexbin"),
                    ("chart-timeline", "chart-timeline"),
                    ("chart-timeline-variant", "chart-timeline-variant"),
                    ("chart-timeline-variant-shimmer", "chart-timeline-variant-shimmer"),
                    ("chart-tree", "chart-tree"),
                    ("chart-waterfall", "chart-waterfall"),
                    ("chat", "chat"),
                    ("chat-alert", "chat-alert"),
                    ("chat-alert-outline", "chat-alert-outline"),
                    ("chat-minus", "chat-minus"),
                    ("chat-minus-outline", "chat-minus-outline"),
                    ("chat-outline", "chat-outline"),
                    ("chat-plus", "chat-plus"),
                    ("chat-plus-outline", "chat-plus-outline"),
                    ("chat-processing", "chat-processing"),
                    ("chat-processing-outline", "chat-processing-outline"),
                    ("chat-question", "chat-question"),
                    ("chat-question-outline", "chat-question-outline"),
                    ("chat-remove", "chat-remove"),
                    ("chat-remove-outline", "chat-remove-outline"),
                    ("chat-sleep", "chat-sleep"),
                    ("chat-sleep-outline", "chat-sleep-outline"),
                    ("check", "check"),
                    ("check-all", "check-all"),
                    ("check-bold", "check-bold"),
                    ("check-bookmark", "check-bookmark"),
                    ("check-circle", "check-circle"),
                    ("check-circle-outline", "check-circle-outline"),
                    ("check-decagram", "check-decagram"),
                    ("check-decagram-outline", "check-decagram-outline"),
                    ("check-network", "check-network"),
                    ("check-network-outline", "check-network-outline"),
                    ("check-outline", "check-outline"),
                    ("check-underline", "check-underline"),
                    ("check-underline-circle", "check-underline-circle"),
                    ("check-underline-circle-outline", "check-underline-circle-outline"),
                    ("checkbook", "checkbook"),
                    ("checkbook-arrow-left", "checkbook-arrow-left"),
                    ("checkbook-arrow-right", "checkbook-arrow-right"),
                    ("checkbox-blank", "checkbox-blank"),
                    ("checkbox-blank-badge", "checkbox-blank-badge"),
                    ("checkbox-blank-badge-outline", "checkbox-blank-badge-outline"),
                    ("checkbox-blank-circle", "checkbox-blank-circle"),
                    ("checkbox-blank-circle-outline", "checkbox-blank-circle-outline"),
                    ("checkbox-blank-off", "checkbox-blank-off"),
                    ("checkbox-blank-off-outline", "checkbox-blank-off-outline"),
                    ("checkbox-blank-outline", "checkbox-blank-outline"),
                    ("checkbox-intermediate", "checkbox-intermediate"),
                    ("checkbox-intermediate-variant", "checkbox-intermediate-variant"),
                    ("checkbox-marked", "checkbox-marked"),
                    ("checkbox-marked-circle", "checkbox-marked-circle"),
                    ("checkbox-marked-circle-auto-outline", "checkbox-marked-circle-auto-outline"),
                    (
                        "checkbox-marked-circle-minus-outline",
                        "checkbox-marked-circle-minus-outline",
                    ),
                    ("checkbox-marked-circle-outline", "checkbox-marked-circle-outline"),
                    ("checkbox-marked-circle-plus-outline", "checkbox-marked-circle-plus-outline"),
                    ("checkbox-marked-outline", "checkbox-marked-outline"),
                    ("checkbox-multiple-blank", "checkbox-multiple-blank"),
                    ("checkbox-multiple-blank-circle", "checkbox-multiple-blank-circle"),
                    (
                        "checkbox-multiple-blank-circle-outline",
                        "checkbox-multiple-blank-circle-outline",
                    ),
                    ("checkbox-multiple-blank-outline", "checkbox-multiple-blank-outline"),
                    ("checkbox-multiple-marked", "checkbox-multiple-marked"),
                    ("checkbox-multiple-marked-circle", "checkbox-multiple-marked-circle"),
                    (
                        "checkbox-multiple-marked-circle-outline",
                        "checkbox-multiple-marked-circle-outline",
                    ),
                    ("checkbox-multiple-marked-outline", "checkbox-multiple-marked-outline"),
                    ("checkbox-multiple-outline", "checkbox-multiple-outline"),
                    ("checkbox-outline", "checkbox-outline"),
                    ("checkerboard", "checkerboard"),
                    ("checkerboard-minus", "checkerboard-minus"),
                    ("checkerboard-plus", "checkerboard-plus"),
                    ("checkerboard-remove", "checkerboard-remove"),
                    ("cheese", "cheese"),
                    ("cheese-off", "cheese-off"),
                    ("chef-hat", "chef-hat"),
                    ("chemical-weapon", "chemical-weapon"),
                    ("chess-bishop", "chess-bishop"),
                    ("chess-king", "chess-king"),
                    ("chess-knight", "chess-knight"),
                    ("chess-pawn", "chess-pawn"),
                    ("chess-queen", "chess-queen"),
                    ("chess-rook", "chess-rook"),
                    ("chevron-double-down", "chevron-double-down"),
                    ("chevron-double-left", "chevron-double-left"),
                    ("chevron-double-right", "chevron-double-right"),
                    ("chevron-double-up", "chevron-double-up"),
                    ("chevron-down", "chevron-down"),
                    ("chevron-down-box", "chevron-down-box"),
                    ("chevron-down-box-outline", "chevron-down-box-outline"),
                    ("chevron-down-circle", "chevron-down-circle"),
                    ("chevron-down-circle-outline", "chevron-down-circle-outline"),
                    ("chevron-left", "chevron-left"),
                    ("chevron-left-box", "chevron-left-box"),
                    ("chevron-left-box-outline", "chevron-left-box-outline"),
                    ("chevron-left-circle", "chevron-left-circle"),
                    ("chevron-left-circle-outline", "chevron-left-circle-outline"),
                    ("chevron-right", "chevron-right"),
                    ("chevron-right-box", "chevron-right-box"),
                    ("chevron-right-box-outline", "chevron-right-box-outline"),
                    ("chevron-right-circle", "chevron-right-circle"),
                    ("chevron-right-circle-outline", "chevron-right-circle-outline"),
                    ("chevron-triple-down", "chevron-triple-down"),
                    ("chevron-triple-left", "chevron-triple-left"),
                    ("chevron-triple-right", "chevron-triple-right"),
                    ("chevron-triple-up", "chevron-triple-up"),
                    ("chevron-up", "chevron-up"),
                    ("chevron-up-box", "chevron-up-box"),
                    ("chevron-up-box-outline", "chevron-up-box-outline"),
                    ("chevron-up-circle", "chevron-up-circle"),
                    ("chevron-up-circle-outline", "chevron-up-circle-outline"),
                    ("chili-alert", "chili-alert"),
                    ("chili-alert-outline", "chili-alert-outline"),
                    ("chili-hot", "chili-hot"),
                    ("chili-hot-outline", "chili-hot-outline"),
                    ("chili-medium", "chili-medium"),
                    ("chili-medium-outline", "chili-medium-outline"),
                    ("chili-mild", "chili-mild"),
                    ("chili-mild-outline", "chili-mild-outline"),
                    ("chili-off", "chili-off"),
                    ("chili-off-outline", "chili-off-outline"),
                    ("chip", "chip"),
                    ("church", "church"),
                    ("church-outline", "church-outline"),
                    ("cigar", "cigar"),
                    ("cigar-off", "cigar-off"),
                    ("circle", "circle"),
                    ("circle-box", "circle-box"),
                    ("circle-box-outline", "circle-box-outline"),
                    ("circle-double", "circle-double"),
                    ("circle-edit-outline", "circle-edit-outline"),
                    ("circle-expand", "circle-expand"),
                    ("circle-half", "circle-half"),
                    ("circle-half-full", "circle-half-full"),
                    ("circle-medium", "circle-medium"),
                    ("circle-multiple", "circle-multiple"),
                    ("circle-multiple-outline", "circle-multiple-outline"),
                    ("circle-off-outline", "circle-off-outline"),
                    ("circle-opacity", "circle-opacity"),
                    ("circle-outline", "circle-outline"),
                    ("circle-slice-1", "circle-slice-1"),
                    ("circle-slice-2", "circle-slice-2"),
                    ("circle-slice-3", "circle-slice-3"),
                    ("circle-slice-4", "circle-slice-4"),
                    ("circle-slice-5", "circle-slice-5"),
                    ("circle-slice-6", "circle-slice-6"),
                    ("circle-slice-7", "circle-slice-7"),
                    ("circle-slice-8", "circle-slice-8"),
                    ("circle-small", "circle-small"),
                    ("circular-saw", "circular-saw"),
                    ("cisco-webex", "cisco-webex"),
                    ("city", "city"),
                    ("city-switch", "city-switch"),
                    ("city-variant", "city-variant"),
                    ("city-variant-outline", "city-variant-outline"),
                    ("clipboard", "clipboard"),
                    ("clipboard-account", "clipboard-account"),
                    ("clipboard-account-outline", "clipboard-account-outline"),
                    ("clipboard-alert", "clipboard-alert"),
                    ("clipboard-alert-outline", "clipboard-alert-outline"),
                    ("clipboard-arrow-down", "clipboard-arrow-down"),
                    ("clipboard-arrow-down-outline", "clipboard-arrow-down-outline"),
                    ("clipboard-arrow-left", "clipboard-arrow-left"),
                    ("clipboard-arrow-left-outline", "clipboard-arrow-left-outline"),
                    ("clipboard-arrow-right", "clipboard-arrow-right"),
                    ("clipboard-arrow-right-outline", "clipboard-arrow-right-outline"),
                    ("clipboard-arrow-up", "clipboard-arrow-up"),
                    ("clipboard-arrow-up-outline", "clipboard-arrow-up-outline"),
                    ("clipboard-check", "clipboard-check"),
                    ("clipboard-check-multiple", "clipboard-check-multiple"),
                    ("clipboard-check-multiple-outline", "clipboard-check-multiple-outline"),
                    ("clipboard-check-outline", "clipboard-check-outline"),
                    ("clipboard-clock", "clipboard-clock"),
                    ("clipboard-clock-outline", "clipboard-clock-outline"),
                    ("clipboard-edit", "clipboard-edit"),
                    ("clipboard-edit-outline", "clipboard-edit-outline"),
                    ("clipboard-file", "clipboard-file"),
                    ("clipboard-file-outline", "clipboard-file-outline"),
                    ("clipboard-flow", "clipboard-flow"),
                    ("clipboard-flow-outline", "clipboard-flow-outline"),
                    ("clipboard-list", "clipboard-list"),
                    ("clipboard-list-outline", "clipboard-list-outline"),
                    ("clipboard-minus", "clipboard-minus"),
                    ("clipboard-minus-outline", "clipboard-minus-outline"),
                    ("clipboard-multiple", "clipboard-multiple"),
                    ("clipboard-multiple-outline", "clipboard-multiple-outline"),
                    ("clipboard-off", "clipboard-off"),
                    ("clipboard-off-outline", "clipboard-off-outline"),
                    ("clipboard-outline", "clipboard-outline"),
                    ("clipboard-play", "clipboard-play"),
                    ("clipboard-play-multiple", "clipboard-play-multiple"),
                    ("clipboard-play-multiple-outline", "clipboard-play-multiple-outline"),
                    ("clipboard-play-outline", "clipboard-play-outline"),
                    ("clipboard-plus", "clipboard-plus"),
                    ("clipboard-plus-outline", "clipboard-plus-outline"),
                    ("clipboard-pulse", "clipboard-pulse"),
                    ("clipboard-pulse-outline", "clipboard-pulse-outline"),
                    ("clipboard-remove", "clipboard-remove"),
                    ("clipboard-remove-outline", "clipboard-remove-outline"),
                    ("clipboard-search", "clipboard-search"),
                    ("clipboard-search-outline", "clipboard-search-outline"),
                    ("clipboard-text", "clipboard-text"),
                    ("clipboard-text-clock", "clipboard-text-clock"),
                    ("clipboard-text-clock-outline", "clipboard-text-clock-outline"),
                    ("clipboard-text-multiple", "clipboard-text-multiple"),
                    ("clipboard-text-multiple-outline", "clipboard-text-multiple-outline"),
                    ("clipboard-text-off", "clipboard-text-off"),
                    ("clipboard-text-off-outline", "clipboard-text-off-outline"),
                    ("clipboard-text-outline", "clipboard-text-outline"),
                    ("clipboard-text-play", "clipboard-text-play"),
                    ("clipboard-text-play-outline", "clipboard-text-play-outline"),
                    ("clipboard-text-search", "clipboard-text-search"),
                    ("clipboard-text-search-outline", "clipboard-text-search-outline"),
                    ("clippy", "clippy"),
                    ("clock", "clock"),
                    ("clock-alert", "clock-alert"),
                    ("clock-alert-outline", "clock-alert-outline"),
                    ("clock-check", "clock-check"),
                    ("clock-check-outline", "clock-check-outline"),
                    ("clock-digital", "clock-digital"),
                    ("clock-edit", "clock-edit"),
                    ("clock-edit-outline", "clock-edit-outline"),
                    ("clock-end", "clock-end"),
                    ("clock-fast", "clock-fast"),
                    ("clock-in", "clock-in"),
                    ("clock-minus", "clock-minus"),
                    ("clock-minus-outline", "clock-minus-outline"),
                    ("clock-out", "clock-out"),
                    ("clock-outline", "clock-outline"),
                    ("clock-plus", "clock-plus"),
                    ("clock-plus-outline", "clock-plus-outline"),
                    ("clock-remove", "clock-remove"),
                    ("clock-remove-outline", "clock-remove-outline"),
                    ("clock-star-four-points", "clock-star-four-points"),
                    ("clock-star-four-points-outline", "clock-star-four-points-outline"),
                    ("clock-start", "clock-start"),
                    ("clock-time-eight", "clock-time-eight"),
                    ("clock-time-eight-outline", "clock-time-eight-outline"),
                    ("clock-time-eleven", "clock-time-eleven"),
                    ("clock-time-eleven-outline", "clock-time-eleven-outline"),
                    ("clock-time-five", "clock-time-five"),
                    ("clock-time-five-outline", "clock-time-five-outline"),
                    ("clock-time-four", "clock-time-four"),
                    ("clock-time-four-outline", "clock-time-four-outline"),
                    ("clock-time-nine", "clock-time-nine"),
                    ("clock-time-nine-outline", "clock-time-nine-outline"),
                    ("clock-time-one", "clock-time-one"),
                    ("clock-time-one-outline", "clock-time-one-outline"),
                    ("clock-time-seven", "clock-time-seven"),
                    ("clock-time-seven-outline", "clock-time-seven-outline"),
                    ("clock-time-six", "clock-time-six"),
                    ("clock-time-six-outline", "clock-time-six-outline"),
                    ("clock-time-ten", "clock-time-ten"),
                    ("clock-time-ten-outline", "clock-time-ten-outline"),
                    ("clock-time-three", "clock-time-three"),
                    ("clock-time-three-outline", "clock-time-three-outline"),
                    ("clock-time-twelve", "clock-time-twelve"),
                    ("clock-time-twelve-outline", "clock-time-twelve-outline"),
                    ("clock-time-two", "clock-time-two"),
                    ("clock-time-two-outline", "clock-time-two-outline"),
                    ("close", "close"),
                    ("close-box", "close-box"),
                    ("close-box-multiple", "close-box-multiple"),
                    ("close-box-multiple-outline", "close-box-multiple-outline"),
                    ("close-box-outline", "close-box-outline"),
                    ("close-circle", "close-circle"),
                    ("close-circle-multiple", "close-circle-multiple"),
                    ("close-circle-multiple-outline", "close-circle-multiple-outline"),
                    ("close-circle-outline", "close-circle-outline"),
                    ("close-network", "close-network"),
                    ("close-network-outline", "close-network-outline"),
                    ("close-octagon", "close-octagon"),
                    ("close-octagon-outline", "close-octagon-outline"),
                    ("close-outline", "close-outline"),
                    ("close-thick", "close-thick"),
                    ("closed-caption", "closed-caption"),
                    ("closed-caption-outline", "closed-caption-outline"),
                    ("cloud", "cloud"),
                    ("cloud-alert", "cloud-alert"),
                    ("cloud-alert-outline", "cloud-alert-outline"),
                    ("cloud-arrow-down", "cloud-arrow-down"),
                    ("cloud-arrow-down-outline", "cloud-arrow-down-outline"),
                    ("cloud-arrow-left", "cloud-arrow-left"),
                    ("cloud-arrow-left-outline", "cloud-arrow-left-outline"),
                    ("cloud-arrow-right", "cloud-arrow-right"),
                    ("cloud-arrow-right-outline", "cloud-arrow-right-outline"),
                    ("cloud-arrow-up", "cloud-arrow-up"),
                    ("cloud-arrow-up-outline", "cloud-arrow-up-outline"),
                    ("cloud-braces", "cloud-braces"),
                    ("cloud-cancel", "cloud-cancel"),
                    ("cloud-cancel-outline", "cloud-cancel-outline"),
                    ("cloud-check", "cloud-check"),
                    ("cloud-check-outline", "cloud-check-outline"),
                    ("cloud-check-variant", "cloud-check-variant"),
                    ("cloud-check-variant-outline", "cloud-check-variant-outline"),
                    ("cloud-circle", "cloud-circle"),
                    ("cloud-circle-outline", "cloud-circle-outline"),
                    ("cloud-clock", "cloud-clock"),
                    ("cloud-clock-outline", "cloud-clock-outline"),
                    ("cloud-cog", "cloud-cog"),
                    ("cloud-cog-outline", "cloud-cog-outline"),
                    ("cloud-download", "cloud-download"),
                    ("cloud-download-outline", "cloud-download-outline"),
                    ("cloud-key", "cloud-key"),
                    ("cloud-key-outline", "cloud-key-outline"),
                    ("cloud-lock", "cloud-lock"),
                    ("cloud-lock-open", "cloud-lock-open"),
                    ("cloud-lock-open-outline", "cloud-lock-open-outline"),
                    ("cloud-lock-outline", "cloud-lock-outline"),
                    ("cloud-minus", "cloud-minus"),
                    ("cloud-minus-outline", "cloud-minus-outline"),
                    ("cloud-off", "cloud-off"),
                    ("cloud-off-outline", "cloud-off-outline"),
                    ("cloud-outline", "cloud-outline"),
                    ("cloud-percent", "cloud-percent"),
                    ("cloud-percent-outline", "cloud-percent-outline"),
                    ("cloud-plus", "cloud-plus"),
                    ("cloud-plus-outline", "cloud-plus-outline"),
                    ("cloud-print", "cloud-print"),
                    ("cloud-print-outline", "cloud-print-outline"),
                    ("cloud-question", "cloud-question"),
                    ("cloud-question-outline", "cloud-question-outline"),
                    ("cloud-refresh", "cloud-refresh"),
                    ("cloud-refresh-outline", "cloud-refresh-outline"),
                    ("cloud-refresh-variant", "cloud-refresh-variant"),
                    ("cloud-refresh-variant-outline", "cloud-refresh-variant-outline"),
                    ("cloud-remove", "cloud-remove"),
                    ("cloud-remove-outline", "cloud-remove-outline"),
                    ("cloud-search", "cloud-search"),
                    ("cloud-search-outline", "cloud-search-outline"),
                    ("cloud-sync", "cloud-sync"),
                    ("cloud-sync-outline", "cloud-sync-outline"),
                    ("cloud-tags", "cloud-tags"),
                    ("cloud-upload", "cloud-upload"),
                    ("cloud-upload-outline", "cloud-upload-outline"),
                    ("clouds", "clouds"),
                    ("clover", "clover"),
                    ("clover-outline", "clover-outline"),
                    ("coach-lamp", "coach-lamp"),
                    ("coach-lamp-variant", "coach-lamp-variant"),
                    ("coat-rack", "coat-rack"),
                    ("code-array", "code-array"),
                    ("code-block-braces", "code-block-braces"),
                    ("code-block-brackets", "code-block-brackets"),
                    ("code-block-parentheses", "code-block-parentheses"),
                    ("code-block-tags", "code-block-tags"),
                    ("code-braces", "code-braces"),
                    ("code-braces-box", "code-braces-box"),
                    ("code-brackets", "code-brackets"),
                    ("code-equal", "code-equal"),
                    ("code-greater-than", "code-greater-than"),
                    ("code-greater-than-or-equal", "code-greater-than-or-equal"),
                    ("code-json", "code-json"),
                    ("code-less-than", "code-less-than"),
                    ("code-less-than-or-equal", "code-less-than-or-equal"),
                    ("code-not-equal", "code-not-equal"),
                    ("code-not-equal-variant", "code-not-equal-variant"),
                    ("code-parentheses", "code-parentheses"),
                    ("code-parentheses-box", "code-parentheses-box"),
                    ("code-string", "code-string"),
                    ("code-tags", "code-tags"),
                    ("code-tags-check", "code-tags-check"),
                    ("codepen", "codepen"),
                    ("coffee", "coffee"),
                    ("coffee-maker", "coffee-maker"),
                    ("coffee-maker-check", "coffee-maker-check"),
                    ("coffee-maker-check-outline", "coffee-maker-check-outline"),
                    ("coffee-maker-outline", "coffee-maker-outline"),
                    ("coffee-off", "coffee-off"),
                    ("coffee-off-outline", "coffee-off-outline"),
                    ("coffee-outline", "coffee-outline"),
                    ("coffee-to-go", "coffee-to-go"),
                    ("coffee-to-go-outline", "coffee-to-go-outline"),
                    ("coffin", "coffin"),
                    ("cog", "cog"),
                    ("cog-box", "cog-box"),
                    ("cog-clockwise", "cog-clockwise"),
                    ("cog-counterclockwise", "cog-counterclockwise"),
                    ("cog-off", "cog-off"),
                    ("cog-off-outline", "cog-off-outline"),
                    ("cog-outline", "cog-outline"),
                    ("cog-pause", "cog-pause"),
                    ("cog-pause-outline", "cog-pause-outline"),
                    ("cog-play", "cog-play"),
                    ("cog-play-outline", "cog-play-outline"),
                    ("cog-refresh", "cog-refresh"),
                    ("cog-refresh-outline", "cog-refresh-outline"),
                    ("cog-stop", "cog-stop"),
                    ("cog-stop-outline", "cog-stop-outline"),
                    ("cog-sync", "cog-sync"),
                    ("cog-sync-outline", "cog-sync-outline"),
                    ("cog-transfer", "cog-transfer"),
                    ("cog-transfer-outline", "cog-transfer-outline"),
                    ("cogs", "cogs"),
                    ("collage", "collage"),
                    ("collapse-all", "collapse-all"),
                    ("collapse-all-outline", "collapse-all-outline"),
                    ("color-helper", "color-helper"),
                    ("comma", "comma"),
                    ("comma-box", "comma-box"),
                    ("comma-box-outline", "comma-box-outline"),
                    ("comma-circle", "comma-circle"),
                    ("comma-circle-outline", "comma-circle-outline"),
                    ("comment", "comment"),
                    ("comment-account", "comment-account"),
                    ("comment-account-outline", "comment-account-outline"),
                    ("comment-alert", "comment-alert"),
                    ("comment-alert-outline", "comment-alert-outline"),
                    ("comment-arrow-left", "comment-arrow-left"),
                    ("comment-arrow-left-outline", "comment-arrow-left-outline"),
                    ("comment-arrow-right", "comment-arrow-right"),
                    ("comment-arrow-right-outline", "comment-arrow-right-outline"),
                    ("comment-bookmark", "comment-bookmark"),
                    ("comment-bookmark-outline", "comment-bookmark-outline"),
                    ("comment-check", "comment-check"),
                    ("comment-check-outline", "comment-check-outline"),
                    ("comment-edit", "comment-edit"),
                    ("comment-edit-outline", "comment-edit-outline"),
                    ("comment-eye", "comment-eye"),
                    ("comment-eye-outline", "comment-eye-outline"),
                    ("comment-flash", "comment-flash"),
                    ("comment-flash-outline", "comment-flash-outline"),
                    ("comment-minus", "comment-minus"),
                    ("comment-minus-outline", "comment-minus-outline"),
                    ("comment-multiple", "comment-multiple"),
                    ("comment-multiple-outline", "comment-multiple-outline"),
                    ("comment-off", "comment-off"),
                    ("comment-off-outline", "comment-off-outline"),
                    ("comment-outline", "comment-outline"),
                    ("comment-plus", "comment-plus"),
                    ("comment-plus-outline", "comment-plus-outline"),
                    ("comment-processing", "comment-processing"),
                    ("comment-processing-outline", "comment-processing-outline"),
                    ("comment-question", "comment-question"),
                    ("comment-question-outline", "comment-question-outline"),
                    ("comment-quote", "comment-quote"),
                    ("comment-quote-outline", "comment-quote-outline"),
                    ("comment-remove", "comment-remove"),
                    ("comment-remove-outline", "comment-remove-outline"),
                    ("comment-search", "comment-search"),
                    ("comment-search-outline", "comment-search-outline"),
                    ("comment-text", "comment-text"),
                    ("comment-text-multiple", "comment-text-multiple"),
                    ("comment-text-multiple-outline", "comment-text-multiple-outline"),
                    ("comment-text-outline", "comment-text-outline"),
                    ("compare", "compare"),
                    ("compare-horizontal", "compare-horizontal"),
                    ("compare-remove", "compare-remove"),
                    ("compare-vertical", "compare-vertical"),
                    ("compass", "compass"),
                    ("compass-off", "compass-off"),
                    ("compass-off-outline", "compass-off-outline"),
                    ("compass-outline", "compass-outline"),
                    ("compass-rose", "compass-rose"),
                    ("compost", "compost"),
                    ("concourse-ci", "concourse-ci"),
                    ("cone", "cone"),
                    ("cone-off", "cone-off"),
                    ("connection", "connection"),
                    ("console", "console"),
                    ("console-line", "console-line"),
                    ("console-network", "console-network"),
                    ("console-network-outline", "console-network-outline"),
                    ("consolidate", "consolidate"),
                    ("contactless-payment", "contactless-payment"),
                    ("contactless-payment-circle", "contactless-payment-circle"),
                    ("contactless-payment-circle-outline", "contactless-payment-circle-outline"),
                    ("contacts", "contacts"),
                    ("contacts-outline", "contacts-outline"),
                    ("contain", "contain"),
                    ("contain-end", "contain-end"),
                    ("contain-start", "contain-start"),
                    ("content-copy", "content-copy"),
                    ("content-cut", "content-cut"),
                    ("content-duplicate", "content-duplicate"),
                    ("content-paste", "content-paste"),
                    ("content-save", "content-save"),
                    ("content-save-alert", "content-save-alert"),
                    ("content-save-alert-outline", "content-save-alert-outline"),
                    ("content-save-all", "content-save-all"),
                    ("content-save-all-outline", "content-save-all-outline"),
                    ("content-save-check", "content-save-check"),
                    ("content-save-check-outline", "content-save-check-outline"),
                    ("content-save-cog", "content-save-cog"),
                    ("content-save-cog-outline", "content-save-cog-outline"),
                    ("content-save-edit", "content-save-edit"),
                    ("content-save-edit-outline", "content-save-edit-outline"),
                    ("content-save-minus", "content-save-minus"),
                    ("content-save-minus-outline", "content-save-minus-outline"),
                    ("content-save-move", "content-save-move"),
                    ("content-save-move-outline", "content-save-move-outline"),
                    ("content-save-off", "content-save-off"),
                    ("content-save-off-outline", "content-save-off-outline"),
                    ("content-save-outline", "content-save-outline"),
                    ("content-save-plus", "content-save-plus"),
                    ("content-save-plus-outline", "content-save-plus-outline"),
                    ("content-save-settings", "content-save-settings"),
                    ("content-save-settings-outline", "content-save-settings-outline"),
                    ("contrast", "contrast"),
                    ("contrast-box", "contrast-box"),
                    ("contrast-circle", "contrast-circle"),
                    ("controller", "controller"),
                    ("controller-classic", "controller-classic"),
                    ("controller-classic-outline", "controller-classic-outline"),
                    ("controller-off", "controller-off"),
                    ("controller-xbox", "controller-xbox"),
                    ("cookie", "cookie"),
                    ("cookie-alert", "cookie-alert"),
                    ("cookie-alert-outline", "cookie-alert-outline"),
                    ("cookie-check", "cookie-check"),
                    ("cookie-check-outline", "cookie-check-outline"),
                    ("cookie-clock", "cookie-clock"),
                    ("cookie-clock-outline", "cookie-clock-outline"),
                    ("cookie-cog", "cookie-cog"),
                    ("cookie-cog-outline", "cookie-cog-outline"),
                    ("cookie-edit", "cookie-edit"),
                    ("cookie-edit-outline", "cookie-edit-outline"),
                    ("cookie-lock", "cookie-lock"),
                    ("cookie-lock-outline", "cookie-lock-outline"),
                    ("cookie-minus", "cookie-minus"),
                    ("cookie-minus-outline", "cookie-minus-outline"),
                    ("cookie-off", "cookie-off"),
                    ("cookie-off-outline", "cookie-off-outline"),
                    ("cookie-outline", "cookie-outline"),
                    ("cookie-plus", "cookie-plus"),
                    ("cookie-plus-outline", "cookie-plus-outline"),
                    ("cookie-refresh", "cookie-refresh"),
                    ("cookie-refresh-outline", "cookie-refresh-outline"),
                    ("cookie-remove", "cookie-remove"),
                    ("cookie-remove-outline", "cookie-remove-outline"),
                    ("cookie-settings", "cookie-settings"),
                    ("cookie-settings-outline", "cookie-settings-outline"),
                    ("coolant-temperature", "coolant-temperature"),
                    ("copyleft", "copyleft"),
                    ("copyright", "copyright"),
                    ("cordova", "cordova"),
                    ("corn", "corn"),
                    ("corn-off", "corn-off"),
                    ("cosine-wave", "cosine-wave"),
                    ("counter", "counter"),
                    ("countertop", "countertop"),
                    ("countertop-outline", "countertop-outline"),
                    ("cow", "cow"),
                    ("cow-off", "cow-off"),
                    ("cpu-32-bit", "cpu-32-bit"),
                    ("cpu-64-bit", "cpu-64-bit"),
                    ("cradle", "cradle"),
                    ("cradle-outline", "cradle-outline"),
                    ("crane", "crane"),
                    ("creation", "creation"),
                    ("creation-outline", "creation-outline"),
                    ("creative-commons", "creative-commons"),
                    ("credit-card", "credit-card"),
                    ("credit-card-check", "credit-card-check"),
                    ("credit-card-check-outline", "credit-card-check-outline"),
                    ("credit-card-chip", "credit-card-chip"),
                    ("credit-card-chip-outline", "credit-card-chip-outline"),
                    ("credit-card-clock", "credit-card-clock"),
                    ("credit-card-clock-outline", "credit-card-clock-outline"),
                    ("credit-card-edit", "credit-card-edit"),
                    ("credit-card-edit-outline", "credit-card-edit-outline"),
                    ("credit-card-fast", "credit-card-fast"),
                    ("credit-card-fast-outline", "credit-card-fast-outline"),
                    ("credit-card-lock", "credit-card-lock"),
                    ("credit-card-lock-outline", "credit-card-lock-outline"),
                    ("credit-card-marker", "credit-card-marker"),
                    ("credit-card-marker-outline", "credit-card-marker-outline"),
                    ("credit-card-minus", "credit-card-minus"),
                    ("credit-card-minus-outline", "credit-card-minus-outline"),
                    ("credit-card-multiple", "credit-card-multiple"),
                    ("credit-card-multiple-outline", "credit-card-multiple-outline"),
                    ("credit-card-off", "credit-card-off"),
                    ("credit-card-off-outline", "credit-card-off-outline"),
                    ("credit-card-outline", "credit-card-outline"),
                    ("credit-card-plus", "credit-card-plus"),
                    ("credit-card-plus-outline", "credit-card-plus-outline"),
                    ("credit-card-refresh", "credit-card-refresh"),
                    ("credit-card-refresh-outline", "credit-card-refresh-outline"),
                    ("credit-card-refund", "credit-card-refund"),
                    ("credit-card-refund-outline", "credit-card-refund-outline"),
                    ("credit-card-remove", "credit-card-remove"),
                    ("credit-card-remove-outline", "credit-card-remove-outline"),
                    ("credit-card-scan", "credit-card-scan"),
                    ("credit-card-scan-outline", "credit-card-scan-outline"),
                    ("credit-card-search", "credit-card-search"),
                    ("credit-card-search-outline", "credit-card-search-outline"),
                    ("credit-card-settings", "credit-card-settings"),
                    ("credit-card-settings-outline", "credit-card-settings-outline"),
                    ("credit-card-sync", "credit-card-sync"),
                    ("credit-card-sync-outline", "credit-card-sync-outline"),
                    ("credit-card-wireless", "credit-card-wireless"),
                    ("credit-card-wireless-off", "credit-card-wireless-off"),
                    ("credit-card-wireless-off-outline", "credit-card-wireless-off-outline"),
                    ("credit-card-wireless-outline", "credit-card-wireless-outline"),
                    ("cricket", "cricket"),
                    ("crop", "crop"),
                    ("crop-free", "crop-free"),
                    ("crop-landscape", "crop-landscape"),
                    ("crop-portrait", "crop-portrait"),
                    ("crop-rotate", "crop-rotate"),
                    ("crop-square", "crop-square"),
                    ("cross", "cross"),
                    ("cross-bolnisi", "cross-bolnisi"),
                    ("cross-celtic", "cross-celtic"),
                    ("cross-outline", "cross-outline"),
                    ("crosshairs", "crosshairs"),
                    ("crosshairs-gps", "crosshairs-gps"),
                    ("crosshairs-off", "crosshairs-off"),
                    ("crosshairs-question", "crosshairs-question"),
                    ("crowd", "crowd"),
                    ("crown", "crown"),
                    ("crown-circle", "crown-circle"),
                    ("crown-circle-outline", "crown-circle-outline"),
                    ("crown-outline", "crown-outline"),
                    ("cryengine", "cryengine"),
                    ("crystal-ball", "crystal-ball"),
                    ("cube", "cube"),
                    ("cube-off", "cube-off"),
                    ("cube-off-outline", "cube-off-outline"),
                    ("cube-outline", "cube-outline"),
                    ("cube-scan", "cube-scan"),
                    ("cube-send", "cube-send"),
                    ("cube-unfolded", "cube-unfolded"),
                    ("cup", "cup"),
                    ("cup-off", "cup-off"),
                    ("cup-off-outline", "cup-off-outline"),
                    ("cup-outline", "cup-outline"),
                    ("cup-water", "cup-water"),
                    ("cupboard", "cupboard"),
                    ("cupboard-outline", "cupboard-outline"),
                    ("cupcake", "cupcake"),
                    ("curling", "curling"),
                    ("currency-bdt", "currency-bdt"),
                    ("currency-brl", "currency-brl"),
                    ("currency-btc", "currency-btc"),
                    ("currency-chf", "currency-chf"),
                    ("currency-cny", "currency-cny"),
                    ("currency-eth", "currency-eth"),
                    ("currency-eur", "currency-eur"),
                    ("currency-eur-off", "currency-eur-off"),
                    ("currency-fra", "currency-fra"),
                    ("currency-gbp", "currency-gbp"),
                    ("currency-ils", "currency-ils"),
                    ("currency-inr", "currency-inr"),
                    ("currency-jpy", "currency-jpy"),
                    ("currency-krw", "currency-krw"),
                    ("currency-kzt", "currency-kzt"),
                    ("currency-mnt", "currency-mnt"),
                    ("currency-ngn", "currency-ngn"),
                    ("currency-php", "currency-php"),
                    ("currency-rial", "currency-rial"),
                    ("currency-rub", "currency-rub"),
                    ("currency-rupee", "currency-rupee"),
                    ("currency-sign", "currency-sign"),
                    ("currency-thb", "currency-thb"),
                    ("currency-try", "currency-try"),
                    ("currency-twd", "currency-twd"),
                    ("currency-uah", "currency-uah"),
                    ("currency-usd", "currency-usd"),
                    ("currency-usd-circle", "currency-usd-circle"),
                    ("currency-usd-circle-outline", "currency-usd-circle-outline"),
                    ("currency-usd-off", "currency-usd-off"),
                    ("current-ac", "current-ac"),
                    ("current-dc", "current-dc"),
                    ("cursor-default", "cursor-default"),
                    ("cursor-default-click", "cursor-default-click"),
                    ("cursor-default-click-outline", "cursor-default-click-outline"),
                    ("cursor-default-gesture", "cursor-default-gesture"),
                    ("cursor-default-gesture-outline", "cursor-default-gesture-outline"),
                    ("cursor-default-outline", "cursor-default-outline"),
                    ("cursor-move", "cursor-move"),
                    ("cursor-pointer", "cursor-pointer"),
                    ("cursor-text", "cursor-text"),
                    ("curtains", "curtains"),
                    ("curtains-closed", "curtains-closed"),
                    ("cylinder", "cylinder"),
                    ("cylinder-off", "cylinder-off"),
                    ("dance-ballroom", "dance-ballroom"),
                    ("dance-pole", "dance-pole"),
                    ("data", "data"),
                    ("data-matrix", "data-matrix"),
                    ("data-matrix-edit", "data-matrix-edit"),
                    ("data-matrix-minus", "data-matrix-minus"),
                    ("data-matrix-plus", "data-matrix-plus"),
                    ("data-matrix-remove", "data-matrix-remove"),
                    ("data-matrix-scan", "data-matrix-scan"),
                    ("database", "database"),
                    ("database-alert", "database-alert"),
                    ("database-alert-outline", "database-alert-outline"),
                    ("database-arrow-down", "database-arrow-down"),
                    ("database-arrow-down-outline", "database-arrow-down-outline"),
                    ("database-arrow-left", "database-arrow-left"),
                    ("database-arrow-left-outline", "database-arrow-left-outline"),
                    ("database-arrow-right", "database-arrow-right"),
                    ("database-arrow-right-outline", "database-arrow-right-outline"),
                    ("database-arrow-up", "database-arrow-up"),
                    ("database-arrow-up-outline", "database-arrow-up-outline"),
                    ("database-check", "database-check"),
                    ("database-check-outline", "database-check-outline"),
                    ("database-clock", "database-clock"),
                    ("database-clock-outline", "database-clock-outline"),
                    ("database-cog", "database-cog"),
                    ("database-cog-outline", "database-cog-outline"),
                    ("database-edit", "database-edit"),
                    ("database-edit-outline", "database-edit-outline"),
                    ("database-export", "database-export"),
                    ("database-export-outline", "database-export-outline"),
                    ("database-eye", "database-eye"),
                    ("database-eye-off", "database-eye-off"),
                    ("database-eye-off-outline", "database-eye-off-outline"),
                    ("database-eye-outline", "database-eye-outline"),
                    ("database-import", "database-import"),
                    ("database-import-outline", "database-import-outline"),
                    ("database-lock", "database-lock"),
                    ("database-lock-outline", "database-lock-outline"),
                    ("database-marker", "database-marker"),
                    ("database-marker-outline", "database-marker-outline"),
                    ("database-minus", "database-minus"),
                    ("database-minus-outline", "database-minus-outline"),
                    ("database-off", "database-off"),
                    ("database-off-outline", "database-off-outline"),
                    ("database-outline", "database-outline"),
                    ("database-plus", "database-plus"),
                    ("database-plus-outline", "database-plus-outline"),
                    ("database-refresh", "database-refresh"),
                    ("database-refresh-outline", "database-refresh-outline"),
                    ("database-remove", "database-remove"),
                    ("database-remove-outline", "database-remove-outline"),
                    ("database-search", "database-search"),
                    ("database-search-outline", "database-search-outline"),
                    ("database-settings", "database-settings"),
                    ("database-settings-outline", "database-settings-outline"),
                    ("database-sync", "database-sync"),
                    ("database-sync-outline", "database-sync-outline"),
                    ("death-star", "death-star"),
                    ("death-star-variant", "death-star-variant"),
                    ("deathly-hallows", "deathly-hallows"),
                    ("debian", "debian"),
                    ("debug-step-into", "debug-step-into"),
                    ("debug-step-out", "debug-step-out"),
                    ("debug-step-over", "debug-step-over"),
                    ("decagram", "decagram"),
                    ("decagram-outline", "decagram-outline"),
                    ("decimal", "decimal"),
                    ("decimal-comma", "decimal-comma"),
                    ("decimal-comma-decrease", "decimal-comma-decrease"),
                    ("decimal-comma-increase", "decimal-comma-increase"),
                    ("decimal-decrease", "decimal-decrease"),
                    ("decimal-increase", "decimal-increase"),
                    ("delete", "delete"),
                    ("delete-alert", "delete-alert"),
                    ("delete-alert-outline", "delete-alert-outline"),
                    ("delete-circle", "delete-circle"),
                    ("delete-circle-outline", "delete-circle-outline"),
                    ("delete-clock", "delete-clock"),
                    ("delete-clock-outline", "delete-clock-outline"),
                    ("delete-empty", "delete-empty"),
                    ("delete-empty-outline", "delete-empty-outline"),
                    ("delete-forever", "delete-forever"),
                    ("delete-forever-outline", "delete-forever-outline"),
                    ("delete-off", "delete-off"),
                    ("delete-off-outline", "delete-off-outline"),
                    ("delete-outline", "delete-outline"),
                    ("delete-restore", "delete-restore"),
                    ("delete-sweep", "delete-sweep"),
                    ("delete-sweep-outline", "delete-sweep-outline"),
                    ("delete-variant", "delete-variant"),
                    ("delta", "delta"),
                    ("desk", "desk"),
                    ("desk-lamp", "desk-lamp"),
                    ("desk-lamp-off", "desk-lamp-off"),
                    ("desk-lamp-on", "desk-lamp-on"),
                    ("deskphone", "deskphone"),
                    ("desktop-classic", "desktop-classic"),
                    ("desktop-mac", "desktop-mac"),
                    ("desktop-mac-dashboard", "desktop-mac-dashboard"),
                    ("desktop-tower", "desktop-tower"),
                    ("desktop-tower-monitor", "desktop-tower-monitor"),
                    ("details", "details"),
                    ("dev-to", "dev-to"),
                    ("developer-board", "developer-board"),
                    ("deviantart", "deviantart"),
                    ("devices", "devices"),
                    ("dharmachakra", "dharmachakra"),
                    ("diabetes", "diabetes"),
                    ("dialpad", "dialpad"),
                    ("diameter", "diameter"),
                    ("diameter-outline", "diameter-outline"),
                    ("diameter-variant", "diameter-variant"),
                    ("diamond", "diamond"),
                    ("diamond-outline", "diamond-outline"),
                    ("diamond-stone", "diamond-stone"),
                    ("dice", "dice"),
                    ("dice-1", "dice-1"),
                    ("dice-1-outline", "dice-1-outline"),
                    ("dice-2", "dice-2"),
                    ("dice-2-outline", "dice-2-outline"),
                    ("dice-3", "dice-3"),
                    ("dice-3-outline", "dice-3-outline"),
                    ("dice-4", "dice-4"),
                    ("dice-4-outline", "dice-4-outline"),
                    ("dice-5", "dice-5"),
                    ("dice-5-outline", "dice-5-outline"),
                    ("dice-6", "dice-6"),
                    ("dice-6-outline", "dice-6-outline"),
                    ("dice-d10", "dice-d10"),
                    ("dice-d10-outline", "dice-d10-outline"),
                    ("dice-d12", "dice-d12"),
                    ("dice-d12-outline", "dice-d12-outline"),
                    ("dice-d20", "dice-d20"),
                    ("dice-d20-outline", "dice-d20-outline"),
                    ("dice-d4", "dice-d4"),
                    ("dice-d4-outline", "dice-d4-outline"),
                    ("dice-d6", "dice-d6"),
                    ("dice-d6-outline", "dice-d6-outline"),
                    ("dice-d8", "dice-d8"),
                    ("dice-d8-outline", "dice-d8-outline"),
                    ("dice-multiple", "dice-multiple"),
                    ("dice-multiple-outline", "dice-multiple-outline"),
                    ("digital-ocean", "digital-ocean"),
                    ("dip-switch", "dip-switch"),
                    ("directions", "directions"),
                    ("directions-fork", "directions-fork"),
                    ("disc", "disc"),
                    ("disc-alert", "disc-alert"),
                    ("disc-player", "disc-player"),
                    ("discord", "discord"),
                    ("dishwasher", "dishwasher"),
                    ("dishwasher-alert", "dishwasher-alert"),
                    ("dishwasher-off", "dishwasher-off"),
                    ("disk", "disk"),
                    ("disk-alert", "disk-alert"),
                    ("disk-player", "disk-player"),
                    ("disqus", "disqus"),
                    ("disqus-outline", "disqus-outline"),
                    ("distribute-horizontal-center", "distribute-horizontal-center"),
                    ("distribute-horizontal-left", "distribute-horizontal-left"),
                    ("distribute-horizontal-right", "distribute-horizontal-right"),
                    ("distribute-vertical-bottom", "distribute-vertical-bottom"),
                    ("distribute-vertical-center", "distribute-vertical-center"),
                    ("distribute-vertical-top", "distribute-vertical-top"),
                    ("diversify", "diversify"),
                    ("diving", "diving"),
                    ("diving-flippers", "diving-flippers"),
                    ("diving-helmet", "diving-helmet"),
                    ("diving-scuba", "diving-scuba"),
                    ("diving-scuba-flag", "diving-scuba-flag"),
                    ("diving-scuba-mask", "diving-scuba-mask"),
                    ("diving-scuba-tank", "diving-scuba-tank"),
                    ("diving-scuba-tank-multiple", "diving-scuba-tank-multiple"),
                    ("diving-snorkel", "diving-snorkel"),
                    ("division", "division"),
                    ("division-box", "division-box"),
                    ("dlna", "dlna"),
                    ("dna", "dna"),
                    ("dns", "dns"),
                    ("dns-outline", "dns-outline"),
                    ("do-not-disturb", "do-not-disturb"),
                    ("dock-bottom", "dock-bottom"),
                    ("dock-left", "dock-left"),
                    ("dock-right", "dock-right"),
                    ("dock-top", "dock-top"),
                    ("dock-window", "dock-window"),
                    ("docker", "docker"),
                    ("doctor", "doctor"),
                    ("document", "document"),
                    ("dog", "dog"),
                    ("dog-service", "dog-service"),
                    ("dog-side", "dog-side"),
                    ("dog-side-off", "dog-side-off"),
                    ("dolby", "dolby"),
                    ("dolly", "dolly"),
                    ("dolphin", "dolphin"),
                    ("domain", "domain"),
                    ("domain-off", "domain-off"),
                    ("domain-plus", "domain-plus"),
                    ("domain-remove", "domain-remove"),
                    ("domain-switch", "domain-switch"),
                    ("dome-light", "dome-light"),
                    ("domino-mask", "domino-mask"),
                    ("donkey", "donkey"),
                    ("door", "door"),
                    ("door-closed", "door-closed"),
                    ("door-closed-cancel", "door-closed-cancel"),
                    ("door-closed-lock", "door-closed-lock"),
                    ("door-open", "door-open"),
                    ("door-sliding", "door-sliding"),
                    ("door-sliding-lock", "door-sliding-lock"),
                    ("door-sliding-open", "door-sliding-open"),
                    ("doorbell", "doorbell"),
                    ("doorbell-video", "doorbell-video"),
                    ("dot-net", "dot-net"),
                    ("dots-circle", "dots-circle"),
                    ("dots-grid", "dots-grid"),
                    ("dots-hexagon", "dots-hexagon"),
                    ("dots-horizontal", "dots-horizontal"),
                    ("dots-horizontal-circle", "dots-horizontal-circle"),
                    ("dots-horizontal-circle-outline", "dots-horizontal-circle-outline"),
                    ("dots-square", "dots-square"),
                    ("dots-triangle", "dots-triangle"),
                    ("dots-vertical", "dots-vertical"),
                    ("dots-vertical-circle", "dots-vertical-circle"),
                    ("dots-vertical-circle-outline", "dots-vertical-circle-outline"),
                    ("douban", "douban"),
                    ("download", "download"),
                    ("download-box", "download-box"),
                    ("download-box-outline", "download-box-outline"),
                    ("download-circle", "download-circle"),
                    ("download-circle-outline", "download-circle-outline"),
                    ("download-lock", "download-lock"),
                    ("download-lock-outline", "download-lock-outline"),
                    ("download-multiple", "download-multiple"),
                    ("download-network", "download-network"),
                    ("download-network-outline", "download-network-outline"),
                    ("download-off", "download-off"),
                    ("download-off-outline", "download-off-outline"),
                    ("download-outline", "download-outline"),
                    ("drag", "drag"),
                    ("drag-horizontal", "drag-horizontal"),
                    ("drag-horizontal-variant", "drag-horizontal-variant"),
                    ("drag-variant", "drag-variant"),
                    ("drag-vertical", "drag-vertical"),
                    ("drag-vertical-variant", "drag-vertical-variant"),
                    ("drama-masks", "drama-masks"),
                    ("draw", "draw"),
                    ("draw-pen", "draw-pen"),
                    ("drawing", "drawing"),
                    ("drawing-box", "drawing-box"),
                    ("dresser", "dresser"),
                    ("dresser-outline", "dresser-outline"),
                    ("dribbble", "dribbble"),
                    ("dribbble-box", "dribbble-box"),
                    ("drone", "drone"),
                    ("dropbox", "dropbox"),
                    ("drupal", "drupal"),
                    ("duck", "duck"),
                    ("dumbbell", "dumbbell"),
                    ("dump-truck", "dump-truck"),
                    ("ear-hearing", "ear-hearing"),
                    ("ear-hearing-loop", "ear-hearing-loop"),
                    ("ear-hearing-off", "ear-hearing-off"),
                    ("earbuds", "earbuds"),
                    ("earbuds-off", "earbuds-off"),
                    ("earbuds-off-outline", "earbuds-off-outline"),
                    ("earbuds-outline", "earbuds-outline"),
                    ("earth", "earth"),
                    ("earth-arrow-down", "earth-arrow-down"),
                    ("earth-arrow-left", "earth-arrow-left"),
                    ("earth-arrow-right", "earth-arrow-right"),
                    ("earth-arrow-up", "earth-arrow-up"),
                    ("earth-box", "earth-box"),
                    ("earth-box-minus", "earth-box-minus"),
                    ("earth-box-off", "earth-box-off"),
                    ("earth-box-plus", "earth-box-plus"),
                    ("earth-box-remove", "earth-box-remove"),
                    ("earth-minus", "earth-minus"),
                    ("earth-off", "earth-off"),
                    ("earth-plus", "earth-plus"),
                    ("earth-remove", "earth-remove"),
                    ("ebay", "ebay"),
                    ("egg", "egg"),
                    ("egg-easter", "egg-easter"),
                    ("egg-fried", "egg-fried"),
                    ("egg-off", "egg-off"),
                    ("egg-off-outline", "egg-off-outline"),
                    ("egg-outline", "egg-outline"),
                    ("eiffel-tower", "eiffel-tower"),
                    ("eight-track", "eight-track"),
                    ("eject", "eject"),
                    ("eject-circle", "eject-circle"),
                    ("eject-circle-outline", "eject-circle-outline"),
                    ("eject-outline", "eject-outline"),
                    ("electric-switch", "electric-switch"),
                    ("electric-switch-closed", "electric-switch-closed"),
                    ("electron-framework", "electron-framework"),
                    ("elephant", "elephant"),
                    ("elevation-decline", "elevation-decline"),
                    ("elevation-rise", "elevation-rise"),
                    ("elevator", "elevator"),
                    ("elevator-down", "elevator-down"),
                    ("elevator-passenger", "elevator-passenger"),
                    ("elevator-passenger-off", "elevator-passenger-off"),
                    ("elevator-passenger-off-outline", "elevator-passenger-off-outline"),
                    ("elevator-passenger-outline", "elevator-passenger-outline"),
                    ("elevator-up", "elevator-up"),
                    ("ellipse", "ellipse"),
                    ("ellipse-outline", "ellipse-outline"),
                    ("email", "email"),
                    ("email-alert", "email-alert"),
                    ("email-alert-outline", "email-alert-outline"),
                    ("email-arrow-left", "email-arrow-left"),
                    ("email-arrow-left-outline", "email-arrow-left-outline"),
                    ("email-arrow-right", "email-arrow-right"),
                    ("email-arrow-right-outline", "email-arrow-right-outline"),
                    ("email-box", "email-box"),
                    ("email-check", "email-check"),
                    ("email-check-outline", "email-check-outline"),
                    ("email-edit", "email-edit"),
                    ("email-edit-outline", "email-edit-outline"),
                    ("email-fast", "email-fast"),
                    ("email-fast-outline", "email-fast-outline"),
                    ("email-heart-outline", "email-heart-outline"),
                    ("email-lock", "email-lock"),
                    ("email-lock-outline", "email-lock-outline"),
                    ("email-mark-as-unread", "email-mark-as-unread"),
                    ("email-minus", "email-minus"),
                    ("email-minus-outline", "email-minus-outline"),
                    ("email-multiple", "email-multiple"),
                    ("email-multiple-outline", "email-multiple-outline"),
                    ("email-newsletter", "email-newsletter"),
                    ("email-off", "email-off"),
                    ("email-off-outline", "email-off-outline"),
                    ("email-open", "email-open"),
                    ("email-open-heart-outline", "email-open-heart-outline"),
                    ("email-open-multiple", "email-open-multiple"),
                    ("email-open-multiple-outline", "email-open-multiple-outline"),
                    ("email-open-outline", "email-open-outline"),
                    ("email-outline", "email-outline"),
                    ("email-plus", "email-plus"),
                    ("email-plus-outline", "email-plus-outline"),
                    ("email-remove", "email-remove"),
                    ("email-remove-outline", "email-remove-outline"),
                    ("email-seal", "email-seal"),
                    ("email-seal-outline", "email-seal-outline"),
                    ("email-search", "email-search"),
                    ("email-search-outline", "email-search-outline"),
                    ("email-sync", "email-sync"),
                    ("email-sync-outline", "email-sync-outline"),
                    ("email-variant", "email-variant"),
                    ("ember", "ember"),
                    ("emby", "emby"),
                    ("emoticon", "emoticon"),
                    ("emoticon-angry", "emoticon-angry"),
                    ("emoticon-angry-outline", "emoticon-angry-outline"),
                    ("emoticon-confused", "emoticon-confused"),
                    ("emoticon-confused-outline", "emoticon-confused-outline"),
                    ("emoticon-cool", "emoticon-cool"),
                    ("emoticon-cool-outline", "emoticon-cool-outline"),
                    ("emoticon-cry", "emoticon-cry"),
                    ("emoticon-cry-outline", "emoticon-cry-outline"),
                    ("emoticon-dead", "emoticon-dead"),
                    ("emoticon-dead-outline", "emoticon-dead-outline"),
                    ("emoticon-devil", "emoticon-devil"),
                    ("emoticon-devil-outline", "emoticon-devil-outline"),
                    ("emoticon-excited", "emoticon-excited"),
                    ("emoticon-excited-outline", "emoticon-excited-outline"),
                    ("emoticon-frown", "emoticon-frown"),
                    ("emoticon-frown-outline", "emoticon-frown-outline"),
                    ("emoticon-happy", "emoticon-happy"),
                    ("emoticon-happy-outline", "emoticon-happy-outline"),
                    ("emoticon-kiss", "emoticon-kiss"),
                    ("emoticon-kiss-outline", "emoticon-kiss-outline"),
                    ("emoticon-lol", "emoticon-lol"),
                    ("emoticon-lol-outline", "emoticon-lol-outline"),
                    ("emoticon-neutral", "emoticon-neutral"),
                    ("emoticon-neutral-outline", "emoticon-neutral-outline"),
                    ("emoticon-outline", "emoticon-outline"),
                    ("emoticon-poop", "emoticon-poop"),
                    ("emoticon-poop-outline", "emoticon-poop-outline"),
                    ("emoticon-sad", "emoticon-sad"),
                    ("emoticon-sad-outline", "emoticon-sad-outline"),
                    ("emoticon-sick", "emoticon-sick"),
                    ("emoticon-sick-outline", "emoticon-sick-outline"),
                    ("emoticon-tongue", "emoticon-tongue"),
                    ("emoticon-tongue-outline", "emoticon-tongue-outline"),
                    ("emoticon-wink", "emoticon-wink"),
                    ("emoticon-wink-outline", "emoticon-wink-outline"),
                    ("engine", "engine"),
                    ("engine-off", "engine-off"),
                    ("engine-off-outline", "engine-off-outline"),
                    ("engine-outline", "engine-outline"),
                    ("epsilon", "epsilon"),
                    ("equal", "equal"),
                    ("equal-box", "equal-box"),
                    ("equalizer", "equalizer"),
                    ("equalizer-outline", "equalizer-outline"),
                    ("eraser", "eraser"),
                    ("eraser-variant", "eraser-variant"),
                    ("escalator", "escalator"),
                    ("escalator-box", "escalator-box"),
                    ("escalator-down", "escalator-down"),
                    ("escalator-up", "escalator-up"),
                    ("eslint", "eslint"),
                    ("et", "et"),
                    ("ethereum", "ethereum"),
                    ("ethernet", "ethernet"),
                    ("ethernet-cable", "ethernet-cable"),
                    ("ethernet-cable-off", "ethernet-cable-off"),
                    ("etsy", "etsy"),
                    ("ev-plug-ccs1", "ev-plug-ccs1"),
                    ("ev-plug-ccs2", "ev-plug-ccs2"),
                    ("ev-plug-chademo", "ev-plug-chademo"),
                    ("ev-plug-tesla", "ev-plug-tesla"),
                    ("ev-plug-type1", "ev-plug-type1"),
                    ("ev-plug-type2", "ev-plug-type2"),
                    ("ev-station", "ev-station"),
                    ("eventbrite", "eventbrite"),
                    ("evernote", "evernote"),
                    ("excavator", "excavator"),
                    ("exclamation", "exclamation"),
                    ("exclamation-thick", "exclamation-thick"),
                    ("exit-run", "exit-run"),
                    ("exit-to-app", "exit-to-app"),
                    ("expand-all", "expand-all"),
                    ("expand-all-outline", "expand-all-outline"),
                    ("expansion-card", "expansion-card"),
                    ("expansion-card-variant", "expansion-card-variant"),
                    ("exponent", "exponent"),
                    ("exponent-box", "exponent-box"),
                    ("export", "export"),
                    ("export-variant", "export-variant"),
                    ("eye", "eye"),
                    ("eye-arrow-left", "eye-arrow-left"),
                    ("eye-arrow-left-outline", "eye-arrow-left-outline"),
                    ("eye-arrow-right", "eye-arrow-right"),
                    ("eye-arrow-right-outline", "eye-arrow-right-outline"),
                    ("eye-check", "eye-check"),
                    ("eye-check-outline", "eye-check-outline"),
                    ("eye-circle", "eye-circle"),
                    ("eye-circle-outline", "eye-circle-outline"),
                    ("eye-closed", "eye-closed"),
                    ("eye-lock", "eye-lock"),
                    ("eye-lock-open", "eye-lock-open"),
                    ("eye-lock-open-outline", "eye-lock-open-outline"),
                    ("eye-lock-outline", "eye-lock-outline"),
                    ("eye-minus", "eye-minus"),
                    ("eye-minus-outline", "eye-minus-outline"),
                    ("eye-off", "eye-off"),
                    ("eye-off-outline", "eye-off-outline"),
                    ("eye-outline", "eye-outline"),
                    ("eye-plus", "eye-plus"),
                    ("eye-plus-outline", "eye-plus-outline"),
                    ("eye-refresh", "eye-refresh"),
                    ("eye-refresh-outline", "eye-refresh-outline"),
                    ("eye-remove", "eye-remove"),
                    ("eye-remove-outline", "eye-remove-outline"),
                    ("eye-settings", "eye-settings"),
                    ("eye-settings-outline", "eye-settings-outline"),
                    ("eyedropper", "eyedropper"),
                    ("eyedropper-minus", "eyedropper-minus"),
                    ("eyedropper-off", "eyedropper-off"),
                    ("eyedropper-plus", "eyedropper-plus"),
                    ("eyedropper-remove", "eyedropper-remove"),
                    ("eyedropper-variant", "eyedropper-variant"),
                    ("face-agent", "face-agent"),
                    ("face-man", "face-man"),
                    ("face-man-outline", "face-man-outline"),
                    ("face-man-profile", "face-man-profile"),
                    ("face-man-shimmer", "face-man-shimmer"),
                    ("face-man-shimmer-outline", "face-man-shimmer-outline"),
                    ("face-mask", "face-mask"),
                    ("face-mask-outline", "face-mask-outline"),
                    ("face-recognition", "face-recognition"),
                    ("face-woman", "face-woman"),
                    ("face-woman-outline", "face-woman-outline"),
                    ("face-woman-profile", "face-woman-profile"),
                    ("face-woman-shimmer", "face-woman-shimmer"),
                    ("face-woman-shimmer-outline", "face-woman-shimmer-outline"),
                    ("facebook", "facebook"),
                    ("facebook-box", "facebook-box"),
                    ("facebook-gaming", "facebook-gaming"),
                    ("facebook-messenger", "facebook-messenger"),
                    ("facebook-workplace", "facebook-workplace"),
                    ("factory", "factory"),
                    ("family-tree", "family-tree"),
                    ("fan", "fan"),
                    ("fan-alert", "fan-alert"),
                    ("fan-auto", "fan-auto"),
                    ("fan-chevron-down", "fan-chevron-down"),
                    ("fan-chevron-up", "fan-chevron-up"),
                    ("fan-clock", "fan-clock"),
                    ("fan-minus", "fan-minus"),
                    ("fan-off", "fan-off"),
                    ("fan-plus", "fan-plus"),
                    ("fan-remove", "fan-remove"),
                    ("fan-speed-1", "fan-speed-1"),
                    ("fan-speed-2", "fan-speed-2"),
                    ("fan-speed-3", "fan-speed-3"),
                    ("fast-forward", "fast-forward"),
                    ("fast-forward-10", "fast-forward-10"),
                    ("fast-forward-15", "fast-forward-15"),
                    ("fast-forward-30", "fast-forward-30"),
                    ("fast-forward-45", "fast-forward-45"),
                    ("fast-forward-5", "fast-forward-5"),
                    ("fast-forward-60", "fast-forward-60"),
                    ("fast-forward-outline", "fast-forward-outline"),
                    ("faucet", "faucet"),
                    ("faucet-variant", "faucet-variant"),
                    ("fax", "fax"),
                    ("feather", "feather"),
                    ("feature-search", "feature-search"),
                    ("feature-search-outline", "feature-search-outline"),
                    ("fedora", "fedora"),
                    ("fence", "fence"),
                    ("fence-electric", "fence-electric"),
                    ("fencing", "fencing"),
                    ("ferris-wheel", "ferris-wheel"),
                    ("ferry", "ferry"),
                    ("file", "file"),
                    ("file-account", "file-account"),
                    ("file-account-outline", "file-account-outline"),
                    ("file-alert", "file-alert"),
                    ("file-alert-outline", "file-alert-outline"),
                    ("file-arrow-left-right", "file-arrow-left-right"),
                    ("file-arrow-left-right-outline", "file-arrow-left-right-outline"),
                    ("file-arrow-up-down", "file-arrow-up-down"),
                    ("file-arrow-up-down-outline", "file-arrow-up-down-outline"),
                    ("file-cabinet", "file-cabinet"),
                    ("file-cad", "file-cad"),
                    ("file-cad-box", "file-cad-box"),
                    ("file-cancel", "file-cancel"),
                    ("file-cancel-outline", "file-cancel-outline"),
                    ("file-certificate", "file-certificate"),
                    ("file-certificate-outline", "file-certificate-outline"),
                    ("file-chart", "file-chart"),
                    ("file-chart-check", "file-chart-check"),
                    ("file-chart-check-outline", "file-chart-check-outline"),
                    ("file-chart-outline", "file-chart-outline"),
                    ("file-check", "file-check"),
                    ("file-check-outline", "file-check-outline"),
                    ("file-clock", "file-clock"),
                    ("file-clock-outline", "file-clock-outline"),
                    ("file-cloud", "file-cloud"),
                    ("file-cloud-outline", "file-cloud-outline"),
                    ("file-code", "file-code"),
                    ("file-code-outline", "file-code-outline"),
                    ("file-cog", "file-cog"),
                    ("file-cog-outline", "file-cog-outline"),
                    ("file-compare", "file-compare"),
                    ("file-delimited", "file-delimited"),
                    ("file-delimited-outline", "file-delimited-outline"),
                    ("file-document", "file-document"),
                    ("file-document-alert", "file-document-alert"),
                    ("file-document-alert-outline", "file-document-alert-outline"),
                    ("file-document-arrow-right", "file-document-arrow-right"),
                    ("file-document-arrow-right-outline", "file-document-arrow-right-outline"),
                    ("file-document-check", "file-document-check"),
                    ("file-document-check-outline", "file-document-check-outline"),
                    ("file-document-edit", "file-document-edit"),
                    ("file-document-edit-outline", "file-document-edit-outline"),
                    ("file-document-minus", "file-document-minus"),
                    ("file-document-minus-outline", "file-document-minus-outline"),
                    ("file-document-multiple", "file-document-multiple"),
                    ("file-document-multiple-outline", "file-document-multiple-outline"),
                    ("file-document-outline", "file-document-outline"),
                    ("file-document-plus", "file-document-plus"),
                    ("file-document-plus-outline", "file-document-plus-outline"),
                    ("file-document-refresh", "file-document-refresh"),
                    ("file-document-refresh-outline", "file-document-refresh-outline"),
                    ("file-document-remove", "file-document-remove"),
                    ("file-document-remove-outline", "file-document-remove-outline"),
                    ("file-download", "file-download"),
                    ("file-download-outline", "file-download-outline"),
                    ("file-edit", "file-edit"),
                    ("file-edit-outline", "file-edit-outline"),
                    ("file-excel", "file-excel"),
                    ("file-excel-box", "file-excel-box"),
                    ("file-excel-box-outline", "file-excel-box-outline"),
                    ("file-excel-outline", "file-excel-outline"),
                    ("file-export", "file-export"),
                    ("file-export-outline", "file-export-outline"),
                    ("file-eye", "file-eye"),
                    ("file-eye-outline", "file-eye-outline"),
                    ("file-find", "file-find"),
                    ("file-find-outline", "file-find-outline"),
                    ("file-gif-box", "file-gif-box"),
                    ("file-hidden", "file-hidden"),
                    ("file-image", "file-image"),
                    ("file-image-box", "file-image-box"),
                    ("file-image-marker", "file-image-marker"),
                    ("file-image-marker-outline", "file-image-marker-outline"),
                    ("file-image-minus", "file-image-minus"),
                    ("file-image-minus-outline", "file-image-minus-outline"),
                    ("file-image-outline", "file-image-outline"),
                    ("file-image-plus", "file-image-plus"),
                    ("file-image-plus-outline", "file-image-plus-outline"),
                    ("file-image-remove", "file-image-remove"),
                    ("file-image-remove-outline", "file-image-remove-outline"),
                    ("file-import", "file-import"),
                    ("file-import-outline", "file-import-outline"),
                    ("file-jpg-box", "file-jpg-box"),
                    ("file-key", "file-key"),
                    ("file-key-outline", "file-key-outline"),
                    ("file-link", "file-link"),
                    ("file-link-outline", "file-link-outline"),
                    ("file-lock", "file-lock"),
                    ("file-lock-open", "file-lock-open"),
                    ("file-lock-open-outline", "file-lock-open-outline"),
                    ("file-lock-outline", "file-lock-outline"),
                    ("file-marker", "file-marker"),
                    ("file-marker-outline", "file-marker-outline"),
                    ("file-minus", "file-minus"),
                    ("file-minus-outline", "file-minus-outline"),
                    ("file-move", "file-move"),
                    ("file-move-outline", "file-move-outline"),
                    ("file-multiple", "file-multiple"),
                    ("file-multiple-outline", "file-multiple-outline"),
                    ("file-music", "file-music"),
                    ("file-music-outline", "file-music-outline"),
                    ("file-outline", "file-outline"),
                    ("file-pdf", "file-pdf"),
                    ("file-pdf-box", "file-pdf-box"),
                    ("file-pdf-box-outline", "file-pdf-box-outline"),
                    ("file-pdf-outline", "file-pdf-outline"),
                    ("file-percent", "file-percent"),
                    ("file-percent-outline", "file-percent-outline"),
                    ("file-phone", "file-phone"),
                    ("file-phone-outline", "file-phone-outline"),
                    ("file-plus", "file-plus"),
                    ("file-plus-outline", "file-plus-outline"),
                    ("file-png-box", "file-png-box"),
                    ("file-powerpoint", "file-powerpoint"),
                    ("file-powerpoint-box", "file-powerpoint-box"),
                    ("file-powerpoint-box-outline", "file-powerpoint-box-outline"),
                    ("file-powerpoint-outline", "file-powerpoint-outline"),
                    ("file-presentation-box", "file-presentation-box"),
                    ("file-question", "file-question"),
                    ("file-question-outline", "file-question-outline"),
                    ("file-refresh", "file-refresh"),
                    ("file-refresh-outline", "file-refresh-outline"),
                    ("file-remove", "file-remove"),
                    ("file-remove-outline", "file-remove-outline"),
                    ("file-replace", "file-replace"),
                    ("file-replace-outline", "file-replace-outline"),
                    ("file-restore", "file-restore"),
                    ("file-restore-outline", "file-restore-outline"),
                    ("file-rotate-left", "file-rotate-left"),
                    ("file-rotate-left-outline", "file-rotate-left-outline"),
                    ("file-rotate-right", "file-rotate-right"),
                    ("file-rotate-right-outline", "file-rotate-right-outline"),
                    ("file-search", "file-search"),
                    ("file-search-outline", "file-search-outline"),
                    ("file-send", "file-send"),
                    ("file-send-outline", "file-send-outline"),
                    ("file-settings", "file-settings"),
                    ("file-settings-outline", "file-settings-outline"),
                    ("file-sign", "file-sign"),
                    ("file-star", "file-star"),
                    ("file-star-four-points", "file-star-four-points"),
                    ("file-star-four-points-outline", "file-star-four-points-outline"),
                    ("file-star-outline", "file-star-outline"),
                    ("file-swap", "file-swap"),
                    ("file-swap-outline", "file-swap-outline"),
                    ("file-sync", "file-sync"),
                    ("file-sync-outline", "file-sync-outline"),
                    ("file-table", "file-table"),
                    ("file-table-box", "file-table-box"),
                    ("file-table-box-multiple", "file-table-box-multiple"),
                    ("file-table-box-multiple-outline", "file-table-box-multiple-outline"),
                    ("file-table-box-outline", "file-table-box-outline"),
                    ("file-table-outline", "file-table-outline"),
                    ("file-tree", "file-tree"),
                    ("file-tree-outline", "file-tree-outline"),
                    ("file-undo", "file-undo"),
                    ("file-undo-outline", "file-undo-outline"),
                    ("file-upload", "file-upload"),
                    ("file-upload-outline", "file-upload-outline"),
                    ("file-video", "file-video"),
                    ("file-video-outline", "file-video-outline"),
                    ("file-word", "file-word"),
                    ("file-word-box", "file-word-box"),
                    ("file-word-box-outline", "file-word-box-outline"),
                    ("file-word-outline", "file-word-outline"),
                    ("file-xml", "file-xml"),
                    ("file-xml-box", "file-xml-box"),
                    ("fill", "fill"),
                    ("film", "film"),
                    ("filmstrip", "filmstrip"),
                    ("filmstrip-box", "filmstrip-box"),
                    ("filmstrip-box-multiple", "filmstrip-box-multiple"),
                    ("filmstrip-off", "filmstrip-off"),
                    ("filter", "filter"),
                    ("filter-check", "filter-check"),
                    ("filter-check-outline", "filter-check-outline"),
                    ("filter-cog", "filter-cog"),
                    ("filter-cog-outline", "filter-cog-outline"),
                    ("filter-menu", "filter-menu"),
                    ("filter-menu-outline", "filter-menu-outline"),
                    ("filter-minus", "filter-minus"),
                    ("filter-minus-outline", "filter-minus-outline"),
                    ("filter-multiple", "filter-multiple"),
                    ("filter-multiple-outline", "filter-multiple-outline"),
                    ("filter-off", "filter-off"),
                    ("filter-off-outline", "filter-off-outline"),
                    ("filter-outline", "filter-outline"),
                    ("filter-plus", "filter-plus"),
                    ("filter-plus-outline", "filter-plus-outline"),
                    ("filter-remove", "filter-remove"),
                    ("filter-remove-outline", "filter-remove-outline"),
                    ("filter-settings", "filter-settings"),
                    ("filter-settings-outline", "filter-settings-outline"),
                    ("filter-variant", "filter-variant"),
                    ("filter-variant-minus", "filter-variant-minus"),
                    ("filter-variant-plus", "filter-variant-plus"),
                    ("filter-variant-remove", "filter-variant-remove"),
                    ("finance", "finance"),
                    ("find-replace", "find-replace"),
                    ("fingerprint", "fingerprint"),
                    ("fingerprint-off", "fingerprint-off"),
                    ("fire", "fire"),
                    ("fire-alert", "fire-alert"),
                    ("fire-circle", "fire-circle"),
                    ("fire-extinguisher", "fire-extinguisher"),
                    ("fire-hydrant", "fire-hydrant"),
                    ("fire-hydrant-alert", "fire-hydrant-alert"),
                    ("fire-hydrant-off", "fire-hydrant-off"),
                    ("fire-off", "fire-off"),
                    ("fire-truck", "fire-truck"),
                    ("firebase", "firebase"),
                    ("firefox", "firefox"),
                    ("fireplace", "fireplace"),
                    ("fireplace-off", "fireplace-off"),
                    ("firewire", "firewire"),
                    ("firework", "firework"),
                    ("firework-off", "firework-off"),
                    ("fish", "fish"),
                    ("fish-off", "fish-off"),
                    ("fishbowl", "fishbowl"),
                    ("fishbowl-outline", "fishbowl-outline"),
                    ("fit-to-page", "fit-to-page"),
                    ("fit-to-page-outline", "fit-to-page-outline"),
                    ("fit-to-screen", "fit-to-screen"),
                    ("fit-to-screen-outline", "fit-to-screen-outline"),
                    ("flag", "flag"),
                    ("flag-checkered", "flag-checkered"),
                    ("flag-checkered-variant", "flag-checkered-variant"),
                    ("flag-minus", "flag-minus"),
                    ("flag-minus-outline", "flag-minus-outline"),
                    ("flag-off", "flag-off"),
                    ("flag-off-outline", "flag-off-outline"),
                    ("flag-outline", "flag-outline"),
                    ("flag-outline-variant", "flag-outline-variant"),
                    ("flag-plus", "flag-plus"),
                    ("flag-plus-outline", "flag-plus-outline"),
                    ("flag-remove", "flag-remove"),
                    ("flag-remove-outline", "flag-remove-outline"),
                    ("flag-triangle", "flag-triangle"),
                    ("flag-variant", "flag-variant"),
                    ("flag-variant-minus", "flag-variant-minus"),
                    ("flag-variant-minus-outline", "flag-variant-minus-outline"),
                    ("flag-variant-off", "flag-variant-off"),
                    ("flag-variant-off-outline", "flag-variant-off-outline"),
                    ("flag-variant-outline", "flag-variant-outline"),
                    ("flag-variant-plus", "flag-variant-plus"),
                    ("flag-variant-plus-outline", "flag-variant-plus-outline"),
                    ("flag-variant-remove", "flag-variant-remove"),
                    ("flag-variant-remove-outline", "flag-variant-remove-outline"),
                    ("flare", "flare"),
                    ("flash", "flash"),
                    ("flash-alert", "flash-alert"),
                    ("flash-alert-outline", "flash-alert-outline"),
                    ("flash-auto", "flash-auto"),
                    ("flash-off", "flash-off"),
                    ("flash-off-outline", "flash-off-outline"),
                    ("flash-outline", "flash-outline"),
                    ("flash-red-eye", "flash-red-eye"),
                    ("flash-triangle", "flash-triangle"),
                    ("flash-triangle-outline", "flash-triangle-outline"),
                    ("flashlight", "flashlight"),
                    ("flashlight-off", "flashlight-off"),
                    ("flask", "flask"),
                    ("flask-empty", "flask-empty"),
                    ("flask-empty-minus", "flask-empty-minus"),
                    ("flask-empty-minus-outline", "flask-empty-minus-outline"),
                    ("flask-empty-off", "flask-empty-off"),
                    ("flask-empty-off-outline", "flask-empty-off-outline"),
                    ("flask-empty-outline", "flask-empty-outline"),
                    ("flask-empty-plus", "flask-empty-plus"),
                    ("flask-empty-plus-outline", "flask-empty-plus-outline"),
                    ("flask-empty-remove", "flask-empty-remove"),
                    ("flask-empty-remove-outline", "flask-empty-remove-outline"),
                    ("flask-minus", "flask-minus"),
                    ("flask-minus-outline", "flask-minus-outline"),
                    ("flask-off", "flask-off"),
                    ("flask-off-outline", "flask-off-outline"),
                    ("flask-outline", "flask-outline"),
                    ("flask-plus", "flask-plus"),
                    ("flask-plus-outline", "flask-plus-outline"),
                    ("flask-remove", "flask-remove"),
                    ("flask-remove-outline", "flask-remove-outline"),
                    ("flask-round-bottom", "flask-round-bottom"),
                    ("flask-round-bottom-empty", "flask-round-bottom-empty"),
                    ("flask-round-bottom-empty-outline", "flask-round-bottom-empty-outline"),
                    ("flask-round-bottom-outline", "flask-round-bottom-outline"),
                    ("flattr", "flattr"),
                    ("fleur-de-lis", "fleur-de-lis"),
                    ("flickr", "flickr"),
                    ("flickr-after", "flickr-after"),
                    ("flickr-before", "flickr-before"),
                    ("flip-horizontal", "flip-horizontal"),
                    ("flip-to-back", "flip-to-back"),
                    ("flip-to-front", "flip-to-front"),
                    ("flip-vertical", "flip-vertical"),
                    ("floor-1", "floor-1"),
                    ("floor-2", "floor-2"),
                    ("floor-3", "floor-3"),
                    ("floor-a", "floor-a"),
                    ("floor-b", "floor-b"),
                    ("floor-g", "floor-g"),
                    ("floor-l", "floor-l"),
                    ("floor-lamp", "floor-lamp"),
                    ("floor-lamp-dual", "floor-lamp-dual"),
                    ("floor-lamp-dual-outline", "floor-lamp-dual-outline"),
                    ("floor-lamp-outline", "floor-lamp-outline"),
                    ("floor-lamp-torchiere", "floor-lamp-torchiere"),
                    ("floor-lamp-torchiere-outline", "floor-lamp-torchiere-outline"),
                    ("floor-lamp-torchiere-variant", "floor-lamp-torchiere-variant"),
                    (
                        "floor-lamp-torchiere-variant-outline",
                        "floor-lamp-torchiere-variant-outline",
                    ),
                    ("floor-plan", "floor-plan"),
                    ("floppy", "floppy"),
                    ("floppy-variant", "floppy-variant"),
                    ("flower", "flower"),
                    ("flower-outline", "flower-outline"),
                    ("flower-pollen", "flower-pollen"),
                    ("flower-pollen-outline", "flower-pollen-outline"),
                    ("flower-poppy", "flower-poppy"),
                    ("flower-tulip", "flower-tulip"),
                    ("flower-tulip-outline", "flower-tulip-outline"),
                    ("focus-auto", "focus-auto"),
                    ("focus-field", "focus-field"),
                    ("focus-field-horizontal", "focus-field-horizontal"),
                    ("focus-field-vertical", "focus-field-vertical"),
                    ("folder", "folder"),
                    ("folder-account", "folder-account"),
                    ("folder-account-outline", "folder-account-outline"),
                    ("folder-alert", "folder-alert"),
                    ("folder-alert-outline", "folder-alert-outline"),
                    ("folder-arrow-down", "folder-arrow-down"),
                    ("folder-arrow-down-outline", "folder-arrow-down-outline"),
                    ("folder-arrow-left", "folder-arrow-left"),
                    ("folder-arrow-left-outline", "folder-arrow-left-outline"),
                    ("folder-arrow-left-right", "folder-arrow-left-right"),
                    ("folder-arrow-left-right-outline", "folder-arrow-left-right-outline"),
                    ("folder-arrow-right", "folder-arrow-right"),
                    ("folder-arrow-right-outline", "folder-arrow-right-outline"),
                    ("folder-arrow-up", "folder-arrow-up"),
                    ("folder-arrow-up-down", "folder-arrow-up-down"),
                    ("folder-arrow-up-down-outline", "folder-arrow-up-down-outline"),
                    ("folder-arrow-up-outline", "folder-arrow-up-outline"),
                    ("folder-cancel", "folder-cancel"),
                    ("folder-cancel-outline", "folder-cancel-outline"),
                    ("folder-check", "folder-check"),
                    ("folder-check-outline", "folder-check-outline"),
                    ("folder-clock", "folder-clock"),
                    ("folder-clock-outline", "folder-clock-outline"),
                    ("folder-cog", "folder-cog"),
                    ("folder-cog-outline", "folder-cog-outline"),
                    ("folder-download", "folder-download"),
                    ("folder-download-outline", "folder-download-outline"),
                    ("folder-edit", "folder-edit"),
                    ("folder-edit-outline", "folder-edit-outline"),
                    ("folder-eye", "folder-eye"),
                    ("folder-eye-outline", "folder-eye-outline"),
                    ("folder-file", "folder-file"),
                    ("folder-file-outline", "folder-file-outline"),
                    ("folder-google-drive", "folder-google-drive"),
                    ("folder-heart", "folder-heart"),
                    ("folder-heart-outline", "folder-heart-outline"),
                    ("folder-hidden", "folder-hidden"),
                    ("folder-home", "folder-home"),
                    ("folder-home-outline", "folder-home-outline"),
                    ("folder-image", "folder-image"),
                    ("folder-information", "folder-information"),
                    ("folder-information-outline", "folder-information-outline"),
                    ("folder-key", "folder-key"),
                    ("folder-key-network", "folder-key-network"),
                    ("folder-key-network-outline", "folder-key-network-outline"),
                    ("folder-key-outline", "folder-key-outline"),
                    ("folder-lock", "folder-lock"),
                    ("folder-lock-open", "folder-lock-open"),
                    ("folder-lock-open-outline", "folder-lock-open-outline"),
                    ("folder-lock-outline", "folder-lock-outline"),
                    ("folder-marker", "folder-marker"),
                    ("folder-marker-outline", "folder-marker-outline"),
                    ("folder-minus", "folder-minus"),
                    ("folder-minus-outline", "folder-minus-outline"),
                    ("folder-move", "folder-move"),
                    ("folder-move-outline", "folder-move-outline"),
                    ("folder-multiple", "folder-multiple"),
                    ("folder-multiple-image", "folder-multiple-image"),
                    ("folder-multiple-outline", "folder-multiple-outline"),
                    ("folder-multiple-plus", "folder-multiple-plus"),
                    ("folder-multiple-plus-outline", "folder-multiple-plus-outline"),
                    ("folder-music", "folder-music"),
                    ("folder-music-outline", "folder-music-outline"),
                    ("folder-network", "folder-network"),
                    ("folder-network-outline", "folder-network-outline"),
                    ("folder-off", "folder-off"),
                    ("folder-off-outline", "folder-off-outline"),
                    ("folder-open", "folder-open"),
                    ("folder-open-outline", "folder-open-outline"),
                    ("folder-outline", "folder-outline"),
                    ("folder-outline-lock", "folder-outline-lock"),
                    ("folder-play", "folder-play"),
                    ("folder-play-outline", "folder-play-outline"),
                    ("folder-plus", "folder-plus"),
                    ("folder-plus-outline", "folder-plus-outline"),
                    ("folder-pound", "folder-pound"),
                    ("folder-pound-outline", "folder-pound-outline"),
                    ("folder-question", "folder-question"),
                    ("folder-question-outline", "folder-question-outline"),
                    ("folder-refresh", "folder-refresh"),
                    ("folder-refresh-outline", "folder-refresh-outline"),
                    ("folder-remove", "folder-remove"),
                    ("folder-remove-outline", "folder-remove-outline"),
                    ("folder-search", "folder-search"),
                    ("folder-search-outline", "folder-search-outline"),
                    ("folder-settings", "folder-settings"),
                    ("folder-settings-outline", "folder-settings-outline"),
                    ("folder-star", "folder-star"),
                    ("folder-star-multiple", "folder-star-multiple"),
                    ("folder-star-multiple-outline", "folder-star-multiple-outline"),
                    ("folder-star-outline", "folder-star-outline"),
                    ("folder-swap", "folder-swap"),
                    ("folder-swap-outline", "folder-swap-outline"),
                    ("folder-sync", "folder-sync"),
                    ("folder-sync-outline", "folder-sync-outline"),
                    ("folder-table", "folder-table"),
                    ("folder-table-outline", "folder-table-outline"),
                    ("folder-text", "folder-text"),
                    ("folder-text-outline", "folder-text-outline"),
                    ("folder-upload", "folder-upload"),
                    ("folder-upload-outline", "folder-upload-outline"),
                    ("folder-wrench", "folder-wrench"),
                    ("folder-wrench-outline", "folder-wrench-outline"),
                    ("folder-zip", "folder-zip"),
                    ("folder-zip-outline", "folder-zip-outline"),
                    ("font-awesome", "font-awesome"),
                    ("food", "food"),
                    ("food-apple", "food-apple"),
                    ("food-apple-outline", "food-apple-outline"),
                    ("food-croissant", "food-croissant"),
                    ("food-drumstick", "food-drumstick"),
                    ("food-drumstick-off", "food-drumstick-off"),
                    ("food-drumstick-off-outline", "food-drumstick-off-outline"),
                    ("food-drumstick-outline", "food-drumstick-outline"),
                    ("food-fork-drink", "food-fork-drink"),
                    ("food-halal", "food-halal"),
                    ("food-hot-dog", "food-hot-dog"),
                    ("food-kosher", "food-kosher"),
                    ("food-off", "food-off"),
                    ("food-off-outline", "food-off-outline"),
                    ("food-outline", "food-outline"),
                    ("food-steak", "food-steak"),
                    ("food-steak-off", "food-steak-off"),
                    ("food-takeout-box", "food-takeout-box"),
                    ("food-takeout-box-outline", "food-takeout-box-outline"),
                    ("food-turkey", "food-turkey"),
                    ("food-variant", "food-variant"),
                    ("food-variant-off", "food-variant-off"),
                    ("foot-print", "foot-print"),
                    ("football", "football"),
                    ("football-australian", "football-australian"),
                    ("football-helmet", "football-helmet"),
                    ("footer", "footer"),
                    ("forest", "forest"),
                    ("forest-outline", "forest-outline"),
                    ("forklift", "forklift"),
                    ("form-dropdown", "form-dropdown"),
                    ("form-select", "form-select"),
                    ("form-textarea", "form-textarea"),
                    ("form-textbox", "form-textbox"),
                    ("form-textbox-lock", "form-textbox-lock"),
                    ("form-textbox-password", "form-textbox-password"),
                    ("format-align-bottom", "format-align-bottom"),
                    ("format-align-center", "format-align-center"),
                    ("format-align-justify", "format-align-justify"),
                    ("format-align-left", "format-align-left"),
                    ("format-align-middle", "format-align-middle"),
                    ("format-align-right", "format-align-right"),
                    ("format-align-top", "format-align-top"),
                    ("format-annotation-minus", "format-annotation-minus"),
                    ("format-annotation-plus", "format-annotation-plus"),
                    ("format-bold", "format-bold"),
                    ("format-clear", "format-clear"),
                    ("format-color", "format-color"),
                    ("format-color-fill", "format-color-fill"),
                    ("format-color-highlight", "format-color-highlight"),
                    ("format-color-marker-cancel", "format-color-marker-cancel"),
                    ("format-color-text", "format-color-text"),
                    ("format-columns", "format-columns"),
                    ("format-float-center", "format-float-center"),
                    ("format-float-left", "format-float-left"),
                    ("format-float-none", "format-float-none"),
                    ("format-float-right", "format-float-right"),
                    ("format-font", "format-font"),
                    ("format-font-size-decrease", "format-font-size-decrease"),
                    ("format-font-size-increase", "format-font-size-increase"),
                    ("format-header-1", "format-header-1"),
                    ("format-header-2", "format-header-2"),
                    ("format-header-3", "format-header-3"),
                    ("format-header-4", "format-header-4"),
                    ("format-header-5", "format-header-5"),
                    ("format-header-6", "format-header-6"),
                    ("format-header-decrease", "format-header-decrease"),
                    ("format-header-down", "format-header-down"),
                    ("format-header-equal", "format-header-equal"),
                    ("format-header-increase", "format-header-increase"),
                    ("format-header-pound", "format-header-pound"),
                    ("format-header-up", "format-header-up"),
                    ("format-horizontal-align-center", "format-horizontal-align-center"),
                    ("format-horizontal-align-left", "format-horizontal-align-left"),
                    ("format-horizontal-align-right", "format-horizontal-align-right"),
                    ("format-indent-decrease", "format-indent-decrease"),
                    ("format-indent-increase", "format-indent-increase"),
                    ("format-italic", "format-italic"),
                    ("format-letter-case", "format-letter-case"),
                    ("format-letter-case-lower", "format-letter-case-lower"),
                    ("format-letter-case-upper", "format-letter-case-upper"),
                    ("format-letter-ends-with", "format-letter-ends-with"),
                    ("format-letter-matches", "format-letter-matches"),
                    ("format-letter-spacing", "format-letter-spacing"),
                    ("format-letter-spacing-variant", "format-letter-spacing-variant"),
                    ("format-letter-starts-with", "format-letter-starts-with"),
                    ("format-line-height", "format-line-height"),
                    ("format-line-spacing", "format-line-spacing"),
                    ("format-line-style", "format-line-style"),
                    ("format-line-weight", "format-line-weight"),
                    ("format-list-bulleted", "format-list-bulleted"),
                    ("format-list-bulleted-square", "format-list-bulleted-square"),
                    ("format-list-bulleted-triangle", "format-list-bulleted-triangle"),
                    ("format-list-bulleted-type", "format-list-bulleted-type"),
                    ("format-list-checkbox", "format-list-checkbox"),
                    ("format-list-checks", "format-list-checks"),
                    ("format-list-group", "format-list-group"),
                    ("format-list-group-plus", "format-list-group-plus"),
                    ("format-list-numbered", "format-list-numbered"),
                    ("format-list-numbered-rtl", "format-list-numbered-rtl"),
                    ("format-list-text", "format-list-text"),
                    ("format-list-triangle", "format-list-triangle"),
                    ("format-overline", "format-overline"),
                    ("format-page-break", "format-page-break"),
                    ("format-page-split", "format-page-split"),
                    ("format-paint", "format-paint"),
                    ("format-paragraph", "format-paragraph"),
                    ("format-paragraph-spacing", "format-paragraph-spacing"),
                    ("format-pilcrow", "format-pilcrow"),
                    ("format-pilcrow-arrow-left", "format-pilcrow-arrow-left"),
                    ("format-pilcrow-arrow-right", "format-pilcrow-arrow-right"),
                    ("format-quote-close", "format-quote-close"),
                    ("format-quote-close-outline", "format-quote-close-outline"),
                    ("format-quote-open", "format-quote-open"),
                    ("format-quote-open-outline", "format-quote-open-outline"),
                    ("format-rotate-90", "format-rotate-90"),
                    ("format-section", "format-section"),
                    ("format-size", "format-size"),
                    ("format-strikethrough", "format-strikethrough"),
                    ("format-strikethrough-variant", "format-strikethrough-variant"),
                    ("format-subscript", "format-subscript"),
                    ("format-superscript", "format-superscript"),
                    ("format-text", "format-text"),
                    ("format-text-rotation-angle-down", "format-text-rotation-angle-down"),
                    ("format-text-rotation-angle-up", "format-text-rotation-angle-up"),
                    ("format-text-rotation-down", "format-text-rotation-down"),
                    ("format-text-rotation-down-vertical", "format-text-rotation-down-vertical"),
                    ("format-text-rotation-none", "format-text-rotation-none"),
                    ("format-text-rotation-up", "format-text-rotation-up"),
                    ("format-text-rotation-vertical", "format-text-rotation-vertical"),
                    ("format-text-variant", "format-text-variant"),
                    ("format-text-variant-outline", "format-text-variant-outline"),
                    ("format-text-wrapping-clip", "format-text-wrapping-clip"),
                    ("format-text-wrapping-overflow", "format-text-wrapping-overflow"),
                    ("format-text-wrapping-wrap", "format-text-wrapping-wrap"),
                    ("format-textbox", "format-textbox"),
                    ("format-title", "format-title"),
                    ("format-underline", "format-underline"),
                    ("format-underline-wavy", "format-underline-wavy"),
                    ("format-vertical-align-bottom", "format-vertical-align-bottom"),
                    ("format-vertical-align-center", "format-vertical-align-center"),
                    ("format-vertical-align-top", "format-vertical-align-top"),
                    ("format-wrap-inline", "format-wrap-inline"),
                    ("format-wrap-square", "format-wrap-square"),
                    ("format-wrap-tight", "format-wrap-tight"),
                    ("format-wrap-top-bottom", "format-wrap-top-bottom"),
                    ("forum", "forum"),
                    ("forum-minus", "forum-minus"),
                    ("forum-minus-outline", "forum-minus-outline"),
                    ("forum-outline", "forum-outline"),
                    ("forum-plus", "forum-plus"),
                    ("forum-plus-outline", "forum-plus-outline"),
                    ("forum-remove", "forum-remove"),
                    ("forum-remove-outline", "forum-remove-outline"),
                    ("forward", "forward"),
                    ("forwardburger", "forwardburger"),
                    ("fountain", "fountain"),
                    ("fountain-pen", "fountain-pen"),
                    ("fountain-pen-tip", "fountain-pen-tip"),
                    ("foursquare", "foursquare"),
                    ("fraction-one-half", "fraction-one-half"),
                    ("freebsd", "freebsd"),
                    ("french-fries", "french-fries"),
                    ("frequently-asked-questions", "frequently-asked-questions"),
                    ("fridge", "fridge"),
                    ("fridge-alert", "fridge-alert"),
                    ("fridge-alert-outline", "fridge-alert-outline"),
                    ("fridge-bottom", "fridge-bottom"),
                    ("fridge-industrial", "fridge-industrial"),
                    ("fridge-industrial-alert", "fridge-industrial-alert"),
                    ("fridge-industrial-alert-outline", "fridge-industrial-alert-outline"),
                    ("fridge-industrial-off", "fridge-industrial-off"),
                    ("fridge-industrial-off-outline", "fridge-industrial-off-outline"),
                    ("fridge-industrial-outline", "fridge-industrial-outline"),
                    ("fridge-off", "fridge-off"),
                    ("fridge-off-outline", "fridge-off-outline"),
                    ("fridge-outline", "fridge-outline"),
                    ("fridge-top", "fridge-top"),
                    ("fridge-variant", "fridge-variant"),
                    ("fridge-variant-alert", "fridge-variant-alert"),
                    ("fridge-variant-alert-outline", "fridge-variant-alert-outline"),
                    ("fridge-variant-off", "fridge-variant-off"),
                    ("fridge-variant-off-outline", "fridge-variant-off-outline"),
                    ("fridge-variant-outline", "fridge-variant-outline"),
                    ("fruit-cherries", "fruit-cherries"),
                    ("fruit-cherries-off", "fruit-cherries-off"),
                    ("fruit-citrus", "fruit-citrus"),
                    ("fruit-citrus-off", "fruit-citrus-off"),
                    ("fruit-grapes", "fruit-grapes"),
                    ("fruit-grapes-outline", "fruit-grapes-outline"),
                    ("fruit-pear", "fruit-pear"),
                    ("fruit-pineapple", "fruit-pineapple"),
                    ("fruit-watermelon", "fruit-watermelon"),
                    ("fuel", "fuel"),
                    ("fuel-cell", "fuel-cell"),
                    ("fullscreen", "fullscreen"),
                    ("fullscreen-exit", "fullscreen-exit"),
                    ("function", "function"),
                    ("function-variant", "function-variant"),
                    ("furigana-horizontal", "furigana-horizontal"),
                    ("furigana-vertical", "furigana-vertical"),
                    ("fuse", "fuse"),
                    ("fuse-alert", "fuse-alert"),
                    ("fuse-blade", "fuse-blade"),
                    ("fuse-off", "fuse-off"),
                    ("gamepad", "gamepad"),
                    ("gamepad-circle", "gamepad-circle"),
                    ("gamepad-circle-down", "gamepad-circle-down"),
                    ("gamepad-circle-left", "gamepad-circle-left"),
                    ("gamepad-circle-outline", "gamepad-circle-outline"),
                    ("gamepad-circle-right", "gamepad-circle-right"),
                    ("gamepad-circle-up", "gamepad-circle-up"),
                    ("gamepad-down", "gamepad-down"),
                    ("gamepad-left", "gamepad-left"),
                    ("gamepad-outline", "gamepad-outline"),
                    ("gamepad-right", "gamepad-right"),
                    ("gamepad-round", "gamepad-round"),
                    ("gamepad-round-down", "gamepad-round-down"),
                    ("gamepad-round-left", "gamepad-round-left"),
                    ("gamepad-round-outline", "gamepad-round-outline"),
                    ("gamepad-round-right", "gamepad-round-right"),
                    ("gamepad-round-up", "gamepad-round-up"),
                    ("gamepad-square", "gamepad-square"),
                    ("gamepad-square-outline", "gamepad-square-outline"),
                    ("gamepad-up", "gamepad-up"),
                    ("gamepad-variant", "gamepad-variant"),
                    ("gamepad-variant-outline", "gamepad-variant-outline"),
                    ("gamma", "gamma"),
                    ("gantry-crane", "gantry-crane"),
                    ("garage", "garage"),
                    ("garage-alert", "garage-alert"),
                    ("garage-alert-variant", "garage-alert-variant"),
                    ("garage-lock", "garage-lock"),
                    ("garage-open", "garage-open"),
                    ("garage-open-variant", "garage-open-variant"),
                    ("garage-variant", "garage-variant"),
                    ("garage-variant-lock", "garage-variant-lock"),
                    ("gas-burner", "gas-burner"),
                    ("gas-cylinder", "gas-cylinder"),
                    ("gas-station", "gas-station"),
                    ("gas-station-off", "gas-station-off"),
                    ("gas-station-off-outline", "gas-station-off-outline"),
                    ("gas-station-outline", "gas-station-outline"),
                    ("gate", "gate"),
                    ("gate-alert", "gate-alert"),
                    ("gate-and", "gate-and"),
                    ("gate-arrow-left", "gate-arrow-left"),
                    ("gate-arrow-right", "gate-arrow-right"),
                    ("gate-buffer", "gate-buffer"),
                    ("gate-nand", "gate-nand"),
                    ("gate-nor", "gate-nor"),
                    ("gate-not", "gate-not"),
                    ("gate-open", "gate-open"),
                    ("gate-or", "gate-or"),
                    ("gate-xnor", "gate-xnor"),
                    ("gate-xor", "gate-xor"),
                    ("gatsby", "gatsby"),
                    ("gauge", "gauge"),
                    ("gauge-empty", "gauge-empty"),
                    ("gauge-full", "gauge-full"),
                    ("gauge-low", "gauge-low"),
                    ("gavel", "gavel"),
                    ("gender-female", "gender-female"),
                    ("gender-male", "gender-male"),
                    ("gender-male-female", "gender-male-female"),
                    ("gender-male-female-variant", "gender-male-female-variant"),
                    ("gender-non-binary", "gender-non-binary"),
                    ("gender-transgender", "gender-transgender"),
                    ("generator-mobile", "generator-mobile"),
                    ("generator-portable", "generator-portable"),
                    ("generator-stationary", "generator-stationary"),
                    ("gentoo", "gentoo"),
                    ("gesture", "gesture"),
                    ("gesture-double-tap", "gesture-double-tap"),
                    ("gesture-pinch", "gesture-pinch"),
                    ("gesture-spread", "gesture-spread"),
                    ("gesture-swipe", "gesture-swipe"),
                    ("gesture-swipe-down", "gesture-swipe-down"),
                    ("gesture-swipe-horizontal", "gesture-swipe-horizontal"),
                    ("gesture-swipe-left", "gesture-swipe-left"),
                    ("gesture-swipe-right", "gesture-swipe-right"),
                    ("gesture-swipe-up", "gesture-swipe-up"),
                    ("gesture-swipe-vertical", "gesture-swipe-vertical"),
                    ("gesture-tap", "gesture-tap"),
                    ("gesture-tap-box", "gesture-tap-box"),
                    ("gesture-tap-button", "gesture-tap-button"),
                    ("gesture-tap-hold", "gesture-tap-hold"),
                    ("gesture-two-double-tap", "gesture-two-double-tap"),
                    ("gesture-two-tap", "gesture-two-tap"),
                    ("ghost", "ghost"),
                    ("ghost-off", "ghost-off"),
                    ("ghost-off-outline", "ghost-off-outline"),
                    ("ghost-outline", "ghost-outline"),
                    ("gif", "gif"),
                    ("gift", "gift"),
                    ("gift-off", "gift-off"),
                    ("gift-off-outline", "gift-off-outline"),
                    ("gift-open", "gift-open"),
                    ("gift-open-outline", "gift-open-outline"),
                    ("gift-outline", "gift-outline"),
                    ("git", "git"),
                    ("github", "github"),
                    ("github-box", "github-box"),
                    ("github-face", "github-face"),
                    ("gitlab", "gitlab"),
                    ("glass-cocktail", "glass-cocktail"),
                    ("glass-cocktail-off", "glass-cocktail-off"),
                    ("glass-flute", "glass-flute"),
                    ("glass-fragile", "glass-fragile"),
                    ("glass-mug", "glass-mug"),
                    ("glass-mug-off", "glass-mug-off"),
                    ("glass-mug-variant", "glass-mug-variant"),
                    ("glass-mug-variant-off", "glass-mug-variant-off"),
                    ("glass-pint-outline", "glass-pint-outline"),
                    ("glass-stange", "glass-stange"),
                    ("glass-tulip", "glass-tulip"),
                    ("glass-wine", "glass-wine"),
                    ("glassdoor", "glassdoor"),
                    ("glasses", "glasses"),
                    ("globe-light", "globe-light"),
                    ("globe-light-outline", "globe-light-outline"),
                    ("globe-model", "globe-model"),
                    ("gmail", "gmail"),
                    ("gnome", "gnome"),
                    ("go-kart", "go-kart"),
                    ("go-kart-track", "go-kart-track"),
                    ("gog", "gog"),
                    ("gold", "gold"),
                    ("golf", "golf"),
                    ("golf-cart", "golf-cart"),
                    ("golf-tee", "golf-tee"),
                    ("gondola", "gondola"),
                    ("goodreads", "goodreads"),
                    ("google", "google"),
                    ("google-ads", "google-ads"),
                    ("google-allo", "google-allo"),
                    ("google-analytics", "google-analytics"),
                    ("google-assistant", "google-assistant"),
                    ("google-cardboard", "google-cardboard"),
                    ("google-chrome", "google-chrome"),
                    ("google-circles", "google-circles"),
                    ("google-circles-communities", "google-circles-communities"),
                    ("google-circles-extended", "google-circles-extended"),
                    ("google-circles-group", "google-circles-group"),
                    ("google-classroom", "google-classroom"),
                    ("google-cloud", "google-cloud"),
                    ("google-downasaur", "google-downasaur"),
                    ("google-drive", "google-drive"),
                    ("google-earth", "google-earth"),
                    ("google-fit", "google-fit"),
                    ("google-glass", "google-glass"),
                    ("google-hangouts", "google-hangouts"),
                    ("google-home", "google-home"),
                    ("google-keep", "google-keep"),
                    ("google-lens", "google-lens"),
                    ("google-maps", "google-maps"),
                    ("google-my-business", "google-my-business"),
                    ("google-nearby", "google-nearby"),
                    ("google-pages", "google-pages"),
                    ("google-photos", "google-photos"),
                    ("google-physical-web", "google-physical-web"),
                    ("google-play", "google-play"),
                    ("google-plus", "google-plus"),
                    ("google-plus-box", "google-plus-box"),
                    ("google-podcast", "google-podcast"),
                    ("google-spreadsheet", "google-spreadsheet"),
                    ("google-street-view", "google-street-view"),
                    ("google-translate", "google-translate"),
                    ("google-wallet", "google-wallet"),
                    ("gradient-horizontal", "gradient-horizontal"),
                    ("gradient-vertical", "gradient-vertical"),
                    ("grain", "grain"),
                    ("graph", "graph"),
                    ("graph-outline", "graph-outline"),
                    ("graphql", "graphql"),
                    ("grass", "grass"),
                    ("grave-stone", "grave-stone"),
                    ("grease-pencil", "grease-pencil"),
                    ("greater-than", "greater-than"),
                    ("greater-than-or-equal", "greater-than-or-equal"),
                    ("greenhouse", "greenhouse"),
                    ("grid", "grid"),
                    ("grid-large", "grid-large"),
                    ("grid-off", "grid-off"),
                    ("grill", "grill"),
                    ("grill-outline", "grill-outline"),
                    ("group", "group"),
                    ("guitar-acoustic", "guitar-acoustic"),
                    ("guitar-electric", "guitar-electric"),
                    ("guitar-pick", "guitar-pick"),
                    ("guitar-pick-outline", "guitar-pick-outline"),
                    ("guy-fawkes-mask", "guy-fawkes-mask"),
                    ("gymnastics", "gymnastics"),
                    ("hail", "hail"),
                    ("hair-dryer", "hair-dryer"),
                    ("hair-dryer-outline", "hair-dryer-outline"),
                    ("halloween", "halloween"),
                    ("hamburger", "hamburger"),
                    ("hamburger-check", "hamburger-check"),
                    ("hamburger-minus", "hamburger-minus"),
                    ("hamburger-off", "hamburger-off"),
                    ("hamburger-plus", "hamburger-plus"),
                    ("hamburger-remove", "hamburger-remove"),
                    ("hammer", "hammer"),
                    ("hammer-screwdriver", "hammer-screwdriver"),
                    ("hammer-sickle", "hammer-sickle"),
                    ("hammer-wrench", "hammer-wrench"),
                    ("hand-back-left", "hand-back-left"),
                    ("hand-back-left-off", "hand-back-left-off"),
                    ("hand-back-left-off-outline", "hand-back-left-off-outline"),
                    ("hand-back-left-outline", "hand-back-left-outline"),
                    ("hand-back-right", "hand-back-right"),
                    ("hand-back-right-off", "hand-back-right-off"),
                    ("hand-back-right-off-outline", "hand-back-right-off-outline"),
                    ("hand-back-right-outline", "hand-back-right-outline"),
                    ("hand-clap", "hand-clap"),
                    ("hand-clap-off", "hand-clap-off"),
                    ("hand-coin", "hand-coin"),
                    ("hand-coin-outline", "hand-coin-outline"),
                    ("hand-cycle", "hand-cycle"),
                    ("hand-extended", "hand-extended"),
                    ("hand-extended-outline", "hand-extended-outline"),
                    ("hand-front-left", "hand-front-left"),
                    ("hand-front-left-outline", "hand-front-left-outline"),
                    ("hand-front-right", "hand-front-right"),
                    ("hand-front-right-outline", "hand-front-right-outline"),
                    ("hand-heart", "hand-heart"),
                    ("hand-heart-outline", "hand-heart-outline"),
                    ("hand-left", "hand-left"),
                    ("hand-okay", "hand-okay"),
                    ("hand-peace", "hand-peace"),
                    ("hand-peace-variant", "hand-peace-variant"),
                    ("hand-pointing-down", "hand-pointing-down"),
                    ("hand-pointing-left", "hand-pointing-left"),
                    ("hand-pointing-right", "hand-pointing-right"),
                    ("hand-pointing-up", "hand-pointing-up"),
                    ("hand-right", "hand-right"),
                    ("hand-saw", "hand-saw"),
                    ("hand-wash", "hand-wash"),
                    ("hand-wash-outline", "hand-wash-outline"),
                    ("hand-water", "hand-water"),
                    ("hand-wave", "hand-wave"),
                    ("hand-wave-outline", "hand-wave-outline"),
                    ("handball", "handball"),
                    ("handcuffs", "handcuffs"),
                    ("hands-pray", "hands-pray"),
                    ("handshake", "handshake"),
                    ("handshake-outline", "handshake-outline"),
                    ("hanger", "hanger"),
                    ("hangouts", "hangouts"),
                    ("hard-hat", "hard-hat"),
                    ("harddisk", "harddisk"),
                    ("harddisk-plus", "harddisk-plus"),
                    ("harddisk-remove", "harddisk-remove"),
                    ("hat-fedora", "hat-fedora"),
                    ("hazard-lights", "hazard-lights"),
                    ("hdmi-port", "hdmi-port"),
                    ("hdr", "hdr"),
                    ("hdr-off", "hdr-off"),
                    ("head", "head"),
                    ("head-alert", "head-alert"),
                    ("head-alert-outline", "head-alert-outline"),
                    ("head-check", "head-check"),
                    ("head-check-outline", "head-check-outline"),
                    ("head-cog", "head-cog"),
                    ("head-cog-outline", "head-cog-outline"),
                    ("head-dots-horizontal", "head-dots-horizontal"),
                    ("head-dots-horizontal-outline", "head-dots-horizontal-outline"),
                    ("head-flash", "head-flash"),
                    ("head-flash-outline", "head-flash-outline"),
                    ("head-heart", "head-heart"),
                    ("head-heart-outline", "head-heart-outline"),
                    ("head-lightbulb", "head-lightbulb"),
                    ("head-lightbulb-outline", "head-lightbulb-outline"),
                    ("head-minus", "head-minus"),
                    ("head-minus-outline", "head-minus-outline"),
                    ("head-outline", "head-outline"),
                    ("head-plus", "head-plus"),
                    ("head-plus-outline", "head-plus-outline"),
                    ("head-question", "head-question"),
                    ("head-question-outline", "head-question-outline"),
                    ("head-remove", "head-remove"),
                    ("head-remove-outline", "head-remove-outline"),
                    ("head-snowflake", "head-snowflake"),
                    ("head-snowflake-outline", "head-snowflake-outline"),
                    ("head-sync", "head-sync"),
                    ("head-sync-outline", "head-sync-outline"),
                    ("headphones", "headphones"),
                    ("headphones-bluetooth", "headphones-bluetooth"),
                    ("headphones-box", "headphones-box"),
                    ("headphones-off", "headphones-off"),
                    ("headphones-settings", "headphones-settings"),
                    ("headset", "headset"),
                    ("headset-dock", "headset-dock"),
                    ("headset-off", "headset-off"),
                    ("heart", "heart"),
                    ("heart-box", "heart-box"),
                    ("heart-box-outline", "heart-box-outline"),
                    ("heart-broken", "heart-broken"),
                    ("heart-broken-outline", "heart-broken-outline"),
                    ("heart-circle", "heart-circle"),
                    ("heart-circle-outline", "heart-circle-outline"),
                    ("heart-cog", "heart-cog"),
                    ("heart-cog-outline", "heart-cog-outline"),
                    ("heart-flash", "heart-flash"),
                    ("heart-half", "heart-half"),
                    ("heart-half-full", "heart-half-full"),
                    ("heart-half-outline", "heart-half-outline"),
                    ("heart-minus", "heart-minus"),
                    ("heart-minus-outline", "heart-minus-outline"),
                    ("heart-multiple", "heart-multiple"),
                    ("heart-multiple-outline", "heart-multiple-outline"),
                    ("heart-off", "heart-off"),
                    ("heart-off-outline", "heart-off-outline"),
                    ("heart-outline", "heart-outline"),
                    ("heart-plus", "heart-plus"),
                    ("heart-plus-outline", "heart-plus-outline"),
                    ("heart-pulse", "heart-pulse"),
                    ("heart-remove", "heart-remove"),
                    ("heart-remove-outline", "heart-remove-outline"),
                    ("heart-search", "heart-search"),
                    ("heart-settings", "heart-settings"),
                    ("heart-settings-outline", "heart-settings-outline"),
                    ("heat-pump", "heat-pump"),
                    ("heat-pump-outline", "heat-pump-outline"),
                    ("heat-wave", "heat-wave"),
                    ("heating-coil", "heating-coil"),
                    ("helicopter", "helicopter"),
                    ("help", "help"),
                    ("help-box", "help-box"),
                    ("help-box-multiple", "help-box-multiple"),
                    ("help-box-multiple-outline", "help-box-multiple-outline"),
                    ("help-box-outline", "help-box-outline"),
                    ("help-circle", "help-circle"),
                    ("help-circle-outline", "help-circle-outline"),
                    ("help-network", "help-network"),
                    ("help-network-outline", "help-network-outline"),
                    ("help-rhombus", "help-rhombus"),
                    ("help-rhombus-outline", "help-rhombus-outline"),
                    ("hexadecimal", "hexadecimal"),
                    ("hexagon", "hexagon"),
                    ("hexagon-multiple", "hexagon-multiple"),
                    ("hexagon-multiple-outline", "hexagon-multiple-outline"),
                    ("hexagon-outline", "hexagon-outline"),
                    ("hexagon-slice-1", "hexagon-slice-1"),
                    ("hexagon-slice-2", "hexagon-slice-2"),
                    ("hexagon-slice-3", "hexagon-slice-3"),
                    ("hexagon-slice-4", "hexagon-slice-4"),
                    ("hexagon-slice-5", "hexagon-slice-5"),
                    ("hexagon-slice-6", "hexagon-slice-6"),
                    ("hexagram", "hexagram"),
                    ("hexagram-outline", "hexagram-outline"),
                    ("high-definition", "high-definition"),
                    ("high-definition-box", "high-definition-box"),
                    ("highway", "highway"),
                    ("hiking", "hiking"),
                    ("history", "history"),
                    ("hockey-puck", "hockey-puck"),
                    ("hockey-sticks", "hockey-sticks"),
                    ("hololens", "hololens"),
                    ("home", "home"),
                    ("home-account", "home-account"),
                    ("home-alert", "home-alert"),
                    ("home-alert-outline", "home-alert-outline"),
                    ("home-analytics", "home-analytics"),
                    ("home-assistant", "home-assistant"),
                    ("home-automation", "home-automation"),
                    ("home-battery", "home-battery"),
                    ("home-battery-outline", "home-battery-outline"),
                    ("home-circle", "home-circle"),
                    ("home-circle-outline", "home-circle-outline"),
                    ("home-city", "home-city"),
                    ("home-city-outline", "home-city-outline"),
                    ("home-clock", "home-clock"),
                    ("home-clock-outline", "home-clock-outline"),
                    ("home-currency-usd", "home-currency-usd"),
                    ("home-edit", "home-edit"),
                    ("home-edit-outline", "home-edit-outline"),
                    ("home-export-outline", "home-export-outline"),
                    ("home-flood", "home-flood"),
                    ("home-floor-0", "home-floor-0"),
                    ("home-floor-1", "home-floor-1"),
                    ("home-floor-2", "home-floor-2"),
                    ("home-floor-3", "home-floor-3"),
                    ("home-floor-a", "home-floor-a"),
                    ("home-floor-b", "home-floor-b"),
                    ("home-floor-g", "home-floor-g"),
                    ("home-floor-l", "home-floor-l"),
                    ("home-floor-negative-1", "home-floor-negative-1"),
                    ("home-group", "home-group"),
                    ("home-group-minus", "home-group-minus"),
                    ("home-group-plus", "home-group-plus"),
                    ("home-group-remove", "home-group-remove"),
                    ("home-heart", "home-heart"),
                    ("home-import-outline", "home-import-outline"),
                    ("home-lightbulb", "home-lightbulb"),
                    ("home-lightbulb-outline", "home-lightbulb-outline"),
                    ("home-lightning-bolt", "home-lightning-bolt"),
                    ("home-lightning-bolt-outline", "home-lightning-bolt-outline"),
                    ("home-lock", "home-lock"),
                    ("home-lock-open", "home-lock-open"),
                    ("home-map-marker", "home-map-marker"),
                    ("home-minus", "home-minus"),
                    ("home-minus-outline", "home-minus-outline"),
                    ("home-modern", "home-modern"),
                    ("home-off", "home-off"),
                    ("home-off-outline", "home-off-outline"),
                    ("home-outline", "home-outline"),
                    ("home-percent", "home-percent"),
                    ("home-percent-outline", "home-percent-outline"),
                    ("home-plus", "home-plus"),
                    ("home-plus-outline", "home-plus-outline"),
                    ("home-remove", "home-remove"),
                    ("home-remove-outline", "home-remove-outline"),
                    ("home-roof", "home-roof"),
                    ("home-search", "home-search"),
                    ("home-search-outline", "home-search-outline"),
                    ("home-silo", "home-silo"),
                    ("home-silo-outline", "home-silo-outline"),
                    ("home-sound-in", "home-sound-in"),
                    ("home-sound-in-outline", "home-sound-in-outline"),
                    ("home-sound-out", "home-sound-out"),
                    ("home-sound-out-outline", "home-sound-out-outline"),
                    ("home-switch", "home-switch"),
                    ("home-switch-outline", "home-switch-outline"),
                    ("home-thermometer", "home-thermometer"),
                    ("home-thermometer-outline", "home-thermometer-outline"),
                    ("home-variant", "home-variant"),
                    ("home-variant-outline", "home-variant-outline"),
                    ("hook", "hook"),
                    ("hook-off", "hook-off"),
                    ("hoop-house", "hoop-house"),
                    ("hops", "hops"),
                    ("horizontal-rotate-clockwise", "horizontal-rotate-clockwise"),
                    ("horizontal-rotate-counterclockwise", "horizontal-rotate-counterclockwise"),
                    ("horse", "horse"),
                    ("horse-human", "horse-human"),
                    ("horse-variant", "horse-variant"),
                    ("horse-variant-fast", "horse-variant-fast"),
                    ("horseshoe", "horseshoe"),
                    ("hospital", "hospital"),
                    ("hospital-box", "hospital-box"),
                    ("hospital-box-outline", "hospital-box-outline"),
                    ("hospital-building", "hospital-building"),
                    ("hospital-marker", "hospital-marker"),
                    ("hot-tub", "hot-tub"),
                    ("hours-12", "hours-12"),
                    ("hours-24", "hours-24"),
                    ("houzz", "houzz"),
                    ("houzz-box", "houzz-box"),
                    ("hub", "hub"),
                    ("hub-outline", "hub-outline"),
                    ("hubspot", "hubspot"),
                    ("hulu", "hulu"),
                    ("human", "human"),
                    ("human-baby-changing-table", "human-baby-changing-table"),
                    ("human-cane", "human-cane"),
                    ("human-capacity-decrease", "human-capacity-decrease"),
                    ("human-capacity-increase", "human-capacity-increase"),
                    ("human-child", "human-child"),
                    ("human-dolly", "human-dolly"),
                    ("human-edit", "human-edit"),
                    ("human-female", "human-female"),
                    ("human-female-boy", "human-female-boy"),
                    ("human-female-dance", "human-female-dance"),
                    ("human-female-female", "human-female-female"),
                    ("human-female-female-child", "human-female-female-child"),
                    ("human-female-girl", "human-female-girl"),
                    ("human-greeting", "human-greeting"),
                    ("human-greeting-proximity", "human-greeting-proximity"),
                    ("human-greeting-variant", "human-greeting-variant"),
                    ("human-handsdown", "human-handsdown"),
                    ("human-handsup", "human-handsup"),
                    ("human-male", "human-male"),
                    ("human-male-board", "human-male-board"),
                    ("human-male-board-poll", "human-male-board-poll"),
                    ("human-male-boy", "human-male-boy"),
                    ("human-male-child", "human-male-child"),
                    ("human-male-female", "human-male-female"),
                    ("human-male-female-child", "human-male-female-child"),
                    ("human-male-girl", "human-male-girl"),
                    ("human-male-height", "human-male-height"),
                    ("human-male-height-variant", "human-male-height-variant"),
                    ("human-male-male", "human-male-male"),
                    ("human-male-male-child", "human-male-male-child"),
                    ("human-non-binary", "human-non-binary"),
                    ("human-pregnant", "human-pregnant"),
                    ("human-queue", "human-queue"),
                    ("human-scooter", "human-scooter"),
                    ("human-walker", "human-walker"),
                    ("human-wheelchair", "human-wheelchair"),
                    ("human-white-cane", "human-white-cane"),
                    ("humble-bundle", "humble-bundle"),
                    ("hurricane", "hurricane"),
                    ("hvac", "hvac"),
                    ("hvac-off", "hvac-off"),
                    ("hydraulic-oil-level", "hydraulic-oil-level"),
                    ("hydraulic-oil-temperature", "hydraulic-oil-temperature"),
                    ("hydro-power", "hydro-power"),
                    ("hydrogen-station", "hydrogen-station"),
                    ("ice-cream", "ice-cream"),
                    ("ice-cream-off", "ice-cream-off"),
                    ("ice-pop", "ice-pop"),
                    ("id-card", "id-card"),
                    ("identifier", "identifier"),
                    ("ideogram-cjk", "ideogram-cjk"),
                    ("ideogram-cjk-variant", "ideogram-cjk-variant"),
                    ("image", "image"),
                    ("image-album", "image-album"),
                    ("image-area", "image-area"),
                    ("image-area-close", "image-area-close"),
                    ("image-auto-adjust", "image-auto-adjust"),
                    ("image-broken", "image-broken"),
                    ("image-broken-variant", "image-broken-variant"),
                    ("image-check", "image-check"),
                    ("image-check-outline", "image-check-outline"),
                    ("image-edit", "image-edit"),
                    ("image-edit-outline", "image-edit-outline"),
                    ("image-filter-black-white", "image-filter-black-white"),
                    ("image-filter-center-focus", "image-filter-center-focus"),
                    ("image-filter-center-focus-strong", "image-filter-center-focus-strong"),
                    (
                        "image-filter-center-focus-strong-outline",
                        "image-filter-center-focus-strong-outline",
                    ),
                    ("image-filter-center-focus-weak", "image-filter-center-focus-weak"),
                    ("image-filter-drama", "image-filter-drama"),
                    ("image-filter-drama-outline", "image-filter-drama-outline"),
                    ("image-filter-frames", "image-filter-frames"),
                    ("image-filter-hdr", "image-filter-hdr"),
                    ("image-filter-hdr-outline", "image-filter-hdr-outline"),
                    ("image-filter-none", "image-filter-none"),
                    ("image-filter-tilt-shift", "image-filter-tilt-shift"),
                    ("image-filter-vintage", "image-filter-vintage"),
                    ("image-frame", "image-frame"),
                    ("image-lock", "image-lock"),
                    ("image-lock-outline", "image-lock-outline"),
                    ("image-marker", "image-marker"),
                    ("image-marker-outline", "image-marker-outline"),
                    ("image-minus", "image-minus"),
                    ("image-minus-outline", "image-minus-outline"),
                    ("image-move", "image-move"),
                    ("image-multiple", "image-multiple"),
                    ("image-multiple-outline", "image-multiple-outline"),
                    ("image-off", "image-off"),
                    ("image-off-outline", "image-off-outline"),
                    ("image-outline", "image-outline"),
                    ("image-plus", "image-plus"),
                    ("image-plus-outline", "image-plus-outline"),
                    ("image-refresh", "image-refresh"),
                    ("image-refresh-outline", "image-refresh-outline"),
                    ("image-remove", "image-remove"),
                    ("image-remove-outline", "image-remove-outline"),
                    ("image-search", "image-search"),
                    ("image-search-outline", "image-search-outline"),
                    ("image-size-select-actual", "image-size-select-actual"),
                    ("image-size-select-large", "image-size-select-large"),
                    ("image-size-select-small", "image-size-select-small"),
                    ("image-sync", "image-sync"),
                    ("image-sync-outline", "image-sync-outline"),
                    ("image-text", "image-text"),
                    ("import", "import"),
                    ("inbox", "inbox"),
                    ("inbox-arrow-down", "inbox-arrow-down"),
                    ("inbox-arrow-down-outline", "inbox-arrow-down-outline"),
                    ("inbox-arrow-up", "inbox-arrow-up"),
                    ("inbox-arrow-up-outline", "inbox-arrow-up-outline"),
                    ("inbox-full", "inbox-full"),
                    ("inbox-full-outline", "inbox-full-outline"),
                    ("inbox-multiple", "inbox-multiple"),
                    ("inbox-multiple-outline", "inbox-multiple-outline"),
                    ("inbox-outline", "inbox-outline"),
                    ("inbox-remove", "inbox-remove"),
                    ("inbox-remove-outline", "inbox-remove-outline"),
                    ("incognito", "incognito"),
                    ("incognito-circle", "incognito-circle"),
                    ("incognito-circle-off", "incognito-circle-off"),
                    ("incognito-off", "incognito-off"),
                    ("indent", "indent"),
                    ("induction", "induction"),
                    ("infinity", "infinity"),
                    ("information", "information"),
                    ("information-box", "information-box"),
                    ("information-box-outline", "information-box-outline"),
                    ("information-off", "information-off"),
                    ("information-off-outline", "information-off-outline"),
                    ("information-outline", "information-outline"),
                    ("information-slab-box", "information-slab-box"),
                    ("information-slab-box-outline", "information-slab-box-outline"),
                    ("information-slab-circle", "information-slab-circle"),
                    ("information-slab-circle-outline", "information-slab-circle-outline"),
                    ("information-slab-symbol", "information-slab-symbol"),
                    ("information-symbol", "information-symbol"),
                    ("information-variant", "information-variant"),
                    ("information-variant-box", "information-variant-box"),
                    ("information-variant-box-outline", "information-variant-box-outline"),
                    ("information-variant-circle", "information-variant-circle"),
                    ("information-variant-circle-outline", "information-variant-circle-outline"),
                    ("instagram", "instagram"),
                    ("instapaper", "instapaper"),
                    ("instrument-triangle", "instrument-triangle"),
                    ("integrated-circuit-chip", "integrated-circuit-chip"),
                    ("invert-colors", "invert-colors"),
                    ("invert-colors-off", "invert-colors-off"),
                    ("iobroker", "iobroker"),
                    ("ip", "ip"),
                    ("ip-network", "ip-network"),
                    ("ip-network-outline", "ip-network-outline"),
                    ("ip-outline", "ip-outline"),
                    ("ipod", "ipod"),
                    ("iron", "iron"),
                    ("iron-board", "iron-board"),
                    ("iron-outline", "iron-outline"),
                    ("island", "island"),
                    ("itunes", "itunes"),
                    ("iv-bag", "iv-bag"),
                    ("jabber", "jabber"),
                    ("jeepney", "jeepney"),
                    ("jellyfish", "jellyfish"),
                    ("jellyfish-outline", "jellyfish-outline"),
                    ("jira", "jira"),
                    ("jquery", "jquery"),
                    ("jsfiddle", "jsfiddle"),
                    ("jump-rope", "jump-rope"),
                    ("kabaddi", "kabaddi"),
                    ("kangaroo", "kangaroo"),
                    ("karate", "karate"),
                    ("kayaking", "kayaking"),
                    ("keg", "keg"),
                    ("kettle", "kettle"),
                    ("kettle-alert", "kettle-alert"),
                    ("kettle-alert-outline", "kettle-alert-outline"),
                    ("kettle-off", "kettle-off"),
                    ("kettle-off-outline", "kettle-off-outline"),
                    ("kettle-outline", "kettle-outline"),
                    ("kettle-pour-over", "kettle-pour-over"),
                    ("kettle-steam", "kettle-steam"),
                    ("kettle-steam-outline", "kettle-steam-outline"),
                    ("kettlebell", "kettlebell"),
                    ("key", "key"),
                    ("key-alert", "key-alert"),
                    ("key-alert-outline", "key-alert-outline"),
                    ("key-arrow-right", "key-arrow-right"),
                    ("key-chain", "key-chain"),
                    ("key-chain-variant", "key-chain-variant"),
                    ("key-change", "key-change"),
                    ("key-link", "key-link"),
                    ("key-minus", "key-minus"),
                    ("key-outline", "key-outline"),
                    ("key-plus", "key-plus"),
                    ("key-remove", "key-remove"),
                    ("key-star", "key-star"),
                    ("key-variant", "key-variant"),
                    ("key-wireless", "key-wireless"),
                    ("keyboard", "keyboard"),
                    ("keyboard-backspace", "keyboard-backspace"),
                    ("keyboard-caps", "keyboard-caps"),
                    ("keyboard-close", "keyboard-close"),
                    ("keyboard-close-outline", "keyboard-close-outline"),
                    ("keyboard-esc", "keyboard-esc"),
                    ("keyboard-f1", "keyboard-f1"),
                    ("keyboard-f10", "keyboard-f10"),
                    ("keyboard-f11", "keyboard-f11"),
                    ("keyboard-f12", "keyboard-f12"),
                    ("keyboard-f2", "keyboard-f2"),
                    ("keyboard-f3", "keyboard-f3"),
                    ("keyboard-f4", "keyboard-f4"),
                    ("keyboard-f5", "keyboard-f5"),
                    ("keyboard-f6", "keyboard-f6"),
                    ("keyboard-f7", "keyboard-f7"),
                    ("keyboard-f8", "keyboard-f8"),
                    ("keyboard-f9", "keyboard-f9"),
                    ("keyboard-off", "keyboard-off"),
                    ("keyboard-off-outline", "keyboard-off-outline"),
                    ("keyboard-outline", "keyboard-outline"),
                    ("keyboard-return", "keyboard-return"),
                    ("keyboard-settings", "keyboard-settings"),
                    ("keyboard-settings-outline", "keyboard-settings-outline"),
                    ("keyboard-space", "keyboard-space"),
                    ("keyboard-tab", "keyboard-tab"),
                    ("keyboard-tab-reverse", "keyboard-tab-reverse"),
                    ("keyboard-variant", "keyboard-variant"),
                    ("khanda", "khanda"),
                    ("kickstarter", "kickstarter"),
                    ("kite", "kite"),
                    ("kite-outline", "kite-outline"),
                    ("kitesurfing", "kitesurfing"),
                    ("klingon", "klingon"),
                    ("knife", "knife"),
                    ("knife-military", "knife-military"),
                    ("knob", "knob"),
                    ("koala", "koala"),
                    ("kodi", "kodi"),
                    ("kubernetes", "kubernetes"),
                    ("label", "label"),
                    ("label-multiple", "label-multiple"),
                    ("label-multiple-outline", "label-multiple-outline"),
                    ("label-off", "label-off"),
                    ("label-off-outline", "label-off-outline"),
                    ("label-outline", "label-outline"),
                    ("label-percent", "label-percent"),
                    ("label-percent-outline", "label-percent-outline"),
                    ("label-variant", "label-variant"),
                    ("label-variant-outline", "label-variant-outline"),
                    ("ladder", "ladder"),
                    ("ladybug", "ladybug"),
                    ("lambda", "lambda"),
                    ("lamp", "lamp"),
                    ("lamp-outline", "lamp-outline"),
                    ("lamps", "lamps"),
                    ("lamps-outline", "lamps-outline"),
                    ("lan", "lan"),
                    ("lan-check", "lan-check"),
                    ("lan-connect", "lan-connect"),
                    ("lan-disconnect", "lan-disconnect"),
                    ("lan-pending", "lan-pending"),
                    ("land-fields", "land-fields"),
                    ("land-plots", "land-plots"),
                    ("land-plots-circle", "land-plots-circle"),
                    ("land-plots-circle-variant", "land-plots-circle-variant"),
                    ("land-plots-marker", "land-plots-marker"),
                    ("land-rows-horizontal", "land-rows-horizontal"),
                    ("land-rows-vertical", "land-rows-vertical"),
                    ("landslide", "landslide"),
                    ("landslide-outline", "landslide-outline"),
                    ("language-c", "language-c"),
                    ("language-cpp", "language-cpp"),
                    ("language-csharp", "language-csharp"),
                    ("language-css3", "language-css3"),
                    ("language-fortran", "language-fortran"),
                    ("language-go", "language-go"),
                    ("language-haskell", "language-haskell"),
                    ("language-html5", "language-html5"),
                    ("language-java", "language-java"),
                    ("language-javascript", "language-javascript"),
                    ("language-jsx", "language-jsx"),
                    ("language-kotlin", "language-kotlin"),
                    ("language-lua", "language-lua"),
                    ("language-markdown", "language-markdown"),
                    ("language-markdown-outline", "language-markdown-outline"),
                    ("language-php", "language-php"),
                    ("language-python", "language-python"),
                    ("language-python-text", "language-python-text"),
                    ("language-r", "language-r"),
                    ("language-ruby", "language-ruby"),
                    ("language-ruby-on-rails", "language-ruby-on-rails"),
                    ("language-rust", "language-rust"),
                    ("language-swift", "language-swift"),
                    ("language-typescript", "language-typescript"),
                    ("language-xaml", "language-xaml"),
                    ("laptop", "laptop"),
                    ("laptop-account", "laptop-account"),
                    ("laptop-chromebook", "laptop-chromebook"),
                    ("laptop-mac", "laptop-mac"),
                    ("laptop-off", "laptop-off"),
                    ("laptop-windows", "laptop-windows"),
                    ("laravel", "laravel"),
                    ("laser-pointer", "laser-pointer"),
                    ("lasso", "lasso"),
                    ("lastfm", "lastfm"),
                    ("lastpass", "lastpass"),
                    ("latitude", "latitude"),
                    ("launch", "launch"),
                    ("lava-lamp", "lava-lamp"),
                    ("layers", "layers"),
                    ("layers-edit", "layers-edit"),
                    ("layers-minus", "layers-minus"),
                    ("layers-off", "layers-off"),
                    ("layers-off-outline", "layers-off-outline"),
                    ("layers-outline", "layers-outline"),
                    ("layers-plus", "layers-plus"),
                    ("layers-remove", "layers-remove"),
                    ("layers-search", "layers-search"),
                    ("layers-search-outline", "layers-search-outline"),
                    ("layers-triple", "layers-triple"),
                    ("layers-triple-outline", "layers-triple-outline"),
                    ("lead-pencil", "lead-pencil"),
                    ("leaf", "leaf"),
                    ("leaf-circle", "leaf-circle"),
                    ("leaf-circle-outline", "leaf-circle-outline"),
                    ("leaf-maple", "leaf-maple"),
                    ("leaf-maple-off", "leaf-maple-off"),
                    ("leaf-off", "leaf-off"),
                    ("leak", "leak"),
                    ("leak-off", "leak-off"),
                    ("lectern", "lectern"),
                    ("led-off", "led-off"),
                    ("led-on", "led-on"),
                    ("led-outline", "led-outline"),
                    ("led-strip", "led-strip"),
                    ("led-strip-variant", "led-strip-variant"),
                    ("led-strip-variant-off", "led-strip-variant-off"),
                    ("led-variant-off", "led-variant-off"),
                    ("led-variant-on", "led-variant-on"),
                    ("led-variant-outline", "led-variant-outline"),
                    ("leek", "leek"),
                    ("less-than", "less-than"),
                    ("less-than-or-equal", "less-than-or-equal"),
                    ("library", "library"),
                    ("library-books", "library-books"),
                    ("library-outline", "library-outline"),
                    ("library-shelves", "library-shelves"),
                    ("license", "license"),
                    ("lifebuoy", "lifebuoy"),
                    ("light-flood-down", "light-flood-down"),
                    ("light-flood-up", "light-flood-up"),
                    ("light-recessed", "light-recessed"),
                    ("light-switch", "light-switch"),
                    ("light-switch-off", "light-switch-off"),
                    ("lightbulb", "lightbulb"),
                    ("lightbulb-alert", "lightbulb-alert"),
                    ("lightbulb-alert-outline", "lightbulb-alert-outline"),
                    ("lightbulb-auto", "lightbulb-auto"),
                    ("lightbulb-auto-outline", "lightbulb-auto-outline"),
                    ("lightbulb-cfl", "lightbulb-cfl"),
                    ("lightbulb-cfl-off", "lightbulb-cfl-off"),
                    ("lightbulb-cfl-spiral", "lightbulb-cfl-spiral"),
                    ("lightbulb-cfl-spiral-off", "lightbulb-cfl-spiral-off"),
                    ("lightbulb-fluorescent-tube", "lightbulb-fluorescent-tube"),
                    ("lightbulb-fluorescent-tube-outline", "lightbulb-fluorescent-tube-outline"),
                    ("lightbulb-group", "lightbulb-group"),
                    ("lightbulb-group-off", "lightbulb-group-off"),
                    ("lightbulb-group-off-outline", "lightbulb-group-off-outline"),
                    ("lightbulb-group-outline", "lightbulb-group-outline"),
                    ("lightbulb-multiple", "lightbulb-multiple"),
                    ("lightbulb-multiple-off", "lightbulb-multiple-off"),
                    ("lightbulb-multiple-off-outline", "lightbulb-multiple-off-outline"),
                    ("lightbulb-multiple-outline", "lightbulb-multiple-outline"),
                    ("lightbulb-night", "lightbulb-night"),
                    ("lightbulb-night-outline", "lightbulb-night-outline"),
                    ("lightbulb-off", "lightbulb-off"),
                    ("lightbulb-off-outline", "lightbulb-off-outline"),
                    ("lightbulb-on", "lightbulb-on"),
                    ("lightbulb-on-10", "lightbulb-on-10"),
                    ("lightbulb-on-20", "lightbulb-on-20"),
                    ("lightbulb-on-30", "lightbulb-on-30"),
                    ("lightbulb-on-40", "lightbulb-on-40"),
                    ("lightbulb-on-50", "lightbulb-on-50"),
                    ("lightbulb-on-60", "lightbulb-on-60"),
                    ("lightbulb-on-70", "lightbulb-on-70"),
                    ("lightbulb-on-80", "lightbulb-on-80"),
                    ("lightbulb-on-90", "lightbulb-on-90"),
                    ("lightbulb-on-outline", "lightbulb-on-outline"),
                    ("lightbulb-outline", "lightbulb-outline"),
                    ("lightbulb-question", "lightbulb-question"),
                    ("lightbulb-question-outline", "lightbulb-question-outline"),
                    ("lightbulb-spot", "lightbulb-spot"),
                    ("lightbulb-spot-off", "lightbulb-spot-off"),
                    ("lightbulb-variant", "lightbulb-variant"),
                    ("lightbulb-variant-outline", "lightbulb-variant-outline"),
                    ("lighthouse", "lighthouse"),
                    ("lighthouse-on", "lighthouse-on"),
                    ("lightning-bolt", "lightning-bolt"),
                    ("lightning-bolt-circle", "lightning-bolt-circle"),
                    ("lightning-bolt-outline", "lightning-bolt-outline"),
                    ("line-scan", "line-scan"),
                    ("lingerie", "lingerie"),
                    ("link", "link"),
                    ("link-box", "link-box"),
                    ("link-box-outline", "link-box-outline"),
                    ("link-box-variant", "link-box-variant"),
                    ("link-box-variant-outline", "link-box-variant-outline"),
                    ("link-lock", "link-lock"),
                    ("link-off", "link-off"),
                    ("link-plus", "link-plus"),
                    ("link-variant", "link-variant"),
                    ("link-variant-minus", "link-variant-minus"),
                    ("link-variant-off", "link-variant-off"),
                    ("link-variant-plus", "link-variant-plus"),
                    ("link-variant-remove", "link-variant-remove"),
                    ("linkedin", "linkedin"),
                    ("linode", "linode"),
                    ("linux", "linux"),
                    ("linux-mint", "linux-mint"),
                    ("lipstick", "lipstick"),
                    ("liquid-spot", "liquid-spot"),
                    ("liquor", "liquor"),
                    ("list-box", "list-box"),
                    ("list-box-outline", "list-box-outline"),
                    ("list-status", "list-status"),
                    ("litecoin", "litecoin"),
                    ("loading", "loading"),
                    ("location-enter", "location-enter"),
                    ("location-exit", "location-exit"),
                    ("lock", "lock"),
                    ("lock-alert", "lock-alert"),
                    ("lock-alert-outline", "lock-alert-outline"),
                    ("lock-check", "lock-check"),
                    ("lock-check-outline", "lock-check-outline"),
                    ("lock-clock", "lock-clock"),
                    ("lock-minus", "lock-minus"),
                    ("lock-minus-outline", "lock-minus-outline"),
                    ("lock-off", "lock-off"),
                    ("lock-off-outline", "lock-off-outline"),
                    ("lock-open", "lock-open"),
                    ("lock-open-alert", "lock-open-alert"),
                    ("lock-open-alert-outline", "lock-open-alert-outline"),
                    ("lock-open-check", "lock-open-check"),
                    ("lock-open-check-outline", "lock-open-check-outline"),
                    ("lock-open-minus", "lock-open-minus"),
                    ("lock-open-minus-outline", "lock-open-minus-outline"),
                    ("lock-open-outline", "lock-open-outline"),
                    ("lock-open-plus", "lock-open-plus"),
                    ("lock-open-plus-outline", "lock-open-plus-outline"),
                    ("lock-open-remove", "lock-open-remove"),
                    ("lock-open-remove-outline", "lock-open-remove-outline"),
                    ("lock-open-variant", "lock-open-variant"),
                    ("lock-open-variant-outline", "lock-open-variant-outline"),
                    ("lock-outline", "lock-outline"),
                    ("lock-pattern", "lock-pattern"),
                    ("lock-percent", "lock-percent"),
                    ("lock-percent-open", "lock-percent-open"),
                    ("lock-percent-open-outline", "lock-percent-open-outline"),
                    ("lock-percent-open-variant", "lock-percent-open-variant"),
                    ("lock-percent-open-variant-outline", "lock-percent-open-variant-outline"),
                    ("lock-percent-outline", "lock-percent-outline"),
                    ("lock-plus", "lock-plus"),
                    ("lock-plus-outline", "lock-plus-outline"),
                    ("lock-question", "lock-question"),
                    ("lock-remove", "lock-remove"),
                    ("lock-remove-outline", "lock-remove-outline"),
                    ("lock-reset", "lock-reset"),
                    ("lock-smart", "lock-smart"),
                    ("locker", "locker"),
                    ("locker-multiple", "locker-multiple"),
                    ("login", "login"),
                    ("login-variant", "login-variant"),
                    ("logout", "logout"),
                    ("logout-variant", "logout-variant"),
                    ("longitude", "longitude"),
                    ("looks", "looks"),
                    ("lotion", "lotion"),
                    ("lotion-outline", "lotion-outline"),
                    ("lotion-plus", "lotion-plus"),
                    ("lotion-plus-outline", "lotion-plus-outline"),
                    ("loupe", "loupe"),
                    ("lumx", "lumx"),
                    ("lungs", "lungs"),
                    ("lyft", "lyft"),
                    ("mace", "mace"),
                    ("magazine-pistol", "magazine-pistol"),
                    ("magazine-rifle", "magazine-rifle"),
                    ("magic-staff", "magic-staff"),
                    ("magnet", "magnet"),
                    ("magnet-on", "magnet-on"),
                    ("magnify", "magnify"),
                    ("magnify-close", "magnify-close"),
                    ("magnify-expand", "magnify-expand"),
                    ("magnify-minus", "magnify-minus"),
                    ("magnify-minus-cursor", "magnify-minus-cursor"),
                    ("magnify-minus-outline", "magnify-minus-outline"),
                    ("magnify-plus", "magnify-plus"),
                    ("magnify-plus-cursor", "magnify-plus-cursor"),
                    ("magnify-plus-outline", "magnify-plus-outline"),
                    ("magnify-remove-cursor", "magnify-remove-cursor"),
                    ("magnify-remove-outline", "magnify-remove-outline"),
                    ("magnify-scan", "magnify-scan"),
                    ("mail", "mail"),
                    ("mail-ru", "mail-ru"),
                    ("mailbox", "mailbox"),
                    ("mailbox-open", "mailbox-open"),
                    ("mailbox-open-outline", "mailbox-open-outline"),
                    ("mailbox-open-up", "mailbox-open-up"),
                    ("mailbox-open-up-outline", "mailbox-open-up-outline"),
                    ("mailbox-outline", "mailbox-outline"),
                    ("mailbox-up", "mailbox-up"),
                    ("mailbox-up-outline", "mailbox-up-outline"),
                    ("manjaro", "manjaro"),
                    ("map", "map"),
                    ("map-check", "map-check"),
                    ("map-check-outline", "map-check-outline"),
                    ("map-clock", "map-clock"),
                    ("map-clock-outline", "map-clock-outline"),
                    ("map-legend", "map-legend"),
                    ("map-marker", "map-marker"),
                    ("map-marker-account", "map-marker-account"),
                    ("map-marker-account-outline", "map-marker-account-outline"),
                    ("map-marker-alert", "map-marker-alert"),
                    ("map-marker-alert-outline", "map-marker-alert-outline"),
                    ("map-marker-check", "map-marker-check"),
                    ("map-marker-check-outline", "map-marker-check-outline"),
                    ("map-marker-circle", "map-marker-circle"),
                    ("map-marker-distance", "map-marker-distance"),
                    ("map-marker-down", "map-marker-down"),
                    ("map-marker-left", "map-marker-left"),
                    ("map-marker-left-outline", "map-marker-left-outline"),
                    ("map-marker-minus", "map-marker-minus"),
                    ("map-marker-minus-outline", "map-marker-minus-outline"),
                    ("map-marker-multiple", "map-marker-multiple"),
                    ("map-marker-multiple-outline", "map-marker-multiple-outline"),
                    ("map-marker-off", "map-marker-off"),
                    ("map-marker-off-outline", "map-marker-off-outline"),
                    ("map-marker-outline", "map-marker-outline"),
                    ("map-marker-path", "map-marker-path"),
                    ("map-marker-plus", "map-marker-plus"),
                    ("map-marker-plus-outline", "map-marker-plus-outline"),
                    ("map-marker-question", "map-marker-question"),
                    ("map-marker-question-outline", "map-marker-question-outline"),
                    ("map-marker-radius", "map-marker-radius"),
                    ("map-marker-radius-outline", "map-marker-radius-outline"),
                    ("map-marker-remove", "map-marker-remove"),
                    ("map-marker-remove-outline", "map-marker-remove-outline"),
                    ("map-marker-remove-variant", "map-marker-remove-variant"),
                    ("map-marker-right", "map-marker-right"),
                    ("map-marker-right-outline", "map-marker-right-outline"),
                    ("map-marker-star", "map-marker-star"),
                    ("map-marker-star-outline", "map-marker-star-outline"),
                    ("map-marker-up", "map-marker-up"),
                    ("map-minus", "map-minus"),
                    ("map-outline", "map-outline"),
                    ("map-plus", "map-plus"),
                    ("map-search", "map-search"),
                    ("map-search-outline", "map-search-outline"),
                    ("mapbox", "mapbox"),
                    ("margin", "margin"),
                    ("marker", "marker"),
                    ("marker-cancel", "marker-cancel"),
                    ("marker-check", "marker-check"),
                    ("mastodon", "mastodon"),
                    ("mastodon-variant", "mastodon-variant"),
                    ("material-design", "material-design"),
                    ("material-ui", "material-ui"),
                    ("math-compass", "math-compass"),
                    ("math-cos", "math-cos"),
                    ("math-integral", "math-integral"),
                    ("math-integral-box", "math-integral-box"),
                    ("math-log", "math-log"),
                    ("math-norm", "math-norm"),
                    ("math-norm-box", "math-norm-box"),
                    ("math-sin", "math-sin"),
                    ("math-tan", "math-tan"),
                    ("matrix", "matrix"),
                    ("maxcdn", "maxcdn"),
                    ("medal", "medal"),
                    ("medal-outline", "medal-outline"),
                    ("medical-bag", "medical-bag"),
                    ("medical-cotton-swab", "medical-cotton-swab"),
                    ("medication", "medication"),
                    ("medication-outline", "medication-outline"),
                    ("meditation", "meditation"),
                    ("medium", "medium"),
                    ("meetup", "meetup"),
                    ("memory", "memory"),
                    ("memory-arrow-down", "memory-arrow-down"),
                    ("menorah", "menorah"),
                    ("menorah-fire", "menorah-fire"),
                    ("menu", "menu"),
                    ("menu-close", "menu-close"),
                    ("menu-down", "menu-down"),
                    ("menu-down-outline", "menu-down-outline"),
                    ("menu-left", "menu-left"),
                    ("menu-left-outline", "menu-left-outline"),
                    ("menu-open", "menu-open"),
                    ("menu-right", "menu-right"),
                    ("menu-right-outline", "menu-right-outline"),
                    ("menu-swap", "menu-swap"),
                    ("menu-swap-outline", "menu-swap-outline"),
                    ("menu-up", "menu-up"),
                    ("menu-up-outline", "menu-up-outline"),
                    ("merge", "merge"),
                    ("message", "message"),
                    ("message-alert", "message-alert"),
                    ("message-alert-outline", "message-alert-outline"),
                    ("message-arrow-left", "message-arrow-left"),
                    ("message-arrow-left-outline", "message-arrow-left-outline"),
                    ("message-arrow-right", "message-arrow-right"),
                    ("message-arrow-right-outline", "message-arrow-right-outline"),
                    ("message-badge", "message-badge"),
                    ("message-badge-outline", "message-badge-outline"),
                    ("message-bookmark", "message-bookmark"),
                    ("message-bookmark-outline", "message-bookmark-outline"),
                    ("message-bulleted", "message-bulleted"),
                    ("message-bulleted-off", "message-bulleted-off"),
                    ("message-check", "message-check"),
                    ("message-check-outline", "message-check-outline"),
                    ("message-cog", "message-cog"),
                    ("message-cog-outline", "message-cog-outline"),
                    ("message-draw", "message-draw"),
                    ("message-fast", "message-fast"),
                    ("message-fast-outline", "message-fast-outline"),
                    ("message-flash", "message-flash"),
                    ("message-flash-outline", "message-flash-outline"),
                    ("message-image", "message-image"),
                    ("message-image-outline", "message-image-outline"),
                    ("message-lock", "message-lock"),
                    ("message-lock-outline", "message-lock-outline"),
                    ("message-minus", "message-minus"),
                    ("message-minus-outline", "message-minus-outline"),
                    ("message-off", "message-off"),
                    ("message-off-outline", "message-off-outline"),
                    ("message-outline", "message-outline"),
                    ("message-plus", "message-plus"),
                    ("message-plus-outline", "message-plus-outline"),
                    ("message-processing", "message-processing"),
                    ("message-processing-outline", "message-processing-outline"),
                    ("message-question", "message-question"),
                    ("message-question-outline", "message-question-outline"),
                    ("message-reply", "message-reply"),
                    ("message-reply-outline", "message-reply-outline"),
                    ("message-reply-text", "message-reply-text"),
                    ("message-reply-text-outline", "message-reply-text-outline"),
                    ("message-settings", "message-settings"),
                    ("message-settings-outline", "message-settings-outline"),
                    ("message-star", "message-star"),
                    ("message-star-outline", "message-star-outline"),
                    ("message-text", "message-text"),
                    ("message-text-clock", "message-text-clock"),
                    ("message-text-clock-outline", "message-text-clock-outline"),
                    ("message-text-fast", "message-text-fast"),
                    ("message-text-fast-outline", "message-text-fast-outline"),
                    ("message-text-lock", "message-text-lock"),
                    ("message-text-lock-outline", "message-text-lock-outline"),
                    ("message-text-outline", "message-text-outline"),
                    ("message-video", "message-video"),
                    ("meteor", "meteor"),
                    ("meter-electric", "meter-electric"),
                    ("meter-electric-outline", "meter-electric-outline"),
                    ("meter-gas", "meter-gas"),
                    ("meter-gas-outline", "meter-gas-outline"),
                    ("metronome", "metronome"),
                    ("metronome-tick", "metronome-tick"),
                    ("micro-sd", "micro-sd"),
                    ("microphone", "microphone"),
                    ("microphone-message", "microphone-message"),
                    ("microphone-message-off", "microphone-message-off"),
                    ("microphone-minus", "microphone-minus"),
                    ("microphone-off", "microphone-off"),
                    ("microphone-outline", "microphone-outline"),
                    ("microphone-plus", "microphone-plus"),
                    ("microphone-question", "microphone-question"),
                    ("microphone-question-outline", "microphone-question-outline"),
                    ("microphone-settings", "microphone-settings"),
                    ("microphone-variant", "microphone-variant"),
                    ("microphone-variant-off", "microphone-variant-off"),
                    ("microscope", "microscope"),
                    ("microsoft", "microsoft"),
                    ("microsoft-access", "microsoft-access"),
                    ("microsoft-azure", "microsoft-azure"),
                    ("microsoft-azure-devops", "microsoft-azure-devops"),
                    ("microsoft-bing", "microsoft-bing"),
                    ("microsoft-dynamics-365", "microsoft-dynamics-365"),
                    ("microsoft-edge", "microsoft-edge"),
                    ("microsoft-edge-legacy", "microsoft-edge-legacy"),
                    ("microsoft-excel", "microsoft-excel"),
                    ("microsoft-internet-explorer", "microsoft-internet-explorer"),
                    ("microsoft-office", "microsoft-office"),
                    ("microsoft-onedrive", "microsoft-onedrive"),
                    ("microsoft-onenote", "microsoft-onenote"),
                    ("microsoft-outlook", "microsoft-outlook"),
                    ("microsoft-powerpoint", "microsoft-powerpoint"),
                    ("microsoft-sharepoint", "microsoft-sharepoint"),
                    ("microsoft-teams", "microsoft-teams"),
                    ("microsoft-visual-studio", "microsoft-visual-studio"),
                    ("microsoft-visual-studio-code", "microsoft-visual-studio-code"),
                    ("microsoft-windows", "microsoft-windows"),
                    ("microsoft-windows-classic", "microsoft-windows-classic"),
                    ("microsoft-word", "microsoft-word"),
                    ("microsoft-xbox", "microsoft-xbox"),
                    ("microsoft-xbox-controller", "microsoft-xbox-controller"),
                    (
                        "microsoft-xbox-controller-battery-alert",
                        "microsoft-xbox-controller-battery-alert",
                    ),
                    (
                        "microsoft-xbox-controller-battery-charging",
                        "microsoft-xbox-controller-battery-charging",
                    ),
                    (
                        "microsoft-xbox-controller-battery-empty",
                        "microsoft-xbox-controller-battery-empty",
                    ),
                    (
                        "microsoft-xbox-controller-battery-full",
                        "microsoft-xbox-controller-battery-full",
                    ),
                    (
                        "microsoft-xbox-controller-battery-low",
                        "microsoft-xbox-controller-battery-low",
                    ),
                    (
                        "microsoft-xbox-controller-battery-medium",
                        "microsoft-xbox-controller-battery-medium",
                    ),
                    (
                        "microsoft-xbox-controller-battery-unknown",
                        "microsoft-xbox-controller-battery-unknown",
                    ),
                    ("microsoft-xbox-controller-menu", "microsoft-xbox-controller-menu"),
                    ("microsoft-xbox-controller-off", "microsoft-xbox-controller-off"),
                    ("microsoft-xbox-controller-view", "microsoft-xbox-controller-view"),
                    ("microsoft-yammer", "microsoft-yammer"),
                    ("microwave", "microwave"),
                    ("microwave-off", "microwave-off"),
                    ("middleware", "middleware"),
                    ("middleware-outline", "middleware-outline"),
                    ("midi", "midi"),
                    ("midi-input", "midi-input"),
                    ("midi-port", "midi-port"),
                    ("mine", "mine"),
                    ("minecraft", "minecraft"),
                    ("mini-sd", "mini-sd"),
                    ("minidisc", "minidisc"),
                    ("minus", "minus"),
                    ("minus-box", "minus-box"),
                    ("minus-box-multiple", "minus-box-multiple"),
                    ("minus-box-multiple-outline", "minus-box-multiple-outline"),
                    ("minus-box-outline", "minus-box-outline"),
                    ("minus-circle", "minus-circle"),
                    ("minus-circle-multiple", "minus-circle-multiple"),
                    ("minus-circle-multiple-outline", "minus-circle-multiple-outline"),
                    ("minus-circle-off", "minus-circle-off"),
                    ("minus-circle-off-outline", "minus-circle-off-outline"),
                    ("minus-circle-outline", "minus-circle-outline"),
                    ("minus-network", "minus-network"),
                    ("minus-network-outline", "minus-network-outline"),
                    ("minus-thick", "minus-thick"),
                    ("mirror", "mirror"),
                    ("mirror-rectangle", "mirror-rectangle"),
                    ("mirror-variant", "mirror-variant"),
                    ("mixcloud", "mixcloud"),
                    ("mixed-martial-arts", "mixed-martial-arts"),
                    ("mixed-reality", "mixed-reality"),
                    ("mixer", "mixer"),
                    ("molecule", "molecule"),
                    ("molecule-co", "molecule-co"),
                    ("molecule-co2", "molecule-co2"),
                    ("monitor", "monitor"),
                    ("monitor-account", "monitor-account"),
                    ("monitor-arrow-down", "monitor-arrow-down"),
                    ("monitor-arrow-down-variant", "monitor-arrow-down-variant"),
                    ("monitor-cellphone", "monitor-cellphone"),
                    ("monitor-cellphone-star", "monitor-cellphone-star"),
                    ("monitor-dashboard", "monitor-dashboard"),
                    ("monitor-edit", "monitor-edit"),
                    ("monitor-eye", "monitor-eye"),
                    ("monitor-lock", "monitor-lock"),
                    ("monitor-multiple", "monitor-multiple"),
                    ("monitor-off", "monitor-off"),
                    ("monitor-screenshot", "monitor-screenshot"),
                    ("monitor-share", "monitor-share"),
                    ("monitor-shimmer", "monitor-shimmer"),
                    ("monitor-small", "monitor-small"),
                    ("monitor-speaker", "monitor-speaker"),
                    ("monitor-speaker-off", "monitor-speaker-off"),
                    ("monitor-star", "monitor-star"),
                    ("monitor-vertical", "monitor-vertical"),
                    ("moon-first-quarter", "moon-first-quarter"),
                    ("moon-full", "moon-full"),
                    ("moon-last-quarter", "moon-last-quarter"),
                    ("moon-new", "moon-new"),
                    ("moon-waning-crescent", "moon-waning-crescent"),
                    ("moon-waning-gibbous", "moon-waning-gibbous"),
                    ("moon-waxing-crescent", "moon-waxing-crescent"),
                    ("moon-waxing-gibbous", "moon-waxing-gibbous"),
                    ("moped", "moped"),
                    ("moped-electric", "moped-electric"),
                    ("moped-electric-outline", "moped-electric-outline"),
                    ("moped-outline", "moped-outline"),
                    ("more", "more"),
                    ("mortar-pestle", "mortar-pestle"),
                    ("mortar-pestle-plus", "mortar-pestle-plus"),
                    ("mosque", "mosque"),
                    ("mosque-outline", "mosque-outline"),
                    ("mother-heart", "mother-heart"),
                    ("mother-nurse", "mother-nurse"),
                    ("motion", "motion"),
                    ("motion-outline", "motion-outline"),
                    ("motion-pause", "motion-pause"),
                    ("motion-pause-outline", "motion-pause-outline"),
                    ("motion-play", "motion-play"),
                    ("motion-play-outline", "motion-play-outline"),
                    ("motion-sensor", "motion-sensor"),
                    ("motion-sensor-off", "motion-sensor-off"),
                    ("motorbike", "motorbike"),
                    ("motorbike-electric", "motorbike-electric"),
                    ("motorbike-off", "motorbike-off"),
                    ("mouse", "mouse"),
                    ("mouse-bluetooth", "mouse-bluetooth"),
                    ("mouse-move-down", "mouse-move-down"),
                    ("mouse-move-up", "mouse-move-up"),
                    ("mouse-move-vertical", "mouse-move-vertical"),
                    ("mouse-off", "mouse-off"),
                    ("mouse-variant", "mouse-variant"),
                    ("mouse-variant-off", "mouse-variant-off"),
                    ("move-resize", "move-resize"),
                    ("move-resize-variant", "move-resize-variant"),
                    ("movie", "movie"),
                    ("movie-check", "movie-check"),
                    ("movie-check-outline", "movie-check-outline"),
                    ("movie-cog", "movie-cog"),
                    ("movie-cog-outline", "movie-cog-outline"),
                    ("movie-edit", "movie-edit"),
                    ("movie-edit-outline", "movie-edit-outline"),
                    ("movie-filter", "movie-filter"),
                    ("movie-filter-outline", "movie-filter-outline"),
                    ("movie-minus", "movie-minus"),
                    ("movie-minus-outline", "movie-minus-outline"),
                    ("movie-off", "movie-off"),
                    ("movie-off-outline", "movie-off-outline"),
                    ("movie-open", "movie-open"),
                    ("movie-open-check", "movie-open-check"),
                    ("movie-open-check-outline", "movie-open-check-outline"),
                    ("movie-open-cog", "movie-open-cog"),
                    ("movie-open-cog-outline", "movie-open-cog-outline"),
                    ("movie-open-edit", "movie-open-edit"),
                    ("movie-open-edit-outline", "movie-open-edit-outline"),
                    ("movie-open-minus", "movie-open-minus"),
                    ("movie-open-minus-outline", "movie-open-minus-outline"),
                    ("movie-open-off", "movie-open-off"),
                    ("movie-open-off-outline", "movie-open-off-outline"),
                    ("movie-open-outline", "movie-open-outline"),
                    ("movie-open-play", "movie-open-play"),
                    ("movie-open-play-outline", "movie-open-play-outline"),
                    ("movie-open-plus", "movie-open-plus"),
                    ("movie-open-plus-outline", "movie-open-plus-outline"),
                    ("movie-open-remove", "movie-open-remove"),
                    ("movie-open-remove-outline", "movie-open-remove-outline"),
                    ("movie-open-settings", "movie-open-settings"),
                    ("movie-open-settings-outline", "movie-open-settings-outline"),
                    ("movie-open-star", "movie-open-star"),
                    ("movie-open-star-outline", "movie-open-star-outline"),
                    ("movie-outline", "movie-outline"),
                    ("movie-play", "movie-play"),
                    ("movie-play-outline", "movie-play-outline"),
                    ("movie-plus", "movie-plus"),
                    ("movie-plus-outline", "movie-plus-outline"),
                    ("movie-remove", "movie-remove"),
                    ("movie-remove-outline", "movie-remove-outline"),
                    ("movie-roll", "movie-roll"),
                    ("movie-search", "movie-search"),
                    ("movie-search-outline", "movie-search-outline"),
                    ("movie-settings", "movie-settings"),
                    ("movie-settings-outline", "movie-settings-outline"),
                    ("movie-star", "movie-star"),
                    ("movie-star-outline", "movie-star-outline"),
                    ("mower", "mower"),
                    ("mower-bag", "mower-bag"),
                    ("mower-bag-on", "mower-bag-on"),
                    ("mower-on", "mower-on"),
                    ("muffin", "muffin"),
                    ("multicast", "multicast"),
                    ("multimedia", "multimedia"),
                    ("multiplication", "multiplication"),
                    ("multiplication-box", "multiplication-box"),
                    ("mushroom", "mushroom"),
                    ("mushroom-off", "mushroom-off"),
                    ("mushroom-off-outline", "mushroom-off-outline"),
                    ("mushroom-outline", "mushroom-outline"),
                    ("music", "music"),
                    ("music-accidental-double-flat", "music-accidental-double-flat"),
                    ("music-accidental-double-sharp", "music-accidental-double-sharp"),
                    ("music-accidental-flat", "music-accidental-flat"),
                    ("music-accidental-natural", "music-accidental-natural"),
                    ("music-accidental-sharp", "music-accidental-sharp"),
                    ("music-box", "music-box"),
                    ("music-box-multiple", "music-box-multiple"),
                    ("music-box-multiple-outline", "music-box-multiple-outline"),
                    ("music-box-outline", "music-box-outline"),
                    ("music-circle", "music-circle"),
                    ("music-circle-outline", "music-circle-outline"),
                    ("music-clef-alto", "music-clef-alto"),
                    ("music-clef-bass", "music-clef-bass"),
                    ("music-clef-treble", "music-clef-treble"),
                    ("music-note", "music-note"),
                    ("music-note-bluetooth", "music-note-bluetooth"),
                    ("music-note-bluetooth-off", "music-note-bluetooth-off"),
                    ("music-note-eighth", "music-note-eighth"),
                    ("music-note-eighth-dotted", "music-note-eighth-dotted"),
                    ("music-note-half", "music-note-half"),
                    ("music-note-half-dotted", "music-note-half-dotted"),
                    ("music-note-minus", "music-note-minus"),
                    ("music-note-off", "music-note-off"),
                    ("music-note-off-outline", "music-note-off-outline"),
                    ("music-note-outline", "music-note-outline"),
                    ("music-note-plus", "music-note-plus"),
                    ("music-note-quarter", "music-note-quarter"),
                    ("music-note-quarter-dotted", "music-note-quarter-dotted"),
                    ("music-note-sixteenth", "music-note-sixteenth"),
                    ("music-note-sixteenth-dotted", "music-note-sixteenth-dotted"),
                    ("music-note-whole", "music-note-whole"),
                    ("music-note-whole-dotted", "music-note-whole-dotted"),
                    ("music-off", "music-off"),
                    ("music-rest-eighth", "music-rest-eighth"),
                    ("music-rest-half", "music-rest-half"),
                    ("music-rest-quarter", "music-rest-quarter"),
                    ("music-rest-sixteenth", "music-rest-sixteenth"),
                    ("music-rest-whole", "music-rest-whole"),
                    ("mustache", "mustache"),
                    ("nail", "nail"),
                    ("nas", "nas"),
                    ("nativescript", "nativescript"),
                    ("nature", "nature"),
                    ("nature-outline", "nature-outline"),
                    ("nature-people", "nature-people"),
                    ("nature-people-outline", "nature-people-outline"),
                    ("navigation", "navigation"),
                    ("navigation-outline", "navigation-outline"),
                    ("navigation-variant", "navigation-variant"),
                    ("navigation-variant-outline", "navigation-variant-outline"),
                    ("near-me", "near-me"),
                    ("necklace", "necklace"),
                    ("needle", "needle"),
                    ("needle-off", "needle-off"),
                    ("nest-thermostat", "nest-thermostat"),
                    ("netflix", "netflix"),
                    ("network", "network"),
                    ("network-off", "network-off"),
                    ("network-off-outline", "network-off-outline"),
                    ("network-outline", "network-outline"),
                    ("network-pos", "network-pos"),
                    ("network-strength-1", "network-strength-1"),
                    ("network-strength-1-alert", "network-strength-1-alert"),
                    ("network-strength-2", "network-strength-2"),
                    ("network-strength-2-alert", "network-strength-2-alert"),
                    ("network-strength-3", "network-strength-3"),
                    ("network-strength-3-alert", "network-strength-3-alert"),
                    ("network-strength-4", "network-strength-4"),
                    ("network-strength-4-alert", "network-strength-4-alert"),
                    ("network-strength-4-cog", "network-strength-4-cog"),
                    ("network-strength-alert", "network-strength-alert"),
                    ("network-strength-alert-outline", "network-strength-alert-outline"),
                    ("network-strength-off", "network-strength-off"),
                    ("network-strength-off-outline", "network-strength-off-outline"),
                    ("network-strength-outline", "network-strength-outline"),
                    ("new-box", "new-box"),
                    ("newspaper", "newspaper"),
                    ("newspaper-check", "newspaper-check"),
                    ("newspaper-minus", "newspaper-minus"),
                    ("newspaper-plus", "newspaper-plus"),
                    ("newspaper-remove", "newspaper-remove"),
                    ("newspaper-variant", "newspaper-variant"),
                    ("newspaper-variant-multiple", "newspaper-variant-multiple"),
                    ("newspaper-variant-multiple-outline", "newspaper-variant-multiple-outline"),
                    ("newspaper-variant-outline", "newspaper-variant-outline"),
                    ("nfc", "nfc"),
                    ("nfc-off", "nfc-off"),
                    ("nfc-search-variant", "nfc-search-variant"),
                    ("nfc-tap", "nfc-tap"),
                    ("nfc-variant", "nfc-variant"),
                    ("nfc-variant-off", "nfc-variant-off"),
                    ("ninja", "ninja"),
                    ("nintendo-game-boy", "nintendo-game-boy"),
                    ("nintendo-switch", "nintendo-switch"),
                    ("nintendo-wii", "nintendo-wii"),
                    ("nintendo-wiiu", "nintendo-wiiu"),
                    ("nix", "nix"),
                    ("nodejs", "nodejs"),
                    ("noodles", "noodles"),
                    ("not-equal", "not-equal"),
                    ("not-equal-variant", "not-equal-variant"),
                    ("note", "note"),
                    ("note-alert", "note-alert"),
                    ("note-alert-outline", "note-alert-outline"),
                    ("note-check", "note-check"),
                    ("note-check-outline", "note-check-outline"),
                    ("note-edit", "note-edit"),
                    ("note-edit-outline", "note-edit-outline"),
                    ("note-minus", "note-minus"),
                    ("note-minus-outline", "note-minus-outline"),
                    ("note-multiple", "note-multiple"),
                    ("note-multiple-outline", "note-multiple-outline"),
                    ("note-off", "note-off"),
                    ("note-off-outline", "note-off-outline"),
                    ("note-outline", "note-outline"),
                    ("note-plus", "note-plus"),
                    ("note-plus-outline", "note-plus-outline"),
                    ("note-remove", "note-remove"),
                    ("note-remove-outline", "note-remove-outline"),
                    ("note-search", "note-search"),
                    ("note-search-outline", "note-search-outline"),
                    ("note-text", "note-text"),
                    ("note-text-outline", "note-text-outline"),
                    ("notebook", "notebook"),
                    ("notebook-check", "notebook-check"),
                    ("notebook-check-outline", "notebook-check-outline"),
                    ("notebook-edit", "notebook-edit"),
                    ("notebook-edit-outline", "notebook-edit-outline"),
                    ("notebook-heart", "notebook-heart"),
                    ("notebook-heart-outline", "notebook-heart-outline"),
                    ("notebook-minus", "notebook-minus"),
                    ("notebook-minus-outline", "notebook-minus-outline"),
                    ("notebook-multiple", "notebook-multiple"),
                    ("notebook-outline", "notebook-outline"),
                    ("notebook-plus", "notebook-plus"),
                    ("notebook-plus-outline", "notebook-plus-outline"),
                    ("notebook-remove", "notebook-remove"),
                    ("notebook-remove-outline", "notebook-remove-outline"),
                    ("notification-clear-all", "notification-clear-all"),
                    ("npm", "npm"),
                    ("npm-variant", "npm-variant"),
                    ("npm-variant-outline", "npm-variant-outline"),
                    ("nuke", "nuke"),
                    ("null", "null"),
                    ("numeric", "numeric"),
                    ("numeric-0", "numeric-0"),
                    ("numeric-0-box", "numeric-0-box"),
                    ("numeric-0-box-multiple", "numeric-0-box-multiple"),
                    ("numeric-0-box-multiple-outline", "numeric-0-box-multiple-outline"),
                    ("numeric-0-box-outline", "numeric-0-box-outline"),
                    ("numeric-0-circle", "numeric-0-circle"),
                    ("numeric-0-circle-outline", "numeric-0-circle-outline"),
                    ("numeric-1", "numeric-1"),
                    ("numeric-1-box", "numeric-1-box"),
                    ("numeric-1-box-multiple", "numeric-1-box-multiple"),
                    ("numeric-1-box-multiple-outline", "numeric-1-box-multiple-outline"),
                    ("numeric-1-box-outline", "numeric-1-box-outline"),
                    ("numeric-1-circle", "numeric-1-circle"),
                    ("numeric-1-circle-outline", "numeric-1-circle-outline"),
                    ("numeric-10", "numeric-10"),
                    ("numeric-10-box", "numeric-10-box"),
                    ("numeric-10-box-multiple", "numeric-10-box-multiple"),
                    ("numeric-10-box-multiple-outline", "numeric-10-box-multiple-outline"),
                    ("numeric-10-box-outline", "numeric-10-box-outline"),
                    ("numeric-10-circle", "numeric-10-circle"),
                    ("numeric-10-circle-outline", "numeric-10-circle-outline"),
                    ("numeric-2", "numeric-2"),
                    ("numeric-2-box", "numeric-2-box"),
                    ("numeric-2-box-multiple", "numeric-2-box-multiple"),
                    ("numeric-2-box-multiple-outline", "numeric-2-box-multiple-outline"),
                    ("numeric-2-box-outline", "numeric-2-box-outline"),
                    ("numeric-2-circle", "numeric-2-circle"),
                    ("numeric-2-circle-outline", "numeric-2-circle-outline"),
                    ("numeric-3", "numeric-3"),
                    ("numeric-3-box", "numeric-3-box"),
                    ("numeric-3-box-multiple", "numeric-3-box-multiple"),
                    ("numeric-3-box-multiple-outline", "numeric-3-box-multiple-outline"),
                    ("numeric-3-box-outline", "numeric-3-box-outline"),
                    ("numeric-3-circle", "numeric-3-circle"),
                    ("numeric-3-circle-outline", "numeric-3-circle-outline"),
                    ("numeric-4", "numeric-4"),
                    ("numeric-4-box", "numeric-4-box"),
                    ("numeric-4-box-multiple", "numeric-4-box-multiple"),
                    ("numeric-4-box-multiple-outline", "numeric-4-box-multiple-outline"),
                    ("numeric-4-box-outline", "numeric-4-box-outline"),
                    ("numeric-4-circle", "numeric-4-circle"),
                    ("numeric-4-circle-outline", "numeric-4-circle-outline"),
                    ("numeric-5", "numeric-5"),
                    ("numeric-5-box", "numeric-5-box"),
                    ("numeric-5-box-multiple", "numeric-5-box-multiple"),
                    ("numeric-5-box-multiple-outline", "numeric-5-box-multiple-outline"),
                    ("numeric-5-box-outline", "numeric-5-box-outline"),
                    ("numeric-5-circle", "numeric-5-circle"),
                    ("numeric-5-circle-outline", "numeric-5-circle-outline"),
                    ("numeric-6", "numeric-6"),
                    ("numeric-6-box", "numeric-6-box"),
                    ("numeric-6-box-multiple", "numeric-6-box-multiple"),
                    ("numeric-6-box-multiple-outline", "numeric-6-box-multiple-outline"),
                    ("numeric-6-box-outline", "numeric-6-box-outline"),
                    ("numeric-6-circle", "numeric-6-circle"),
                    ("numeric-6-circle-outline", "numeric-6-circle-outline"),
                    ("numeric-7", "numeric-7"),
                    ("numeric-7-box", "numeric-7-box"),
                    ("numeric-7-box-multiple", "numeric-7-box-multiple"),
                    ("numeric-7-box-multiple-outline", "numeric-7-box-multiple-outline"),
                    ("numeric-7-box-outline", "numeric-7-box-outline"),
                    ("numeric-7-circle", "numeric-7-circle"),
                    ("numeric-7-circle-outline", "numeric-7-circle-outline"),
                    ("numeric-8", "numeric-8"),
                    ("numeric-8-box", "numeric-8-box"),
                    ("numeric-8-box-multiple", "numeric-8-box-multiple"),
                    ("numeric-8-box-multiple-outline", "numeric-8-box-multiple-outline"),
                    ("numeric-8-box-outline", "numeric-8-box-outline"),
                    ("numeric-8-circle", "numeric-8-circle"),
                    ("numeric-8-circle-outline", "numeric-8-circle-outline"),
                    ("numeric-9", "numeric-9"),
                    ("numeric-9-box", "numeric-9-box"),
                    ("numeric-9-box-multiple", "numeric-9-box-multiple"),
                    ("numeric-9-box-multiple-outline", "numeric-9-box-multiple-outline"),
                    ("numeric-9-box-outline", "numeric-9-box-outline"),
                    ("numeric-9-circle", "numeric-9-circle"),
                    ("numeric-9-circle-outline", "numeric-9-circle-outline"),
                    ("numeric-9-plus", "numeric-9-plus"),
                    ("numeric-9-plus-box", "numeric-9-plus-box"),
                    ("numeric-9-plus-box-multiple", "numeric-9-plus-box-multiple"),
                    ("numeric-9-plus-box-multiple-outline", "numeric-9-plus-box-multiple-outline"),
                    ("numeric-9-plus-box-outline", "numeric-9-plus-box-outline"),
                    ("numeric-9-plus-circle", "numeric-9-plus-circle"),
                    ("numeric-9-plus-circle-outline", "numeric-9-plus-circle-outline"),
                    ("numeric-negative-1", "numeric-negative-1"),
                    ("numeric-off", "numeric-off"),
                    ("numeric-positive-1", "numeric-positive-1"),
                    ("nut", "nut"),
                    ("nutrition", "nutrition"),
                    ("nuxt", "nuxt"),
                    ("oar", "oar"),
                    ("ocarina", "ocarina"),
                    ("oci", "oci"),
                    ("ocr", "ocr"),
                    ("octagon", "octagon"),
                    ("octagon-outline", "octagon-outline"),
                    ("octagram", "octagram"),
                    ("octagram-edit", "octagram-edit"),
                    ("octagram-edit-outline", "octagram-edit-outline"),
                    ("octagram-minus", "octagram-minus"),
                    ("octagram-minus-outline", "octagram-minus-outline"),
                    ("octagram-outline", "octagram-outline"),
                    ("octagram-plus", "octagram-plus"),
                    ("octagram-plus-outline", "octagram-plus-outline"),
                    ("octahedron", "octahedron"),
                    ("octahedron-off", "octahedron-off"),
                    ("odnoklassniki", "odnoklassniki"),
                    ("offer", "offer"),
                    ("office-building", "office-building"),
                    ("office-building-cog", "office-building-cog"),
                    ("office-building-cog-outline", "office-building-cog-outline"),
                    ("office-building-marker", "office-building-marker"),
                    ("office-building-marker-outline", "office-building-marker-outline"),
                    ("office-building-minus", "office-building-minus"),
                    ("office-building-minus-outline", "office-building-minus-outline"),
                    ("office-building-outline", "office-building-outline"),
                    ("office-building-plus", "office-building-plus"),
                    ("office-building-plus-outline", "office-building-plus-outline"),
                    ("office-building-remove", "office-building-remove"),
                    ("office-building-remove-outline", "office-building-remove-outline"),
                    ("oil", "oil"),
                    ("oil-lamp", "oil-lamp"),
                    ("oil-level", "oil-level"),
                    ("oil-temperature", "oil-temperature"),
                    ("om", "om"),
                    ("omega", "omega"),
                    ("one-up", "one-up"),
                    ("onedrive", "onedrive"),
                    ("onenote", "onenote"),
                    ("onepassword", "onepassword"),
                    ("opacity", "opacity"),
                    ("open-in-app", "open-in-app"),
                    ("open-in-new", "open-in-new"),
                    ("open-source-initiative", "open-source-initiative"),
                    ("openid", "openid"),
                    ("opera", "opera"),
                    ("orbit", "orbit"),
                    ("orbit-variant", "orbit-variant"),
                    ("order-alphabetical-ascending", "order-alphabetical-ascending"),
                    ("order-alphabetical-descending", "order-alphabetical-descending"),
                    ("order-bool-ascending", "order-bool-ascending"),
                    ("order-bool-ascending-variant", "order-bool-ascending-variant"),
                    ("order-bool-descending", "order-bool-descending"),
                    ("order-bool-descending-variant", "order-bool-descending-variant"),
                    ("order-numeric-ascending", "order-numeric-ascending"),
                    ("order-numeric-descending", "order-numeric-descending"),
                    ("origin", "origin"),
                    ("ornament", "ornament"),
                    ("ornament-variant", "ornament-variant"),
                    ("outbox", "outbox"),
                    ("outdent", "outdent"),
                    ("outdoor-lamp", "outdoor-lamp"),
                    ("outlook", "outlook"),
                    ("overscan", "overscan"),
                    ("owl", "owl"),
                    ("pac-man", "pac-man"),
                    ("package", "package"),
                    ("package-check", "package-check"),
                    ("package-down", "package-down"),
                    ("package-up", "package-up"),
                    ("package-variant", "package-variant"),
                    ("package-variant-closed", "package-variant-closed"),
                    ("package-variant-closed-check", "package-variant-closed-check"),
                    ("package-variant-closed-minus", "package-variant-closed-minus"),
                    ("package-variant-closed-plus", "package-variant-closed-plus"),
                    ("package-variant-closed-remove", "package-variant-closed-remove"),
                    ("package-variant-minus", "package-variant-minus"),
                    ("package-variant-plus", "package-variant-plus"),
                    ("package-variant-remove", "package-variant-remove"),
                    ("page-first", "page-first"),
                    ("page-last", "page-last"),
                    ("page-layout-body", "page-layout-body"),
                    ("page-layout-footer", "page-layout-footer"),
                    ("page-layout-header", "page-layout-header"),
                    ("page-layout-header-footer", "page-layout-header-footer"),
                    ("page-layout-sidebar-left", "page-layout-sidebar-left"),
                    ("page-layout-sidebar-right", "page-layout-sidebar-right"),
                    ("page-next", "page-next"),
                    ("page-next-outline", "page-next-outline"),
                    ("page-previous", "page-previous"),
                    ("page-previous-outline", "page-previous-outline"),
                    ("pail", "pail"),
                    ("pail-minus", "pail-minus"),
                    ("pail-minus-outline", "pail-minus-outline"),
                    ("pail-off", "pail-off"),
                    ("pail-off-outline", "pail-off-outline"),
                    ("pail-outline", "pail-outline"),
                    ("pail-plus", "pail-plus"),
                    ("pail-plus-outline", "pail-plus-outline"),
                    ("pail-remove", "pail-remove"),
                    ("pail-remove-outline", "pail-remove-outline"),
                    ("palette", "palette"),
                    ("palette-advanced", "palette-advanced"),
                    ("palette-outline", "palette-outline"),
                    ("palette-swatch", "palette-swatch"),
                    ("palette-swatch-outline", "palette-swatch-outline"),
                    ("palette-swatch-variant", "palette-swatch-variant"),
                    ("palm-tree", "palm-tree"),
                    ("pan", "pan"),
                    ("pan-bottom-left", "pan-bottom-left"),
                    ("pan-bottom-right", "pan-bottom-right"),
                    ("pan-down", "pan-down"),
                    ("pan-horizontal", "pan-horizontal"),
                    ("pan-left", "pan-left"),
                    ("pan-right", "pan-right"),
                    ("pan-top-left", "pan-top-left"),
                    ("pan-top-right", "pan-top-right"),
                    ("pan-up", "pan-up"),
                    ("pan-vertical", "pan-vertical"),
                    ("panda", "panda"),
                    ("pandora", "pandora"),
                    ("panorama", "panorama"),
                    ("panorama-fisheye", "panorama-fisheye"),
                    ("panorama-horizontal", "panorama-horizontal"),
                    ("panorama-horizontal-outline", "panorama-horizontal-outline"),
                    ("panorama-outline", "panorama-outline"),
                    ("panorama-sphere", "panorama-sphere"),
                    ("panorama-sphere-outline", "panorama-sphere-outline"),
                    ("panorama-variant", "panorama-variant"),
                    ("panorama-variant-outline", "panorama-variant-outline"),
                    ("panorama-vertical", "panorama-vertical"),
                    ("panorama-vertical-outline", "panorama-vertical-outline"),
                    ("panorama-wide-angle", "panorama-wide-angle"),
                    ("panorama-wide-angle-outline", "panorama-wide-angle-outline"),
                    ("paper-cut-vertical", "paper-cut-vertical"),
                    ("paper-roll", "paper-roll"),
                    ("paper-roll-outline", "paper-roll-outline"),
                    ("paperclip", "paperclip"),
                    ("paperclip-check", "paperclip-check"),
                    ("paperclip-lock", "paperclip-lock"),
                    ("paperclip-minus", "paperclip-minus"),
                    ("paperclip-off", "paperclip-off"),
                    ("paperclip-plus", "paperclip-plus"),
                    ("paperclip-remove", "paperclip-remove"),
                    ("parachute", "parachute"),
                    ("parachute-outline", "parachute-outline"),
                    ("paragliding", "paragliding"),
                    ("parking", "parking"),
                    ("party-popper", "party-popper"),
                    ("passport", "passport"),
                    ("passport-biometric", "passport-biometric"),
                    ("pasta", "pasta"),
                    ("patio-heater", "patio-heater"),
                    ("patreon", "patreon"),
                    ("pause", "pause"),
                    ("pause-box", "pause-box"),
                    ("pause-box-outline", "pause-box-outline"),
                    ("pause-circle", "pause-circle"),
                    ("pause-circle-outline", "pause-circle-outline"),
                    ("pause-octagon", "pause-octagon"),
                    ("pause-octagon-outline", "pause-octagon-outline"),
                    ("paw", "paw"),
                    ("paw-off", "paw-off"),
                    ("paw-off-outline", "paw-off-outline"),
                    ("paw-outline", "paw-outline"),
                    ("paypal", "paypal"),
                    ("peace", "peace"),
                    ("peanut", "peanut"),
                    ("peanut-off", "peanut-off"),
                    ("peanut-off-outline", "peanut-off-outline"),
                    ("peanut-outline", "peanut-outline"),
                    ("pen", "pen"),
                    ("pen-lock", "pen-lock"),
                    ("pen-minus", "pen-minus"),
                    ("pen-off", "pen-off"),
                    ("pen-plus", "pen-plus"),
                    ("pen-remove", "pen-remove"),
                    ("pencil", "pencil"),
                    ("pencil-box", "pencil-box"),
                    ("pencil-box-multiple", "pencil-box-multiple"),
                    ("pencil-box-multiple-outline", "pencil-box-multiple-outline"),
                    ("pencil-box-outline", "pencil-box-outline"),
                    ("pencil-circle", "pencil-circle"),
                    ("pencil-circle-outline", "pencil-circle-outline"),
                    ("pencil-lock", "pencil-lock"),
                    ("pencil-lock-outline", "pencil-lock-outline"),
                    ("pencil-minus", "pencil-minus"),
                    ("pencil-minus-outline", "pencil-minus-outline"),
                    ("pencil-off", "pencil-off"),
                    ("pencil-off-outline", "pencil-off-outline"),
                    ("pencil-outline", "pencil-outline"),
                    ("pencil-plus", "pencil-plus"),
                    ("pencil-plus-outline", "pencil-plus-outline"),
                    ("pencil-remove", "pencil-remove"),
                    ("pencil-remove-outline", "pencil-remove-outline"),
                    ("pencil-ruler", "pencil-ruler"),
                    ("pencil-ruler-outline", "pencil-ruler-outline"),
                    ("penguin", "penguin"),
                    ("pentagon", "pentagon"),
                    ("pentagon-outline", "pentagon-outline"),
                    ("pentagram", "pentagram"),
                    ("percent", "percent"),
                    ("percent-box", "percent-box"),
                    ("percent-box-outline", "percent-box-outline"),
                    ("percent-circle", "percent-circle"),
                    ("percent-circle-outline", "percent-circle-outline"),
                    ("percent-outline", "percent-outline"),
                    ("periodic-table", "periodic-table"),
                    ("periscope", "periscope"),
                    ("perspective-less", "perspective-less"),
                    ("perspective-more", "perspective-more"),
                    ("ph", "ph"),
                    ("phone", "phone"),
                    ("phone-alert", "phone-alert"),
                    ("phone-alert-outline", "phone-alert-outline"),
                    ("phone-bluetooth", "phone-bluetooth"),
                    ("phone-bluetooth-outline", "phone-bluetooth-outline"),
                    ("phone-cancel", "phone-cancel"),
                    ("phone-cancel-outline", "phone-cancel-outline"),
                    ("phone-check", "phone-check"),
                    ("phone-check-outline", "phone-check-outline"),
                    ("phone-classic", "phone-classic"),
                    ("phone-classic-off", "phone-classic-off"),
                    ("phone-clock", "phone-clock"),
                    ("phone-dial", "phone-dial"),
                    ("phone-dial-outline", "phone-dial-outline"),
                    ("phone-forward", "phone-forward"),
                    ("phone-forward-outline", "phone-forward-outline"),
                    ("phone-hangup", "phone-hangup"),
                    ("phone-hangup-outline", "phone-hangup-outline"),
                    ("phone-in-talk", "phone-in-talk"),
                    ("phone-in-talk-outline", "phone-in-talk-outline"),
                    ("phone-incoming", "phone-incoming"),
                    ("phone-incoming-outgoing", "phone-incoming-outgoing"),
                    ("phone-incoming-outgoing-outline", "phone-incoming-outgoing-outline"),
                    ("phone-incoming-outline", "phone-incoming-outline"),
                    ("phone-lock", "phone-lock"),
                    ("phone-lock-outline", "phone-lock-outline"),
                    ("phone-log", "phone-log"),
                    ("phone-log-outline", "phone-log-outline"),
                    ("phone-message", "phone-message"),
                    ("phone-message-outline", "phone-message-outline"),
                    ("phone-minus", "phone-minus"),
                    ("phone-minus-outline", "phone-minus-outline"),
                    ("phone-missed", "phone-missed"),
                    ("phone-missed-outline", "phone-missed-outline"),
                    ("phone-off", "phone-off"),
                    ("phone-off-outline", "phone-off-outline"),
                    ("phone-outgoing", "phone-outgoing"),
                    ("phone-outgoing-outline", "phone-outgoing-outline"),
                    ("phone-outline", "phone-outline"),
                    ("phone-paused", "phone-paused"),
                    ("phone-paused-outline", "phone-paused-outline"),
                    ("phone-plus", "phone-plus"),
                    ("phone-plus-outline", "phone-plus-outline"),
                    ("phone-refresh", "phone-refresh"),
                    ("phone-refresh-outline", "phone-refresh-outline"),
                    ("phone-remove", "phone-remove"),
                    ("phone-remove-outline", "phone-remove-outline"),
                    ("phone-return", "phone-return"),
                    ("phone-return-outline", "phone-return-outline"),
                    ("phone-ring", "phone-ring"),
                    ("phone-ring-outline", "phone-ring-outline"),
                    ("phone-rotate-landscape", "phone-rotate-landscape"),
                    ("phone-rotate-portrait", "phone-rotate-portrait"),
                    ("phone-settings", "phone-settings"),
                    ("phone-settings-outline", "phone-settings-outline"),
                    ("phone-sync", "phone-sync"),
                    ("phone-sync-outline", "phone-sync-outline"),
                    ("phone-voip", "phone-voip"),
                    ("pi", "pi"),
                    ("pi-box", "pi-box"),
                    ("pi-hole", "pi-hole"),
                    ("piano", "piano"),
                    ("piano-off", "piano-off"),
                    ("pickaxe", "pickaxe"),
                    ("picture-in-picture-bottom-right", "picture-in-picture-bottom-right"),
                    (
                        "picture-in-picture-bottom-right-outline",
                        "picture-in-picture-bottom-right-outline",
                    ),
                    ("picture-in-picture-top-right", "picture-in-picture-top-right"),
                    (
                        "picture-in-picture-top-right-outline",
                        "picture-in-picture-top-right-outline",
                    ),
                    ("pier", "pier"),
                    ("pier-crane", "pier-crane"),
                    ("pig", "pig"),
                    ("pig-variant", "pig-variant"),
                    ("pig-variant-outline", "pig-variant-outline"),
                    ("piggy-bank", "piggy-bank"),
                    ("piggy-bank-outline", "piggy-bank-outline"),
                    ("pill", "pill"),
                    ("pill-multiple", "pill-multiple"),
                    ("pill-off", "pill-off"),
                    ("pillar", "pillar"),
                    ("pin", "pin"),
                    ("pin-off", "pin-off"),
                    ("pin-off-outline", "pin-off-outline"),
                    ("pin-outline", "pin-outline"),
                    ("pine-tree", "pine-tree"),
                    ("pine-tree-box", "pine-tree-box"),
                    ("pine-tree-fire", "pine-tree-fire"),
                    ("pine-tree-variant", "pine-tree-variant"),
                    ("pine-tree-variant-outline", "pine-tree-variant-outline"),
                    ("pinterest", "pinterest"),
                    ("pinterest-box", "pinterest-box"),
                    ("pinwheel", "pinwheel"),
                    ("pinwheel-outline", "pinwheel-outline"),
                    ("pipe", "pipe"),
                    ("pipe-disconnected", "pipe-disconnected"),
                    ("pipe-leak", "pipe-leak"),
                    ("pipe-valve", "pipe-valve"),
                    ("pipe-wrench", "pipe-wrench"),
                    ("pirate", "pirate"),
                    ("pistol", "pistol"),
                    ("piston", "piston"),
                    ("pitchfork", "pitchfork"),
                    ("pizza", "pizza"),
                    ("plane-car", "plane-car"),
                    ("plane-train", "plane-train"),
                    ("play", "play"),
                    ("play-box", "play-box"),
                    ("play-box-edit-outline", "play-box-edit-outline"),
                    ("play-box-lock", "play-box-lock"),
                    ("play-box-lock-open", "play-box-lock-open"),
                    ("play-box-lock-open-outline", "play-box-lock-open-outline"),
                    ("play-box-lock-outline", "play-box-lock-outline"),
                    ("play-box-multiple", "play-box-multiple"),
                    ("play-box-multiple-outline", "play-box-multiple-outline"),
                    ("play-box-outline", "play-box-outline"),
                    ("play-circle", "play-circle"),
                    ("play-circle-outline", "play-circle-outline"),
                    ("play-network", "play-network"),
                    ("play-network-outline", "play-network-outline"),
                    ("play-outline", "play-outline"),
                    ("play-pause", "play-pause"),
                    ("play-protected-content", "play-protected-content"),
                    ("play-speed", "play-speed"),
                    ("playlist-check", "playlist-check"),
                    ("playlist-edit", "playlist-edit"),
                    ("playlist-minus", "playlist-minus"),
                    ("playlist-music", "playlist-music"),
                    ("playlist-music-outline", "playlist-music-outline"),
                    ("playlist-play", "playlist-play"),
                    ("playlist-plus", "playlist-plus"),
                    ("playlist-remove", "playlist-remove"),
                    ("playlist-star", "playlist-star"),
                    ("plex", "plex"),
                    ("pliers", "pliers"),
                    ("plus", "plus"),
                    ("plus-box", "plus-box"),
                    ("plus-box-multiple", "plus-box-multiple"),
                    ("plus-box-multiple-outline", "plus-box-multiple-outline"),
                    ("plus-box-outline", "plus-box-outline"),
                    ("plus-circle", "plus-circle"),
                    ("plus-circle-multiple", "plus-circle-multiple"),
                    ("plus-circle-multiple-outline", "plus-circle-multiple-outline"),
                    ("plus-circle-outline", "plus-circle-outline"),
                    ("plus-lock", "plus-lock"),
                    ("plus-lock-open", "plus-lock-open"),
                    ("plus-minus", "plus-minus"),
                    ("plus-minus-box", "plus-minus-box"),
                    ("plus-minus-variant", "plus-minus-variant"),
                    ("plus-network", "plus-network"),
                    ("plus-network-outline", "plus-network-outline"),
                    ("plus-outline", "plus-outline"),
                    ("plus-thick", "plus-thick"),
                    ("pocket", "pocket"),
                    ("podcast", "podcast"),
                    ("podium", "podium"),
                    ("podium-bronze", "podium-bronze"),
                    ("podium-gold", "podium-gold"),
                    ("podium-silver", "podium-silver"),
                    ("point-of-sale", "point-of-sale"),
                    ("pokeball", "pokeball"),
                    ("pokemon-go", "pokemon-go"),
                    ("poker-chip", "poker-chip"),
                    ("polaroid", "polaroid"),
                    ("police-badge", "police-badge"),
                    ("police-badge-outline", "police-badge-outline"),
                    ("police-station", "police-station"),
                    ("poll", "poll"),
                    ("polo", "polo"),
                    ("polymer", "polymer"),
                    ("pool", "pool"),
                    ("pool-thermometer", "pool-thermometer"),
                    ("popcorn", "popcorn"),
                    ("post", "post"),
                    ("post-lamp", "post-lamp"),
                    ("post-outline", "post-outline"),
                    ("postage-stamp", "postage-stamp"),
                    ("pot", "pot"),
                    ("pot-mix", "pot-mix"),
                    ("pot-mix-outline", "pot-mix-outline"),
                    ("pot-outline", "pot-outline"),
                    ("pot-steam", "pot-steam"),
                    ("pot-steam-outline", "pot-steam-outline"),
                    ("pound", "pound"),
                    ("pound-box", "pound-box"),
                    ("pound-box-outline", "pound-box-outline"),
                    ("power", "power"),
                    ("power-cycle", "power-cycle"),
                    ("power-off", "power-off"),
                    ("power-on", "power-on"),
                    ("power-plug", "power-plug"),
                    ("power-plug-battery", "power-plug-battery"),
                    ("power-plug-battery-outline", "power-plug-battery-outline"),
                    ("power-plug-off", "power-plug-off"),
                    ("power-plug-off-outline", "power-plug-off-outline"),
                    ("power-plug-outline", "power-plug-outline"),
                    ("power-settings", "power-settings"),
                    ("power-sleep", "power-sleep"),
                    ("power-socket", "power-socket"),
                    ("power-socket-au", "power-socket-au"),
                    ("power-socket-ch", "power-socket-ch"),
                    ("power-socket-de", "power-socket-de"),
                    ("power-socket-eu", "power-socket-eu"),
                    ("power-socket-fr", "power-socket-fr"),
                    ("power-socket-it", "power-socket-it"),
                    ("power-socket-jp", "power-socket-jp"),
                    ("power-socket-uk", "power-socket-uk"),
                    ("power-socket-us", "power-socket-us"),
                    ("power-standby", "power-standby"),
                    ("powershell", "powershell"),
                    ("prescription", "prescription"),
                    ("presentation", "presentation"),
                    ("presentation-play", "presentation-play"),
                    ("pretzel", "pretzel"),
                    ("prezi", "prezi"),
                    ("printer", "printer"),
                    ("printer-3d", "printer-3d"),
                    ("printer-3d-nozzle", "printer-3d-nozzle"),
                    ("printer-3d-nozzle-alert", "printer-3d-nozzle-alert"),
                    ("printer-3d-nozzle-alert-outline", "printer-3d-nozzle-alert-outline"),
                    ("printer-3d-nozzle-heat", "printer-3d-nozzle-heat"),
                    ("printer-3d-nozzle-heat-outline", "printer-3d-nozzle-heat-outline"),
                    ("printer-3d-nozzle-off", "printer-3d-nozzle-off"),
                    ("printer-3d-nozzle-off-outline", "printer-3d-nozzle-off-outline"),
                    ("printer-3d-nozzle-outline", "printer-3d-nozzle-outline"),
                    ("printer-3d-off", "printer-3d-off"),
                    ("printer-alert", "printer-alert"),
                    ("printer-check", "printer-check"),
                    ("printer-eye", "printer-eye"),
                    ("printer-off", "printer-off"),
                    ("printer-off-outline", "printer-off-outline"),
                    ("printer-outline", "printer-outline"),
                    ("printer-pos", "printer-pos"),
                    ("printer-pos-alert", "printer-pos-alert"),
                    ("printer-pos-alert-outline", "printer-pos-alert-outline"),
                    ("printer-pos-cancel", "printer-pos-cancel"),
                    ("printer-pos-cancel-outline", "printer-pos-cancel-outline"),
                    ("printer-pos-check", "printer-pos-check"),
                    ("printer-pos-check-outline", "printer-pos-check-outline"),
                    ("printer-pos-cog", "printer-pos-cog"),
                    ("printer-pos-cog-outline", "printer-pos-cog-outline"),
                    ("printer-pos-edit", "printer-pos-edit"),
                    ("printer-pos-edit-outline", "printer-pos-edit-outline"),
                    ("printer-pos-minus", "printer-pos-minus"),
                    ("printer-pos-minus-outline", "printer-pos-minus-outline"),
                    ("printer-pos-network", "printer-pos-network"),
                    ("printer-pos-network-outline", "printer-pos-network-outline"),
                    ("printer-pos-off", "printer-pos-off"),
                    ("printer-pos-off-outline", "printer-pos-off-outline"),
                    ("printer-pos-outline", "printer-pos-outline"),
                    ("printer-pos-pause", "printer-pos-pause"),
                    ("printer-pos-pause-outline", "printer-pos-pause-outline"),
                    ("printer-pos-play", "printer-pos-play"),
                    ("printer-pos-play-outline", "printer-pos-play-outline"),
                    ("printer-pos-plus", "printer-pos-plus"),
                    ("printer-pos-plus-outline", "printer-pos-plus-outline"),
                    ("printer-pos-refresh", "printer-pos-refresh"),
                    ("printer-pos-refresh-outline", "printer-pos-refresh-outline"),
                    ("printer-pos-remove", "printer-pos-remove"),
                    ("printer-pos-remove-outline", "printer-pos-remove-outline"),
                    ("printer-pos-star", "printer-pos-star"),
                    ("printer-pos-star-outline", "printer-pos-star-outline"),
                    ("printer-pos-stop", "printer-pos-stop"),
                    ("printer-pos-stop-outline", "printer-pos-stop-outline"),
                    ("printer-pos-sync", "printer-pos-sync"),
                    ("printer-pos-sync-outline", "printer-pos-sync-outline"),
                    ("printer-pos-wrench", "printer-pos-wrench"),
                    ("printer-pos-wrench-outline", "printer-pos-wrench-outline"),
                    ("printer-search", "printer-search"),
                    ("printer-settings", "printer-settings"),
                    ("printer-wireless", "printer-wireless"),
                    ("priority-high", "priority-high"),
                    ("priority-low", "priority-low"),
                    ("professional-hexagon", "professional-hexagon"),
                    ("progress-alert", "progress-alert"),
                    ("progress-check", "progress-check"),
                    ("progress-clock", "progress-clock"),
                    ("progress-close", "progress-close"),
                    ("progress-download", "progress-download"),
                    ("progress-helper", "progress-helper"),
                    ("progress-pencil", "progress-pencil"),
                    ("progress-question", "progress-question"),
                    ("progress-star", "progress-star"),
                    ("progress-star-four-points", "progress-star-four-points"),
                    ("progress-upload", "progress-upload"),
                    ("progress-wrench", "progress-wrench"),
                    ("projector", "projector"),
                    ("projector-off", "projector-off"),
                    ("projector-screen", "projector-screen"),
                    ("projector-screen-off", "projector-screen-off"),
                    ("projector-screen-off-outline", "projector-screen-off-outline"),
                    ("projector-screen-outline", "projector-screen-outline"),
                    ("projector-screen-variant", "projector-screen-variant"),
                    ("projector-screen-variant-off", "projector-screen-variant-off"),
                    (
                        "projector-screen-variant-off-outline",
                        "projector-screen-variant-off-outline",
                    ),
                    ("projector-screen-variant-outline", "projector-screen-variant-outline"),
                    ("propane-tank", "propane-tank"),
                    ("propane-tank-outline", "propane-tank-outline"),
                    ("protocol", "protocol"),
                    ("publish", "publish"),
                    ("publish-off", "publish-off"),
                    ("pulse", "pulse"),
                    ("pump", "pump"),
                    ("pump-off", "pump-off"),
                    ("pumpkin", "pumpkin"),
                    ("purse", "purse"),
                    ("purse-outline", "purse-outline"),
                    ("puzzle", "puzzle"),
                    ("puzzle-check", "puzzle-check"),
                    ("puzzle-check-outline", "puzzle-check-outline"),
                    ("puzzle-edit", "puzzle-edit"),
                    ("puzzle-edit-outline", "puzzle-edit-outline"),
                    ("puzzle-heart", "puzzle-heart"),
                    ("puzzle-heart-outline", "puzzle-heart-outline"),
                    ("puzzle-minus", "puzzle-minus"),
                    ("puzzle-minus-outline", "puzzle-minus-outline"),
                    ("puzzle-outline", "puzzle-outline"),
                    ("puzzle-plus", "puzzle-plus"),
                    ("puzzle-plus-outline", "puzzle-plus-outline"),
                    ("puzzle-remove", "puzzle-remove"),
                    ("puzzle-remove-outline", "puzzle-remove-outline"),
                    ("puzzle-star", "puzzle-star"),
                    ("puzzle-star-outline", "puzzle-star-outline"),
                    ("pyramid", "pyramid"),
                    ("pyramid-off", "pyramid-off"),
                    ("qi", "qi"),
                    ("qqchat", "qqchat"),
                    ("qrcode", "qrcode"),
                    ("qrcode-edit", "qrcode-edit"),
                    ("qrcode-minus", "qrcode-minus"),
                    ("qrcode-plus", "qrcode-plus"),
                    ("qrcode-remove", "qrcode-remove"),
                    ("qrcode-scan", "qrcode-scan"),
                    ("quadcopter", "quadcopter"),
                    ("quality-high", "quality-high"),
                    ("quality-low", "quality-low"),
                    ("quality-medium", "quality-medium"),
                    ("quick-reply", "quick-reply"),
                    ("quicktime", "quicktime"),
                    ("quora", "quora"),
                    ("rabbit", "rabbit"),
                    ("rabbit-variant", "rabbit-variant"),
                    ("rabbit-variant-outline", "rabbit-variant-outline"),
                    ("racing-helmet", "racing-helmet"),
                    ("racquetball", "racquetball"),
                    ("radar", "radar"),
                    ("radiator", "radiator"),
                    ("radiator-disabled", "radiator-disabled"),
                    ("radiator-off", "radiator-off"),
                    ("radio", "radio"),
                    ("radio-am", "radio-am"),
                    ("radio-fm", "radio-fm"),
                    ("radio-handheld", "radio-handheld"),
                    ("radio-off", "radio-off"),
                    ("radio-tower", "radio-tower"),
                    ("radioactive", "radioactive"),
                    ("radioactive-circle", "radioactive-circle"),
                    ("radioactive-circle-outline", "radioactive-circle-outline"),
                    ("radioactive-off", "radioactive-off"),
                    ("radiobox-blank", "radiobox-blank"),
                    ("radiobox-indeterminate-variant", "radiobox-indeterminate-variant"),
                    ("radiobox-marked", "radiobox-marked"),
                    ("radiology-box", "radiology-box"),
                    ("radiology-box-outline", "radiology-box-outline"),
                    ("radius", "radius"),
                    ("radius-outline", "radius-outline"),
                    ("railroad-light", "railroad-light"),
                    ("rake", "rake"),
                    ("raspberry-pi", "raspberry-pi"),
                    ("raw", "raw"),
                    ("raw-off", "raw-off"),
                    ("ray-end", "ray-end"),
                    ("ray-end-arrow", "ray-end-arrow"),
                    ("ray-start", "ray-start"),
                    ("ray-start-arrow", "ray-start-arrow"),
                    ("ray-start-end", "ray-start-end"),
                    ("ray-start-vertex-end", "ray-start-vertex-end"),
                    ("ray-vertex", "ray-vertex"),
                    ("razor-double-edge", "razor-double-edge"),
                    ("razor-single-edge", "razor-single-edge"),
                    ("rdio", "rdio"),
                    ("react", "react"),
                    ("read", "read"),
                    ("receipt", "receipt"),
                    ("receipt-clock", "receipt-clock"),
                    ("receipt-clock-outline", "receipt-clock-outline"),
                    ("receipt-outline", "receipt-outline"),
                    ("receipt-send", "receipt-send"),
                    ("receipt-send-outline", "receipt-send-outline"),
                    ("receipt-text", "receipt-text"),
                    ("receipt-text-arrow-left", "receipt-text-arrow-left"),
                    ("receipt-text-arrow-left-outline", "receipt-text-arrow-left-outline"),
                    ("receipt-text-arrow-right", "receipt-text-arrow-right"),
                    ("receipt-text-arrow-right-outline", "receipt-text-arrow-right-outline"),
                    ("receipt-text-check", "receipt-text-check"),
                    ("receipt-text-check-outline", "receipt-text-check-outline"),
                    ("receipt-text-clock", "receipt-text-clock"),
                    ("receipt-text-clock-outline", "receipt-text-clock-outline"),
                    ("receipt-text-edit", "receipt-text-edit"),
                    ("receipt-text-edit-outline", "receipt-text-edit-outline"),
                    ("receipt-text-minus", "receipt-text-minus"),
                    ("receipt-text-minus-outline", "receipt-text-minus-outline"),
                    ("receipt-text-outline", "receipt-text-outline"),
                    ("receipt-text-plus", "receipt-text-plus"),
                    ("receipt-text-plus-outline", "receipt-text-plus-outline"),
                    ("receipt-text-remove", "receipt-text-remove"),
                    ("receipt-text-remove-outline", "receipt-text-remove-outline"),
                    ("receipt-text-send", "receipt-text-send"),
                    ("receipt-text-send-outline", "receipt-text-send-outline"),
                    ("record", "record"),
                    ("record-circle", "record-circle"),
                    ("record-circle-outline", "record-circle-outline"),
                    ("record-player", "record-player"),
                    ("record-rec", "record-rec"),
                    ("rectangle", "rectangle"),
                    ("rectangle-outline", "rectangle-outline"),
                    ("recycle", "recycle"),
                    ("recycle-variant", "recycle-variant"),
                    ("reddit", "reddit"),
                    ("redhat", "redhat"),
                    ("redo", "redo"),
                    ("redo-variant", "redo-variant"),
                    ("reflect-horizontal", "reflect-horizontal"),
                    ("reflect-vertical", "reflect-vertical"),
                    ("refresh", "refresh"),
                    ("refresh-auto", "refresh-auto"),
                    ("refresh-circle", "refresh-circle"),
                    ("regex", "regex"),
                    ("registered-trademark", "registered-trademark"),
                    ("reiterate", "reiterate"),
                    ("relation-many-to-many", "relation-many-to-many"),
                    ("relation-many-to-one", "relation-many-to-one"),
                    ("relation-many-to-one-or-many", "relation-many-to-one-or-many"),
                    ("relation-many-to-only-one", "relation-many-to-only-one"),
                    ("relation-many-to-zero-or-many", "relation-many-to-zero-or-many"),
                    ("relation-many-to-zero-or-one", "relation-many-to-zero-or-one"),
                    ("relation-one-or-many-to-many", "relation-one-or-many-to-many"),
                    ("relation-one-or-many-to-one", "relation-one-or-many-to-one"),
                    ("relation-one-or-many-to-one-or-many", "relation-one-or-many-to-one-or-many"),
                    ("relation-one-or-many-to-only-one", "relation-one-or-many-to-only-one"),
                    (
                        "relation-one-or-many-to-zero-or-many",
                        "relation-one-or-many-to-zero-or-many",
                    ),
                    ("relation-one-or-many-to-zero-or-one", "relation-one-or-many-to-zero-or-one"),
                    ("relation-one-to-many", "relation-one-to-many"),
                    ("relation-one-to-one", "relation-one-to-one"),
                    ("relation-one-to-one-or-many", "relation-one-to-one-or-many"),
                    ("relation-one-to-only-one", "relation-one-to-only-one"),
                    ("relation-one-to-zero-or-many", "relation-one-to-zero-or-many"),
                    ("relation-one-to-zero-or-one", "relation-one-to-zero-or-one"),
                    ("relation-only-one-to-many", "relation-only-one-to-many"),
                    ("relation-only-one-to-one", "relation-only-one-to-one"),
                    ("relation-only-one-to-one-or-many", "relation-only-one-to-one-or-many"),
                    ("relation-only-one-to-only-one", "relation-only-one-to-only-one"),
                    ("relation-only-one-to-zero-or-many", "relation-only-one-to-zero-or-many"),
                    ("relation-only-one-to-zero-or-one", "relation-only-one-to-zero-or-one"),
                    ("relation-zero-or-many-to-many", "relation-zero-or-many-to-many"),
                    ("relation-zero-or-many-to-one", "relation-zero-or-many-to-one"),
                    (
                        "relation-zero-or-many-to-one-or-many",
                        "relation-zero-or-many-to-one-or-many",
                    ),
                    ("relation-zero-or-many-to-only-one", "relation-zero-or-many-to-only-one"),
                    (
                        "relation-zero-or-many-to-zero-or-many",
                        "relation-zero-or-many-to-zero-or-many",
                    ),
                    (
                        "relation-zero-or-many-to-zero-or-one",
                        "relation-zero-or-many-to-zero-or-one",
                    ),
                    ("relation-zero-or-one-to-many", "relation-zero-or-one-to-many"),
                    ("relation-zero-or-one-to-one", "relation-zero-or-one-to-one"),
                    ("relation-zero-or-one-to-one-or-many", "relation-zero-or-one-to-one-or-many"),
                    ("relation-zero-or-one-to-only-one", "relation-zero-or-one-to-only-one"),
                    (
                        "relation-zero-or-one-to-zero-or-many",
                        "relation-zero-or-one-to-zero-or-many",
                    ),
                    ("relation-zero-or-one-to-zero-or-one", "relation-zero-or-one-to-zero-or-one"),
                    ("relative-scale", "relative-scale"),
                    ("reload", "reload"),
                    ("reload-alert", "reload-alert"),
                    ("reminder", "reminder"),
                    ("remote", "remote"),
                    ("remote-desktop", "remote-desktop"),
                    ("remote-off", "remote-off"),
                    ("remote-tv", "remote-tv"),
                    ("remote-tv-off", "remote-tv-off"),
                    ("rename", "rename"),
                    ("rename-box", "rename-box"),
                    ("rename-box-outline", "rename-box-outline"),
                    ("rename-outline", "rename-outline"),
                    ("reorder-horizontal", "reorder-horizontal"),
                    ("reorder-vertical", "reorder-vertical"),
                    ("repeat", "repeat"),
                    ("repeat-off", "repeat-off"),
                    ("repeat-once", "repeat-once"),
                    ("repeat-variant", "repeat-variant"),
                    ("replay", "replay"),
                    ("reply", "reply"),
                    ("reply-all", "reply-all"),
                    ("reply-all-outline", "reply-all-outline"),
                    ("reply-circle", "reply-circle"),
                    ("reply-outline", "reply-outline"),
                    ("reproduction", "reproduction"),
                    ("resistor", "resistor"),
                    ("resistor-nodes", "resistor-nodes"),
                    ("resize", "resize"),
                    ("resize-bottom-right", "resize-bottom-right"),
                    ("responsive", "responsive"),
                    ("restart", "restart"),
                    ("restart-alert", "restart-alert"),
                    ("restart-off", "restart-off"),
                    ("restore", "restore"),
                    ("restore-alert", "restore-alert"),
                    ("rewind", "rewind"),
                    ("rewind-10", "rewind-10"),
                    ("rewind-15", "rewind-15"),
                    ("rewind-30", "rewind-30"),
                    ("rewind-45", "rewind-45"),
                    ("rewind-5", "rewind-5"),
                    ("rewind-60", "rewind-60"),
                    ("rewind-outline", "rewind-outline"),
                    ("rhombus", "rhombus"),
                    ("rhombus-medium", "rhombus-medium"),
                    ("rhombus-medium-outline", "rhombus-medium-outline"),
                    ("rhombus-outline", "rhombus-outline"),
                    ("rhombus-split", "rhombus-split"),
                    ("rhombus-split-outline", "rhombus-split-outline"),
                    ("ribbon", "ribbon"),
                    ("rice", "rice"),
                    ("rickshaw", "rickshaw"),
                    ("rickshaw-electric", "rickshaw-electric"),
                    ("ring", "ring"),
                    ("rivet", "rivet"),
                    ("road", "road"),
                    ("road-variant", "road-variant"),
                    ("robber", "robber"),
                    ("robot", "robot"),
                    ("robot-angry", "robot-angry"),
                    ("robot-angry-outline", "robot-angry-outline"),
                    ("robot-confused", "robot-confused"),
                    ("robot-confused-outline", "robot-confused-outline"),
                    ("robot-dead", "robot-dead"),
                    ("robot-dead-outline", "robot-dead-outline"),
                    ("robot-excited", "robot-excited"),
                    ("robot-excited-outline", "robot-excited-outline"),
                    ("robot-happy", "robot-happy"),
                    ("robot-happy-outline", "robot-happy-outline"),
                    ("robot-industrial", "robot-industrial"),
                    ("robot-industrial-outline", "robot-industrial-outline"),
                    ("robot-love", "robot-love"),
                    ("robot-love-outline", "robot-love-outline"),
                    ("robot-mower", "robot-mower"),
                    ("robot-mower-outline", "robot-mower-outline"),
                    ("robot-off", "robot-off"),
                    ("robot-off-outline", "robot-off-outline"),
                    ("robot-outline", "robot-outline"),
                    ("robot-vacuum", "robot-vacuum"),
                    ("robot-vacuum-alert", "robot-vacuum-alert"),
                    ("robot-vacuum-off", "robot-vacuum-off"),
                    ("robot-vacuum-variant", "robot-vacuum-variant"),
                    ("robot-vacuum-variant-alert", "robot-vacuum-variant-alert"),
                    ("robot-vacuum-variant-off", "robot-vacuum-variant-off"),
                    ("rocket", "rocket"),
                    ("rocket-launch", "rocket-launch"),
                    ("rocket-launch-outline", "rocket-launch-outline"),
                    ("rocket-outline", "rocket-outline"),
                    ("rodent", "rodent"),
                    ("roller-shade", "roller-shade"),
                    ("roller-shade-closed", "roller-shade-closed"),
                    ("roller-skate", "roller-skate"),
                    ("roller-skate-off", "roller-skate-off"),
                    ("rollerblade", "rollerblade"),
                    ("rollerblade-off", "rollerblade-off"),
                    ("rollupjs", "rollupjs"),
                    ("rolodex", "rolodex"),
                    ("rolodex-outline", "rolodex-outline"),
                    ("roman-numeral-1", "roman-numeral-1"),
                    ("roman-numeral-10", "roman-numeral-10"),
                    ("roman-numeral-2", "roman-numeral-2"),
                    ("roman-numeral-3", "roman-numeral-3"),
                    ("roman-numeral-4", "roman-numeral-4"),
                    ("roman-numeral-5", "roman-numeral-5"),
                    ("roman-numeral-6", "roman-numeral-6"),
                    ("roman-numeral-7", "roman-numeral-7"),
                    ("roman-numeral-8", "roman-numeral-8"),
                    ("roman-numeral-9", "roman-numeral-9"),
                    ("room-service", "room-service"),
                    ("room-service-outline", "room-service-outline"),
                    ("rotate-360", "rotate-360"),
                    ("rotate-3d", "rotate-3d"),
                    ("rotate-3d-variant", "rotate-3d-variant"),
                    ("rotate-left", "rotate-left"),
                    ("rotate-left-variant", "rotate-left-variant"),
                    ("rotate-orbit", "rotate-orbit"),
                    ("rotate-right", "rotate-right"),
                    ("rotate-right-variant", "rotate-right-variant"),
                    ("rounded-corner", "rounded-corner"),
                    ("router", "router"),
                    ("router-network", "router-network"),
                    ("router-network-wireless", "router-network-wireless"),
                    ("router-wireless", "router-wireless"),
                    ("router-wireless-off", "router-wireless-off"),
                    ("router-wireless-settings", "router-wireless-settings"),
                    ("routes", "routes"),
                    ("routes-clock", "routes-clock"),
                    ("rowing", "rowing"),
                    ("rss", "rss"),
                    ("rss-box", "rss-box"),
                    ("rss-off", "rss-off"),
                    ("rug", "rug"),
                    ("rugby", "rugby"),
                    ("ruler", "ruler"),
                    ("ruler-square", "ruler-square"),
                    ("ruler-square-compass", "ruler-square-compass"),
                    ("run", "run"),
                    ("run-fast", "run-fast"),
                    ("rv-truck", "rv-truck"),
                    ("sack", "sack"),
                    ("sack-outline", "sack-outline"),
                    ("sack-percent", "sack-percent"),
                    ("safe", "safe"),
                    ("safe-square", "safe-square"),
                    ("safe-square-outline", "safe-square-outline"),
                    ("safety-goggles", "safety-goggles"),
                    ("safety-googles", "safety-googles"),
                    ("sail-boat", "sail-boat"),
                    ("sail-boat-sink", "sail-boat-sink"),
                    ("sale", "sale"),
                    ("sale-outline", "sale-outline"),
                    ("salesforce", "salesforce"),
                    ("sass", "sass"),
                    ("satellite", "satellite"),
                    ("satellite-uplink", "satellite-uplink"),
                    ("satellite-variant", "satellite-variant"),
                    ("sausage", "sausage"),
                    ("sausage-off", "sausage-off"),
                    ("saw-blade", "saw-blade"),
                    ("sawtooth-wave", "sawtooth-wave"),
                    ("saxophone", "saxophone"),
                    ("scale", "scale"),
                    ("scale-balance", "scale-balance"),
                    ("scale-bathroom", "scale-bathroom"),
                    ("scale-off", "scale-off"),
                    ("scale-unbalanced", "scale-unbalanced"),
                    ("scan-helper", "scan-helper"),
                    ("scanner", "scanner"),
                    ("scanner-off", "scanner-off"),
                    ("scatter-plot", "scatter-plot"),
                    ("scatter-plot-outline", "scatter-plot-outline"),
                    ("scent", "scent"),
                    ("scent-off", "scent-off"),
                    ("school", "school"),
                    ("school-outline", "school-outline"),
                    ("scissors-cutting", "scissors-cutting"),
                    ("scooter", "scooter"),
                    ("scooter-electric", "scooter-electric"),
                    ("scoreboard", "scoreboard"),
                    ("scoreboard-outline", "scoreboard-outline"),
                    ("screen-rotation", "screen-rotation"),
                    ("screen-rotation-lock", "screen-rotation-lock"),
                    ("screw-flat-top", "screw-flat-top"),
                    ("screw-lag", "screw-lag"),
                    ("screw-machine-flat-top", "screw-machine-flat-top"),
                    ("screw-machine-round-top", "screw-machine-round-top"),
                    ("screw-round-top", "screw-round-top"),
                    ("screwdriver", "screwdriver"),
                    ("script", "script"),
                    ("script-outline", "script-outline"),
                    ("script-text", "script-text"),
                    ("script-text-key", "script-text-key"),
                    ("script-text-key-outline", "script-text-key-outline"),
                    ("script-text-outline", "script-text-outline"),
                    ("script-text-play", "script-text-play"),
                    ("script-text-play-outline", "script-text-play-outline"),
                    ("sd", "sd"),
                    ("seal", "seal"),
                    ("seal-variant", "seal-variant"),
                    ("search-web", "search-web"),
                    ("seat", "seat"),
                    ("seat-flat", "seat-flat"),
                    ("seat-flat-angled", "seat-flat-angled"),
                    ("seat-individual-suite", "seat-individual-suite"),
                    ("seat-legroom-extra", "seat-legroom-extra"),
                    ("seat-legroom-normal", "seat-legroom-normal"),
                    ("seat-legroom-reduced", "seat-legroom-reduced"),
                    ("seat-outline", "seat-outline"),
                    ("seat-passenger", "seat-passenger"),
                    ("seat-recline-extra", "seat-recline-extra"),
                    ("seat-recline-normal", "seat-recline-normal"),
                    ("seatbelt", "seatbelt"),
                    ("security", "security"),
                    ("security-close", "security-close"),
                    ("security-network", "security-network"),
                    ("seed", "seed"),
                    ("seed-off", "seed-off"),
                    ("seed-off-outline", "seed-off-outline"),
                    ("seed-outline", "seed-outline"),
                    ("seed-plus", "seed-plus"),
                    ("seed-plus-outline", "seed-plus-outline"),
                    ("seesaw", "seesaw"),
                    ("segment", "segment"),
                    ("select", "select"),
                    ("select-all", "select-all"),
                    ("select-arrow-down", "select-arrow-down"),
                    ("select-arrow-up", "select-arrow-up"),
                    ("select-color", "select-color"),
                    ("select-compare", "select-compare"),
                    ("select-drag", "select-drag"),
                    ("select-group", "select-group"),
                    ("select-inverse", "select-inverse"),
                    ("select-marker", "select-marker"),
                    ("select-multiple", "select-multiple"),
                    ("select-multiple-marker", "select-multiple-marker"),
                    ("select-off", "select-off"),
                    ("select-place", "select-place"),
                    ("select-remove", "select-remove"),
                    ("select-search", "select-search"),
                    ("selection", "selection"),
                    ("selection-drag", "selection-drag"),
                    ("selection-ellipse", "selection-ellipse"),
                    ("selection-ellipse-arrow-inside", "selection-ellipse-arrow-inside"),
                    ("selection-ellipse-remove", "selection-ellipse-remove"),
                    ("selection-lasso", "selection-lasso"),
                    ("selection-marker", "selection-marker"),
                    ("selection-multiple", "selection-multiple"),
                    ("selection-multiple-marker", "selection-multiple-marker"),
                    ("selection-off", "selection-off"),
                    ("selection-remove", "selection-remove"),
                    ("selection-search", "selection-search"),
                    ("semantic-web", "semantic-web"),
                    ("send", "send"),
                    ("send-check", "send-check"),
                    ("send-check-outline", "send-check-outline"),
                    ("send-circle", "send-circle"),
                    ("send-circle-outline", "send-circle-outline"),
                    ("send-clock", "send-clock"),
                    ("send-clock-outline", "send-clock-outline"),
                    ("send-lock", "send-lock"),
                    ("send-lock-outline", "send-lock-outline"),
                    ("send-outline", "send-outline"),
                    ("send-variant", "send-variant"),
                    ("send-variant-clock", "send-variant-clock"),
                    ("send-variant-clock-outline", "send-variant-clock-outline"),
                    ("send-variant-outline", "send-variant-outline"),
                    ("serial-port", "serial-port"),
                    ("server", "server"),
                    ("server-minus", "server-minus"),
                    ("server-minus-outline", "server-minus-outline"),
                    ("server-network", "server-network"),
                    ("server-network-off", "server-network-off"),
                    ("server-network-outline", "server-network-outline"),
                    ("server-off", "server-off"),
                    ("server-outline", "server-outline"),
                    ("server-plus", "server-plus"),
                    ("server-plus-outline", "server-plus-outline"),
                    ("server-remove", "server-remove"),
                    ("server-security", "server-security"),
                    ("set-all", "set-all"),
                    ("set-center", "set-center"),
                    ("set-center-right", "set-center-right"),
                    ("set-left", "set-left"),
                    ("set-left-center", "set-left-center"),
                    ("set-left-right", "set-left-right"),
                    ("set-merge", "set-merge"),
                    ("set-none", "set-none"),
                    ("set-right", "set-right"),
                    ("set-split", "set-split"),
                    ("set-square", "set-square"),
                    ("set-top-box", "set-top-box"),
                    ("settings-helper", "settings-helper"),
                    ("shaker", "shaker"),
                    ("shaker-outline", "shaker-outline"),
                    ("shape", "shape"),
                    ("shape-circle-plus", "shape-circle-plus"),
                    ("shape-outline", "shape-outline"),
                    ("shape-oval-plus", "shape-oval-plus"),
                    ("shape-plus", "shape-plus"),
                    ("shape-plus-outline", "shape-plus-outline"),
                    ("shape-polygon-plus", "shape-polygon-plus"),
                    ("shape-rectangle-plus", "shape-rectangle-plus"),
                    ("shape-square-plus", "shape-square-plus"),
                    ("shape-square-rounded-plus", "shape-square-rounded-plus"),
                    ("share", "share"),
                    ("share-all", "share-all"),
                    ("share-all-outline", "share-all-outline"),
                    ("share-circle", "share-circle"),
                    ("share-off", "share-off"),
                    ("share-off-outline", "share-off-outline"),
                    ("share-outline", "share-outline"),
                    ("share-variant", "share-variant"),
                    ("share-variant-outline", "share-variant-outline"),
                    ("shark", "shark"),
                    ("shark-fin", "shark-fin"),
                    ("shark-fin-outline", "shark-fin-outline"),
                    ("shark-off", "shark-off"),
                    ("sheep", "sheep"),
                    ("shield", "shield"),
                    ("shield-account", "shield-account"),
                    ("shield-account-outline", "shield-account-outline"),
                    ("shield-account-variant", "shield-account-variant"),
                    ("shield-account-variant-outline", "shield-account-variant-outline"),
                    ("shield-airplane", "shield-airplane"),
                    ("shield-airplane-outline", "shield-airplane-outline"),
                    ("shield-alert", "shield-alert"),
                    ("shield-alert-outline", "shield-alert-outline"),
                    ("shield-bug", "shield-bug"),
                    ("shield-bug-outline", "shield-bug-outline"),
                    ("shield-car", "shield-car"),
                    ("shield-check", "shield-check"),
                    ("shield-check-outline", "shield-check-outline"),
                    ("shield-cross", "shield-cross"),
                    ("shield-cross-outline", "shield-cross-outline"),
                    ("shield-crown", "shield-crown"),
                    ("shield-crown-outline", "shield-crown-outline"),
                    ("shield-edit", "shield-edit"),
                    ("shield-edit-outline", "shield-edit-outline"),
                    ("shield-half", "shield-half"),
                    ("shield-half-full", "shield-half-full"),
                    ("shield-home", "shield-home"),
                    ("shield-home-outline", "shield-home-outline"),
                    ("shield-key", "shield-key"),
                    ("shield-key-outline", "shield-key-outline"),
                    ("shield-link-variant", "shield-link-variant"),
                    ("shield-link-variant-outline", "shield-link-variant-outline"),
                    ("shield-lock", "shield-lock"),
                    ("shield-lock-open", "shield-lock-open"),
                    ("shield-lock-open-outline", "shield-lock-open-outline"),
                    ("shield-lock-outline", "shield-lock-outline"),
                    ("shield-moon", "shield-moon"),
                    ("shield-moon-outline", "shield-moon-outline"),
                    ("shield-off", "shield-off"),
                    ("shield-off-outline", "shield-off-outline"),
                    ("shield-outline", "shield-outline"),
                    ("shield-plus", "shield-plus"),
                    ("shield-plus-outline", "shield-plus-outline"),
                    ("shield-refresh", "shield-refresh"),
                    ("shield-refresh-outline", "shield-refresh-outline"),
                    ("shield-remove", "shield-remove"),
                    ("shield-remove-outline", "shield-remove-outline"),
                    ("shield-search", "shield-search"),
                    ("shield-star", "shield-star"),
                    ("shield-star-outline", "shield-star-outline"),
                    ("shield-sun", "shield-sun"),
                    ("shield-sun-outline", "shield-sun-outline"),
                    ("shield-sword", "shield-sword"),
                    ("shield-sword-outline", "shield-sword-outline"),
                    ("shield-sync", "shield-sync"),
                    ("shield-sync-outline", "shield-sync-outline"),
                    ("shimmer", "shimmer"),
                    ("ship-wheel", "ship-wheel"),
                    ("shipping-pallet", "shipping-pallet"),
                    ("shoe-ballet", "shoe-ballet"),
                    ("shoe-cleat", "shoe-cleat"),
                    ("shoe-formal", "shoe-formal"),
                    ("shoe-heel", "shoe-heel"),
                    ("shoe-print", "shoe-print"),
                    ("shoe-sneaker", "shoe-sneaker"),
                    ("shopify", "shopify"),
                    ("shopping", "shopping"),
                    ("shopping-music", "shopping-music"),
                    ("shopping-outline", "shopping-outline"),
                    ("shopping-search", "shopping-search"),
                    ("shopping-search-outline", "shopping-search-outline"),
                    ("shore", "shore"),
                    ("shovel", "shovel"),
                    ("shovel-off", "shovel-off"),
                    ("shower", "shower"),
                    ("shower-head", "shower-head"),
                    ("shredder", "shredder"),
                    ("shuffle", "shuffle"),
                    ("shuffle-disabled", "shuffle-disabled"),
                    ("shuffle-variant", "shuffle-variant"),
                    ("shuriken", "shuriken"),
                    ("sickle", "sickle"),
                    ("sigma", "sigma"),
                    ("sigma-lower", "sigma-lower"),
                    ("sign-caution", "sign-caution"),
                    ("sign-direction", "sign-direction"),
                    ("sign-direction-minus", "sign-direction-minus"),
                    ("sign-direction-plus", "sign-direction-plus"),
                    ("sign-direction-remove", "sign-direction-remove"),
                    ("sign-language", "sign-language"),
                    ("sign-language-outline", "sign-language-outline"),
                    ("sign-pole", "sign-pole"),
                    ("sign-real-estate", "sign-real-estate"),
                    ("sign-text", "sign-text"),
                    ("sign-yield", "sign-yield"),
                    ("signal", "signal"),
                    ("signal-2g", "signal-2g"),
                    ("signal-3g", "signal-3g"),
                    ("signal-4g", "signal-4g"),
                    ("signal-5g", "signal-5g"),
                    ("signal-cellular-1", "signal-cellular-1"),
                    ("signal-cellular-2", "signal-cellular-2"),
                    ("signal-cellular-3", "signal-cellular-3"),
                    ("signal-cellular-outline", "signal-cellular-outline"),
                    ("signal-distance-variant", "signal-distance-variant"),
                    ("signal-hspa", "signal-hspa"),
                    ("signal-hspa-plus", "signal-hspa-plus"),
                    ("signal-off", "signal-off"),
                    ("signal-variant", "signal-variant"),
                    ("signature", "signature"),
                    ("signature-freehand", "signature-freehand"),
                    ("signature-image", "signature-image"),
                    ("signature-text", "signature-text"),
                    ("silo", "silo"),
                    ("silo-outline", "silo-outline"),
                    ("silverware", "silverware"),
                    ("silverware-clean", "silverware-clean"),
                    ("silverware-fork", "silverware-fork"),
                    ("silverware-fork-knife", "silverware-fork-knife"),
                    ("silverware-spoon", "silverware-spoon"),
                    ("silverware-variant", "silverware-variant"),
                    ("sim", "sim"),
                    ("sim-alert", "sim-alert"),
                    ("sim-alert-outline", "sim-alert-outline"),
                    ("sim-off", "sim-off"),
                    ("sim-off-outline", "sim-off-outline"),
                    ("sim-outline", "sim-outline"),
                    ("simple-icons", "simple-icons"),
                    ("sina-weibo", "sina-weibo"),
                    ("sine-wave", "sine-wave"),
                    ("sitemap", "sitemap"),
                    ("sitemap-outline", "sitemap-outline"),
                    ("size-l", "size-l"),
                    ("size-m", "size-m"),
                    ("size-s", "size-s"),
                    ("size-xl", "size-xl"),
                    ("size-xs", "size-xs"),
                    ("size-xxl", "size-xxl"),
                    ("size-xxs", "size-xxs"),
                    ("size-xxxl", "size-xxxl"),
                    ("skate", "skate"),
                    ("skate-off", "skate-off"),
                    ("skateboard", "skateboard"),
                    ("skateboarding", "skateboarding"),
                    ("skew-less", "skew-less"),
                    ("skew-more", "skew-more"),
                    ("ski", "ski"),
                    ("ski-cross-country", "ski-cross-country"),
                    ("ski-water", "ski-water"),
                    ("skip-backward", "skip-backward"),
                    ("skip-backward-outline", "skip-backward-outline"),
                    ("skip-forward", "skip-forward"),
                    ("skip-forward-outline", "skip-forward-outline"),
                    ("skip-next", "skip-next"),
                    ("skip-next-circle", "skip-next-circle"),
                    ("skip-next-circle-outline", "skip-next-circle-outline"),
                    ("skip-next-outline", "skip-next-outline"),
                    ("skip-previous", "skip-previous"),
                    ("skip-previous-circle", "skip-previous-circle"),
                    ("skip-previous-circle-outline", "skip-previous-circle-outline"),
                    ("skip-previous-outline", "skip-previous-outline"),
                    ("skull", "skull"),
                    ("skull-crossbones", "skull-crossbones"),
                    ("skull-crossbones-outline", "skull-crossbones-outline"),
                    ("skull-outline", "skull-outline"),
                    ("skull-scan", "skull-scan"),
                    ("skull-scan-outline", "skull-scan-outline"),
                    ("skype", "skype"),
                    ("skype-business", "skype-business"),
                    ("slack", "slack"),
                    ("slackware", "slackware"),
                    ("slash-forward", "slash-forward"),
                    ("slash-forward-box", "slash-forward-box"),
                    ("sledding", "sledding"),
                    ("sleep", "sleep"),
                    ("sleep-off", "sleep-off"),
                    ("slide", "slide"),
                    ("slope-downhill", "slope-downhill"),
                    ("slope-uphill", "slope-uphill"),
                    ("slot-machine", "slot-machine"),
                    ("slot-machine-outline", "slot-machine-outline"),
                    ("smart-card", "smart-card"),
                    ("smart-card-off", "smart-card-off"),
                    ("smart-card-off-outline", "smart-card-off-outline"),
                    ("smart-card-outline", "smart-card-outline"),
                    ("smart-card-reader", "smart-card-reader"),
                    ("smart-card-reader-outline", "smart-card-reader-outline"),
                    ("smog", "smog"),
                    ("smoke", "smoke"),
                    ("smoke-detector", "smoke-detector"),
                    ("smoke-detector-alert", "smoke-detector-alert"),
                    ("smoke-detector-alert-outline", "smoke-detector-alert-outline"),
                    ("smoke-detector-off", "smoke-detector-off"),
                    ("smoke-detector-off-outline", "smoke-detector-off-outline"),
                    ("smoke-detector-outline", "smoke-detector-outline"),
                    ("smoke-detector-variant", "smoke-detector-variant"),
                    ("smoke-detector-variant-alert", "smoke-detector-variant-alert"),
                    ("smoke-detector-variant-off", "smoke-detector-variant-off"),
                    ("smoking", "smoking"),
                    ("smoking-off", "smoking-off"),
                    ("smoking-pipe", "smoking-pipe"),
                    ("smoking-pipe-off", "smoking-pipe-off"),
                    ("snail", "snail"),
                    ("snake", "snake"),
                    ("snapchat", "snapchat"),
                    ("snowboard", "snowboard"),
                    ("snowflake", "snowflake"),
                    ("snowflake-alert", "snowflake-alert"),
                    ("snowflake-check", "snowflake-check"),
                    ("snowflake-melt", "snowflake-melt"),
                    ("snowflake-off", "snowflake-off"),
                    ("snowflake-thermometer", "snowflake-thermometer"),
                    ("snowflake-variant", "snowflake-variant"),
                    ("snowman", "snowman"),
                    ("snowmobile", "snowmobile"),
                    ("snowshoeing", "snowshoeing"),
                    ("soccer", "soccer"),
                    ("soccer-field", "soccer-field"),
                    ("social-distance-2-meters", "social-distance-2-meters"),
                    ("social-distance-6-feet", "social-distance-6-feet"),
                    ("sofa", "sofa"),
                    ("sofa-outline", "sofa-outline"),
                    ("sofa-single", "sofa-single"),
                    ("sofa-single-outline", "sofa-single-outline"),
                    ("solar-panel", "solar-panel"),
                    ("solar-panel-large", "solar-panel-large"),
                    ("solar-power", "solar-power"),
                    ("solar-power-variant", "solar-power-variant"),
                    ("solar-power-variant-outline", "solar-power-variant-outline"),
                    ("soldering-iron", "soldering-iron"),
                    ("solid", "solid"),
                    ("sony-playstation", "sony-playstation"),
                    ("sort", "sort"),
                    ("sort-alphabetical-ascending", "sort-alphabetical-ascending"),
                    ("sort-alphabetical-ascending-variant", "sort-alphabetical-ascending-variant"),
                    ("sort-alphabetical-descending", "sort-alphabetical-descending"),
                    (
                        "sort-alphabetical-descending-variant",
                        "sort-alphabetical-descending-variant",
                    ),
                    ("sort-alphabetical-variant", "sort-alphabetical-variant"),
                    ("sort-ascending", "sort-ascending"),
                    ("sort-bool-ascending", "sort-bool-ascending"),
                    ("sort-bool-ascending-variant", "sort-bool-ascending-variant"),
                    ("sort-bool-descending", "sort-bool-descending"),
                    ("sort-bool-descending-variant", "sort-bool-descending-variant"),
                    ("sort-calendar-ascending", "sort-calendar-ascending"),
                    ("sort-calendar-descending", "sort-calendar-descending"),
                    ("sort-clock-ascending", "sort-clock-ascending"),
                    ("sort-clock-ascending-outline", "sort-clock-ascending-outline"),
                    ("sort-clock-descending", "sort-clock-descending"),
                    ("sort-clock-descending-outline", "sort-clock-descending-outline"),
                    ("sort-descending", "sort-descending"),
                    ("sort-numeric-ascending", "sort-numeric-ascending"),
                    ("sort-numeric-ascending-variant", "sort-numeric-ascending-variant"),
                    ("sort-numeric-descending", "sort-numeric-descending"),
                    ("sort-numeric-descending-variant", "sort-numeric-descending-variant"),
                    ("sort-numeric-variant", "sort-numeric-variant"),
                    ("sort-reverse-variant", "sort-reverse-variant"),
                    ("sort-variant", "sort-variant"),
                    ("sort-variant-lock", "sort-variant-lock"),
                    ("sort-variant-lock-open", "sort-variant-lock-open"),
                    ("sort-variant-off", "sort-variant-off"),
                    ("sort-variant-remove", "sort-variant-remove"),
                    ("soundbar", "soundbar"),
                    ("soundcloud", "soundcloud"),
                    ("source-branch", "source-branch"),
                    ("source-branch-check", "source-branch-check"),
                    ("source-branch-minus", "source-branch-minus"),
                    ("source-branch-plus", "source-branch-plus"),
                    ("source-branch-refresh", "source-branch-refresh"),
                    ("source-branch-remove", "source-branch-remove"),
                    ("source-branch-sync", "source-branch-sync"),
                    ("source-commit", "source-commit"),
                    ("source-commit-end", "source-commit-end"),
                    ("source-commit-end-local", "source-commit-end-local"),
                    ("source-commit-local", "source-commit-local"),
                    ("source-commit-next-local", "source-commit-next-local"),
                    ("source-commit-start", "source-commit-start"),
                    ("source-commit-start-next-local", "source-commit-start-next-local"),
                    ("source-fork", "source-fork"),
                    ("source-merge", "source-merge"),
                    ("source-pull", "source-pull"),
                    ("source-repository", "source-repository"),
                    ("source-repository-multiple", "source-repository-multiple"),
                    ("soy-sauce", "soy-sauce"),
                    ("soy-sauce-off", "soy-sauce-off"),
                    ("spa", "spa"),
                    ("spa-outline", "spa-outline"),
                    ("space-invaders", "space-invaders"),
                    ("space-station", "space-station"),
                    ("spade", "spade"),
                    ("speaker", "speaker"),
                    ("speaker-bluetooth", "speaker-bluetooth"),
                    ("speaker-message", "speaker-message"),
                    ("speaker-multiple", "speaker-multiple"),
                    ("speaker-off", "speaker-off"),
                    ("speaker-pause", "speaker-pause"),
                    ("speaker-play", "speaker-play"),
                    ("speaker-stop", "speaker-stop"),
                    ("speaker-wireless", "speaker-wireless"),
                    ("spear", "spear"),
                    ("speedometer", "speedometer"),
                    ("speedometer-medium", "speedometer-medium"),
                    ("speedometer-slow", "speedometer-slow"),
                    ("spellcheck", "spellcheck"),
                    ("sphere", "sphere"),
                    ("sphere-off", "sphere-off"),
                    ("spider", "spider"),
                    ("spider-outline", "spider-outline"),
                    ("spider-thread", "spider-thread"),
                    ("spider-web", "spider-web"),
                    ("spirit-level", "spirit-level"),
                    ("split-horizontal", "split-horizontal"),
                    ("split-vertical", "split-vertical"),
                    ("spoon-sugar", "spoon-sugar"),
                    ("spotify", "spotify"),
                    ("spotlight", "spotlight"),
                    ("spotlight-beam", "spotlight-beam"),
                    ("spray", "spray"),
                    ("spray-bottle", "spray-bottle"),
                    ("spreadsheet", "spreadsheet"),
                    ("sprinkler", "sprinkler"),
                    ("sprinkler-fire", "sprinkler-fire"),
                    ("sprinkler-variant", "sprinkler-variant"),
                    ("sprout", "sprout"),
                    ("sprout-outline", "sprout-outline"),
                    ("square", "square"),
                    ("square-circle", "square-circle"),
                    ("square-circle-outline", "square-circle-outline"),
                    ("square-edit-outline", "square-edit-outline"),
                    ("square-inc", "square-inc"),
                    ("square-inc-cash", "square-inc-cash"),
                    ("square-medium", "square-medium"),
                    ("square-medium-outline", "square-medium-outline"),
                    ("square-off", "square-off"),
                    ("square-off-outline", "square-off-outline"),
                    ("square-opacity", "square-opacity"),
                    ("square-outline", "square-outline"),
                    ("square-root", "square-root"),
                    ("square-root-box", "square-root-box"),
                    ("square-rounded", "square-rounded"),
                    ("square-rounded-badge", "square-rounded-badge"),
                    ("square-rounded-badge-outline", "square-rounded-badge-outline"),
                    ("square-rounded-outline", "square-rounded-outline"),
                    ("square-small", "square-small"),
                    ("square-wave", "square-wave"),
                    ("squeegee", "squeegee"),
                    ("ssh", "ssh"),
                    ("stack-exchange", "stack-exchange"),
                    ("stack-overflow", "stack-overflow"),
                    ("stackpath", "stackpath"),
                    ("stadium", "stadium"),
                    ("stadium-outline", "stadium-outline"),
                    ("stadium-variant", "stadium-variant"),
                    ("stairs", "stairs"),
                    ("stairs-box", "stairs-box"),
                    ("stairs-down", "stairs-down"),
                    ("stairs-up", "stairs-up"),
                    ("stamper", "stamper"),
                    ("standard-definition", "standard-definition"),
                    ("star", "star"),
                    ("star-box", "star-box"),
                    ("star-box-multiple", "star-box-multiple"),
                    ("star-box-multiple-outline", "star-box-multiple-outline"),
                    ("star-box-outline", "star-box-outline"),
                    ("star-check", "star-check"),
                    ("star-check-outline", "star-check-outline"),
                    ("star-circle", "star-circle"),
                    ("star-circle-outline", "star-circle-outline"),
                    ("star-cog", "star-cog"),
                    ("star-cog-outline", "star-cog-outline"),
                    ("star-crescent", "star-crescent"),
                    ("star-david", "star-david"),
                    ("star-face", "star-face"),
                    ("star-four-points", "star-four-points"),
                    ("star-four-points-box", "star-four-points-box"),
                    ("star-four-points-box-outline", "star-four-points-box-outline"),
                    ("star-four-points-circle", "star-four-points-circle"),
                    ("star-four-points-circle-outline", "star-four-points-circle-outline"),
                    ("star-four-points-outline", "star-four-points-outline"),
                    ("star-four-points-small", "star-four-points-small"),
                    ("star-half", "star-half"),
                    ("star-half-full", "star-half-full"),
                    ("star-minus", "star-minus"),
                    ("star-minus-outline", "star-minus-outline"),
                    ("star-off", "star-off"),
                    ("star-off-outline", "star-off-outline"),
                    ("star-outline", "star-outline"),
                    ("star-plus", "star-plus"),
                    ("star-plus-outline", "star-plus-outline"),
                    ("star-remove", "star-remove"),
                    ("star-remove-outline", "star-remove-outline"),
                    ("star-settings", "star-settings"),
                    ("star-settings-outline", "star-settings-outline"),
                    ("star-shooting", "star-shooting"),
                    ("star-shooting-outline", "star-shooting-outline"),
                    ("star-three-points", "star-three-points"),
                    ("star-three-points-outline", "star-three-points-outline"),
                    ("state-machine", "state-machine"),
                    ("steam", "steam"),
                    ("steam-box", "steam-box"),
                    ("steering", "steering"),
                    ("steering-off", "steering-off"),
                    ("step-backward", "step-backward"),
                    ("step-backward-2", "step-backward-2"),
                    ("step-forward", "step-forward"),
                    ("step-forward-2", "step-forward-2"),
                    ("stethoscope", "stethoscope"),
                    ("sticker", "sticker"),
                    ("sticker-alert", "sticker-alert"),
                    ("sticker-alert-outline", "sticker-alert-outline"),
                    ("sticker-check", "sticker-check"),
                    ("sticker-check-outline", "sticker-check-outline"),
                    ("sticker-circle-outline", "sticker-circle-outline"),
                    ("sticker-emoji", "sticker-emoji"),
                    ("sticker-minus", "sticker-minus"),
                    ("sticker-minus-outline", "sticker-minus-outline"),
                    ("sticker-outline", "sticker-outline"),
                    ("sticker-plus", "sticker-plus"),
                    ("sticker-plus-outline", "sticker-plus-outline"),
                    ("sticker-remove", "sticker-remove"),
                    ("sticker-remove-outline", "sticker-remove-outline"),
                    ("sticker-text", "sticker-text"),
                    ("sticker-text-outline", "sticker-text-outline"),
                    ("stocking", "stocking"),
                    ("stomach", "stomach"),
                    ("stool", "stool"),
                    ("stool-outline", "stool-outline"),
                    ("stop", "stop"),
                    ("stop-circle", "stop-circle"),
                    ("stop-circle-outline", "stop-circle-outline"),
                    ("storage-tank", "storage-tank"),
                    ("storage-tank-outline", "storage-tank-outline"),
                    ("store", "store"),
                    ("store-24-hour", "store-24-hour"),
                    ("store-alert", "store-alert"),
                    ("store-alert-outline", "store-alert-outline"),
                    ("store-check", "store-check"),
                    ("store-check-outline", "store-check-outline"),
                    ("store-clock", "store-clock"),
                    ("store-clock-outline", "store-clock-outline"),
                    ("store-cog", "store-cog"),
                    ("store-cog-outline", "store-cog-outline"),
                    ("store-edit", "store-edit"),
                    ("store-edit-outline", "store-edit-outline"),
                    ("store-marker", "store-marker"),
                    ("store-marker-outline", "store-marker-outline"),
                    ("store-minus", "store-minus"),
                    ("store-minus-outline", "store-minus-outline"),
                    ("store-off", "store-off"),
                    ("store-off-outline", "store-off-outline"),
                    ("store-outline", "store-outline"),
                    ("store-plus", "store-plus"),
                    ("store-plus-outline", "store-plus-outline"),
                    ("store-remove", "store-remove"),
                    ("store-remove-outline", "store-remove-outline"),
                    ("store-search", "store-search"),
                    ("store-search-outline", "store-search-outline"),
                    ("store-settings", "store-settings"),
                    ("store-settings-outline", "store-settings-outline"),
                    ("storefront", "storefront"),
                    ("storefront-check", "storefront-check"),
                    ("storefront-check-outline", "storefront-check-outline"),
                    ("storefront-edit", "storefront-edit"),
                    ("storefront-edit-outline", "storefront-edit-outline"),
                    ("storefront-minus", "storefront-minus"),
                    ("storefront-minus-outline", "storefront-minus-outline"),
                    ("storefront-outline", "storefront-outline"),
                    ("storefront-plus", "storefront-plus"),
                    ("storefront-plus-outline", "storefront-plus-outline"),
                    ("storefront-remove", "storefront-remove"),
                    ("storefront-remove-outline", "storefront-remove-outline"),
                    ("stove", "stove"),
                    ("strategy", "strategy"),
                    ("strava", "strava"),
                    ("stretch-to-page", "stretch-to-page"),
                    ("stretch-to-page-outline", "stretch-to-page-outline"),
                    ("string-lights", "string-lights"),
                    ("string-lights-off", "string-lights-off"),
                    ("subdirectory-arrow-left", "subdirectory-arrow-left"),
                    ("subdirectory-arrow-right", "subdirectory-arrow-right"),
                    ("submarine", "submarine"),
                    ("subtitles", "subtitles"),
                    ("subtitles-outline", "subtitles-outline"),
                    ("subway", "subway"),
                    ("subway-alert-variant", "subway-alert-variant"),
                    ("subway-variant", "subway-variant"),
                    ("summit", "summit"),
                    ("sun-angle", "sun-angle"),
                    ("sun-angle-outline", "sun-angle-outline"),
                    ("sun-clock", "sun-clock"),
                    ("sun-clock-outline", "sun-clock-outline"),
                    ("sun-compass", "sun-compass"),
                    ("sun-snowflake", "sun-snowflake"),
                    ("sun-snowflake-variant", "sun-snowflake-variant"),
                    ("sun-thermometer", "sun-thermometer"),
                    ("sun-thermometer-outline", "sun-thermometer-outline"),
                    ("sun-wireless", "sun-wireless"),
                    ("sun-wireless-outline", "sun-wireless-outline"),
                    ("sunglasses", "sunglasses"),
                    ("surfing", "surfing"),
                    ("surround-sound", "surround-sound"),
                    ("surround-sound-2-0", "surround-sound-2-0"),
                    ("surround-sound-2-1", "surround-sound-2-1"),
                    ("surround-sound-3-1", "surround-sound-3-1"),
                    ("surround-sound-5-1", "surround-sound-5-1"),
                    ("surround-sound-5-1-2", "surround-sound-5-1-2"),
                    ("surround-sound-7-1", "surround-sound-7-1"),
                    ("svg", "svg"),
                    ("swap-horizontal", "swap-horizontal"),
                    ("swap-horizontal-bold", "swap-horizontal-bold"),
                    ("swap-horizontal-circle", "swap-horizontal-circle"),
                    ("swap-horizontal-circle-outline", "swap-horizontal-circle-outline"),
                    ("swap-horizontal-variant", "swap-horizontal-variant"),
                    ("swap-vertical", "swap-vertical"),
                    ("swap-vertical-bold", "swap-vertical-bold"),
                    ("swap-vertical-circle", "swap-vertical-circle"),
                    ("swap-vertical-circle-outline", "swap-vertical-circle-outline"),
                    ("swap-vertical-variant", "swap-vertical-variant"),
                    ("swim", "swim"),
                    ("switch", "switch"),
                    ("sword", "sword"),
                    ("sword-cross", "sword-cross"),
                    ("syllabary-hangul", "syllabary-hangul"),
                    ("syllabary-hiragana", "syllabary-hiragana"),
                    ("syllabary-katakana", "syllabary-katakana"),
                    ("syllabary-katakana-halfwidth", "syllabary-katakana-halfwidth"),
                    ("symbol", "symbol"),
                    ("symfony", "symfony"),
                    ("synagogue", "synagogue"),
                    ("synagogue-outline", "synagogue-outline"),
                    ("sync", "sync"),
                    ("sync-alert", "sync-alert"),
                    ("sync-circle", "sync-circle"),
                    ("sync-off", "sync-off"),
                    ("tab", "tab"),
                    ("tab-minus", "tab-minus"),
                    ("tab-plus", "tab-plus"),
                    ("tab-remove", "tab-remove"),
                    ("tab-search", "tab-search"),
                    ("tab-unselected", "tab-unselected"),
                    ("table", "table"),
                    ("table-account", "table-account"),
                    ("table-alert", "table-alert"),
                    ("table-arrow-down", "table-arrow-down"),
                    ("table-arrow-left", "table-arrow-left"),
                    ("table-arrow-right", "table-arrow-right"),
                    ("table-arrow-up", "table-arrow-up"),
                    ("table-border", "table-border"),
                    ("table-cancel", "table-cancel"),
                    ("table-chair", "table-chair"),
                    ("table-check", "table-check"),
                    ("table-clock", "table-clock"),
                    ("table-cog", "table-cog"),
                    ("table-column", "table-column"),
                    ("table-column-plus-after", "table-column-plus-after"),
                    ("table-column-plus-before", "table-column-plus-before"),
                    ("table-column-remove", "table-column-remove"),
                    ("table-column-width", "table-column-width"),
                    ("table-edit", "table-edit"),
                    ("table-eye", "table-eye"),
                    ("table-eye-off", "table-eye-off"),
                    ("table-filter", "table-filter"),
                    ("table-furniture", "table-furniture"),
                    ("table-headers-eye", "table-headers-eye"),
                    ("table-headers-eye-off", "table-headers-eye-off"),
                    ("table-heart", "table-heart"),
                    ("table-key", "table-key"),
                    ("table-large", "table-large"),
                    ("table-large-plus", "table-large-plus"),
                    ("table-large-remove", "table-large-remove"),
                    ("table-lock", "table-lock"),
                    ("table-merge-cells", "table-merge-cells"),
                    ("table-minus", "table-minus"),
                    ("table-multiple", "table-multiple"),
                    ("table-network", "table-network"),
                    ("table-of-contents", "table-of-contents"),
                    ("table-off", "table-off"),
                    ("table-picnic", "table-picnic"),
                    ("table-pivot", "table-pivot"),
                    ("table-plus", "table-plus"),
                    ("table-question", "table-question"),
                    ("table-refresh", "table-refresh"),
                    ("table-remove", "table-remove"),
                    ("table-row", "table-row"),
                    ("table-row-height", "table-row-height"),
                    ("table-row-plus-after", "table-row-plus-after"),
                    ("table-row-plus-before", "table-row-plus-before"),
                    ("table-row-remove", "table-row-remove"),
                    ("table-search", "table-search"),
                    ("table-settings", "table-settings"),
                    ("table-split-cell", "table-split-cell"),
                    ("table-star", "table-star"),
                    ("table-sync", "table-sync"),
                    ("table-tennis", "table-tennis"),
                    ("tablet", "tablet"),
                    ("tablet-android", "tablet-android"),
                    ("tablet-cellphone", "tablet-cellphone"),
                    ("tablet-dashboard", "tablet-dashboard"),
                    ("tablet-ipad", "tablet-ipad"),
                    ("taco", "taco"),
                    ("tag", "tag"),
                    ("tag-arrow-down", "tag-arrow-down"),
                    ("tag-arrow-down-outline", "tag-arrow-down-outline"),
                    ("tag-arrow-left", "tag-arrow-left"),
                    ("tag-arrow-left-outline", "tag-arrow-left-outline"),
                    ("tag-arrow-right", "tag-arrow-right"),
                    ("tag-arrow-right-outline", "tag-arrow-right-outline"),
                    ("tag-arrow-up", "tag-arrow-up"),
                    ("tag-arrow-up-outline", "tag-arrow-up-outline"),
                    ("tag-check", "tag-check"),
                    ("tag-check-outline", "tag-check-outline"),
                    ("tag-edit", "tag-edit"),
                    ("tag-edit-outline", "tag-edit-outline"),
                    ("tag-faces", "tag-faces"),
                    ("tag-heart", "tag-heart"),
                    ("tag-heart-outline", "tag-heart-outline"),
                    ("tag-hidden", "tag-hidden"),
                    ("tag-minus", "tag-minus"),
                    ("tag-minus-outline", "tag-minus-outline"),
                    ("tag-multiple", "tag-multiple"),
                    ("tag-multiple-outline", "tag-multiple-outline"),
                    ("tag-off", "tag-off"),
                    ("tag-off-outline", "tag-off-outline"),
                    ("tag-outline", "tag-outline"),
                    ("tag-plus", "tag-plus"),
                    ("tag-plus-outline", "tag-plus-outline"),
                    ("tag-remove", "tag-remove"),
                    ("tag-remove-outline", "tag-remove-outline"),
                    ("tag-search", "tag-search"),
                    ("tag-search-outline", "tag-search-outline"),
                    ("tag-text", "tag-text"),
                    ("tag-text-outline", "tag-text-outline"),
                    ("tailwind", "tailwind"),
                    ("tally-mark-1", "tally-mark-1"),
                    ("tally-mark-2", "tally-mark-2"),
                    ("tally-mark-3", "tally-mark-3"),
                    ("tally-mark-4", "tally-mark-4"),
                    ("tally-mark-5", "tally-mark-5"),
                    ("tangram", "tangram"),
                    ("tank", "tank"),
                    ("tanker-truck", "tanker-truck"),
                    ("tape-drive", "tape-drive"),
                    ("tape-measure", "tape-measure"),
                    ("target", "target"),
                    ("target-account", "target-account"),
                    ("target-variant", "target-variant"),
                    ("taxi", "taxi"),
                    ("tea", "tea"),
                    ("tea-outline", "tea-outline"),
                    ("teamspeak", "teamspeak"),
                    ("teamviewer", "teamviewer"),
                    ("teddy-bear", "teddy-bear"),
                    ("telegram", "telegram"),
                    ("telescope", "telescope"),
                    ("television", "television"),
                    ("television-ambient-light", "television-ambient-light"),
                    ("television-box", "television-box"),
                    ("television-classic", "television-classic"),
                    ("television-classic-off", "television-classic-off"),
                    ("television-guide", "television-guide"),
                    ("television-off", "television-off"),
                    ("television-pause", "television-pause"),
                    ("television-play", "television-play"),
                    ("television-shimmer", "television-shimmer"),
                    ("television-speaker", "television-speaker"),
                    ("television-speaker-off", "television-speaker-off"),
                    ("television-stop", "television-stop"),
                    ("temperature-celsius", "temperature-celsius"),
                    ("temperature-fahrenheit", "temperature-fahrenheit"),
                    ("temperature-kelvin", "temperature-kelvin"),
                    ("temple-buddhist", "temple-buddhist"),
                    ("temple-buddhist-outline", "temple-buddhist-outline"),
                    ("temple-hindu", "temple-hindu"),
                    ("temple-hindu-outline", "temple-hindu-outline"),
                    ("tennis", "tennis"),
                    ("tennis-ball", "tennis-ball"),
                    ("tennis-ball-outline", "tennis-ball-outline"),
                    ("tent", "tent"),
                    ("terraform", "terraform"),
                    ("terrain", "terrain"),
                    ("test-tube", "test-tube"),
                    ("test-tube-empty", "test-tube-empty"),
                    ("test-tube-off", "test-tube-off"),
                    ("text", "text"),
                    ("text-account", "text-account"),
                    ("text-box", "text-box"),
                    ("text-box-check", "text-box-check"),
                    ("text-box-check-outline", "text-box-check-outline"),
                    ("text-box-edit", "text-box-edit"),
                    ("text-box-edit-outline", "text-box-edit-outline"),
                    ("text-box-minus", "text-box-minus"),
                    ("text-box-minus-outline", "text-box-minus-outline"),
                    ("text-box-multiple", "text-box-multiple"),
                    ("text-box-multiple-outline", "text-box-multiple-outline"),
                    ("text-box-outline", "text-box-outline"),
                    ("text-box-plus", "text-box-plus"),
                    ("text-box-plus-outline", "text-box-plus-outline"),
                    ("text-box-remove", "text-box-remove"),
                    ("text-box-remove-outline", "text-box-remove-outline"),
                    ("text-box-search", "text-box-search"),
                    ("text-box-search-outline", "text-box-search-outline"),
                    ("text-long", "text-long"),
                    ("text-recognition", "text-recognition"),
                    ("text-search", "text-search"),
                    ("text-search-variant", "text-search-variant"),
                    ("text-shadow", "text-shadow"),
                    ("text-short", "text-short"),
                    ("texture", "texture"),
                    ("texture-box", "texture-box"),
                    ("theater", "theater"),
                    ("theme-light-dark", "theme-light-dark"),
                    ("thermometer", "thermometer"),
                    ("thermometer-alert", "thermometer-alert"),
                    ("thermometer-auto", "thermometer-auto"),
                    ("thermometer-bluetooth", "thermometer-bluetooth"),
                    ("thermometer-check", "thermometer-check"),
                    ("thermometer-chevron-down", "thermometer-chevron-down"),
                    ("thermometer-chevron-up", "thermometer-chevron-up"),
                    ("thermometer-high", "thermometer-high"),
                    ("thermometer-lines", "thermometer-lines"),
                    ("thermometer-low", "thermometer-low"),
                    ("thermometer-minus", "thermometer-minus"),
                    ("thermometer-off", "thermometer-off"),
                    ("thermometer-plus", "thermometer-plus"),
                    ("thermometer-probe", "thermometer-probe"),
                    ("thermometer-probe-off", "thermometer-probe-off"),
                    ("thermometer-water", "thermometer-water"),
                    ("thermostat", "thermostat"),
                    ("thermostat-auto", "thermostat-auto"),
                    ("thermostat-box", "thermostat-box"),
                    ("thermostat-box-auto", "thermostat-box-auto"),
                    ("thermostat-cog", "thermostat-cog"),
                    ("thought-bubble", "thought-bubble"),
                    ("thought-bubble-outline", "thought-bubble-outline"),
                    ("thumb-down", "thumb-down"),
                    ("thumb-down-outline", "thumb-down-outline"),
                    ("thumb-up", "thumb-up"),
                    ("thumb-up-outline", "thumb-up-outline"),
                    ("thumbs-up-down", "thumbs-up-down"),
                    ("thumbs-up-down-outline", "thumbs-up-down-outline"),
                    ("ticket", "ticket"),
                    ("ticket-account", "ticket-account"),
                    ("ticket-confirmation", "ticket-confirmation"),
                    ("ticket-confirmation-outline", "ticket-confirmation-outline"),
                    ("ticket-outline", "ticket-outline"),
                    ("ticket-percent", "ticket-percent"),
                    ("ticket-percent-outline", "ticket-percent-outline"),
                    ("tie", "tie"),
                    ("tilde", "tilde"),
                    ("tilde-off", "tilde-off"),
                    ("timelapse", "timelapse"),
                    ("timeline", "timeline"),
                    ("timeline-alert", "timeline-alert"),
                    ("timeline-alert-outline", "timeline-alert-outline"),
                    ("timeline-check", "timeline-check"),
                    ("timeline-check-outline", "timeline-check-outline"),
                    ("timeline-clock", "timeline-clock"),
                    ("timeline-clock-outline", "timeline-clock-outline"),
                    ("timeline-minus", "timeline-minus"),
                    ("timeline-minus-outline", "timeline-minus-outline"),
                    ("timeline-outline", "timeline-outline"),
                    ("timeline-plus", "timeline-plus"),
                    ("timeline-plus-outline", "timeline-plus-outline"),
                    ("timeline-question", "timeline-question"),
                    ("timeline-question-outline", "timeline-question-outline"),
                    ("timeline-remove", "timeline-remove"),
                    ("timeline-remove-outline", "timeline-remove-outline"),
                    ("timeline-text", "timeline-text"),
                    ("timeline-text-outline", "timeline-text-outline"),
                    ("timer", "timer"),
                    ("timer-10", "timer-10"),
                    ("timer-3", "timer-3"),
                    ("timer-alert", "timer-alert"),
                    ("timer-alert-outline", "timer-alert-outline"),
                    ("timer-cancel", "timer-cancel"),
                    ("timer-cancel-outline", "timer-cancel-outline"),
                    ("timer-check", "timer-check"),
                    ("timer-check-outline", "timer-check-outline"),
                    ("timer-cog", "timer-cog"),
                    ("timer-cog-outline", "timer-cog-outline"),
                    ("timer-edit", "timer-edit"),
                    ("timer-edit-outline", "timer-edit-outline"),
                    ("timer-lock", "timer-lock"),
                    ("timer-lock-open", "timer-lock-open"),
                    ("timer-lock-open-outline", "timer-lock-open-outline"),
                    ("timer-lock-outline", "timer-lock-outline"),
                    ("timer-marker", "timer-marker"),
                    ("timer-marker-outline", "timer-marker-outline"),
                    ("timer-minus", "timer-minus"),
                    ("timer-minus-outline", "timer-minus-outline"),
                    ("timer-music", "timer-music"),
                    ("timer-music-outline", "timer-music-outline"),
                    ("timer-off", "timer-off"),
                    ("timer-off-outline", "timer-off-outline"),
                    ("timer-outline", "timer-outline"),
                    ("timer-pause", "timer-pause"),
                    ("timer-pause-outline", "timer-pause-outline"),
                    ("timer-play", "timer-play"),
                    ("timer-play-outline", "timer-play-outline"),
                    ("timer-plus", "timer-plus"),
                    ("timer-plus-outline", "timer-plus-outline"),
                    ("timer-refresh", "timer-refresh"),
                    ("timer-refresh-outline", "timer-refresh-outline"),
                    ("timer-remove", "timer-remove"),
                    ("timer-remove-outline", "timer-remove-outline"),
                    ("timer-sand", "timer-sand"),
                    ("timer-sand-complete", "timer-sand-complete"),
                    ("timer-sand-empty", "timer-sand-empty"),
                    ("timer-sand-full", "timer-sand-full"),
                    ("timer-sand-paused", "timer-sand-paused"),
                    ("timer-settings", "timer-settings"),
                    ("timer-settings-outline", "timer-settings-outline"),
                    ("timer-star", "timer-star"),
                    ("timer-star-outline", "timer-star-outline"),
                    ("timer-stop", "timer-stop"),
                    ("timer-stop-outline", "timer-stop-outline"),
                    ("timer-sync", "timer-sync"),
                    ("timer-sync-outline", "timer-sync-outline"),
                    ("timetable", "timetable"),
                    ("tire", "tire"),
                    ("toaster", "toaster"),
                    ("toaster-off", "toaster-off"),
                    ("toaster-oven", "toaster-oven"),
                    ("toggle-switch", "toggle-switch"),
                    ("toggle-switch-off", "toggle-switch-off"),
                    ("toggle-switch-off-outline", "toggle-switch-off-outline"),
                    ("toggle-switch-outline", "toggle-switch-outline"),
                    ("toggle-switch-variant", "toggle-switch-variant"),
                    ("toggle-switch-variant-off", "toggle-switch-variant-off"),
                    ("toilet", "toilet"),
                    ("toolbox", "toolbox"),
                    ("toolbox-outline", "toolbox-outline"),
                    ("tools", "tools"),
                    ("tooltip", "tooltip"),
                    ("tooltip-account", "tooltip-account"),
                    ("tooltip-cellphone", "tooltip-cellphone"),
                    ("tooltip-check", "tooltip-check"),
                    ("tooltip-check-outline", "tooltip-check-outline"),
                    ("tooltip-edit", "tooltip-edit"),
                    ("tooltip-edit-outline", "tooltip-edit-outline"),
                    ("tooltip-image", "tooltip-image"),
                    ("tooltip-image-outline", "tooltip-image-outline"),
                    ("tooltip-minus", "tooltip-minus"),
                    ("tooltip-minus-outline", "tooltip-minus-outline"),
                    ("tooltip-outline", "tooltip-outline"),
                    ("tooltip-plus", "tooltip-plus"),
                    ("tooltip-plus-outline", "tooltip-plus-outline"),
                    ("tooltip-question", "tooltip-question"),
                    ("tooltip-question-outline", "tooltip-question-outline"),
                    ("tooltip-remove", "tooltip-remove"),
                    ("tooltip-remove-outline", "tooltip-remove-outline"),
                    ("tooltip-text", "tooltip-text"),
                    ("tooltip-text-outline", "tooltip-text-outline"),
                    ("tooth", "tooth"),
                    ("tooth-outline", "tooth-outline"),
                    ("toothbrush", "toothbrush"),
                    ("toothbrush-electric", "toothbrush-electric"),
                    ("toothbrush-paste", "toothbrush-paste"),
                    ("tor", "tor"),
                    ("torch", "torch"),
                    ("tortoise", "tortoise"),
                    ("toslink", "toslink"),
                    ("touch-text-outline", "touch-text-outline"),
                    ("tournament", "tournament"),
                    ("tow-truck", "tow-truck"),
                    ("tower-beach", "tower-beach"),
                    ("tower-fire", "tower-fire"),
                    ("town-hall", "town-hall"),
                    ("toy-brick", "toy-brick"),
                    ("toy-brick-marker", "toy-brick-marker"),
                    ("toy-brick-marker-outline", "toy-brick-marker-outline"),
                    ("toy-brick-minus", "toy-brick-minus"),
                    ("toy-brick-minus-outline", "toy-brick-minus-outline"),
                    ("toy-brick-outline", "toy-brick-outline"),
                    ("toy-brick-plus", "toy-brick-plus"),
                    ("toy-brick-plus-outline", "toy-brick-plus-outline"),
                    ("toy-brick-remove", "toy-brick-remove"),
                    ("toy-brick-remove-outline", "toy-brick-remove-outline"),
                    ("toy-brick-search", "toy-brick-search"),
                    ("toy-brick-search-outline", "toy-brick-search-outline"),
                    ("track-light", "track-light"),
                    ("track-light-off", "track-light-off"),
                    ("trackpad", "trackpad"),
                    ("trackpad-lock", "trackpad-lock"),
                    ("tractor", "tractor"),
                    ("tractor-variant", "tractor-variant"),
                    ("trademark", "trademark"),
                    ("traffic-cone", "traffic-cone"),
                    ("traffic-light", "traffic-light"),
                    ("traffic-light-outline", "traffic-light-outline"),
                    ("train", "train"),
                    ("train-car", "train-car"),
                    ("train-car-autorack", "train-car-autorack"),
                    ("train-car-box", "train-car-box"),
                    ("train-car-box-full", "train-car-box-full"),
                    ("train-car-box-open", "train-car-box-open"),
                    ("train-car-caboose", "train-car-caboose"),
                    ("train-car-centerbeam", "train-car-centerbeam"),
                    ("train-car-centerbeam-full", "train-car-centerbeam-full"),
                    ("train-car-container", "train-car-container"),
                    ("train-car-flatbed", "train-car-flatbed"),
                    ("train-car-flatbed-car", "train-car-flatbed-car"),
                    ("train-car-flatbed-tank", "train-car-flatbed-tank"),
                    ("train-car-gondola", "train-car-gondola"),
                    ("train-car-gondola-full", "train-car-gondola-full"),
                    ("train-car-hopper", "train-car-hopper"),
                    ("train-car-hopper-covered", "train-car-hopper-covered"),
                    ("train-car-hopper-full", "train-car-hopper-full"),
                    ("train-car-intermodal", "train-car-intermodal"),
                    ("train-car-passenger", "train-car-passenger"),
                    ("train-car-passenger-door", "train-car-passenger-door"),
                    ("train-car-passenger-door-open", "train-car-passenger-door-open"),
                    ("train-car-passenger-variant", "train-car-passenger-variant"),
                    ("train-car-tank", "train-car-tank"),
                    ("train-variant", "train-variant"),
                    ("tram", "tram"),
                    ("tram-side", "tram-side"),
                    ("transcribe", "transcribe"),
                    ("transcribe-close", "transcribe-close"),
                    ("transfer", "transfer"),
                    ("transfer-down", "transfer-down"),
                    ("transfer-left", "transfer-left"),
                    ("transfer-right", "transfer-right"),
                    ("transfer-up", "transfer-up"),
                    ("transit-connection", "transit-connection"),
                    ("transit-connection-horizontal", "transit-connection-horizontal"),
                    ("transit-connection-variant", "transit-connection-variant"),
                    ("transit-detour", "transit-detour"),
                    ("transit-skip", "transit-skip"),
                    ("transit-transfer", "transit-transfer"),
                    ("transition", "transition"),
                    ("transition-masked", "transition-masked"),
                    ("translate", "translate"),
                    ("translate-off", "translate-off"),
                    ("translate-variant", "translate-variant"),
                    ("transmission-tower", "transmission-tower"),
                    ("transmission-tower-export", "transmission-tower-export"),
                    ("transmission-tower-import", "transmission-tower-import"),
                    ("transmission-tower-off", "transmission-tower-off"),
                    ("trash-can", "trash-can"),
                    ("trash-can-outline", "trash-can-outline"),
                    ("tray", "tray"),
                    ("tray-alert", "tray-alert"),
                    ("tray-arrow-down", "tray-arrow-down"),
                    ("tray-arrow-up", "tray-arrow-up"),
                    ("tray-full", "tray-full"),
                    ("tray-minus", "tray-minus"),
                    ("tray-plus", "tray-plus"),
                    ("tray-remove", "tray-remove"),
                    ("treasure-chest", "treasure-chest"),
                    ("treasure-chest-outline", "treasure-chest-outline"),
                    ("tree", "tree"),
                    ("tree-outline", "tree-outline"),
                    ("trello", "trello"),
                    ("trending-down", "trending-down"),
                    ("trending-neutral", "trending-neutral"),
                    ("trending-up", "trending-up"),
                    ("triangle", "triangle"),
                    ("triangle-down", "triangle-down"),
                    ("triangle-down-outline", "triangle-down-outline"),
                    ("triangle-outline", "triangle-outline"),
                    ("triangle-small-down", "triangle-small-down"),
                    ("triangle-small-up", "triangle-small-up"),
                    ("triangle-wave", "triangle-wave"),
                    ("triforce", "triforce"),
                    ("trophy", "trophy"),
                    ("trophy-award", "trophy-award"),
                    ("trophy-broken", "trophy-broken"),
                    ("trophy-outline", "trophy-outline"),
                    ("trophy-variant", "trophy-variant"),
                    ("trophy-variant-outline", "trophy-variant-outline"),
                    ("truck", "truck"),
                    ("truck-alert", "truck-alert"),
                    ("truck-alert-outline", "truck-alert-outline"),
                    ("truck-cargo-container", "truck-cargo-container"),
                    ("truck-check", "truck-check"),
                    ("truck-check-outline", "truck-check-outline"),
                    ("truck-delivery", "truck-delivery"),
                    ("truck-delivery-outline", "truck-delivery-outline"),
                    ("truck-fast", "truck-fast"),
                    ("truck-fast-outline", "truck-fast-outline"),
                    ("truck-flatbed", "truck-flatbed"),
                    ("truck-minus", "truck-minus"),
                    ("truck-minus-outline", "truck-minus-outline"),
                    ("truck-off-road", "truck-off-road"),
                    ("truck-off-road-off", "truck-off-road-off"),
                    ("truck-outline", "truck-outline"),
                    ("truck-plus", "truck-plus"),
                    ("truck-plus-outline", "truck-plus-outline"),
                    ("truck-remove", "truck-remove"),
                    ("truck-remove-outline", "truck-remove-outline"),
                    ("truck-snowflake", "truck-snowflake"),
                    ("truck-trailer", "truck-trailer"),
                    ("trumpet", "trumpet"),
                    ("tshirt-crew", "tshirt-crew"),
                    ("tshirt-crew-outline", "tshirt-crew-outline"),
                    ("tshirt-v", "tshirt-v"),
                    ("tshirt-v-outline", "tshirt-v-outline"),
                    ("tsunami", "tsunami"),
                    ("tumble-dryer", "tumble-dryer"),
                    ("tumble-dryer-alert", "tumble-dryer-alert"),
                    ("tumble-dryer-off", "tumble-dryer-off"),
                    ("tumblr", "tumblr"),
                    ("tumblr-box", "tumblr-box"),
                    ("tumblr-reblog", "tumblr-reblog"),
                    ("tune", "tune"),
                    ("tune-variant", "tune-variant"),
                    ("tune-vertical", "tune-vertical"),
                    ("tune-vertical-variant", "tune-vertical-variant"),
                    ("tunnel", "tunnel"),
                    ("tunnel-outline", "tunnel-outline"),
                    ("turbine", "turbine"),
                    ("turkey", "turkey"),
                    ("turnstile", "turnstile"),
                    ("turnstile-outline", "turnstile-outline"),
                    ("turtle", "turtle"),
                    ("twitch", "twitch"),
                    ("twitter", "twitter"),
                    ("twitter-box", "twitter-box"),
                    ("twitter-circle", "twitter-circle"),
                    ("two-factor-authentication", "two-factor-authentication"),
                    ("typewriter", "typewriter"),
                    ("uber", "uber"),
                    ("ubisoft", "ubisoft"),
                    ("ubuntu", "ubuntu"),
                    ("ufo", "ufo"),
                    ("ufo-outline", "ufo-outline"),
                    ("ultra-high-definition", "ultra-high-definition"),
                    ("umbraco", "umbraco"),
                    ("umbrella", "umbrella"),
                    ("umbrella-beach", "umbrella-beach"),
                    ("umbrella-beach-outline", "umbrella-beach-outline"),
                    ("umbrella-closed", "umbrella-closed"),
                    ("umbrella-closed-outline", "umbrella-closed-outline"),
                    ("umbrella-closed-variant", "umbrella-closed-variant"),
                    ("umbrella-outline", "umbrella-outline"),
                    ("undo", "undo"),
                    ("undo-variant", "undo-variant"),
                    ("unfold-less-horizontal", "unfold-less-horizontal"),
                    ("unfold-less-vertical", "unfold-less-vertical"),
                    ("unfold-more-horizontal", "unfold-more-horizontal"),
                    ("unfold-more-vertical", "unfold-more-vertical"),
                    ("ungroup", "ungroup"),
                    ("unicode", "unicode"),
                    ("unicorn", "unicorn"),
                    ("unicorn-variant", "unicorn-variant"),
                    ("unicycle", "unicycle"),
                    ("unity", "unity"),
                    ("unreal", "unreal"),
                    ("untappd", "untappd"),
                    ("update", "update"),
                    ("upload", "upload"),
                    ("upload-lock", "upload-lock"),
                    ("upload-lock-outline", "upload-lock-outline"),
                    ("upload-multiple", "upload-multiple"),
                    ("upload-network", "upload-network"),
                    ("upload-network-outline", "upload-network-outline"),
                    ("upload-off", "upload-off"),
                    ("upload-off-outline", "upload-off-outline"),
                    ("upload-outline", "upload-outline"),
                    ("usb", "usb"),
                    ("usb-flash-drive", "usb-flash-drive"),
                    ("usb-flash-drive-outline", "usb-flash-drive-outline"),
                    ("usb-port", "usb-port"),
                    ("vacuum", "vacuum"),
                    ("vacuum-outline", "vacuum-outline"),
                    ("valve", "valve"),
                    ("valve-closed", "valve-closed"),
                    ("valve-open", "valve-open"),
                    ("van-passenger", "van-passenger"),
                    ("van-utility", "van-utility"),
                    ("vanish", "vanish"),
                    ("vanish-quarter", "vanish-quarter"),
                    ("vanity-light", "vanity-light"),
                    ("variable", "variable"),
                    ("variable-box", "variable-box"),
                    ("vector-arrange-above", "vector-arrange-above"),
                    ("vector-arrange-below", "vector-arrange-below"),
                    ("vector-bezier", "vector-bezier"),
                    ("vector-circle", "vector-circle"),
                    ("vector-circle-variant", "vector-circle-variant"),
                    ("vector-combine", "vector-combine"),
                    ("vector-curve", "vector-curve"),
                    ("vector-difference", "vector-difference"),
                    ("vector-difference-ab", "vector-difference-ab"),
                    ("vector-difference-ba", "vector-difference-ba"),
                    ("vector-ellipse", "vector-ellipse"),
                    ("vector-intersection", "vector-intersection"),
                    ("vector-line", "vector-line"),
                    ("vector-link", "vector-link"),
                    ("vector-point", "vector-point"),
                    ("vector-point-edit", "vector-point-edit"),
                    ("vector-point-minus", "vector-point-minus"),
                    ("vector-point-plus", "vector-point-plus"),
                    ("vector-point-select", "vector-point-select"),
                    ("vector-polygon", "vector-polygon"),
                    ("vector-polygon-variant", "vector-polygon-variant"),
                    ("vector-polyline", "vector-polyline"),
                    ("vector-polyline-edit", "vector-polyline-edit"),
                    ("vector-polyline-minus", "vector-polyline-minus"),
                    ("vector-polyline-plus", "vector-polyline-plus"),
                    ("vector-polyline-remove", "vector-polyline-remove"),
                    ("vector-radius", "vector-radius"),
                    ("vector-rectangle", "vector-rectangle"),
                    ("vector-selection", "vector-selection"),
                    ("vector-square", "vector-square"),
                    ("vector-square-close", "vector-square-close"),
                    ("vector-square-edit", "vector-square-edit"),
                    ("vector-square-minus", "vector-square-minus"),
                    ("vector-square-open", "vector-square-open"),
                    ("vector-square-plus", "vector-square-plus"),
                    ("vector-square-remove", "vector-square-remove"),
                    ("vector-triangle", "vector-triangle"),
                    ("vector-union", "vector-union"),
                    ("venmo", "venmo"),
                    ("vhs", "vhs"),
                    ("vibrate", "vibrate"),
                    ("vibrate-off", "vibrate-off"),
                    ("video", "video"),
                    ("video-2d", "video-2d"),
                    ("video-3d", "video-3d"),
                    ("video-3d-off", "video-3d-off"),
                    ("video-3d-variant", "video-3d-variant"),
                    ("video-4k-box", "video-4k-box"),
                    ("video-account", "video-account"),
                    ("video-box", "video-box"),
                    ("video-box-off", "video-box-off"),
                    ("video-check", "video-check"),
                    ("video-check-outline", "video-check-outline"),
                    ("video-high-definition", "video-high-definition"),
                    ("video-image", "video-image"),
                    ("video-input-antenna", "video-input-antenna"),
                    ("video-input-component", "video-input-component"),
                    ("video-input-hdmi", "video-input-hdmi"),
                    ("video-input-scart", "video-input-scart"),
                    ("video-input-svideo", "video-input-svideo"),
                    ("video-marker", "video-marker"),
                    ("video-marker-outline", "video-marker-outline"),
                    ("video-minus", "video-minus"),
                    ("video-minus-outline", "video-minus-outline"),
                    ("video-off", "video-off"),
                    ("video-off-outline", "video-off-outline"),
                    ("video-outline", "video-outline"),
                    ("video-plus", "video-plus"),
                    ("video-plus-outline", "video-plus-outline"),
                    ("video-stabilization", "video-stabilization"),
                    ("video-standard-definition", "video-standard-definition"),
                    ("video-switch", "video-switch"),
                    ("video-switch-outline", "video-switch-outline"),
                    ("video-vintage", "video-vintage"),
                    ("video-wireless", "video-wireless"),
                    ("video-wireless-outline", "video-wireless-outline"),
                    ("view-agenda", "view-agenda"),
                    ("view-agenda-outline", "view-agenda-outline"),
                    ("view-array", "view-array"),
                    ("view-array-outline", "view-array-outline"),
                    ("view-carousel", "view-carousel"),
                    ("view-carousel-outline", "view-carousel-outline"),
                    ("view-column", "view-column"),
                    ("view-column-outline", "view-column-outline"),
                    ("view-comfy", "view-comfy"),
                    ("view-comfy-outline", "view-comfy-outline"),
                    ("view-compact", "view-compact"),
                    ("view-compact-outline", "view-compact-outline"),
                    ("view-dashboard", "view-dashboard"),
                    ("view-dashboard-edit", "view-dashboard-edit"),
                    ("view-dashboard-edit-outline", "view-dashboard-edit-outline"),
                    ("view-dashboard-outline", "view-dashboard-outline"),
                    ("view-dashboard-variant", "view-dashboard-variant"),
                    ("view-dashboard-variant-outline", "view-dashboard-variant-outline"),
                    ("view-day", "view-day"),
                    ("view-day-outline", "view-day-outline"),
                    ("view-gallery", "view-gallery"),
                    ("view-gallery-outline", "view-gallery-outline"),
                    ("view-grid", "view-grid"),
                    ("view-grid-compact", "view-grid-compact"),
                    ("view-grid-outline", "view-grid-outline"),
                    ("view-grid-plus", "view-grid-plus"),
                    ("view-grid-plus-outline", "view-grid-plus-outline"),
                    ("view-headline", "view-headline"),
                    ("view-list", "view-list"),
                    ("view-list-outline", "view-list-outline"),
                    ("view-module", "view-module"),
                    ("view-module-outline", "view-module-outline"),
                    ("view-parallel", "view-parallel"),
                    ("view-parallel-outline", "view-parallel-outline"),
                    ("view-quilt", "view-quilt"),
                    ("view-quilt-outline", "view-quilt-outline"),
                    ("view-sequential", "view-sequential"),
                    ("view-sequential-outline", "view-sequential-outline"),
                    ("view-split-horizontal", "view-split-horizontal"),
                    ("view-split-vertical", "view-split-vertical"),
                    ("view-stream", "view-stream"),
                    ("view-stream-outline", "view-stream-outline"),
                    ("view-week", "view-week"),
                    ("view-week-outline", "view-week-outline"),
                    ("vimeo", "vimeo"),
                    ("vine", "vine"),
                    ("violin", "violin"),
                    ("virtual-reality", "virtual-reality"),
                    ("virus", "virus"),
                    ("virus-off", "virus-off"),
                    ("virus-off-outline", "virus-off-outline"),
                    ("virus-outline", "virus-outline"),
                    ("vk", "vk"),
                    ("vk-box", "vk-box"),
                    ("vk-circle", "vk-circle"),
                    ("vlc", "vlc"),
                    ("voicemail", "voicemail"),
                    ("volcano", "volcano"),
                    ("volcano-outline", "volcano-outline"),
                    ("volleyball", "volleyball"),
                    ("volume", "volume"),
                    ("volume-equal", "volume-equal"),
                    ("volume-high", "volume-high"),
                    ("volume-low", "volume-low"),
                    ("volume-medium", "volume-medium"),
                    ("volume-minus", "volume-minus"),
                    ("volume-mute", "volume-mute"),
                    ("volume-off", "volume-off"),
                    ("volume-plus", "volume-plus"),
                    ("volume-source", "volume-source"),
                    ("volume-variant-off", "volume-variant-off"),
                    ("volume-vibrate", "volume-vibrate"),
                    ("vote", "vote"),
                    ("vote-outline", "vote-outline"),
                    ("vpn", "vpn"),
                    ("vuejs", "vuejs"),
                    ("vuetify", "vuetify"),
                    ("walk", "walk"),
                    ("wall", "wall"),
                    ("wall-fire", "wall-fire"),
                    ("wall-sconce", "wall-sconce"),
                    ("wall-sconce-flat", "wall-sconce-flat"),
                    ("wall-sconce-flat-outline", "wall-sconce-flat-outline"),
                    ("wall-sconce-flat-variant", "wall-sconce-flat-variant"),
                    ("wall-sconce-flat-variant-outline", "wall-sconce-flat-variant-outline"),
                    ("wall-sconce-outline", "wall-sconce-outline"),
                    ("wall-sconce-round", "wall-sconce-round"),
                    ("wall-sconce-round-outline", "wall-sconce-round-outline"),
                    ("wall-sconce-round-variant", "wall-sconce-round-variant"),
                    ("wall-sconce-round-variant-outline", "wall-sconce-round-variant-outline"),
                    ("wall-sconce-variant", "wall-sconce-variant"),
                    ("wallet", "wallet"),
                    ("wallet-bifold", "wallet-bifold"),
                    ("wallet-bifold-outline", "wallet-bifold-outline"),
                    ("wallet-giftcard", "wallet-giftcard"),
                    ("wallet-membership", "wallet-membership"),
                    ("wallet-outline", "wallet-outline"),
                    ("wallet-plus", "wallet-plus"),
                    ("wallet-plus-outline", "wallet-plus-outline"),
                    ("wallet-travel", "wallet-travel"),
                    ("wallpaper", "wallpaper"),
                    ("wan", "wan"),
                    ("wardrobe", "wardrobe"),
                    ("wardrobe-outline", "wardrobe-outline"),
                    ("warehouse", "warehouse"),
                    ("washing-machine", "washing-machine"),
                    ("washing-machine-alert", "washing-machine-alert"),
                    ("washing-machine-off", "washing-machine-off"),
                    ("watch", "watch"),
                    ("watch-export", "watch-export"),
                    ("watch-export-variant", "watch-export-variant"),
                    ("watch-import", "watch-import"),
                    ("watch-import-variant", "watch-import-variant"),
                    ("watch-variant", "watch-variant"),
                    ("watch-vibrate", "watch-vibrate"),
                    ("watch-vibrate-off", "watch-vibrate-off"),
                    ("water", "water"),
                    ("water-alert", "water-alert"),
                    ("water-alert-outline", "water-alert-outline"),
                    ("water-boiler", "water-boiler"),
                    ("water-boiler-alert", "water-boiler-alert"),
                    ("water-boiler-auto", "water-boiler-auto"),
                    ("water-boiler-off", "water-boiler-off"),
                    ("water-check", "water-check"),
                    ("water-check-outline", "water-check-outline"),
                    ("water-circle", "water-circle"),
                    ("water-minus", "water-minus"),
                    ("water-minus-outline", "water-minus-outline"),
                    ("water-off", "water-off"),
                    ("water-off-outline", "water-off-outline"),
                    ("water-opacity", "water-opacity"),
                    ("water-outline", "water-outline"),
                    ("water-percent", "water-percent"),
                    ("water-percent-alert", "water-percent-alert"),
                    ("water-plus", "water-plus"),
                    ("water-plus-outline", "water-plus-outline"),
                    ("water-polo", "water-polo"),
                    ("water-pump", "water-pump"),
                    ("water-pump-off", "water-pump-off"),
                    ("water-remove", "water-remove"),
                    ("water-remove-outline", "water-remove-outline"),
                    ("water-sync", "water-sync"),
                    ("water-thermometer", "water-thermometer"),
                    ("water-thermometer-outline", "water-thermometer-outline"),
                    ("water-well", "water-well"),
                    ("water-well-outline", "water-well-outline"),
                    ("waterfall", "waterfall"),
                    ("watering-can", "watering-can"),
                    ("watering-can-outline", "watering-can-outline"),
                    ("watermark", "watermark"),
                    ("wave", "wave"),
                    ("waveform", "waveform"),
                    ("waves", "waves"),
                    ("waves-arrow-left", "waves-arrow-left"),
                    ("waves-arrow-right", "waves-arrow-right"),
                    ("waves-arrow-up", "waves-arrow-up"),
                    ("waze", "waze"),
                    ("weather-cloudy", "weather-cloudy"),
                    ("weather-cloudy-alert", "weather-cloudy-alert"),
                    ("weather-cloudy-arrow-right", "weather-cloudy-arrow-right"),
                    ("weather-cloudy-clock", "weather-cloudy-clock"),
                    ("weather-dust", "weather-dust"),
                    ("weather-fog", "weather-fog"),
                    ("weather-hail", "weather-hail"),
                    ("weather-hazy", "weather-hazy"),
                    ("weather-hurricane", "weather-hurricane"),
                    ("weather-hurricane-outline", "weather-hurricane-outline"),
                    ("weather-lightning", "weather-lightning"),
                    ("weather-lightning-rainy", "weather-lightning-rainy"),
                    ("weather-night", "weather-night"),
                    ("weather-night-partly-cloudy", "weather-night-partly-cloudy"),
                    ("weather-partly-cloudy", "weather-partly-cloudy"),
                    ("weather-partly-lightning", "weather-partly-lightning"),
                    ("weather-partly-rainy", "weather-partly-rainy"),
                    ("weather-partly-snowy", "weather-partly-snowy"),
                    ("weather-partly-snowy-rainy", "weather-partly-snowy-rainy"),
                    ("weather-pouring", "weather-pouring"),
                    ("weather-rainy", "weather-rainy"),
                    ("weather-snowy", "weather-snowy"),
                    ("weather-snowy-heavy", "weather-snowy-heavy"),
                    ("weather-snowy-rainy", "weather-snowy-rainy"),
                    ("weather-sunny", "weather-sunny"),
                    ("weather-sunny-alert", "weather-sunny-alert"),
                    ("weather-sunny-off", "weather-sunny-off"),
                    ("weather-sunset", "weather-sunset"),
                    ("weather-sunset-down", "weather-sunset-down"),
                    ("weather-sunset-up", "weather-sunset-up"),
                    ("weather-tornado", "weather-tornado"),
                    ("weather-windy", "weather-windy"),
                    ("weather-windy-variant", "weather-windy-variant"),
                    ("web", "web"),
                    ("web-box", "web-box"),
                    ("web-cancel", "web-cancel"),
                    ("web-check", "web-check"),
                    ("web-clock", "web-clock"),
                    ("web-minus", "web-minus"),
                    ("web-off", "web-off"),
                    ("web-plus", "web-plus"),
                    ("web-refresh", "web-refresh"),
                    ("web-remove", "web-remove"),
                    ("web-sync", "web-sync"),
                    ("webcam", "webcam"),
                    ("webcam-off", "webcam-off"),
                    ("webhook", "webhook"),
                    ("webpack", "webpack"),
                    ("webrtc", "webrtc"),
                    ("wechat", "wechat"),
                    ("weight", "weight"),
                    ("weight-gram", "weight-gram"),
                    ("weight-kilogram", "weight-kilogram"),
                    ("weight-lifter", "weight-lifter"),
                    ("weight-pound", "weight-pound"),
                    ("whatsapp", "whatsapp"),
                    ("wheel-barrow", "wheel-barrow"),
                    ("wheelchair", "wheelchair"),
                    ("wheelchair-accessibility", "wheelchair-accessibility"),
                    ("whistle", "whistle"),
                    ("whistle-outline", "whistle-outline"),
                    ("white-balance-auto", "white-balance-auto"),
                    ("white-balance-incandescent", "white-balance-incandescent"),
                    ("white-balance-iridescent", "white-balance-iridescent"),
                    ("white-balance-sunny", "white-balance-sunny"),
                    ("widgets", "widgets"),
                    ("widgets-outline", "widgets-outline"),
                    ("wifi", "wifi"),
                    ("wifi-alert", "wifi-alert"),
                    ("wifi-arrow-down", "wifi-arrow-down"),
                    ("wifi-arrow-left", "wifi-arrow-left"),
                    ("wifi-arrow-left-right", "wifi-arrow-left-right"),
                    ("wifi-arrow-right", "wifi-arrow-right"),
                    ("wifi-arrow-up", "wifi-arrow-up"),
                    ("wifi-arrow-up-down", "wifi-arrow-up-down"),
                    ("wifi-cancel", "wifi-cancel"),
                    ("wifi-check", "wifi-check"),
                    ("wifi-cog", "wifi-cog"),
                    ("wifi-lock", "wifi-lock"),
                    ("wifi-lock-open", "wifi-lock-open"),
                    ("wifi-marker", "wifi-marker"),
                    ("wifi-minus", "wifi-minus"),
                    ("wifi-off", "wifi-off"),
                    ("wifi-plus", "wifi-plus"),
                    ("wifi-refresh", "wifi-refresh"),
                    ("wifi-remove", "wifi-remove"),
                    ("wifi-settings", "wifi-settings"),
                    ("wifi-star", "wifi-star"),
                    ("wifi-strength-1", "wifi-strength-1"),
                    ("wifi-strength-1-alert", "wifi-strength-1-alert"),
                    ("wifi-strength-1-lock", "wifi-strength-1-lock"),
                    ("wifi-strength-1-lock-open", "wifi-strength-1-lock-open"),
                    ("wifi-strength-2", "wifi-strength-2"),
                    ("wifi-strength-2-alert", "wifi-strength-2-alert"),
                    ("wifi-strength-2-lock", "wifi-strength-2-lock"),
                    ("wifi-strength-2-lock-open", "wifi-strength-2-lock-open"),
                    ("wifi-strength-3", "wifi-strength-3"),
                    ("wifi-strength-3-alert", "wifi-strength-3-alert"),
                    ("wifi-strength-3-lock", "wifi-strength-3-lock"),
                    ("wifi-strength-3-lock-open", "wifi-strength-3-lock-open"),
                    ("wifi-strength-4", "wifi-strength-4"),
                    ("wifi-strength-4-alert", "wifi-strength-4-alert"),
                    ("wifi-strength-4-lock", "wifi-strength-4-lock"),
                    ("wifi-strength-4-lock-open", "wifi-strength-4-lock-open"),
                    ("wifi-strength-alert-outline", "wifi-strength-alert-outline"),
                    ("wifi-strength-lock-open-outline", "wifi-strength-lock-open-outline"),
                    ("wifi-strength-lock-outline", "wifi-strength-lock-outline"),
                    ("wifi-strength-off", "wifi-strength-off"),
                    ("wifi-strength-off-outline", "wifi-strength-off-outline"),
                    ("wifi-strength-outline", "wifi-strength-outline"),
                    ("wifi-sync", "wifi-sync"),
                    ("wikipedia", "wikipedia"),
                    ("wind-power", "wind-power"),
                    ("wind-power-outline", "wind-power-outline"),
                    ("wind-turbine", "wind-turbine"),
                    ("wind-turbine-alert", "wind-turbine-alert"),
                    ("wind-turbine-check", "wind-turbine-check"),
                    ("window-close", "window-close"),
                    ("window-closed", "window-closed"),
                    ("window-closed-variant", "window-closed-variant"),
                    ("window-maximize", "window-maximize"),
                    ("window-minimize", "window-minimize"),
                    ("window-open", "window-open"),
                    ("window-open-variant", "window-open-variant"),
                    ("window-restore", "window-restore"),
                    ("window-shutter", "window-shutter"),
                    ("window-shutter-alert", "window-shutter-alert"),
                    ("window-shutter-auto", "window-shutter-auto"),
                    ("window-shutter-cog", "window-shutter-cog"),
                    ("window-shutter-open", "window-shutter-open"),
                    ("window-shutter-settings", "window-shutter-settings"),
                    ("windsock", "windsock"),
                    ("wiper", "wiper"),
                    ("wiper-wash", "wiper-wash"),
                    ("wiper-wash-alert", "wiper-wash-alert"),
                    ("wizard-hat", "wizard-hat"),
                    ("wordpress", "wordpress"),
                    ("wrap", "wrap"),
                    ("wrap-disabled", "wrap-disabled"),
                    ("wrench", "wrench"),
                    ("wrench-check", "wrench-check"),
                    ("wrench-check-outline", "wrench-check-outline"),
                    ("wrench-clock", "wrench-clock"),
                    ("wrench-clock-outline", "wrench-clock-outline"),
                    ("wrench-cog", "wrench-cog"),
                    ("wrench-cog-outline", "wrench-cog-outline"),
                    ("wrench-outline", "wrench-outline"),
                    ("wunderlist", "wunderlist"),
                    ("xamarin", "xamarin"),
                    ("xamarin-outline", "xamarin-outline"),
                    ("xda", "xda"),
                    ("xing", "xing"),
                    ("xing-circle", "xing-circle"),
                    ("xml", "xml"),
                    ("xmpp", "xmpp"),
                    ("y-combinator", "y-combinator"),
                    ("yahoo", "yahoo"),
                    ("yammer", "yammer"),
                    ("yeast", "yeast"),
                    ("yelp", "yelp"),
                    ("yin-yang", "yin-yang"),
                    ("yoga", "yoga"),
                    ("youtube", "youtube"),
                    ("youtube-gaming", "youtube-gaming"),
                    ("youtube-studio", "youtube-studio"),
                    ("youtube-subscription", "youtube-subscription"),
                    ("youtube-tv", "youtube-tv"),
                    ("yurt", "yurt"),
                    ("z-wave", "z-wave"),
                    ("zend", "zend"),
                    ("zigbee", "zigbee"),
                    ("zip-box", "zip-box"),
                    ("zip-box-outline", "zip-box-outline"),
                    ("zip-disk", "zip-disk"),
                    ("zodiac-aquarius", "zodiac-aquarius"),
                    ("zodiac-aries", "zodiac-aries"),
                    ("zodiac-cancer", "zodiac-cancer"),
                    ("zodiac-capricorn", "zodiac-capricorn"),
                    ("zodiac-gemini", "zodiac-gemini"),
                    ("zodiac-leo", "zodiac-leo"),
                    ("zodiac-libra", "zodiac-libra"),
                    ("zodiac-pisces", "zodiac-pisces"),
                    ("zodiac-sagittarius", "zodiac-sagittarius"),
                    ("zodiac-scorpio", "zodiac-scorpio"),
                    ("zodiac-taurus", "zodiac-taurus"),
                    ("zodiac-virgo", "zodiac-virgo"),
                ],
                default="information-outline",
                max_length=50,
                verbose_name="Icon",
            ),
        ),
    ]
